
import { createWidget, isEmpty, isNotEmptyArray, Form, TextEditor, TextValueCombo, TextAreaEditor, VerticalLayout, Button, Decorators, Widget } from "@fui/core"


@Decorators.shortcut()
export class FormDemo extends Widget {
    static xtype = "demo.form";

    props = { baseCls: "demo-form" };

    render() {
        const widget = createWidget({
            type: Form.xtype,
            width: 300,
            labelWidth: 100,
            items: [
                {
                    validate(v) {
                        return v !== "a" && v !== "";
                    },
                    tip(v) {
                        if (isEmpty(v)) {
                            return "不能为空";
                        }

                        return "不合法格式";
                    },
                    label: "E-mail",
                    el: {
                        type: TextEditor.xtype,
                        watermark: "输入a报错",
                        allowBlank: true,
                    },
                },
                {
                    validate(v) {
                        return isNotEmptyArray(v);
                    },
                    tip() {
                        return "不能为空";
                    },
                    label: "性别",
                    el: {
                        type: TextValueCombo.xtype,
                        text: "请选择",
                        items: [
                            {
                                text: "男",
                                value: 1,
                            },
                            {
                                text: "女",
                                value: 2,
                            }
                        ],
                    },
                },
                {
                    validate(v) {
                        return v !== "";
                    },
                    tip() {
                        return "不能为空";
                    },
                    label: "姓名",
                    el: {
                        type: TextEditor.xtype,
                        watermark: "输入姓名",
                        allowBlank: true,
                    },
                },
                {
                    validate(v) {
                        return v !== "";
                    },
                    tip() {
                        return "不能为空";
                    },
                    label: "姓名",
                    el: {
                        type: TextAreaEditor.xtype,
                        cls: "bi-border",
                        watermark: "输入简介",
                        allowBlank: true,
                        height: 200,
                    },
                }
            ],
            layout: {
                type: VerticalLayout.xtype,
                vgap: 30,
            },
        });

        return {
            type: VerticalLayout.xtype,
            hgap: 200,
            vgap: 10,
            items: [
                widget,
                {
                    type: Button.xtype,
                    text: "提交",
                    handler() {
                        widget.validate();

                        console.log(widget.getValue());
                    },
                }
            ],
        };
    }
}

