
import { VerticalLayout, TextButton, BarPopover, Label, ButtonGroup, Decorators, Widget, UUID, map, range, Popovers } from "@fui/core"



@Decorators.shortcut()
export class PopoverDemo extends Widget {
    static xtype = "demo.popover";

    props = { baseCls: "demo-func" };

    render() {
        const id = UUID();
        let body;

        return {
            type: VerticalLayout.xtype,
            vgap: 10,
            items: [
                {
                    type: TextButton.xtype,
                    text: "点击弹出Popover(normal size & fixed)",
                    height: 30,
                    handler() {
                        Popovers.remove(id);
                        Popovers.create(id, {
                            type: BarPopover.xtype,
                            size: "normal",
                            header: {
                                type: Label.xtype,
                                text: "这个是header",
                            },
                            body: {
                                type: Label.xtype,
                                text: "这个是body",
                            },
                        }).open(id);
                    },
                },
                {
                    type: TextButton.xtype,
                    text: "点击弹出Popover(small size & fixed)",
                    height: 30,
                    handler() {
                        Popovers.remove(id);
                        Popovers.create(id, {
                            type: BarPopover.xtype,
                            size: "small",
                            header: {
                                type: Label.xtype,
                                text: "这个是header",
                            },
                            body: {
                                type: Label.xtype,
                                text: "这个是body",
                            },
                        }).open(id);
                    },
                },
                {
                    type: TextButton.xtype,
                    text: "点击弹出Popover(big size & fixed)",
                    height: 30,
                    handler() {
                        Popovers.remove(id);
                        Popovers.create(id, {
                            type: BarPopover.xtype,
                            size: "big",
                            header: {
                                type: Label.xtype,
                                text: "这个是header",
                            },
                            body: {
                                type: Label.xtype,
                                text: "这个是body",
                            },
                        }).open(id);
                    },
                },
                {
                    type: TextButton.xtype,
                    text: "点击弹出Popover(normal size & adapt body区域高度是300)",
                    height: 30,
                    handler() {
                        Popovers.remove(id);
                        Popovers.create(id, {
                            type: BarPopover.xtype,
                            size: "normal",
                            logic: {
                                dynamic: true,
                            },
                            header: {
                                type: Label.xtype,
                                text: "这个是header",
                            },
                            body: {
                                type: VerticalLayout.xtype,
                                items: [
                                    {
                                        type: ButtonGroup.xtype,
                                        ref() {
                                            body = this;
                                        },
                                        items: map(range(0, 10), () => {
                                            return {
                                                type: Label.xtype,
                                                text: "1",
                                                height: 30,
                                            };
                                        }),
                                        layouts: [
                                            {
                                                type: VerticalLayout.xtype,
                                            }
                                        ],
                                    }
                                ],
                            },
                        }).open(id);
                    },
                },
                {
                    type: TextButton.xtype,
                    text: "点击弹出Popover(small size & adapt body区域高度是900)",
                    height: 30,
                    handler() {
                        Popovers.remove(id);
                        Popovers.create(id, {
                            type: BarPopover.xtype,
                            size: "small",
                            logic: {
                                dynamic: true,
                            },
                            header: {
                                type: Label.xtype,
                                text: "这个是header",
                            },
                            body: {
                                type: VerticalLayout.xtype,
                                items: [
                                    {
                                        type: ButtonGroup.xtype,
                                        ref() {
                                            body = this;
                                        },
                                        items: map(range(0, 30), () => {
                                            return {
                                                type: Label.xtype,
                                                text: "1",
                                                height: 30,
                                            };
                                        }),
                                        layouts: [
                                            {
                                                type: VerticalLayout.xtype,
                                            }
                                        ],
                                    }
                                ],
                            },
                        }).open(id);
                    },
                },
                {
                    type: TextButton.xtype,
                    text: "点击弹出Popover(custom)",
                    height: 30,
                    handler() {
                        Popovers.remove(id);
                        Popovers.create(id, {
                            width: 400,
                            height: 300,
                            header: {
                                type: Label.xtype,
                                text: "这个是header",
                            },
                            body: {
                                type: Label.xtype,
                                text: "这个是body",
                            },
                            footer: {
                                type: Label.xtype,
                                text: "这个是footer",
                            },
                        }).open(id);
                    },
                },
                {
                    type: TextButton.xtype,
                    height: 30,
                    text: "弹出一个高度动态的popover层, 此弹出层指定size为small, 但是高度随内容自适应，自适应支持的最大高度为默认为600px",
                    handler() {
                        const id = "弹出层id1";
                        Popovers.create(id, {
                            // String或者是json都行
                            header: "弹出层标题",
                            logic: {
                                dynamic: true,
                                maxHeight: 700,
                            },
                            size: "small",
                            body: {
                                type: VerticalLayout.xtype,
                                items: map(range(0, 50), (idx, v) => {
                                    return {
                                        type: Label.xtype,
                                        text: "弹出层内容",
                                    };
                                }),
                            },
                            footer: {
                                type: Label.xtype,
                                text: "这个是footer",
                            },
                        }).open(id);
                    },
                }
            ],
        };
    }
}

