import {
    Bubbles,
    Msg,
    Widget,
    createWidget,
    each,
    FloatLeftLayout,
    Button,
    VerticalLayout,
    Label,
    Decorators,
    HorizontalAutoLayout
} from "@fui/core";

@Decorators.shortcut()
export class TipsDemo extends Widget {
    static xtype = "demo.tips";

    props = { baseCls: "demo-tips" };

    render() {
        const btns = [];
        const bubble = createWidget({
            type: FloatLeftLayout.xtype,
            items: [
                {
                    el: {
                        type: Button.xtype,
                        text: "bubble测试",
                        height: 30,
                        handler() {
                            Bubbles.show("singleBubble1", "bubble测试", this);
                            btns.push("singleBubble1");
                        }
                    }
                },
                {
                    el: {
                        type: Button.xtype,
                        text: "bubble测试(居中显示)",
                        height: 30,
                        handler() {
                            Bubbles.show("singleBubble2", "bubble测试", this, {
                                offsetStyle: "center"
                            });
                            btns.push("singleBubble2");
                        }
                    }
                },
                {
                    el: {
                        type: Button.xtype,
                        text: "bubble测试(右边显示)",
                        height: 30,
                        handler() {
                            Bubbles.show("singleBubble3", "bubble测试", this, {
                                offsetStyle: "right"
                            });
                            btns.push("singleBubble3");
                        }
                    }
                },
                {
                    el: {
                        type: Button.xtype,
                        text: "隐藏所有 bubble",
                        height: 30,
                        cls: "layout-bg2",
                        handler() {
                            each(btns, (index, value) => {
                                Bubbles.hide(value);
                            });
                        }
                    }
                }
            ],
            hgap: 20
        });

        const title = createWidget({
            type: VerticalLayout.xtype,
            items: [
                {
                    type: Label.xtype,
                    cls: "layout-bg1",
                    height: 50,
                    title: "title提示",
                    text: "移上去有title提示",
                    textAlign: "center"
                },
                {
                    type: Label.xtype,
                    cls: "layout-bg6",
                    height: 50,
                    disabled: true,
                    warningTitle: "title错误提示",
                    text: "移上去有title错误提示",
                    textAlign: "center"
                },
                {
                    type: Label.xtype,
                    cls: "layout-bg2",
                    height: 50,
                    disabled: true,
                    tipType: "success",
                    title: "自定义title提示效果",
                    warningTitle: "自定义title提示效果",
                    text: "自定义title提示效果",
                    textAlign: "center"
                }
            ],
            hgap: 20,
            vgap: 20
        });

        const toast = createWidget({
            type: VerticalLayout.xtype,
            items: [
                {
                    el: {
                        type: Button.xtype,
                        text: "简单Toast测试",
                        height: 30,
                        handler() {
                            Msg.toast("这是一条简单的数据");
                        }
                    }
                },
                {
                    el: {
                        type: Button.xtype,
                        text: "很长的Toast测试",
                        height: 30,
                        handler() {
                            Msg.toast(
                                "这是一条很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长很长的数据"
                            );
                        }
                    }
                },
                {
                    el: {
                        type: Button.xtype,
                        text: "非常长的Toast测试",
                        height: 30,
                        handler() {
                            Msg.toast(
                                "这是一条非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长非常长的数据"
                            );
                        }
                    }
                },
                {
                    el: {
                        type: Button.xtype,
                        text: "错误提示Toast测试",
                        level: "warning",
                        height: 30,
                        handler() {
                            Msg.toast("错误提示Toast测试", "warning");
                        }
                    }
                }
            ],
            vgap: 20
        });

        return {
            type: HorizontalAutoLayout.xtype,
            vgap: 20,
            hgap: 20,
            items: [bubble, title, toast]
        };
    }
}
