import { CONSTANTS } from "@/config/constant";

import { SingleSelectCombo, AbsoluteLayout, Button,  Decorators, Widget, Func, createWidget, bind, each, makeObject, filter, delay } from "@fui/core"


@Decorators.shortcut()
export class SingleSelectComboDemo extends Widget {
    static xtype = "demo.single_select_combo";

    props = { baseCls: "demo-single-select-combo" };

    _createSingleSelectCombo() {
        const self = this;
        const widget = createWidget({
            type: SingleSelectCombo.xtype,
            itemsCreator: bind(this._itemsCreator, this),
            width: 200,
            ref() {
                self.SingleSelectCombo = this;
            },
            value: "柳州市针织总厂",
        });

        widget.populate();

        widget.on(SingleSelectCombo.EVENT_CONFIRM, function () {
            Msg.toast(JSON.stringify(this.getValue()));
        });

        return widget;
    }

    _getItemsByTimes(items, times) {
        const res = [];
        for (let i = (times - 1) * 100; items[i] && i < times * 100; i++) {
            res.push(items[i]);
        }

        return res;
    }

    _hasNextByTimes(items, times) {
        return times * 100 < items.length;
    }

    _itemsCreator(options, callback) {
        const self = this;
        let items = CONSTANTS.ITEMS;
        const keywords = (options.keywords || []).slice();
        if (options.keyword) {
            keywords.push(options.keyword);
        }
        each(keywords, (i, kw) => {
            const search = Func.getSearchResult(items, kw);
            items = search.match.concat(search.find);
        });
        if (options.selectedValues) {
            // 过滤
            const filterItems = makeObject(options.selectedValues, true);
            items = filter(items, (i, ob) => !filterItems[ob.value]);
        }
        if (options.type == SingleSelectCombo.REQ_GET_ALL_DATA) {
            callback({
                items,
            });

            return;
        }
        if (options.type == SingleSelectCombo.REQ_GET_DATA_LENGTH) {
            callback({ count: items.length });

            return;
        }
        delay(() => {
            callback({
                items: self._getItemsByTimes(items, options.times),
                hasNext: self._hasNextByTimes(items, options.times),
            });
        }, 1000);
    }

    render() {
        const self = this;

        return {
            type: AbsoluteLayout.xtype,
            scrolly: false,
            items: [
                {
                    el: this._createSingleSelectCombo(),
                    right: "50%",
                    top: 10,
                },
                {
                    el: {
                        type: Button.xtype,
                        text: "setValue(\"柳州市针织总厂\")",
                        handler() {
                            self.SingleSelectCombo.setValue("柳州市针织总厂");
                        },
                    },
                }
            ],
        };
    }
}

