import {
    SingleSlider,
    SingleSliderNormal,
    SingleSliderLabel,
    IntervalSlider,
    VerticalLayout,
    CenterAdaptLayout,
     Decorators,
    Widget,
    createWidget
} from "@fui/core";

@Decorators.shortcut()
export class SliderDemo extends Widget {
    static xtype = "demo.slider";

    props = {
        baseCls: "demo-slider",
        width: 300,
        height: 50,
        min: 0,
        max: 100
    };

    render() {
        const self = this,
            o = this.options;
        const singleSlider = createWidget({
            type: SingleSlider.xtype,
            digit: 0,
            width: o.width,
            height: o.height,
            cls: "layout-bg-white",
            value: 30,
            min: 10,
            max: 100
        });
        singleSlider.on(SingleSlider.EVENT_CHANGE, function () {
            console.log(this.getValue());
        });

        const normalSingleSlider = createWidget({
            type: SingleSliderNormal.xtype,
            width: o.width,
            height: 30,
            cls: "layout-bg-white",
            min: o.min,
            max: o.max,
            value: 30
        });
        normalSingleSlider.on(SingleSliderNormal.EVENT_DRAG, function () {
            console.log(this.getValue());
        });

        const singleSliderLabel = createWidget({
            type: SingleSliderLabel.xtype,
            width: o.width,
            height: o.height,
            digit: 0,
            unit: "个",
            cls: "layout-bg-white",
            min: o.min,
            max: o.max,
            value: 10
        });

        const intervalSlider = createWidget({
            type: IntervalSlider.xtype,
            width: o.width,
            digit: 0,
            cls: "layout-bg-white",
            min: o.min,
            max: o.max,
            value: {
                min: 10,
                max: 70
            }
        });

        const intervalSliderLabel = createWidget({
            type: IntervalSlider.xtype,
            width: o.width,
            unit: "px",
            cls: "layout-bg-white",
            digit: 1,
            min: 0,
            max: 120,
            value: {
                min: 60,
                max: 120
            }
        });

        return {
            type: VerticalLayout.xtype,
            element: this,
            items: [
                {
                    type: CenterAdaptLayout.xtype,
                    items: [
                        {
                            el: singleSlider
                        }
                    ]
                },
                {
                    type: CenterAdaptLayout.xtype,
                    items: [
                        {
                            el: normalSingleSlider
                        }
                    ]
                },
                {
                    type: CenterAdaptLayout.xtype,
                    items: [
                        {
                            el: singleSliderLabel
                        }
                    ]
                },
                {
                    type: CenterAdaptLayout.xtype,
                    items: [
                        {
                            el: intervalSlider
                        }
                    ]
                },
                {
                    type: CenterAdaptLayout.xtype,
                    items: [
                        {
                            el: intervalSliderLabel
                        }
                    ]
                }
            ],
            vgap: 20
        };
    }
}
