import {
    CenterLayout,
    shortcut,
    Widget,
    Controller,
    extend,
    isFunction,
    isKey,
    isArray,
    map,
    stripEL,
    deepClone,
    formatEL,
    isEmpty,
    each,
    createWidget
} from "@/core";

@shortcut()
export class VirtualGroup extends Widget {
    static xtype = "bi.virtual_group";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-virtual-group",
            items: [],
            layouts: [
                {
                    type: CenterLayout.xtype,
                    hgap: 0,
                    vgap: 0,
                }
            ],
        });
    }

    render() {
        const { items: optionsItems, value } = this.options;
        const items = isFunction(optionsItems)
            ? this.__watch(optionsItems, (context, newValue) => {
                this.populate(newValue);
            })
            : optionsItems;
        this.populate(items);
        this.options.value = isFunction(value)
            ? this.__watch(value, (context, newValue) => {
                this.setValue(newValue);
            })
            : value;
        if (isKey(value)) {
            this.setValue(value);
        }
    }

    _packageBtns(items) {
        const o = this.options;
        const btMap = (this.buttonMap = {});
        const layouts = isArray(o.layouts) ? o.layouts : [o.layouts];
        for (let i = layouts.length - 1; i > 0; i--) {
            items = map(items, (k, it) => {
                const el = stripEL(it);

                return extend({}, layouts[i], {
                    items: [
                        extend({}, layouts[i].el, {
                            el: extend(
                                {
                                    ref: _ref => {
                                        if (isKey(el.value)) {
                                            btMap[el.value] = _ref;
                                        }
                                    },
                                },
                                el
                            ),
                        })
                    ],
                });
            });
        }

        return items;
    }

    _packageLayout(items) {
        const o = this.options;
        const layouts = isArray(o.layouts) ? o.layouts : [o.layouts];
        const layout = deepClone(layouts[0]);

        let lay = formatEL(layout).el;
        while (lay && lay.items && !isEmpty(lay.items)) {
            lay = formatEL(lay.items[0]).el;
        }
        lay.items = items;

        return layout;
    }

    addItems(items) {
        this.layouts.addItems(items, this);
    }

    prependItems(items) {
        this.layouts.prependItems(items, this);
    }

    setValue(v) {
        v = isArray(v) ? v : [v];
        each(this.buttonMap, (key, item) => {
            if (item) {
                if (v.deepContains(key)) {
                    item.setSelected && item.setSelected(true);
                } else {
                    item.setSelected && item.setSelected(false);
                }
            }
        });
    }

    getNotSelectedValue() {
        const v = [];
        each(this.buttonMap, (i, item) => {
            if (item) {
                if (item.isEnabled() && !(item.isSelected && item.isSelected())) {
                    v.push(item.getValue());
                }
            }
        });

        return v;
    }

    getNodeByValue(value) {
        return this.buttonMap[value];
    }

    /**
     * 滚动到指定的节点
     */
    scrollToValue(value, scrollIntoViewOptions) {
        const node = this.getNodeByValue(value);
        if (node) {
            node.element[0].scrollIntoView(scrollIntoViewOptions);
        }
    }

    getValue() {
        const v = [];
        each(this.buttonMap, (i, item) => {
            if (item) {
                if (item.isEnabled() && item.isSelected && item.isSelected()) {
                    v.push(item.getValue());
                }
            }
        });

        return v;
    }

    populate(items) {
        items = items || [];
        this.options.items = items;
        items = this._packageBtns(items);
        if (!this.layouts) {
            this.layouts = createWidget(extend({ element: this }, this._packageLayout(items)));
        } else {
            this.layouts.populate(items, {
                context: this,
            });
        }
    }
}
