import { ButtonGroup } from "../combination/group.button";
import {
    VerticalLayout,
    AbsoluteLayout,
    Layout,
    CenterLayout,
    Widget,
    shortcut,
    extend,
    Controller,
    createWidget,
    createItems,
    clamp,
    Direction,
    zIndex_popup,
    pixFormat,
    LogicFactory,
    Events
} from "@/core";

/**
 * 下拉框弹出层, zIndex在1000w
 * @class PopupView
 * @extends Widget
 */

@shortcut()
export class PopupView extends Widget {
    _const = {
        TRIANGLE_LENGTH: 12,
    };

    static xtype = "bi.popup_view";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig(props) {
        return extend(super._defaultConfig(...arguments), {
            _baseCls: `bi-popup-view${props.primary ? " bi-primary" : ""}`,
            // 品牌色
            primary: false,
            maxWidth: "auto",
            minWidth: 100,
            // maxHeight: 200,
            minHeight: 24,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            vgap: 0,
            hgap: 0,
            innerVgap: 0,
            innerHgap: 0,
            showArrow: false,
            direction: Direction.Top, // 工具栏的方向
            stopEvent: false, // 是否停止mousedown、mouseup事件
            stopPropagation: false, // 是否停止mousedown、mouseup向上冒泡
            logic: {
                dynamic: true,
            },

            tool: false, // 自定义工具栏
            tabs: [], // 导航栏
            buttons: [], // toolbar栏

            el: {
                type: ButtonGroup.xtype,
                items: [],
                chooseType: 0,
                behaviors: {},
                layouts: [
                    {
                        type: VerticalLayout.xtype,
                    }
                ],
            },
        });
    }
    render() {
        const {
            minWidth,
            maxWidth,
            stopPropagation,
            stopEvent,
            direction,
            logic,
            lgap,
            rgap,
            tgap,
            bgap,
            vgap,
            hgap,
            primary,
            showArrow,
        } = this.options;
        function fn(e) {
            e.stopPropagation();
        }
        function stop(e) {
            e.stopEvent();

            return false;
        }
        this.element
            .css({
                "z-index": zIndex_popup,
                "min-width": pixFormat(minWidth),
                "max-width": pixFormat(maxWidth),
            })
            .bind({ click: fn });

        this.element.bind("mousewheel", fn);

        stopPropagation && this.element.bind({ mousedown: fn, mouseup: fn, mouseover: fn });
        stopEvent && this.element.bind({ mousedown: stop, mouseup: stop, mouseover: stop });
        this.tool = this._createTool();
        this.tab = this._createTab();
        this.view = this._createView();
        this.toolbar = this._createToolBar();

        this.view.on(Controller.EVENT_CHANGE, (type, ...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, type, ...args);
            if (type === Events.CLICK) {
                this.fireEvent(PopupView.EVENT_CHANGE);
            }
        });

        createWidget(
            extend(
                {
                    element: this,
                },
                LogicFactory.createLogic(
                    LogicFactory.createLogicTypeByDirection(direction),
                    extend({}, logic, {
                        scrolly: false,
                        lgap,
                        rgap,
                        tgap,
                        bgap,
                        vgap,
                        hgap,
                        items: LogicFactory.createLogicItemsByDirection(
                            direction,
                            extend(
                                {
                                    cls: `list-view-outer bi-card list-view-shadow${primary ? " bi-primary" : ""}`,
                                },
                                LogicFactory.createLogic(
                                    LogicFactory.createLogicTypeByDirection(direction),
                                    extend({}, logic, {
                                        items: LogicFactory.createLogicItemsByDirection(
                                            direction,
                                            this.tool,
                                            this.tab,
                                            this.view,
                                            this.toolbar
                                        ),
                                    })
                                )
                            )
                        ),
                    })
                )
            )
        );
        if (showArrow) {
            this.arrow = createWidget({
                type: AbsoluteLayout.xtype,
                cls: "bi-bubble-arrow",
                items: [
                    {
                        type: Layout.xtype,
                        cls: "bubble-arrow",
                    }
                ],
            });
            this.arrowWrapper = createWidget({
                type: AbsoluteLayout.xtype,
                cls: "bi-bubble-arrow-wrapper",
                items: [
                    {
                        el: this.arrow,
                    }
                ],
            });
            // 因为三角符号的原因位置变大了，需要占位
            this.placeholder = createWidget({
                type: Layout.xtype,
            });
            createWidget({
                type: AbsoluteLayout.xtype,
                element: this,
                items: [
                    {
                        el: this.arrowWrapper,
                        left: 0,
                        top: 0,
                    },
                    {
                        el: this.placeholder,
                    }
                ],
            });
        }
    }
    _createView() {
        const { el, value, minHeight, innerVgap, innerHgap } = this.options;
        this.button_group = createWidget(el, { type: ButtonGroup.xtype, value });
        this.button_group.element.css({
            "min-height": pixFormat(minHeight),
            "padding-top": pixFormat(innerVgap),
            "padding-bottom": pixFormat(innerVgap),
            "padding-left": pixFormat(innerHgap),
            "padding-right": pixFormat(innerHgap),
        });

        return this.button_group;
    }

    _createTool() {
        const { tool } = this.options;
        if (false === tool) {
            return;
        }

        return createWidget(tool);
    }

    _createTab() {
        const { tabs, value } = this.options;
        if (tabs.length === 0) {
            return;
        }

        return createWidget({
            type: CenterLayout.xtype,
            cls: "list-view-tab",
            height: 25,
            items: tabs,
            value,
        });
    }

    _createToolBar() {
        const { buttons } = this.options;
        if (buttons.length === 0) {
            return;
        }

        return createWidget({
            type: CenterLayout.xtype,
            cls: "list-view-toolbar bi-high-light bi-split-top",
            height: 24,
            items: createItems(buttons, {
                once: false,
                shadow: true,
                isShadowShowingOnSelected: true,
            }),
        });
    }

    setDirection(direction, position) {
        const { showArrow, tgap, vgap, bgap, rgap, hgap, lgap } = this.options;
        if (showArrow) {
            let style = {},
                wrapperStyle = {},
                placeholderStyle = {};
            const adjustXOffset = position.adjustXOffset || 0;
            const adjustYOffset = position.adjustYOffset || 0;
            const bodyBounds = Widget._renderEngine.createElement("body").bounds();
            const bodyWidth = bodyBounds.width;
            const bodyHeight = bodyBounds.height;
            const popupWidth = this.element.outerWidth();
            const popupHeight = this.element.outerHeight();
            const offset = position.offset;
            const offsetStyle = position.offsetStyle;
            const middle = offsetStyle === "center" || offsetStyle === "middle";

            const minLeft = Math.max(4, offset.left + 4 + popupWidth - bodyWidth);
            const minRight = Math.max(4, popupWidth - (offset.left + 4));
            const minTop = Math.max(4, offset.top + 4 + popupHeight - bodyHeight);
            const minBottom = Math.max(4, popupHeight - (offset.top + 4));

            const maxLeft = Math.min(popupWidth - 16 - 4, offset.left + position.width - 16 - 4);
            const maxRight = Math.min(popupWidth - 16 - 4, bodyWidth - (offset.left + position.width - 16 - 4));
            const maxTop = Math.min(popupHeight - 16 - 4, offset.top + position.height - 16 - 4);
            const maxBottom = Math.min(popupHeight - 16 - 4, bodyHeight - (offset.top + position.height - 16 - 4));
            switch (direction) {
                case "bottom":
                case "bottom,right":
                    direction = "bottom";
                    style = {
                    // 5表示留出一定的空间
                        left: clamp(((middle ? popupWidth : position.width) - adjustXOffset) / 2 - 8, minLeft, maxLeft),
                    };
                    wrapperStyle = {
                        top: tgap + vgap,
                        left: 0,
                        right: "",
                        bottom: "",
                    };
                    placeholderStyle = {
                        left: 0,
                        right: 0,
                        height: this._const.TRIANGLE_LENGTH,
                        top: -this._const.TRIANGLE_LENGTH,
                        bottom: "",
                    };
                    break;
                case "bottom,left":
                    direction = "bottom";
                    style = {
                        right: clamp(
                            ((middle ? popupWidth : position.width) + adjustXOffset) / 2 - 8,
                            minRight,
                            maxRight
                        ),
                    };
                    wrapperStyle = {
                        top: bgap + vgap,
                        left: "",
                        right: 0,
                        bottom: "",
                    };
                    placeholderStyle = {
                        left: 0,
                        right: 0,
                        height: this._const.TRIANGLE_LENGTH,
                        top: -this._const.TRIANGLE_LENGTH,
                        bottom: "",
                    };
                    break;
                case "top":
                case "top,right":
                    direction = "top";
                    style = {
                        left: clamp(((middle ? popupWidth : position.width) - adjustXOffset) / 2 - 8, minLeft, maxLeft),
                    };
                    wrapperStyle = {
                        bottom: bgap + vgap,
                        left: 0,
                        right: "",
                        top: "",
                    };
                    placeholderStyle = {
                        left: 0,
                        right: 0,
                        height: this._const.TRIANGLE_LENGTH,
                        top: "",
                        bottom: -this._const.TRIANGLE_LENGTH,
                    };
                    break;
                case "top,left":
                    direction = "top";
                    style = {
                        right: clamp(
                            ((middle ? popupWidth : position.width) + adjustXOffset) / 2 - 8,
                            minRight,
                            maxRight
                        ),
                    };
                    wrapperStyle = {
                        bottom: bgap + vgap,
                        right: 0,
                        left: "",
                        top: "",
                    };
                    placeholderStyle = {
                        left: 0,
                        right: 0,
                        height: this._const.TRIANGLE_LENGTH,
                        top: "",
                        bottom: -this._const.TRIANGLE_LENGTH,
                    };
                    break;
                case "left":
                case "left,bottom":
                    direction = "left";
                    style = {
                        top: clamp(((middle ? popupHeight : position.height) - adjustYOffset) / 2 - 8, minTop, maxTop),
                    };
                    wrapperStyle = {
                        right: rgap + hgap,
                        top: 0,
                        bottom: "",
                        left: "",
                    };
                    placeholderStyle = {
                        top: 0,
                        bottom: 0,
                        width: this._const.TRIANGLE_LENGTH,
                        right: -this._const.TRIANGLE_LENGTH,
                        left: "",
                    };
                    break;
                case "left,top":
                    direction = "left";
                    style = {
                        bottom: clamp(
                            ((middle ? popupHeight : position.height) + adjustYOffset) / 2 - 8,
                            minBottom,
                            maxBottom
                        ),
                    };
                    wrapperStyle = {
                        right: rgap + hgap,
                        bottom: 0,
                        top: "",
                        left: "",
                    };
                    placeholderStyle = {
                        top: 0,
                        bottom: 0,
                        width: this._const.TRIANGLE_LENGTH,
                        right: -this._const.TRIANGLE_LENGTH,
                        left: "",
                    };
                    break;
                case "right":
                case "right,bottom":
                    direction = "right";
                    style = {
                        top: clamp(((middle ? popupHeight : position.height) - adjustYOffset) / 2 - 8, minTop, maxTop),
                    };
                    wrapperStyle = {
                        left: lgap + hgap,
                        top: 0,
                        bottom: "",
                        right: "",
                    };
                    placeholderStyle = {
                        top: 0,
                        bottom: 0,
                        width: this._const.TRIANGLE_LENGTH,
                        left: -this._const.TRIANGLE_LENGTH,
                        right: "",
                    };
                    break;
                case "right,top":
                    direction = "right";
                    style = {
                        bottom: clamp(
                            ((middle ? popupHeight : position.height) + adjustYOffset) / 2 - 8,
                            minBottom,
                            maxBottom
                        ),
                    };
                    wrapperStyle = {
                        left: lgap + hgap,
                        bottom: 0,
                        top: "",
                        right: "",
                    };
                    placeholderStyle = {
                        top: 0,
                        bottom: 0,
                        width: this._const.TRIANGLE_LENGTH,
                        left: -this._const.TRIANGLE_LENGTH,
                        right: "",
                    };
                    break;
                case "right,innerRight":
                    break;
                case "right,innerLeft":
                    break;
                case "innerRight":
                    break;
                case "innerLeft":
                    break;
                default:
                    break;
            }
            this.element
                .removeClass("left")
                .removeClass("right")
                .removeClass("top")
                .removeClass("bottom")
                .addClass(direction);
            this.arrow.element.css(style);
            this.arrowWrapper.element.css(wrapperStyle);
            this.placeholder.element.css(placeholderStyle);
        }
    }

    getView() {
        return this.view;
    }

    populate(items) {
        this.view.populate(...arguments);
    }

    resetWidth(w) {
        this.options.width = w;
        this.element.width(w);
    }

    resetHeight(h) {
        const tbHeight = this.toolbar ? this.toolbar.attr("height") || 24 : 0,
            tabHeight = this.tab ? this.tab.attr("height") || 24 : 0,
            toolHeight = ((this.tool && this.tool.attr("height")) || 24) * (this.tool && this.tool.isVisible() ? 1 : 0);
        const resetHeight = h - tbHeight - tabHeight - toolHeight - 2 * this.options.innerVgap;
        this.view.resetHeight
            ? this.view.resetHeight(resetHeight)
            : this.view.element.css({ "max-height": pixFormat(resetHeight) });
    }

    setValue(selectedValues) {
        this.tab && this.tab.setValue(selectedValues);
        this.view.setValue(selectedValues);
    }

    getValue() {
        return this.view.getValue();
    }
}
