import { ButtonGroup } from "../combination";
import { VerticalLayout, Layout, shortcut, extend, createWidget, Controller, isNotEmptyArray, Selection, i18nText, Events } from "@/core";
import { Pane } from "../1.pane";

/**
 * 搜索面板
 *
 * Created by GUY on 2015/9/28.
 * @class SearcherView
 * @extends Pane
 */

@shortcut()
export class SearcherView extends Pane {
    static xtype = "bi.searcher_view";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-searcher-view bi-card`,
            tipText: i18nText("BI-No_Select"),
            chooseType: Selection.Single,

            matcher: {
                // 完全匹配的构造器
                type: ButtonGroup.xtype,
                behaviors: {
                    redmark: () => true,
                },
                items: [],
                layouts: [
                    {
                        type: VerticalLayout.xtype,
                    }
                ],
            },
            searcher: {
                type: ButtonGroup.xtype,
                behaviors: {
                    redmark: () => true,
                },
                items: [],
                layouts: [
                    {
                        type: VerticalLayout.xtype,
                    }
                ],
            },
        });
    }
    render() {
        const { matcher, chooseType, value, searcher } = this.options;

        this.matcher = createWidget(matcher, {
            type: ButtonGroup.xtype,
            chooseType,
            behaviors: {
                redmark: () => true,
            },
            layouts: [
                {
                    type: VerticalLayout.xtype,
                }
            ],
            value,
        });
        this.matcher.on(Controller.EVENT_CHANGE, (type, val, ob, ...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, type, val, ob, ...args);
            if (type === Events.CLICK) {
                this.fireEvent(SearcherView.EVENT_CHANGE, val, ob);
            }
        });
        this.spliter = createWidget({
            type: VerticalLayout.xtype,
            height: 1,
            hgap: 10,
            items: [
                {
                    type: Layout.xtype,
                    height: 1,
                    cls: "searcher-view-spliter bi-background",
                }
            ],
        });
        this.searcher = createWidget(searcher, {
            type: ButtonGroup.xtype,
            chooseType,
            behaviors: {
                redmark: () => true,
            },
            layouts: [
                {
                    type: VerticalLayout.xtype,
                }
            ],
            value,
        });
        this.searcher.on(Controller.EVENT_CHANGE, (type, val, ob, ...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, type, val, ob, ...args);
            if (type === Events.CLICK) {
                this.fireEvent(SearcherView.EVENT_CHANGE, val, ob);
            }
        });

        createWidget({
            type: VerticalLayout.xtype,
            element: this,
            items: [this.matcher, this.spliter, this.searcher],
        });
    }

    startSearch() {}

    stopSearch() {}

    setValue(v) {
        this.matcher.setValue(v);
        this.searcher.setValue(v);
    }

    getValue() {
        return this.matcher.getValue().concat(this.searcher.getValue());
    }

    populate(searchResult, matchResult, keyword) {
        searchResult || (searchResult = []);
        matchResult || (matchResult = []);
        this.setTipVisible(searchResult.length + matchResult.length === 0);
        this.spliter.setVisible(isNotEmptyArray(matchResult) && isNotEmptyArray(searchResult));
        this.matcher.populate(matchResult, keyword);
        this.searcher.populate(searchResult, keyword);
    }

    empty() {
        this.searcher.empty();
        this.matcher.empty();
    }

    hasMatched() {
        return this.matcher.getAllButtons().length > 0;
    }
}
