import { Img } from "../../img/img";
import { CenterAdaptLayout, AdaptiveLayout, shortcut, extend, isNumber, createWidget } from "@/core";
import { BasicButton } from "../button.basic";

/**
 * 图片的button
 *
 * Created by GUY on 2016/1/27.
 * @class ImageButton
 * @extends BasicButton
 */
@shortcut()
export class ImageButton extends BasicButton {
    static EVENT_CHANGE = "EVENT_CHANGE";
    static xtype = "bi.image_button";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-image-button`,
            src: "",
            iconWidth: "100%",
            iconHeight: "100%",
        });
    }

    render() {
        const o = this.options;
        this.image = createWidget({
            type: Img.xtype,
            width: o.iconWidth,
            height: o.iconHeight,
            src: o.src,
        });
        if (isNumber(o.iconWidth) || isNumber(o.iconHeight)) {
            createWidget({
                type: CenterAdaptLayout.xtype,
                element: this,
                items: [this.image],
            });
        } else {
            createWidget({
                type: AdaptiveLayout.xtype,
                element: this,
                items: [this.image],
                scrollable: false,
            });
        }
    }

    setWidth(w) {
        super.setWidth(...arguments);
        this.options.width = w;
    }

    setHeight(h) {
        super.setHeight(...arguments);
        this.options.height = h;
    }

    setImageWidth(w) {
        this.image.setWidth(w);
    }

    setImageHeight(h) {
        this.image.setHeight(h);
    }

    getImageWidth() {
        return this.image.element.width();
    }

    getImageHeight() {
        return this.image.element.height();
    }

    setSrc(src) {
        this.options.src = src;
        this.image.setSrc(src);
    }

    getSrc() {
        return this.image.getSrc();
    }

    doClick() {
        super.doClick(...arguments);
        if (this.isValid()) {
            this.fireEvent(ImageButton.EVENT_CHANGE, this);
        }
    }
}
