import { Label } from "../../label";
import { BasicButton } from "../button.basic";
import { shortcut, extend, createWidget, isArray } from "@/core";

/**
 * guy
 * 可以点击的一行文字
 * @class TextButton
 * @extends BasicButton
 * 文字button
 */
@shortcut()
export class TextButton extends BasicButton {
    static xtype = "bi.text_button";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-text-button`,
            textAlign: "center",
            whiteSpace: "nowrap",
            textWidth: null,
            textHeight: null,
            hgap: 0,
            lgap: 0,
            rgap: 0,
            vgap: 0,
            py: "",
        });
    }

    render() {
        const o = this.options;
        this.text = createWidget({
            type: Label.xtype,
            element: this,
            textAlign: o.textAlign,
            whiteSpace: o.whiteSpace,
            textWidth: o.textWidth,
            textHeight: o.textHeight,
            width: o.width,
            height: o.height,
            hgap: o.hgap,
            vgap: o.vgap,
            lgap: o.lgap,
            rgap: o.rgap,
            text: o.text,
            value: o.value,
            py: o.py,
            keyword: o.keyword,
        });
    }

    doClick() {
        super.doClick(...arguments);
        if (this.isValid()) {
            this.fireEvent(TextButton.EVENT_CHANGE, this.getValue(), this);
        }
    }

    doRedMark() {
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    doHighLight() {
        this.text.doHighLight(...arguments);
    }

    unHighLight() {
        this.text.unHighLight(...arguments);
    }

    setText(text) {
        super.setText(...arguments);
        text = isArray(text) ? text.join(",") : text;
        this.text.setText(text);
    }

    setStyle(style) {
        this.text.setStyle(style);
    }

    setValue(text) {
        super.setValue(...arguments);
        if (!this.isReadOnly()) {
            text = isArray(text) ? text.join(",") : text;
            this.text.setValue(text);
        }
    }
}
