import { VerticalAdaptLayout, extend, shortcut } from "@/core";
import { IconLabel, Label } from "../../label";
import { NodeButton } from "../button.node";

/**
 * guy
 * Created by GUY on 2015/9/9.
 * @class IconTextIconNode
 * @extends NodeButton
 */
@shortcut()
export class IconTextIconNode extends NodeButton {
    static xtype = "bi.icon_text_icon_node";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-icon-text-icon-node`,
            iconCls1: "close-ha-font",
            iconCls2: "close-ha-font",
            iconHeight: null,
            iconWidth: null,
            textHgap: 0,
            textVgap: 0,
            textLgap: 0,
            textRgap: 0,
        });
    }

    render() {
        const o = this.options;

        return {
            type: VerticalAdaptLayout.xtype,
            columnSize: [o.leftIconWrapperWidth || o.height, "fill", o.rightIconWrapperWidth || o.height],
            items: [
                {
                    type: IconLabel.xtype,
                    cls: o.iconCls1,
                    width: o.leftIconWrapperWidth || o.height,
                    height: o.height,
                    iconWidth: o.iconWidth,
                    iconHeight: o.iconHeight,
                },
                {
                    el: {
                        type: Label.xtype,
                        ref: _ref => {
                            this.text = _ref;
                        },
                        textAlign: "left",
                        hgap: o.textHgap,
                        vgap: o.textVgap,
                        lgap: o.textLgap,
                        rgap: o.textRgap,
                        text: o.text,
                        value: o.value,
                        keyword: o.keyword,
                        height: o.height,
                    },
                },
                {
                    type: IconLabel.xtype,
                    cls: o.iconCls2,
                    width: o.rightIconWrapperWidth || o.height,
                    height: o.height,
                    iconWidth: o.iconWidth,
                    iconHeight: o.iconHeight,
                }
            ],
        };
    }

    doClick() {
        super.doClick(...arguments);
        if (this.isValid()) {
            this.fireEvent(IconTextIconNode.EVENT_CHANGE, this.getValue(), this);
        }
    }

    doRedMark() {
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    setValue() {
        if (!this.isReadOnly()) {
            this.text.setValue(...arguments);
        }
    }

    getValue() {
        return this.text.getValue();
    }

    setText() {
        this.text.setText(...arguments);
    }

    getText() {
        return this.text.getText();
    }
}
