import { File } from "../input";
import { AbsoluteLayout, AdaptiveLayout, shortcut, Widget, createWidget, extend } from "@/core";

/**
 * 多文件
 *
 * Created by GUY on 2016/4/13.
 * @class MultifileEditor
 * @extends Single
 * @abstract
 */

@shortcut()
export class MultifileEditor extends Widget {
    static xtype = "bi.multifile_editor";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_UPLOADSTART = "EVENT_UPLOADSTART";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_PROGRESS = "EVENT_PROGRESS";
    static EVENT_UPLOADED = "EVENT_UPLOADED";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-multifile-editor`,
            multiple: false,
            maxSize: -1, // 1024 * 1024
            accept: "",
            url: "",
        });
    }

    render() {
        const { name, url, multiple, accept, maxSize, maxLength, title, errorText } = this.options;
        this.file = createWidget({
            type: File.xtype,
            cls: "multifile-editor",
            width: "100%",
            height: "100%",
            name,
            url,
            multiple,
            accept,
            maxSize,
            maxLength,
            title,
            errorText,
        });
        this.file.on(File.EVENT_CHANGE, (...args) => {
            this.fireEvent(MultifileEditor.EVENT_CHANGE, ...args);
        });
        this.file.on(File.EVENT_UPLOADSTART, (...args) => {
            this.fireEvent(MultifileEditor.EVENT_UPLOADSTART, ...args);
        });
        this.file.on(File.EVENT_ERROR, (...args) => {
            this.fireEvent(MultifileEditor.EVENT_ERROR, ...args);
        });
        this.file.on(File.EVENT_PROGRESS, (...args) => {
            this.fireEvent(MultifileEditor.EVENT_PROGRESS, ...args);
        });
        this.file.on(File.EVENT_UPLOADED, (...args) => {
            this.fireEvent(MultifileEditor.EVENT_UPLOADED, ...args);
        });

        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: {
                        type: AdaptiveLayout.xtype,
                        scrollable: false,
                        items: [this.file],
                    },
                    top: 0,
                    right: 0,
                    left: 0,
                    bottom: 0,
                }
            ],
        });
    }

    _reset() {
        this.file.reset();
    }

    setUrl(v) {
        this.file.setUrl(v);
    }

    setMaxFileLength(v) {
        this.file.setMaxFileLength(v);
    }

    select() {
        this.file.select();
    }

    getQueue() {
        return this.file.getQueue();
    }

    getValue() {
        return this.file.getValue();
    }

    upload() {
        this._reset();
        this.file.upload();
    }

    sendFiles(files) {
        this._reset();

        this.file.sendFiles(files);
    }

    reset() {
        this._reset();
    }
}
