import { IconLabel, Label } from "../label";
import { IconButton } from "../button";
import { HorizontalLayout, shortcut, extend, isPlainObject, pixFormat, HorizontalAlign } from "@/core";
import { Tip } from "./0.tip";

/**
 * toast提示
 *
 * Created by GUY on 2015/9/7.
 * @class Toast
 * @extends Tip
 */

@shortcut()
export class Toast extends Tip {
    _const = {
        closableMinWidth: 146,
        minWidth: 100,
        closableMaxWidth: 410,
        maxWidth: 400,
    };

    static EVENT_DESTORY = "EVENT_DESTORY";
    static xtype = "bi.toast";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            extraCls: "bi-toast",
            text: "",
            level: "success", // success或warning
            autoClose: true,
            closable: null,
            textHeight: 20,
            vgap: 10,
            innerHgap: 4,
            hgap: 8,
        });
    }

    render() {
        const { closable, level, autoClose, textHeight, text, hgap, vgap, innerHgap } = this.options;
        const { closableMinWidth, minWidth, maxWidth, closableMaxWidth } = this._const;
        this.element.css({
            minWidth: pixFormat(closable ? closableMinWidth : minWidth),
            maxWidth: pixFormat(closable ? closableMaxWidth : maxWidth),
        });
        this.element.addClass(`toast-${level}`);
        function fn(e) {
            e.stopPropagation();
            e.stopEvent();

            return false;
        }
        this.element.bind({
            click: fn,
            mousedown: fn,
            mouseup: fn,
            mouseover: fn,
            mouseenter: fn,
            mouseleave: fn,
            mousemove: fn,
        });
        let cls;
        switch (level) {
            case "success":
                cls = "toast-success-font";
                break;
            case "error":
                cls = "toast-error-font";
                break;
            case "warning":
                cls = "toast-warning-font";
                break;
            case "loading":
                cls = "toast-loading-font anim-rotate";
                break;
            case "normal":
            default:
                cls = "toast-message-font";
                break;
        }

        function hasCloseIcon() {
            return closable === true || (closable === null && autoClose === false);
        }
        const items = [
            {
                type: IconLabel.xtype,
                cls: `${cls} toast-icon`,
                height: textHeight,
            },
            {
                el: isPlainObject(text)
                    ? text
                    : {
                        type: Label.xtype,
                        whiteSpace: "normal",
                        text,
                        textHeight,
                        textAlign: "left",
                    },
            }
        ];

        const columnSize = ["", "fill"];

        if (hasCloseIcon()) {
            items.push({
                type: IconButton.xtype,
                cls: "close-font toast-icon",
                handler: () => {
                    this.destroy();
                },
                height: textHeight,
            });
            columnSize.push("");
        }

        return {
            type: HorizontalLayout.xtype,
            horizontalAlign: HorizontalAlign.Stretch,
            items,
            hgap,
            vgap,
            innerHgap,
            columnSize,
        };
    }

    beforeDestroy() {
        this.fireEvent(Toast.EVENT_DESTORY);
    }
}
