import { Checkbox, Label, BasicButton } from "@/base";
import { VerticalAdaptLayout, CenterAdaptLayout, shortcut, extend, createWidget, SIZE_CONSANTS } from "@/core";

/**
 * guy
 * 复选框item
 * @type {*|void|Object}
 */
@shortcut()
export class MultiSelectItem extends BasicButton {
    static xtype = "bi.multi_select_item";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            extraCls: "bi-multi-select-item",
            attributes: {
                tabIndex: 1,
            },
            height: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
            iconWrapperWidth: 26,
            el: {},
        });
    }

    render() {
        const o = this.options;
        this.checkbox = createWidget({
            type: Checkbox.xtype,
        });

        return {
            type: VerticalAdaptLayout.xtype,
            columnSize: [o.iconWrapperWidth || o.height, "fill"],
            items: [
                {
                    type: CenterAdaptLayout.xtype,
                    items: [this.checkbox],
                },
                {
                    el: {
                        type: Label.xtype,
                        ref: _ref => {
                            this.text = _ref;
                        },
                        cls: "list-item-text",
                        textAlign: "left",
                        whiteSpace: "nowrap",
                        textHeight: o.height,
                        height: o.height,
                        hgap: o.textHgap,
                        rgap: o.textRgap,
                        lgap: o.textLgap,
                        vgap: o.textVgap,
                        text: o.text,
                        keyword: o.keyword,
                        value: o.value,
                        py: o.py,
                        ...o.el
                    },
                }
            ],
        };
    }

    // _setEnable: function (enable) {
    //     MultiSelectItem.superclass._setEnable.apply(this, arguments);
    //     if (enable === true) {
    //         this.element.attr("tabIndex", 1);
    //     } else if (enable === false) {
    //         this.element.removeAttr("tabIndex");
    //     }
    // },

    doRedMark() {
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    doClick() {
        super.doClick(...arguments);
        if (this.isValid()) {
            this.fireEvent(MultiSelectItem.EVENT_CHANGE, this.getValue(), this);
        }
    }

    setSelected(v) {
        super.setSelected(...arguments);
        this.checkbox.setSelected(v);
    }
}
