import { Label, BasicButton } from "@/base";
import { shortcut, extend, createWidget, SIZE_CONSANTS } from "@/core";

@shortcut()
export class SingleSelectItem extends BasicButton {
    static xtype = "bi.single_select_item";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            extraCls: "bi-single-select-item bi-list-item-active",
            attributes: {
                tabIndex: 1,
            },
            textHgap: 10,
            height: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
            textAlign: "left",
        });
    }

    render() {
        const o = this.options;
        this.text = createWidget({
            type: Label.xtype,
            element: this,
            textAlign: o.textAlign,
            whiteSpace: "nowrap",
            textHeight: o.height,
            height: o.height,
            hgap: o.hgap || o.textHgap,
            vgap: o.textVgap,
            lgap: o.textLgap,
            rgap: o.textRgap,
            text: o.text,
            keyword: o.keyword,
            value: o.value,
            py: o.py,
        });
    }

    _setEnable(enable) {
        super._setEnable(...arguments);
        if (enable === true) {
            this.element.attr("tabIndex", 1);
        } else if (enable === false) {
            this.element.removeAttr("tabIndex");
        }
    }

    doRedMark() {
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    doClick() {
        super.doClick(...arguments);
        if (this.isValid()) {
            this.fireEvent(SingleSelectItem.EVENT_CHANGE, this.isSelected(), this);
        }
    }

    setSelected(v) {
        super.setSelected(...arguments);
    }
}
