import { IconArrowNode } from "./node.icon.arrow";
import { Layout, HorizontalAdaptLayout, shortcut, extend, createWidget, Controller, count, makeArray } from "@/core";
import { NodeButton } from "@/base";

@shortcut()
export class MultiLayerIconArrowNode extends NodeButton {
    static xtype = "bi.multilayer_icon_arrow_node";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            extraCls: "bi-multilayer-icon-arrow-node bi-list-item",
            layer: 0, // 第几层级
            id: "",
            pId: "",
            open: false,
            height: 24,
            iconHeight: 16,
            iconWidth: 16,
            iconCls: "",
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.node = createWidget({
            type: IconArrowNode.xtype,
            iconCls: o.iconCls,
            cls: "bi-list-item-none",
            id: o.id,
            pId: o.pId,
            open: o.open,
            height: o.height,
            iconHeight: o.iconHeight,
            iconWidth: o.iconWidth,
            hgap: o.hgap,
            text: o.text,
            value: o.value,
            py: o.py,
            keyword: o.keyword,
        });
        this.node.on(Controller.EVENT_CHANGE, (...args) => {
            this.setSelected(this.isSelected());
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });

        const items = [];
        count(0, o.layer, () => {
            items.push({
                type: Layout.xtype,
                width: 15,
                height: o.height,
            });
        });
        items.push(this.node);
        createWidget({
            type: HorizontalAdaptLayout.xtype,
            element: this,
            columnSize: makeArray(o.layer, 15),
            items,
        });
    }

    isOnce() {
        return true;
    }

    doRedMark() {
        this.node.doRedMark(...arguments);
    }

    unRedMark() {
        this.node.unRedMark(...arguments);
    }

    isSelected() {
        return this.node.isSelected();
    }

    setSelected(b) {
        super.setSelected(...arguments);
        this.node.setSelected(b);
    }

    doClick() {
        super.doClick(...arguments);
        this.node.setSelected(this.isSelected());
    }

    setOpened(v) {
        super.setOpened(...arguments);
        this.node.setOpened(v);
    }
}
