import { IconLabel, NodeButton } from "@/base";
import { shortcut, extend, STYLE_CONSTANTS } from "@/core";

@shortcut()
export class TreeNodeSwitcher extends NodeButton {
    static xtype = "bi.tree_node_switcher";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-tree-node-switcher",
            iconWidth: 24,
            iconHeight: 24,
            isFirstNode: false,
            isLastNode: false,
            layer: 0,
        });
    }

    render() {
        const [collapse, expand] = this.getIconCls();

        return {
            type: IconLabel.xtype,
            iconWidth: this.options.iconWidth,
            iconHeight: this.options.iconHeight,
            cls: this.options.open ? expand : collapse,
        };
    }

    getIconCls() {
        const options = this.options;
        if (options.layer === 0 && options.isFirstNode && options.isLastNode) {
            //  只有一层，并且是第一个节点，并且是最后一个节点
            return STYLE_CONSTANTS.LINK_LINE_TYPE === "solid"
                ? ["tree-solid-collapse-icon-type1", "tree-solid-expand-icon-type1"]
                : ["tree-collapse-icon-type1", "tree-expand-icon-type1"];
        } else if (options.layer === 0 && options.isFirstNode) {
            //  第一层，并且是第一个节点
            return STYLE_CONSTANTS.LINK_LINE_TYPE === "solid"
                ? ["tree-solid-collapse-icon-type1", "tree-solid-expand-icon-type1"]
                : ["tree-collapse-icon-type2", "tree-expand-icon-type2"];
        } else if (options.isLastNode) {
            //  最后一个节点
            return STYLE_CONSTANTS.LINK_LINE_TYPE === "solid"
                ? ["tree-solid-collapse-icon-type1", "tree-solid-expand-icon-type1"]
                : ["tree-collapse-icon-type4", "tree-expand-icon-type4"];
        } else {
            //  其他情况
            return STYLE_CONSTANTS.LINK_LINE_TYPE === "solid"
                ? ["tree-solid-collapse-icon-type1", "tree-solid-expand-icon-type1"]
                : ["tree-collapse-icon-type3", "tree-expand-icon-type3"];
        }
    }

    setOpened(b) {
        super.setOpened(...arguments);
        const [collapse, expand] = this.getIconCls();
        if (b) {
            this.element.addClass(expand).removeClass(collapse);
        } else {
            this.element.addClass(collapse).removeClass(expand);
        }
    }

    doClick() {
        super.doClick(...arguments);
        this.fireEvent(TreeNodeSwitcher.EVENT_CHANGE, this);
    }
}
