import { AbsoluteLayout, shortcut, i18nText } from "@/core";
import { TextButton, Label, BasicButton } from "@/base";

const CIRCLE_SIZE = 12;

@shortcut()
export class Switch extends BasicButton {
    static xtype = "bi.switch";
    static EVENT_CHANGE = "EVENT_CHANGE";

    props = {
        extraCls: "bi-switch",
        attributes: {
            tabIndex: 1,
        },
        height: 20,
        width: 44,
        showTip: false,
        textGap: 8,
    };

    render() {
        const { selected, width, height, textGap, showTip } = this.options;
        const tgap = (height - CIRCLE_SIZE) / 2;

        return {
            type: AbsoluteLayout.xtype,
            ref: _ref => {
                this.layout = _ref;
            },
            items: [
                {
                    el: {
                        type: TextButton.xtype,
                        cls: "circle-button",
                    },
                    width: 12,
                    height: 12,
                    top: tgap,
                    left: selected ? width - height + tgap : tgap,
                },
                {
                    type: Label.xtype,
                    text: i18nText("BI-Basic_Simple_Open"),
                    cls: "content-tip",
                    left: textGap,
                    top: tgap - 2,
                    invisible: !(showTip && selected),
                    ref: _ref => {
                        this.openTip = _ref;
                    },
                },
                {
                    type: Label.xtype,
                    text: i18nText("BI-Basic_Simple_Close"),
                    cls: "content-tip",
                    right: textGap,
                    top: tgap - 2,
                    invisible: !(showTip && !selected),
                    ref: _ref => {
                        this.closeTip = _ref;
                    },
                }
            ],
        };
    }

    _setEnable(enable) {
        super._setEnable(...arguments);
        if (enable === true) {
            this.element.attr("tabIndex", 1);
        } else if (enable === false) {
            this.element.removeAttr("tabIndex");
        }
    }

    setSelected(v) {
        super.setSelected(...arguments);
        const { width, height } = this.options;
        const tgap = (height - CIRCLE_SIZE) / 2;
        this.layout.attr("items")[0].left = v ? width - height + tgap : tgap;
        this.layout.resize();
        this.options.showTip && this.openTip.setVisible(v);
        this.options.showTip && this.closeTip.setVisible(!v);
    }

    doClick() {
        super.doClick(...arguments);
        this.fireEvent(Switch.EVENT_CHANGE, this.isSelected());
    }
}
