import { SimpleHexColorPickerEditor, ColorPickerEditor } from "./colorpicker";
import { Farbtastic } from "./farbtastic/farbtastic";
import { VTapeLayout, AbsoluteLayout, shortcut, Widget, extend, createWidget } from "@/core";


/**
 * 自定义选色
 *
 * Created by GUY on 2015/11/17.
 * @class CustomColorChooser
 * @extends Widget
 */
@shortcut()
export class CustomColorChooser extends Widget {
    static xtype = "bi.custom_color_chooser";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-custom-color-chooser",
            width: 292,
            height: 265,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.editor = createWidget(o.editor, {
            type: SimpleHexColorPickerEditor.xtype,
            value: o.value,
        });
        this.editor.on(ColorPickerEditor.EVENT_CHANGE, () => {
            this.setValue(this.editor.getValue());
        });
        this.farbtastic = createWidget({
            type: Farbtastic.xtype,
            value: o.value,
        });
        this.farbtastic.on(Farbtastic.EVENT_CHANGE, () => {
            this.setValue(this.farbtastic.getValue());
        });

        createWidget({
            type: VTapeLayout.xtype,
            element: this,
            items: [
                {
                    type: AbsoluteLayout.xtype,
                    items: [
                        {
                            el: this.editor,
                            left: 10,
                            top: 0,
                            right: 10,
                        },
                    ],
                    height: 50,
                },
                {
                    type: AbsoluteLayout.xtype,
                    items: [
                        {
                            el: this.farbtastic,
                            left: 46,
                            right: 46,
                            top: 7,
                        },
                    ],
                    height: 215,
                },
            ],
        });
    }

    setValue(color) {
        this.editor.setValue(color);
        this.farbtastic.setValue(color);
    }

    getValue() {
        return this.editor.getValue();
    }
}
