import { Combo } from "@/base";
import { HexColorChooserPopup } from "./colorchooser.popup.hex";
import { shortcut, Widget, extend, createWidget, toPix, isNotEmptyString } from "@/core";
import { ColorChooserPopup } from "./colorchooser.popup";

/**
 * 选色控件
 *
 * Created by GUY on 2015/11/17.
 * @class ColorChooser
 * @extends Widget
 */
@shortcut()
export class ColorChooser extends Widget {
    static xtype = "bi.color_chooser";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_AFTER_POPUPVIEW = "EVENT_AFTER_POPUPVIEW";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-color-chooser",
            value: "",
            height: 24,
            el: {},
            simple: false,
            colorBoardHeight: 80,
        });
    }

    _init() {
        const o = this.options;
        const fn = () => {
            const color = this.colorPicker.getValue();
            this.setValue(color);
        };

        super._init(...arguments);
        o.value = (o.value || "").toLowerCase();
        this.combo = createWidget({
            type: Combo.xtype,
            element: this,
            container: o.container,
            adjustLength: 1,
            destroyWhenHide: o.destroyWhenHide,
            isNeedAdjustWidth: false,
            isNeedAdjustHeight: false,
            el: extend(
                {
                    type: o.width <= 24 ? "bi.color_chooser_trigger" : "bi.long_color_chooser_trigger",
                    simple: o.simple,
                    ref: _ref => {
                        this.trigger = _ref;
                    },
                    value: o.value,
                    width: o.el.type ? o.width : toPix(o.width, 2),
                    height: o.el.type ? o.height : toPix(o.height, 2),
                },
                o.el
            ),
            popup: () => {
                return {
                    el: extend(
                        {
                            type: HexColorChooserPopup.xtype,
                            colorItems: o.colorItems,
                            colorBoardHeight: o.colorBoardHeight,
                            recommendColorsGetter: o.recommendColorsGetter,
                            ref: _ref => {
                                this.colorPicker = _ref;
                            },
                            listeners: [
                                {
                                    eventName: ColorChooserPopup.EVENT_VALUE_CHANGE,
                                    action: () => {
                                        fn();
                                        if (!this._isRGBColor(this.colorPicker.getValue())) {
                                            this.combo.hideView();
                                        }
                                    },
                                },
                                {
                                    eventName: ColorChooserPopup.EVENT_CHANGE,
                                    action: () => {
                                        fn();
                                        this.combo.hideView();
                                    },
                                }
                            ],
                        },
                        o.popup
                    ),
                    value: o.value,
                    width: 300,
                };
            },
            value: o.value,
        });

        this.combo.on(Combo.EVENT_BEFORE_HIDEVIEW, (...args) => {
            this.fireEvent(ColorChooser.EVENT_CHANGE, ...args);
        });
        this.combo.on(Combo.EVENT_AFTER_POPUPVIEW, (...args) => {
            this.fireEvent(ColorChooser.EVENT_AFTER_POPUPVIEW, ...args);
        });
    }

    _isRGBColor(color) {
        return isNotEmptyString(color) && color !== "transparent";
    }

    isViewVisible() {
        return this.combo.isViewVisible();
    }

    hideView() {
        this.combo.hideView();
    }

    showView() {
        this.combo.showView();
    }

    setValue(color) {
        this.options.value = (color || "").toLowerCase();
        this.combo.setValue(this.options.value);
    }

    getValue() {
        return this.combo.getValue();
    }
}
