import { CustomColorChooser } from "./colorchooser.custom";
import { PopupPanel } from "../layer";
import { Combo, TextItem } from "@/base";
import {
    AbsoluteLayout,
    VTapeLayout,
    Layout,
    shortcut,
    Widget,
    createWidget,
    Cache,
    string2Array,
    isNotNull,
    Queue,
    array2String,
    map,
    count,
    filter,
    isNotEmptyString,
    isArray,
    i18nText
} from "@/core";
import { ColorPickerEditor, ColorPicker } from "./colorpicker";

/**
 * 选色控件
 *
 * Created by GUY on 2015/11/17.
 * @class ColorChooserPopup
 * @extends Widget
 */
@shortcut()
export class ColorChooserPopup extends Widget {
    static xtype = "bi.color_chooser_popup";

    static EVENT_VALUE_CHANGE = "EVENT_VALUE_CHANGE";
    static EVENT_CHANGE = "EVENT_CHANGE";

    props = {
        baseCls: "bi-color-chooser-popup",
        width: 230,
        height: 145,
        simple: false, // 简单模式, popup中没有自动和透明
    };

    render() {
        const o = this.options;
        this.colorEditor = createWidget(o.editor, {
            type: o.simple ? "bi.simple_color_picker_editor" : "bi.color_picker_editor",
            value: o.value,
            cls: "bi-header-background bi-border-bottom",
            height: 30,
        });

        this.colorEditor.on(ColorPickerEditor.EVENT_CHANGE, (...args) => {
            this.setValue(this.colorEditor.getValue());
            this._dealStoreColors();
            this.fireEvent(ColorChooserPopup.EVENT_VALUE_CHANGE, ...args);
        });

        this.storeColors = createWidget({
            type: ColorPicker.xtype,
            cls: "bi-border-bottom bi-border-right",
            items: [this._digestStoreColors(this._getStoreColors())],
            width: 210,
            height: 24,
            value: o.value,
        });
        this.storeColors.on(ColorPicker.EVENT_CHANGE, (...args) => {
            this.setValue(this.storeColors.getValue()[0]);
            this._dealStoreColors();
            this.fireEvent(ColorChooserPopup.EVENT_CHANGE, ...args);
        });

        this.colorPicker = createWidget({
            type: ColorPicker.xtype,
            width: 210,
            height: 50,
            value: o.value,
        });

        this.colorPicker.on(ColorPicker.EVENT_CHANGE, (...args) => {
            this.setValue(this.colorPicker.getValue()[0]);
            this._dealStoreColors();
            this.fireEvent(ColorChooserPopup.EVENT_CHANGE, ...args);
        });

        this.customColorChooser = createWidget({
            type: CustomColorChooser.xtype,
            editor: o.editor,
        });

        const panel = createWidget({
            type: PopupPanel.xtype,
            buttons: [i18nText("BI-Basic_Cancel"), i18nText("BI-Basic_Save")],
            title: i18nText("BI-Custom_Color"),
            el: this.customColorChooser,
            stopPropagation: false,
            bgap: -1,
            rgap: 1,
            lgap: 1,
            minWidth: 227,
        });

        this.more = createWidget({
            type: Combo.xtype,
            cls: "bi-border-top",
            container: null,
            direction: "right,top",
            isNeedAdjustHeight: false,
            el: {
                type: TextItem.xtype,
                cls: "color-chooser-popup-more bi-list-item",
                textAlign: "center",
                height: 24,
                textLgap: 10,
                text: `${i18nText("BI-Basic_More")}...`,
            },
            popup: panel,
        });

        this.more.on(Combo.EVENT_AFTER_POPUPVIEW, () => {
            this.customColorChooser.setValue(this.getValue());
        });
        panel.on(PopupPanel.EVENT_CLICK_TOOLBAR_BUTTON, (index, ...args) => {
            switch (index) {
                case 0:
                    this.more.hideView();
                    break;
                case 1:
                    this.setValue(this.customColorChooser.getValue());
                    this._dealStoreColors();
                    this.more.hideView();
                    this.fireEvent(ColorChooserPopup.EVENT_CHANGE, index, ...args);
                    break;
                default:
                    break;
            }
        });

        return {
            type: AbsoluteLayout.xtype,
            items: [
                {
                    el: {
                        type: VTapeLayout.xtype,
                        items: [
                            this.colorEditor,
                            {
                                el: {
                                    type: AbsoluteLayout.xtype,
                                    items: [
                                        {
                                            el: this.storeColors,
                                            left: 10,
                                            right: 10,
                                            top: 5,
                                        }
                                    ],
                                },
                                height: 29,
                            },
                            {
                                el: {
                                    type: AbsoluteLayout.xtype,
                                    items: [
                                        {
                                            el: this.colorPicker,
                                            left: 10,
                                            right: 10,
                                            top: 5,
                                            bottom: 5,
                                        }
                                    ],
                                },
                                height: 60,
                            },
                            {
                                el: this.more,
                                height: 24,
                            }
                        ],
                    },
                    left: 0,
                    right: 0,
                    top: 0,
                    bottom: 0,
                },
                {
                    el: {
                        type: Layout.xtype,
                        cls: "disable-mask",
                        invisible: !o.disabled,
                        ref: (ref) => {
                            this.mask = ref;
                        },
                    },
                    left: 0,
                    right: 0,
                    top: 0,
                    bottom: 0,
                }
            ],
        };
    }

    // 这里就实现的不好了，setValue里面有个editor，editor的setValue会检测错误然后出bubble提示
    mounted() {
        const o = this.options;
        if (isNotNull(o.value)) {
            this.setValue(o.value);
        }
    }

    _setEnable(enable) {
        super._setEnable(...arguments);
        this.mask.setVisible(!enable);
    }

    _dealStoreColors() {
        const color = this.getValue();
        const colors = this._getStoreColors();
        const que = new Queue(8);
        que.fromArray(colors);
        que.remove(color);
        que.unshift(color);
        const array = que.toArray();
        Cache.setItem("colors", array2String(array));
        this.setStoreColors(array);
    }

    _digestStoreColors(colors) {
        const items = map(colors, (i, color) => {
            return {
                value: color,
            };
        });
        count(colors.length, 8, i => {
            items.push({
                value: "",
                disabled: true,
            });
        });

        return items;
    }

    _getStoreColors() {
        const o = this.options;
        const colorsArray = string2Array(Cache.getItem("colors") || "");

        return filter(colorsArray, (idx, color) => (o.simple ? this._isRGBColor(color) : true));
    }

    _isRGBColor(color) {
        return isNotEmptyString(color) && color !== "transparent";
    }

    setStoreColors(colors) {
        if (isArray(colors)) {
            this.storeColors.populate([this._digestStoreColors(colors)]);
            // BI-66973 选中颜色的同时选中历史
            this.storeColors.setValue(this.getValue());
        }
    }

    setValue(color) {
        this.colorEditor.setValue(color);
        this.colorPicker.setValue(color);
        this.storeColors.setValue(color);
    }

    getValue() {
        return this.colorEditor.getValue();
    }
}
