import { shortcut, Widget, extend, createWidget } from "@/core";
import { ColorChooserPopup } from "./colorchooser.popup";

/**
 * 选色控件
 *
 * Created by GUY on 2015/11/17.
 * @class SimpleColorChooserPopup
 * @extends Widget
 */
@shortcut()
export class SimpleColorChooserPopup extends Widget {
    static xtype = "bi.simple_color_chooser_popup";

    static EVENT_VALUE_CHANGE = "EVENT_VALUE_CHANGE";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-color-chooser-popup",
        });
    }

    _init() {
        super._init(...arguments);
        const { hex, value } = this.options;
        this.popup = createWidget({
            type: hex ? "bi.hex_color_chooser_popup" : "bi.color_chooser_popup",
            value,
            element: this,
            simple: true, // 是否有自动
        });
        this.popup.on(ColorChooserPopup.EVENT_CHANGE, (...args) => {
            this.fireEvent(SimpleColorChooserPopup.EVENT_CHANGE, ...args);
        });
        this.popup.on(ColorChooserPopup.EVENT_VALUE_CHANGE, (...args) => {
            this.fireEvent(SimpleColorChooserPopup.EVENT_VALUE_CHANGE, ...args);
        });
    }

    setStoreColors(colors) {
        this.popup.setStoreColors(colors);
    }

    setValue(color) {
        this.popup.setValue(color);
    }

    getValue() {
        return this.popup.getValue();
    }
}
