import { SimpleHexColorChooserPopup } from "./colorchooser.popup.hex.simple";
import { shortcut, Widget, extend, createWidget } from "@/core";
import { ColorChooser } from "./colorchooser";

/**
 * 简单选色控件，没有自动和透明
 *
 * Created by GUY on 2015/11/17.
 * @class SimpleColorChooser
 * @extends Widget
 */
@shortcut()
export class SimpleColorChooser extends Widget {
    static xtype = "bi.simple_color_chooser";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_AFTER_POPUPVIEW = "EVENT_AFTER_POPUPVIEW";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-simple-color-chooser",
            value: "#ffffff",
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;

        this.combo = createWidget({
            type: ColorChooser.xtype,
            simple: o.simple,
            element: this,
            container: o.container,
            value: o.value,
            width: o.width,
            height: o.height,
            destroyWhenHide: o.destroyWhenHide,
            popup: {
                type: SimpleHexColorChooserPopup.xtype,
                recommendColorsGetter: o.recommendColorsGetter,
            },
        });
        this.combo.on(ColorChooser.EVENT_CHANGE, (...args) => {
            this.fireEvent(SimpleColorChooser.EVENT_CHANGE, ...args);
        });
        this.combo.on(ColorChooser.EVENT_AFTER_POPUPVIEW, (...args) => {
            this.fireEvent(SimpleColorChooser.EVENT_AFTER_POPUPVIEW, ...args);
        });
    }

    isViewVisible() {
        return this.combo.isViewVisible();
    }

    hideView() {
        this.combo.hideView();
    }

    showView() {
        this.combo.showView();
    }

    setValue(color) {
        this.combo.setValue(color);
    }

    getValue() {
        return this.combo.getValue();
    }
}
