import { Layout, AbsoluteLayout, shortcut, extend, createWidget, isNotNull, isIE9Below } from "@/core";
import { IconButton, Trigger } from "@/base";

/**
 * 选色控件
 *
 * Created by GUY on 2015/11/17.
 * @class ColorChooserTrigger
 * @extends Trigger
 */
@shortcut()
export class ColorChooserTrigger extends Trigger {
    static xtype = "bi.color_chooser_trigger";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig(config) {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-color-chooser-trigger bi-focus-shadow ${
                config.simple ? "bi-border-bottom" : "bi-border bi-border-radius"
            }`,
            height: 22,
        });
    }

    _init() {
        super._init(...arguments);
        this.colorContainer = createWidget({
            type: Layout.xtype,
            cls: `color-chooser-trigger-content${isIE9Below && isIE9Below() ? " hack" : ""}`,
        });

        const down = createWidget({
            type: IconButton.xtype,
            disableSelected: true,
            cls: "icon-combo-down-icon trigger-triangle-font icon-size-12",
            width: 12,
            height: 8,
        });

        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: this.colorContainer,
                    left: 2,
                    right: 2,
                    top: 2,
                    bottom: 2,
                },
                {
                    el: down,
                    right: -1,
                    bottom: 1,
                }
            ],
        });
        if (isNotNull(this.options.value)) {
            this.setValue(this.options.value);
        }
    }

    setValue(color) {
        super.setValue(...arguments);
        if (color === "") {
            this.colorContainer.element
                .css("background-color", "")
                .removeClass("trans-color-background")
                .addClass("auto-color-background");
        } else if (color === "transparent") {
            this.colorContainer.element
                .css("background-color", "")
                .removeClass("auto-color-background")
                .addClass("trans-color-background");
        } else {
            this.colorContainer.element
                .css({ "background-color": color })
                .removeClass("auto-color-background")
                .removeClass("trans-color-background");
        }
    }
}
