import { shortcut, isNotNull, extend } from "@/core";
import { BasicButton, Maskers } from "@/base";

/**
 * 简单选色控件按钮
 *
 * Created by GUY on 2015/11/16.
 * @class ColorPickerButton
 * @extends BasicButton
 */
@shortcut()
export class ColorPickerButton extends BasicButton {
    static xtype = "bi.color_picker_button";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-color-picker-button bi-background bi-border-top bi-border-left`,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        if (isNotNull(o.value)) {
            if (o.value === "") {
                this.element.addClass("auto-color-no-square-normal-background");
            } else if (o.value === "transparent") {
                this.element.addClass("trans-color-background");
            } else {
                this.element.css("background-color", o.value);
            }
            const name = this.getName();
            this.element.hover(
                () => {
                    this._createMask();
                    if (this.isEnabled()) {
                        Maskers.show(name);
                    }
                },
                () => {
                    if (!this.isSelected()) {
                        Maskers.hide(name);
                    }
                }
            );
        }
    }

    _createMask() {
        const o = this.options,
            name = this.getName();
        if (this.isEnabled() && !Maskers.has(name)) {
            const w = Maskers.make(name, this, {
                offset: {
                    left: -1,
                    top: -1,
                    right: -1,
                    bottom: -1,
                },
            });
            w.element.addClass("color-picker-button-mask").css("background-color", o.value);
        }
    }

    setSelected(b) {
        super.setSelected(...arguments);
        if (b) {
            this._createMask();
        }
        Maskers[b ? "show" : "hide"](this.getName());
    }
}
