import { ButtonGroup } from "@/base";
import { GridLayout, Layout, shortcut, Widget, extend, each } from "@/core";
import { ColorPickerButton } from "./button";

/**
 * @author windy
 * @version 2.0
 * Created by windy on 2021/7/28
 */
@shortcut()
export class HexColorPicker extends Widget {
    static xtype = "bi.hex_color_picker";

    static EVENT_CHANGE = "EVENT_CHANGE";

    props = {
        baseCls: "bi-hex-color-picker",
        items: null,
    };

    _items = [
        [
            {
                value: "#999999",
            },
            {
                value: "#FFFFFF",
            },
            {
                value: "#FFE5E5",
            },
            {
                value: "#FFF1E5",
            },
            {
                value: "#FFF9E5",
            },
            {
                value: "#E9F5E9",
            },
            {
                value: "#EAEEFF",
            },
            {
                value: "#EFEBF7",
            },
            {
                value: "#FCE8EF",
            }
        ],
        [
            {
                value: "#737373",
            },
            {
                value: "#F2F2F2",
            },
            {
                value: "#FFA6A6",
            },
            {
                value: "#FFD0A6",
            },
            {
                value: "#FFEDA6",
            },
            {
                value: "#B3DCB2",
            },
            {
                value: "#B9C6FF",
            },
            {
                value: "#CABAE6",
            },
            {
                value: "#F8B1C9",
            }
        ],
        [
            {
                value: "#4C4C4C",
            },
            {
                value: "#D9D9D9",
            },
            {
                value: "#FF5959",
            },
            {
                value: "#FFA759",
            },
            {
                value: "#FFDD59",
            },
            {
                value: "#7EBE70",
            },
            {
                value: "#7B95FF",
            },
            {
                value: "#9C7FD0",
            },
            {
                value: "#F06D99",
            }
        ],
        [
            {
                value: "#262626",
            },
            {
                value: "#BFBFBF",
            },
            {
                value: "#FF0000",
            },
            {
                value: "#FF7800",
            },
            {
                value: "#FFCB00",
            },
            {
                value: "#259B23",
            },
            {
                value: "#355CFF",
            },
            {
                value: "#673AB7",
            },
            {
                value: "#E91E63",
            }
        ],
        [
            {
                value: "#000000",
            },
            {
                value: "#A6A6A6",
            },
            {
                value: "#A80000",
            },
            {
                value: "#B65600",
            },
            {
                value: "#CEB000",
            },
            {
                value: "#0E550C",
            },
            {
                value: "#09269C",
            },
            {
                value: "#3A1A73",
            },
            {
                value: "#B30072",
            }
        ]
    ];

    render() {
        const o = this.options;

        return {
            type: ButtonGroup.xtype,
            items: this._digest(o.items || this._items),
            layouts: [
                {
                    type: GridLayout.xtype,
                }
            ],
            value: o.value,
            listeners: [
                {
                    eventName: ButtonGroup.EVENT_CHANGE,
                    action: (...args) => {
                        this.fireEvent(HexColorPicker.EVENT_CHANGE, ...args);
                    },
                }
            ],
            ref: _ref => {
                this.colors = _ref;
            },
        };
    }

    _digest(items) {
        const o = this.options;
        const blocks = [];
        each(items, (idx, row) => {
            const bRow = [];
            each(row, (idx, item) => {
                bRow.push(
                    extend(
                        {
                            type: ColorPickerButton.xtype,
                            once: false,
                            cls: o.space ? "bi-border-right" : "",
                        },
                        item
                    )
                );
                if (o.space && idx < row.length - 1) {
                    bRow.push({ type: Layout.xtype });
                }
            });
            blocks.push(bRow);
        });

        return blocks;
    }

    populate(items) {
        const args = [].slice.call(arguments);
        args[0] = this._digest(items);
        this.colors.populate.apply(this.colors, args);
    }

    setValue(color) {
        this.colors.setValue(color);
    }

    getValue() {
        return this.colors.getValue();
    }
}
