import {
    Layout,
    VerticalAdaptLayout,
    shortcut,
    Widget,
    extend,
    isNull,
    createWidget,
    isNumeric,
    createItems,
    createWidgets,
    each,
    isEmptyString,
    DOM,
    i18nText
} from "@/core";
import { Label } from "@/base";
import { SmallTextEditor } from "@/widget/editor/editor.text.small.js";
import { TextEditor } from "@/widget/editor/editor.text";

const RGB_WIDTH = 32;

/**
 * 简单选色控件
 *
 * Created by GUY on 2015/11/16.
 * @class SimpleColorPickerEditor
 * @extends Widget
 */
@shortcut()
export class SimpleColorPickerEditor extends Widget {
    static xtype = "bi.simple_color_picker_editor";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-color-picker-editor",
            // width: 200,
            height: 30,
        });
    }

    _init() {
        super._init(...arguments);
        this.colorShow = createWidget({
            type: Layout.xtype,
            cls: "color-picker-editor-display bi-card bi-border",
            height: 16,
            width: 16,
        });
        const RGB = createWidgets(
            createItems([{ text: "R" }, { text: "G" }, { text: "B" }], {
                type: Label.xtype,
                cls: "color-picker-editor-label",
                width: 20,
                height: 20,
            })
        );

        const checker = v => isNumeric(v) && (v | 0) >= 0 && (v | 0) <= 255;
        const Ws = createWidgets([{}, {}, {}], {
            type: SmallTextEditor.xtype,
            cls: "color-picker-editor-input bi-border-radius",
            validationChecker: checker,
            errorText: i18nText("BI-Color_Picker_Error_Text"),
            allowBlank: true,
            value: 255,
            width: RGB_WIDTH,
            height: 20,
        });
        each(Ws, (i, w) => {
            w.on(TextEditor.EVENT_CHANGE, () => {
                this._checkEditors();
                if (this.R.isValid() && this.G.isValid() && this.B.isValid()) {
                    this.colorShow.element.css("background-color", this.getValue());
                    this.fireEvent(SimpleColorPickerEditor.EVENT_CHANGE);
                }
            });
        });
        this.R = Ws[0];
        this.G = Ws[1];
        this.B = Ws[2];

        createWidget({
            type: VerticalAdaptLayout.xtype,
            element: this,
            items: [
                {
                    el: this.colorShow,
                    width: 16,
                    lgap: 20,
                    rgap: 15,
                },
                {
                    el: RGB[0],
                    width: 20,
                },
                {
                    el: this.R,
                    width: RGB_WIDTH,
                },
                {
                    el: RGB[1],
                    width: 20,
                },
                {
                    el: this.G,
                    width: RGB_WIDTH,
                },
                {
                    el: RGB[2],
                    width: 20,
                },
                {
                    el: this.B,
                    width: RGB_WIDTH,
                }
            ],
        });
    }

    _checkEditors() {
        if (isEmptyString(this.R.getValue())) {
            this.R.setValue(0);
        }
        if (isEmptyString(this.G.getValue())) {
            this.G.setValue(0);
        }
        if (isEmptyString(this.B.getValue())) {
            this.B.setValue(0);
        }
    }

    setValue(color) {
        this.colorShow.element.css({ "background-color": color });
        const json = DOM.rgb2json(DOM.hex2rgb(color));
        this.R.setValue(isNull(json.r) ? "" : json.r);
        this.G.setValue(isNull(json.g) ? "" : json.g);
        this.B.setValue(isNull(json.b) ? "" : json.b);
    }

    getValue() {
        return DOM.rgb2hex(
            DOM.json2rgb({
                r: this.R.getValue(),
                g: this.G.getValue(),
                b: this.B.getValue(),
            })
        );
    }
}
