import { AbsoluteLayout, Layout, shortcut, isKey, DOM } from "@/core";
import { BasicButton } from "@/base";

const RADIUS = 84,
    SQUARE = 100,
    WIDTH = 194;

@shortcut()
export class Farbtastic extends BasicButton {
    static xtype = "bi.farbtastic";

    static EVENT_CHANGE = "EVENT_CHANGE";

    props = {
        baseCls: "bi-farbtastic",
        width: 195,
        height: 195,
        stopPropagation: true,
        value: "#000000",
    };

    render() {
        this._defaultState();

        return {
            type: AbsoluteLayout.xtype,
            items: [
                {
                    el: {
                        type: Layout.xtype,
                        cls: "",
                        ref: _ref => {
                            this.colorWrapper = _ref;
                        },
                    },
                    top: 47,
                    left: 47,
                    width: 101,
                    height: 101,
                },
                {
                    el: {
                        type: Layout.xtype,
                        cls: "wheel",
                        ref: _ref => {
                            this.wheel = _ref;
                        },
                    },
                    left: 0,
                    right: 0,
                    top: 0,
                    bottom: 0,
                },
                {
                    el: {
                        type: Layout.xtype,
                        cls: "overlay",
                        ref: _ref => {
                            this.overlay = _ref;
                        },
                    },
                    top: 47,
                    left: 47,
                    width: 101,
                    height: 101,
                },
                {
                    el: {
                        type: Layout.xtype,
                        cls: "marker",
                        ref: _ref => {
                            this.hMarker = _ref;
                        },
                        scrollable: false,
                        width: 17,
                        height: 17,
                    },
                },
                {
                    el: {
                        type: Layout.xtype,
                        cls: "marker",
                        ref: _ref => {
                            this.slMarker = _ref;
                        },
                        scrollable: false,
                        width: 17,
                        height: 17,
                    },
                }
            ],
        };
    }

    created() {
        const o = this.options;
        if (isKey(o.value)) {
            this.setValue(o.value);
        }
    }

    _defaultState() {
        this.hsl = [0, 0, 0];
    }

    _unpack(color) {
        if (color.length === 7) {
            return [
                parseInt(`0x${color.substring(1, 3)}`) / 255,
                parseInt(`0x${color.substring(3, 5)}`) / 255,
                parseInt(`0x${color.substring(5, 7)}`) / 255
            ];
        } else if (color.length === 4) {
            return [
                parseInt(`0x${color.substring(1, 2)}`) / 15,
                parseInt(`0x${color.substring(2, 3)}`) / 15,
                parseInt(`0x${color.substring(3, 4)}`) / 15
            ];
        }
    }

    _pack(rgb) {
        const r = Math.round(rgb[0] * 255);
        const g = Math.round(rgb[1] * 255);
        const b = Math.round(rgb[2] * 255);

        return (
            `#${
                r < 16 ? "0" : ""
            }${r.toString(16)
            }${g < 16 ? "0" : ""
            }${g.toString(16)
            }${b < 16 ? "0" : ""
            }${b.toString(16)}`
        );
    }

    _setColor(color) {
        const unpack = this._unpack(color);
        if (this.value !== color && unpack) {
            this.value = color;
            this.rgb = unpack;
            this.hsl = this._RGBToHSL(this.rgb);
            this._updateDisplay();
        }
    }

    _setHSL(hsl) {
        this.hsl = hsl;
        this.rgb = this._HSLToRGB(hsl);
        this.value = this._pack(this.rgb);
        this._updateDisplay();

        return this;
    }

    _HSLToRGB(hsl) {
        return DOM.hsl2rgb(hsl);
    }

    _RGBToHSL(rgb) {
        return DOM.rgb2hsl(rgb);
    }

    _updateDisplay() {
        const angle = this.hsl[0] * 6.28;
        this.hMarker.element.css({
            left: `${Math.round(Math.sin(angle) * RADIUS + WIDTH / 2)}px`,
            top: `${Math.round(-Math.cos(angle) * RADIUS + WIDTH / 2)}px`,
        });

        this.slMarker.element.css({
            left: `${Math.round(SQUARE * (0.5 - this.hsl[1]) + WIDTH / 2)}px`,
            top: `${Math.round(SQUARE * (0.5 - this.hsl[2]) + WIDTH / 2)}px`,
        });

        // Saturation/Luminance gradient
        this.colorWrapper.element.css("backgroundColor", this._pack(this._HSLToRGB([this.hsl[0], 1, 0.5])));
    }

    _absolutePosition(el) {
        const r = { x: el.offsetLeft, y: el.offsetTop };
        // Resolve relative to offsetParent
        if (el.offsetParent) {
            const tmp = this._absolutePosition(el.offsetParent);
            r.x += tmp.x;
            r.y += tmp.y;
        }

        return r;
    }

    _widgetCoords(event) {
        let x, y;
        const el = event.target || event.srcElement;
        const reference = this.wheel.element[0];

        if (typeof event.offsetX !== "undefined") {
            // Use offset coordinates and find common offsetParent
            const pos = { x: event.offsetX, y: event.offsetY };

            // Send the coordinates upwards through the offsetParent chain.
            let e = el;
            while (e) {
                e.mouseX = pos.x;
                e.mouseY = pos.y;
                pos.x += e.offsetLeft;
                pos.y += e.offsetTop;
                e = e.offsetParent;
            }

            // Look for the coordinates starting from the wheel widget.
            e = reference;
            const offset = { x: 0, y: 0 };
            while (e) {
                if (typeof e.mouseX !== "undefined") {
                    x = e.mouseX - offset.x;
                    y = e.mouseY - offset.y;
                    break;
                }
                offset.x += e.offsetLeft;
                offset.y += e.offsetTop;
                e = e.offsetParent;
            }

            // Reset stored coordinates
            e = el;
            while (e) {
                e.mouseX = undefined;
                e.mouseY = undefined;
                e = e.offsetParent;
            }
        } else {
            // Use absolute coordinates
            const pos = this._absolutePosition(reference);
            x = (event.pageX || 0) - pos.x;
            y = (event.pageY || 0) - pos.y;
        }

        // Subtract distance to middle
        return { x: x - WIDTH / 2, y: y - WIDTH / 2 };
    }

    _doMouseMove(event) {
        const pos = this._widgetCoords(event);

        // Set new HSL parameters
        if (this.circleDrag) {
            let hue = Math.atan2(pos.x, -pos.y) / 6.28;
            if (hue < 0) hue += 1;
            this._setHSL([hue, this.hsl[1], this.hsl[2]]);
        } else {
            const sat = Math.max(0, Math.min(1, -(pos.x / SQUARE) + 0.5));
            const lum = Math.max(0, Math.min(1, -(pos.y / SQUARE) + 0.5));
            this._setHSL([this.hsl[0], sat, lum]);
        }
        this.fireEvent(Farbtastic.EVENT_CHANGE, this.getValue(), this);
    }

    doClick(event) {
        const pos = this._widgetCoords(event);
        this.circleDrag = Math.max(Math.abs(pos.x), Math.abs(pos.y)) * 2 > SQUARE;

        // Process
        this._doMouseMove(event);

        return false;
    }

    setValue(color) {
        this._setColor(color);
    }

    getValue() {
        return this.value;
    }
}
