import { BubblePopupView } from "./popup.bubble";
import { shortcut, Widget, extend, emptyFn, createWidget, isFunction } from "@/core";
import { Combo } from "@/base";

@shortcut()
export class BubbleCombo extends Widget {
    static xtype = "bi.bubble_combo";

    static EVENT_TRIGGER_CHANGE = "EVENT_TRIGGER_CHANGE";
    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_EXPAND = "EVENT_EXPAND";
    static EVENT_COLLAPSE = "EVENT_COLLAPSE";
    static EVENT_AFTER_INIT = "EVENT_AFTER_INIT";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";
    static EVENT_AFTER_POPUPVIEW = "EVENT_AFTER_POPUPVIEW";
    static EVENT_BEFORE_HIDEVIEW = "EVENT_BEFORE_HIDEVIEW";
    static EVENT_AFTER_HIDEVIEW = "EVENT_AFTER_HIDEVIEW";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-bubble-combo",
            trigger: "click",
            toggle: true,
            primary: false,
            direction: "bottom,left", // top||bottom||left||right||top,left||top,right||bottom,left||bottom,right
            isDefaultInit: false,
            destroyWhenHide: false,
            hideWhenClickOutside: true,
            hideWhenBlur: true,
            hideWhenAnotherComboOpen: false,
            isNeedAdjustHeight: true, // 是否需要高度调整
            isNeedAdjustWidth: true,
            stopPropagation: false,
            adjustLength: 0, // 调整的距离
            adjustXOffset: 0,
            adjustYOffset: 0,
            hideChecker: emptyFn,
            offsetStyle: "left", // left,right,center
            el: {},
            popup: {},
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.combo = createWidget({
            type: Combo.xtype,
            element: this,
            trigger: o.trigger,
            toggle: o.toggle,
            logic: o.logic,
            container: o.container,
            direction: o.direction,
            isDefaultInit: o.isDefaultInit,
            hideWhenBlur: o.hideWhenBlur,
            hideWhenClickOutside: o.hideWhenClickOutside,
            destroyWhenHide: o.destroyWhenHide,
            hideWhenAnotherComboOpen: o.hideWhenAnotherComboOpen,
            isNeedAdjustHeight: o.isNeedAdjustHeight,
            isNeedAdjustWidth: o.isNeedAdjustWidth,
            stopPropagation: o.stopPropagation,
            adjustXOffset: o.adjustXOffset,
            adjustYOffset: o.adjustYOffset,
            hideChecker: o.hideChecker,
            offsetStyle: o.offsetStyle,
            showArrow: true,
            comboClass: o.comboClass,
            supportCSSTransform: o.supportCSSTransform,
            el: o.el,
            popup: () =>
                extend(
                    {
                        type: BubblePopupView.xtype,
                        animation: "bi-zoom-big",
                        animationDuring: 200,
                        primary: o.primary,
                    },
                    isFunction(this.options.popup) ? this.options.popup() : this.options.popup
                ),
        });
        this.combo.on(Combo.EVENT_TRIGGER_CHANGE, (...args) => {
            this.fireEvent(BubbleCombo.EVENT_TRIGGER_CHANGE, ...args);
        });
        this.combo.on(Combo.EVENT_CHANGE, (...args) => {
            this.fireEvent(BubbleCombo.EVENT_CHANGE, ...args);
        });
        this.combo.on(Combo.EVENT_EXPAND, (...args) => {
            this.fireEvent(BubbleCombo.EVENT_EXPAND, ...args);
        });
        this.combo.on(Combo.EVENT_COLLAPSE, (...args) => {
            this.fireEvent(BubbleCombo.EVENT_COLLAPSE, ...args);
        });
        this.combo.on(Combo.EVENT_AFTER_INIT, (...args) => {
            this.fireEvent(BubbleCombo.EVENT_AFTER_INIT, ...args);
        });
        this.combo.on(Combo.EVENT_BEFORE_POPUPVIEW, (...args) => {
            this.fireEvent(BubbleCombo.EVENT_BEFORE_POPUPVIEW, ...args);
        });
        this.combo.on(Combo.EVENT_AFTER_POPUPVIEW, (...args) => {
            this.fireEvent(BubbleCombo.EVENT_AFTER_POPUPVIEW, ...args);
        });
        this.combo.on(Combo.EVENT_BEFORE_HIDEVIEW, (...args) => {
            this.fireEvent(BubbleCombo.EVENT_BEFORE_HIDEVIEW, ...args);
        });
        this.combo.on(Combo.EVENT_AFTER_HIDEVIEW, (...args) => {
            this.fireEvent(BubbleCombo.EVENT_AFTER_HIDEVIEW, ...args);
        });
    }

    hideView() {
        this.combo && this.combo.hideView();
    }

    showView() {
        this.combo && this.combo.showView();
    }

    isViewVisible() {
        return this.combo.isViewVisible();
    }

    adjustWidth() {
        this.combo.adjustWidth();
    }

    adjustHeight() {
        this.combo.adjustHeight();
    }
}
