import { shortcut, extend, i18nText, each, isWidget, createWidget } from "@/core";
import { PopupView } from "@/base/layer/layer.popup";
import { Button } from "@/base";
import { Label } from "@/base/single/label/label";


@shortcut()
export class BubblePopupView extends PopupView {
    static xtype = "bi.bubble_popup_view";

    _defaultConfig() {
        const config = super._defaultConfig(...arguments);

        return extend(config, {
            baseCls: `${config.baseCls} bi-bubble-popup-view`,
            minWidth: 70,
            maxWidth: 300,
            // minHeight: 50,
            showArrow: true,
        });
    }
}

@shortcut()
export class BubblePopupBarView extends BubblePopupView {
    static xtype = "bi.bubble_bar_popup_view";

    static EVENT_CLICK_TOOLBAR_BUTTON = "EVENT_CLICK_TOOLBAR_BUTTON";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            extraCls: "bi-bubble-bar-popup-view",
            buttons: [{
                value: false,
                text: i18nText("BI-Basic_Cancel"),
                light: true,
            }, {
                text: i18nText(i18nText("BI-Basic_OK")),
                value: true,
            }],
            innerVgap: 16,
            innerHgap: 16,
        });
    }

    _createToolBar() {
        const o = this.options;

        const items = [];
        each(o.buttons, (i, buttonOpt) => {
            if (isWidget(buttonOpt)) {
                items.push({
                    el: buttonOpt,
                    lgap: 12,
                });
            } else {
                items.push({
                    el: extend({
                        type: Button.xtype,
                        height: 24,
                        handler: v => {
                            this.fireEvent(BubblePopupBarView.EVENT_CLICK_TOOLBAR_BUTTON, v);
                        },
                    }, buttonOpt),
                    lgap: 12,
                });
            }
        });

        return createWidget({
            type: "bi.right_vertical_adapt",
            innerVgap: o.innerVgap,
            innerHgap: o.innerHgap,
            items,
        });
    }

    _createContent() {
        return this.options.el;
    }

    _createView() {
        const o = this.options;

        const view = createWidget({
            type: "bi.vertical",
            items: [this._createContent()],
            cls: "bar-popup-container",
            hgap: o.innerHgap,
            tgap: o.innerVgap,
        });

        view.element.css("min-height", o.minHeight);

        return view;
    }
}

@shortcut()
export class TextBubblePopupBarView extends BubblePopupBarView {
    static xtype = "bi.text_bubble_bar_popup_view";

    static EVENT_CHANGE = "EVENT_CLICK_TOOLBAR_BUTTON";

    _defaultConfig() {
        const config = super._defaultConfig(...arguments);

        return extend(config, {
            baseCls: `${config.baseCls} bi-text-bubble-bar-popup-view`,
            text: "",
        });
    }

    _createContent() {
        const o = this.options;

        return {
            type: Label.xtype,
            text: o.text,
            whiteSpace: "normal",
            textAlign: "left",
            ref: _ref => {
                this.text = _ref;
            },
        };
    }

    populate(v) {
        this.text.setText(v || this.options.text);
    }
}
