import { TextValueCheckComboPopup } from "../textvaluecheckcombo/popup.textvaluecheck";
import { shortcut, Widget, extend, emptyFn, createWidget, Controller } from "@/core";
import { ButtonGroup, Combo } from "@/base";
import { EditorTrigger } from "../../trigger";

@shortcut()
export class EditorIconCheckCombo extends Widget {
    static xtype = "bi.editor_icon_check_combo";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_EMPTY = "EVENT_EMPTY";
    static EVENT_VALID = "EVENT_VALID";
    static EVENT_ERROR = "EVENT_ERROR";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseClass: "bi-check-editor-combo",
            width: 100,
            height: 24,
            chooseType: ButtonGroup.CHOOSE_TYPE_SINGLE,
            validationChecker: emptyFn,
            quitChecker: emptyFn,
            allowBlank: true,
            watermark: "",
            errorText: "",
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.trigger = createWidget({
            type: EditorTrigger.xtype,
            items: o.items,
            height: o.height,
            validationChecker: o.validationChecker,
            quitChecker: o.quitChecker,
            allowBlank: o.allowBlank,
            watermark: o.watermark,
            errorText: o.errorText,
            value: o.value,
        });
        this.trigger.on(EditorTrigger.EVENT_CHANGE, (...args) => {
            this.popup.setValue(this.getValue());
            this.fireEvent(EditorIconCheckCombo.EVENT_CHANGE, ...args);
        });
        this.trigger.on(EditorTrigger.EVENT_FOCUS, (...args) => {
            this.fireEvent(EditorIconCheckCombo.EVENT_FOCUS, ...args);
        });
        this.trigger.on(EditorTrigger.EVENT_EMPTY, (...args) => {
            this.fireEvent(EditorIconCheckCombo.EVENT_EMPTY, ...args);
        });
        this.trigger.on(EditorTrigger.EVENT_VALID, (...args) => {
            this.fireEvent(EditorIconCheckCombo.EVENT_VALID, ...args);
        });
        this.trigger.on(EditorTrigger.EVENT_ERROR, (...args) => {
            this.fireEvent(EditorIconCheckCombo.EVENT_ERROR, ...args);
        });

        this.popup = createWidget({
            type: TextValueCheckComboPopup.xtype,
            chooseType: o.chooseType,
            items: o.items,
            value: o.value,
        });
        this.popup.on(TextValueCheckComboPopup.EVENT_CHANGE, () => {
            this.setValue(this.popup.getValue());
            this.editorIconCheckCombo.hideView();
            this.fireEvent(EditorIconCheckCombo.EVENT_CHANGE);
        });
        this.popup.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });
        this.editorIconCheckCombo = createWidget({
            type: Combo.xtype,
            container: o.container,
            direction: o.direction,
            element: this,
            adjustLength: 2,
            el: this.trigger,
            popup: {
                el: this.popup,
                maxHeight: 300,
            },
        });
    }

    setValue(v) {
        this.editorIconCheckCombo.setValue(v);
    }

    getValue() {
        return this.trigger.getValue();
    }

    populate(items) {
        this.options.items = items;
        this.editorIconCheckCombo.populate(items);
    }
}
