import { IconComboTrigger } from "./trigger.iconcombo";
import { shortcut, Widget, extend, isFunction, createWidget, Controller, isNull, isArray } from "@/core";
import { ButtonGroup, Combo } from "@/base";
import { IconComboPopup } from "./popup.iconcombo";

@shortcut()
export class IconCombo extends Widget {
    static xtype = "bi.icon_combo";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-icon-combo",
            width: 24,
            height: 24,
            el: {},
            popup: {},
            minWidth: 100,
            maxWidth: "auto",
            maxHeight: 300,
            direction: "bottom",
            adjustLength: 3, // 调整的距离
            adjustXOffset: 0,
            adjustYOffset: 0,
            offsetStyle: "left",
            chooseType: ButtonGroup.CHOOSE_TYPE_SINGLE,
            isShowDown: true,
            hideWhenAnotherComboOpen: false,
        });
    }

    _init() {
        const o = this.options;
        o.value = isFunction(o.value)
            ? this.__watch(o.value, (context, newValue) => {
                this.setValue(newValue);
            })
            : o.value;
        o.items = isFunction(o.items)
            ? this.__watch(o.items, (context, newValue) => {
                this.populate(newValue);
            })
            : o.items;
        super._init(...arguments);
        this.trigger = createWidget(o.el, {
            type: IconComboTrigger.xtype,
            iconCls: o.iconCls,
            title: o.title,
            items: o.items,
            width: o.width,
            height: o.height,
            iconWidth: o.iconWidth,
            iconHeight: o.iconHeight,
            value: o.value,
            isShowDown: o.isShowDown,
        });
        this.popup = createWidget(o.popup, {
            type: IconComboPopup.xtype,
            chooseType: o.chooseType,
            items: o.items,
            value: o.value,
        });
        this.popup.on(IconComboPopup.EVENT_CHANGE, () => {
            this.setValue(this.popup.getValue());
            this.iconCombo.hideView();
            this.fireEvent(IconCombo.EVENT_CHANGE);
        });
        this.popup.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });
        this.iconCombo = createWidget({
            type: Combo.xtype,
            element: this,
            direction: o.direction,
            trigger: o.trigger,
            container: o.container,
            adjustLength: o.adjustLength,
            adjustXOffset: o.adjustXOffset,
            adjustYOffset: o.adjustYOffset,
            offsetStyle: o.offsetStyle,
            el: this.trigger,
            hideWhenAnotherComboOpen: o.hideWhenAnotherComboOpen,
            popup: {
                el: this.popup,
                maxWidth: o.maxWidth,
                maxHeight: o.maxHeight,
                minWidth: o.minWidth,
            },
        });
    }

    showView() {
        this.iconCombo.showView();
    }

    hideView() {
        this.iconCombo.hideView();
    }

    setValue(v) {
        this.trigger.setValue(v);
        this.popup.setValue(v);
    }

    getValue() {
        const value = this.popup.getValue();

        return isNull(value) ? [] : isArray(value) ? value : [value];
    }

    populate(items) {
        this.options.items = items;
        this.iconCombo.populate(items);
    }
}
