import { shortcut, extend, isKey, createWidget, isNotEmptyString, AbsoluteLayout, isArray, any } from "@/core";
import { Trigger, IconButton } from "@/base";
import { IconChangeButton } from "../../button";

@shortcut()
export class IconComboTrigger extends Trigger {
    static xtype = "bi.icon_combo_trigger";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            extraCls: "bi-icon-combo-trigger",
            el: {},
            items: [],
            iconCls: "",
            width: 24,
            height: 24,
            isShowDown: true,
            value: "",
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        let iconCls = "";
        if (isKey(o.value)) {
            iconCls = this._digest(o.value, o.items);
        }
        this.button = createWidget(o.el, {
            type: IconChangeButton.xtype,
            cls: "icon-combo-trigger-icon",
            iconCls,
            disableSelected: true,
            width: o.isShowDown ? o.width - 12 : o.width,
            height: o.height,
            iconWidth: o.iconWidth,
            iconHeight: o.iconHeight,
            selected: isNotEmptyString(iconCls),
        });
        this.down = createWidget({
            type: IconButton.xtype,
            disableSelected: true,
            cls: "icon-combo-down-icon trigger-triangle-font font-size-12",
            width: 12,
            height: 8,
            selected: isNotEmptyString(iconCls),
            invisible: !o.isShowDown,
        });
        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: this.button,
                    left: 0,
                    right: 0,
                    top: 0,
                    bottom: 0,
                },
                {
                    el: this.down,
                    right: 3,
                    bottom: 0,
                }
            ],
        });
    }

    _digest(v, items) {
        let iconCls = "";
        v = isArray(v) ? v[0] : v;
        any(items, (i, item) => {
            if (v === item.value) {
                iconCls = item.iconCls;

                return true;
            }
        });

        return iconCls;
    }

    populate(items) {
        const o = this.options;
        this.options.items = items || [];
        this.button.setIcon(o.iconCls);
        this.button.setSelected(false);
        this.down.setSelected(false);
    }

    setValue(v) {
        super.setValue(...arguments);
        const o = this.options;
        const iconCls = this._digest(v, this.options.items);
        v = isArray(v) ? v[0] : v;
        if (isNotEmptyString(iconCls)) {
            this.button.setIcon(iconCls);
            this.button.setSelected(true);
            this.down.setSelected(true);
        } else {
            this.button.setIcon(o.iconCls);
            this.button.setSelected(false);
            this.down.setSelected(false);
        }
    }
}
