import { SearchTextValueTrigger } from "./trigger.searchtextvalue";
import { TextValueComboPopup } from "../textvaluecombo/popup.textvalue";
import {
    shortcut,
    Widget,
    isFunction,
    toPix,
    isKey,
    isNull,
    isEmptyArray,
    isEmptyString,
    isArray,
    find,
    contains
} from "@/core";
import { ButtonGroup, Combo } from "@/base";

@shortcut()
export class SearchTextValueCombo extends Widget {
    static xtype = "bi.search_text_value_combo";

    props = {
        baseCls: "bi-search-text-value-combo",
        height: 24,
        text: "",
        defaultText: "",
        items: [],
        tipType: "",
        warningTitle: "",
        allowClear: false,
    };

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";
    static EVENT_AFTER_HIDEVIEW = "EVENT_AFTER_HIDEVIEW";

    render() {
        const o = this.options;
        o.value = isFunction(o.value)
            ? this.__watch(o.value, (context, newValue) => {
                  this.setValue(newValue);
              })
            : o.value;
        o.items = isFunction(o.items)
            ? this.__watch(o.items, (context, newValue) => {
                  this.populate(newValue);
              })
            : o.items;

        return {
            type: Combo.xtype,
            cls: `${o.simple ? "bi-border-bottom" : "bi-border bi-border-radius"} bi-focus-shadow`,
            container: o.container,
            adjustLength: 2,
            height: toPix(o.height, o.simple ? 1 : 2),
            width: toPix(o.width, 2),
            ref: (_ref) => {
                this.combo = _ref;
            },
            el: {
                type: SearchTextValueTrigger.xtype,
                cls: "search-text-value-trigger",
                watermark: o.watermark,
                ref: (_ref) => {
                    this.trigger = _ref;
                },
                items: o.items,
                height: toPix(o.height, o.simple ? 1 : 2),
                text: o.text,
                defaultText: o.defaultText,
                value: o.value,
                tipType: o.tipType,
                warningTitle: o.warningTitle,
                title: o.title,
                allowClear: o.allowClear,
                listeners: [
                    {
                        eventName: SearchTextValueTrigger.EVENT_CHANGE,
                        action: () => {
                            this.setValue(this.getValue()[0]);
                            this.combo.hideView();
                            this.fireEvent(SearchTextValueCombo.EVENT_CHANGE);
                        },
                    },
                    {
                        eventName: SearchTextValueTrigger.EVENT_CLEAR,
                        action: () => {
                            this._clear();
                            this.fireEvent(SearchTextValueCombo.EVENT_CHANGE);
                        },
                    },
                ],
            },
            popup: {
                el: {
                    type: TextValueComboPopup.xtype,
                    chooseType: ButtonGroup.CHOOSE_TYPE_SINGLE,
                    value: o.value,
                    items: o.items,
                    ref: (_ref) => {
                        this.popup = _ref;
                        this.trigger.getSearcher().setAdapter(this.popup);
                    },
                    listeners: [
                        {
                            eventName: TextValueComboPopup.EVENT_CHANGE,
                            action: () => {
                                this.setValue(this.getValue()[0]);
                                this.combo.hideView();
                                this.fireEvent(SearchTextValueCombo.EVENT_CHANGE);
                            },
                        },
                    ],
                },
                value: o.value,
                maxHeight: 252,
                minHeight: 25,
            },
            listeners: [
                {
                    eventName: Combo.EVENT_AFTER_HIDEVIEW,
                    action: () => {
                        this.trigger.stopEditing();
                        this.fireEvent(SearchTextValueCombo.EVENT_AFTER_HIDEVIEW);
                    },
                },
                {
                    eventName: Combo.EVENT_BEFORE_POPUPVIEW,
                    action: () => {
                        this.fireEvent(SearchTextValueCombo.EVENT_BEFORE_POPUPVIEW);
                    },
                },
            ],
        };
    }

    created() {
        const o = this.options;
        if (isKey(o.value)) {
            this._checkError(o.value);
        }
    }

    _clear() {
        this.setValue();
    }

    _checkError(v) {
        if (isNull(v) || isEmptyArray(v) || isEmptyString(v)) {
            this.trigger.options.tipType = "success";
            this.element.removeClass("combo-error");
        } else {
            v = isArray(v) ? v : [v];
            const result = find(this.options.items, (idx, item) => contains(v, item.value));
            if (isNull(result)) {
                this.element.removeClass("combo-error").addClass("combo-error");
                this.trigger.attr("tipType", "warning");
            } else {
                this.element.removeClass("combo-error");
                this.trigger.attr("tipType", "success");
            }
        }
    }

    populate(items) {
        this.options.items = items;
        this.combo.populate(items);
    }

    setValue(v) {
        this.combo.setValue(v);
        this._checkError(v);
    }

    getValue() {
        const value = this.combo.getValue();

        return isNull(value) ? [] : isArray(value) ? value : [value];
    }
}
