import { shortcut, Controller, Events, VerticalLayout, map, extend, concat } from "@/core";
import { ButtonGroup, Pane } from "@/base";
import { SingleSelectItem } from "../../button";

@shortcut()
export class SearchTextValueComboPopup extends Pane {
    static xtype = "bi.search_text_value_combo_popup";

    props = { baseCls: "bi-search-text-value-popup" };

    static EVENT_CHANGE = "EVENT_CHANGE";

    render() {
        const o = this.options;

        return {
            type: VerticalLayout.xtype,
            vgap: 5,
            items: [
                {
                    type: ButtonGroup.xtype,
                    ref: _ref => {
                        this.popup = _ref;
                    },
                    items: this._formatItems(o.items),
                    chooseType: ButtonGroup.CHOOSE_TYPE_SINGLE,
                    layouts: [
                        {
                            type: VerticalLayout.xtype,
                        }
                    ],
                    behaviors: {
                        redmark() {
                            return true;
                        },
                    },
                    value: o.value,
                    listeners: [
                        {
                            eventName: Controller.EVENT_CHANGE,
                            action: (...args) => {
                                const [type, val, obj] = args;
                                this.fireEvent(Controller.EVENT_CHANGE, ...args);
                                if (type === Events.CLICK) {
                                    this.fireEvent(SearchTextValueComboPopup.EVENT_CHANGE, val, obj);
                                }
                            },
                        }
                    ],
                }
            ],
        };
    }

    _formatItems(items) {
        const o = this.options;

        return map(items, (i, item) =>
            extend(
                {
                    type: SingleSelectItem.xtype,
                    textAlign: o.textAlign,
                    title: item.title || item.text,
                },
                item
            )
        );
    }

    mounted() {
        this.check();
    }

    populate(find, match, keyword) {
        const items = concat(find, match);
        super.populate.apply(this, items);
        this.popup.populate(this._formatItems(items), keyword);
    }

    getValue() {
        return this.popup.getValue();
    }

    setValue(v) {
        this.popup.setValue(v);
    }
}
