import { ButtonGroup, TextButton, PopupView } from "@/base";
import { CenterLayout, shortcut, extend, i18nText, each, createWidget, createItems, SIZE_CONSANTS } from "@/core";

/**
 * 下拉框弹出层的多选版本，toolbar带有若干按钮, zIndex在1000w
 * @class MultiPopupView
 * @extends Widget
 */

@shortcut()
export class MultiPopupView extends PopupView {
    static xtype = "bi.multi_popup_view";
    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_CLICK_TOOLBAR_BUTTON = "EVENT_CLICK_TOOLBAR_BUTTON";
    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            _baseCls: `${conf._baseCls || ""} bi-multi-list-view`,
            buttons: [i18nText("BI-Basic_OK")],
        });
    }

    _createToolBar() {
        const o = this.options;
        if (o.buttons.length === 0) {
            return;
        }

        const text = []; // 构造[{text:content},……]
        each(o.buttons, (idx, item) => {
            text.push({
                text: item,
                value: idx,
            });
        });

        this.buttongroup = createWidget({
            type: ButtonGroup.xtype,
            cls: "list-view-toolbar bi-high-light bi-split-top",
            height: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
            items: createItems(text, {
                type: TextButton.xtype,
                once: false,
                shadow: true,
                isShadowShowingOnSelected: true,
            }),
            layouts: [
                {
                    type: CenterLayout.xtype,
                    hgap: 0,
                    vgap: 0,
                }
            ],
        });

        this.buttongroup.on(ButtonGroup.EVENT_CHANGE, (value, obj) => {
            this.fireEvent(MultiPopupView.EVENT_CLICK_TOOLBAR_BUTTON, value, obj);
        });

        return this.buttongroup;
    }
}
