import {
    VerticalFillLayout,
    CenterAdaptLayout,
    LeftRightVerticalAdaptLayout,
    shortcut,
    Widget,
    extend,
    toPix,
    Controller,
    createWidget
} from "@/core";
import { Label, ButtonGroup } from "@/base";

@shortcut()
export class Panel extends Widget {
    static xtype = "bi.panel";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-panel bi-border",
            title: "",
            titleHeight: 30,
            titleButtons: [],
            el: {},
            // logic: {
            //     dynamic: false
            // }
        });
    }

    render() {
        return {
            type: VerticalFillLayout.xtype,
            rowSize: ["", "fill"],
            items: [this._createTitle(), this.options.el],
        };
    }

    _createTitle() {
        const o = this.options;
        this.text = createWidget({
            type: Label.xtype,
            cls: "panel-title-text",
            text: o.title,
            height: o.titleHeight,
        });

        this.button_group = createWidget({
            type: ButtonGroup.xtype,
            items: o.titleButtons,
            layouts: [
                {
                    type: CenterAdaptLayout.xtype,
                    lgap: 10,
                }
            ],
        });

        this.button_group.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });

        this.button_group.on(ButtonGroup.EVENT_CHANGE, (value, obj) => {
            this.fireEvent(Panel.EVENT_CHANGE, value, obj);
        });

        return {
            // el: {
            type: LeftRightVerticalAdaptLayout.xtype,
            cls: "panel-title bi-header-background bi-border-bottom",
            height: toPix(o.titleHeight, 1),
            items: {
                left: [this.text],
                right: [this.button_group],
            },
            lhgap: 10,
            rhgap: 10,
            // },
            // height: toPix(o.titleHeight, 1)
        };
    }

    setTitle(title) {
        this.text.setValue(title);
    }
}
