import { Label, BasicButton } from "@/base";
import { AbsoluteLayout, Layout, shortcut, toPix } from "@/core";

@shortcut()
export class LinearSegmentButton extends BasicButton {
    static xtype = "bi.linear_segment_button";

    props = {
        extraCls: "bi-line-segment-button bi-list-item-effect",
        once: true,
        readonly: true,
        hgap: 10,
        height: 24,
    };

    render() {
        const o = this.options;

        return [
            {
                type: Label.xtype,
                text: o.text,
                height: o.height,
                textHeight: toPix(o.height, 2),
                value: o.value,
                hgap: o.hgap,
                ref: _ref => {
                    this.text = _ref;
                },
            },
            {
                type: AbsoluteLayout.xtype,
                items: [
                    {
                        el: {
                            type: Layout.xtype,
                            cls: "line-segment-button-line",
                            height: 2,
                            ref: _ref => {
                                this.line = _ref;
                            },
                        },
                        left: 0,
                        right: 0,
                        bottom: 0,
                    }
                ],
            }
        ];
    }

    setSelected(v) {
        super.setSelected(...arguments);
        if (v) {
            this.line.element.addClass("bi-high-light-background");
        } else {
            this.line.element.removeClass("bi-high-light-background");
        }
    }

    setText(text) {
        this.text.setText(text);
    }
}
