import { ButtonGroup } from "@/base";
import { LinearSegmentButton } from "./button.linear.segment";
import { TableLayout, shortcut, Widget, createItems, makeArrayByArray } from "@/core";

@shortcut()
export class LinearSegment extends Widget {
    static xtype = "bi.linear_segment";

    props = { baseCls: "bi-linear-segment", items: [], height: 30 };

    render() {
        const o = this.options;

        return {
            type: ButtonGroup.xtype,
            items: [
                createItems(o.items, {
                    type: LinearSegmentButton.xtype,
                    height: o.height,
                })
            ],
            layouts: [
                {
                    type: TableLayout.xtype,
                    columnSize: makeArrayByArray(o.items, "fill"),
                }
            ],
            value: o.value,
            listeners: [
                {
                    eventName: "__EVENT_CHANGE__",
                    action: (...args) => {
                        this.fireEvent("__EVENT_CHANGE__", ...args);
                    },
                },
                {
                    eventName: "EVENT_CHANGE",
                    action: () => {
                        this.fireEvent("EVENT_CHANGE");
                    },
                }
            ],
            ref: _ref => {
                this.buttonGroup = _ref;
            },
        };
    }

    setValue(v) {
        this.buttonGroup.setValue(v);
    }

    setEnabledValue(v) {
        this.buttonGroup.setEnabledValue(v);
    }

    getValue() {
        return this.buttonGroup.getValue();
    }

    populate(buttons) {
        const o = this.options;
        this.buttonGroup.populate([
            createItems(buttons, {
                type: LinearSegmentButton.xtype,
                height: o.height,
            })
        ]);
    }
}
