import { Loader } from "@/base";
import { shortcut, Widget, extend, createWidget, takeRight, take } from "@/core";

@shortcut()
export class LazyLoader extends Widget {
    static xtype = "bi.lazy_loader";

    _const = {
        PAGE: 100,
    };

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-lazy-loader",
            el: {},
            items: [],
        });
    }

    _init() {
        const o = this.options;
        super._init(...arguments);
        const all = o.items.length;
        this.loader = createWidget({
            type: Loader.xtype,
            element: this,
            // 下面是button_group的属性
            el: o.el,
            itemsCreator: (options, populate) => {
                populate(this._getNextItems(options));
            },
            hasNext: option => option.count < all,
        });

        this.loader.on(Loader.EVENT_CHANGE, obj => {
            this.fireEvent(LazyLoader.EVENT_CHANGE, obj);
        });
    }

    _getNextItems(options) {
        const o = this.options;
        const lastNum = o.items.length - this._const.PAGE * (options.times - 1);
        const lastItems = takeRight(o.items, lastNum);
        const nextItems = take(lastItems, this._const.PAGE);

        return nextItems;
    }

    populate(items) {
        this.loader.populate(items);
    }

    addItems(items) {
        this.loader.addItems(items);
    }

    empty() {
        this.loader.empty();
    }

    setNotSelectedValue() {
        this.loader.setNotSelectedValue(...arguments);
    }

    getNotSelectedValue() {
        return this.loader.getNotSelectedValue();
    }

    setValue() {
        this.loader.setValue(...arguments);
    }

    getValue() {
        return this.loader.getValue(...arguments);
    }

    getAllButtons() {
        return this.loader.getAllButtons();
    }

    getAllLeaves() {
        return this.loader.getAllLeaves();
    }

    getSelectedButtons() {
        return this.loader.getSelectedButtons();
    }

    getNotSelectedButtons() {
        return this.loader.getNotSelectedButtons();
    }

    getIndexByValue(value) {
        return this.loader.getIndexByValue(value);
    }

    getNodeById(id) {
        return this.loader.getNodeById(id);
    }

    getNodeByValue(value) {
        return this.loader.getNodeByValue(value);
    }
}
