import { ButtonGroup, LoadingBar } from "@/base";
import {
    VerticalLayout,
    shortcut,
    Widget,
    extend,
    emptyFn,
    Controller,
    createWidget,
    Events,
    nextTick,
    bind,
    isEmpty,
    isNumber,
    isObject,
    isFunction,
    each,
    isNotEmptyArray,
    DOM
} from "@/core";

/**
 * 恶心的加载控件， 为解决排序问题引入的控件
 *
 * Created by GUY on 2015/11/12.
 * @class ListLoader
 * @extends Widget
 */
@shortcut()
export class ListLoader extends Widget {
    static xtype = "bi.list_loader";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-list-loader",
            isDefaultInit: true, // 是否默认初始化数据
            // 下面是button_group的属性
            el: {
                type: ButtonGroup.xtype,
            },
            items: [],
            itemsCreator: emptyFn,
            onLoaded: emptyFn,
            // 下面是分页信息
            count: false,
            next: {},
            hasNext: emptyFn,
        });
    }

    _nextLoad() {
        const o = this.options;
        this.next.setLoading();
        o.itemsCreator.apply(this, [
            {
                times: ++this.times,
            },
            (...args) => {
                this.next.setLoaded();
                this.addItems(...args);
            }
        ]);
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        if (o.itemsCreator === false) {
            o.next = false;
        }

        this.button_group = createWidget(o.el, {
            type: ButtonGroup.xtype,
            element: this,
            chooseType: 0,
            items: o.items,
            behaviors: {},
            layouts: [
                {
                    type: VerticalLayout.xtype,
                }
            ],
        });
        this.button_group.on(Controller.EVENT_CHANGE, (...args) => {
            const [type, , obj] = args;
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
            if (type === Events.CLICK) {
                this.fireEvent(ListLoader.EVENT_CHANGE, obj);
            }
        });

        if (o.next !== false) {
            this.next = createWidget(
                extend(
                    {
                        type: LoadingBar.xtype,
                    },
                    o.next
                )
            );
            this.next.on(Controller.EVENT_CHANGE, type => {
                if (type === Events.CLICK) {
                    this._nextLoad();
                }
            });
        }

        createWidget({
            type: VerticalLayout.xtype,
            element: this,
            items: [this.next],
        });

        o.isDefaultInit &&
            isEmpty(o.items) &&
            nextTick(
                bind(function () {
                    this.populate();
                }, this)
            );
        if (isNotEmptyArray(o.items)) {
            this.populate(o.items);
        }
    }

    hasNext() {
        const o = this.options;
        if (isNumber(o.count)) {
            return this.count < o.count;
        }

        return !!o.hasNext.apply(this, [
            {
                times: this.times,
                count: this.count,
            }
        ]);
    }

    addItems(items) {
        this.count += items.length;
        if (isObject(this.next)) {
            this.options.items = this.options.items.concat(items);
            if (this.hasNext()) {
                this.next.setLoaded();
            } else {
                this.next.setEnd();
            }
        }
        this.button_group.addItems(...arguments);
        this.next.element.appendTo(this.element);
    }

    populate(items) {
        const o = this.options;
        if (arguments.length === 0 && isFunction(o.itemsCreator)) {
            o.itemsCreator.apply(this, [
                {
                    times: 1,
                },
                (...args) => {
                    if (args.length === 0) {
                        throw new Error("parameter cannot be empty");
                    }
                    this.populate(...args);
                    o.onLoaded();
                }
            ]);

            return;
        }
        this.options.items = items;
        this.times = 1;
        this.count = 0;
        this.count += items.length;
        if (isObject(this.next)) {
            if (this.hasNext()) {
                this.next.setLoaded();
            } else {
                this.next.invisible();
            }
        }
        DOM.hang([this.next]);
        this.button_group.populate(...arguments);
        this.next.element.appendTo(this.element);
    }

    empty() {
        DOM.hang([this.next]);
        this.button_group.empty();
        this.next.element.appendTo(this.element);
        each([this.next], (i, ob) => {
            ob && ob.setVisible(false);
        });
    }

    setNotSelectedValue() {
        this.button_group.setNotSelectedValue(...arguments);
    }

    getNotSelectedValue() {
        return this.button_group.getNotSelectedValue();
    }

    setValue() {
        this.button_group.setValue(...arguments);
    }

    getValue() {
        return this.button_group.getValue(...arguments);
    }

    getAllButtons() {
        return this.button_group.getAllButtons();
    }

    getAllLeaves() {
        return this.button_group.getAllLeaves();
    }

    getSelectedButtons() {
        return this.button_group.getSelectedButtons();
    }

    getNotSelectedButtons() {
        return this.button_group.getNotSelectedButtons();
    }

    getIndexByValue(value) {
        return this.button_group.getIndexByValue(value);
    }

    getNodeById(id) {
        return this.button_group.getNodeById(id);
    }

    getNodeByValue(value) {
        return this.button_group.getNodeByValue(value);
    }
}
