import { Checkbox, Label, BasicButton } from "@/base";
import { HalfIconButton } from "../button";
import {
    HTapeLayout,
    CenterAdaptLayout,
    shortcut,
    extend,
    emptyFn,
    i18nText,
    Controller,
    createWidget,
    Events
} from "@/core";

/**
 * guy
 * 复选导航条
 * Created by GUY on 2015/8/25.
 * @class MultiSelectBar
 * @extends BasicButton
 */
@shortcut()
export class MultiSelectBar extends BasicButton {
    static xtype = "bi.multi_select_bar";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            extraCls: "bi-multi-select-bar",
            height: 25,
            text: i18nText("BI-Select_All"),
            isAllCheckedBySelectedValue: emptyFn,
            // 手动控制选中
            disableSelected: true,
            isHalfCheckedBySelectedValue(selectedValues) {
                return selectedValues.length > 0;
            },
            halfSelected: false,
            iconWrapperWidth: 26,
            iconWidth: 14,
            iconHeight: 14,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        const isSelect = o.selected === true;
        const isHalfSelect = !o.selected && o.halfSelected;
        this.checkbox = createWidget({
            type: Checkbox.xtype,
            stopPropagation: true,
            handler: () => {
                this.setSelected(this.isSelected());
            },
            selected: isSelect,
            invisible: isHalfSelect,
            iconWidth: o.iconWidth,
            iconHeight: o.iconHeight,
        });
        this.half = createWidget({
            type: HalfIconButton.xtype,
            stopPropagation: true,
            handler: () => {
                this.setSelected(true);
            },
            invisible: isSelect || !isHalfSelect,
            iconWidth: o.iconWidth,
            iconHeight: o.iconHeight,
        });
        this.checkbox.on(Controller.EVENT_CHANGE, () => {
            this.fireEvent(Controller.EVENT_CHANGE, Events.CLICK, this.isSelected(), this);
        });
        this.checkbox.on(Checkbox.EVENT_CHANGE, () => {
            this.fireEvent(MultiSelectBar.EVENT_CHANGE, this.isSelected(), this);
        });
        this.half.on(Controller.EVENT_CHANGE, () => {
            this.fireEvent(Controller.EVENT_CHANGE, Events.CLICK, this.isSelected(), this);
        });
        this.half.on(HalfIconButton.EVENT_CHANGE, () => {
            this.fireEvent(MultiSelectBar.EVENT_CHANGE, this.isSelected(), this);
        });
        this.text = createWidget({
            type: Label.xtype,
            textAlign: "left",
            whiteSpace: "nowrap",
            textHeight: o.height,
            height: o.height,
            hgap: o.hgap,
            text: o.text,
            keyword: o.keyword,
            value: o.value,
            py: o.py,
        });
        createWidget({
            type: HTapeLayout.xtype,
            element: this,
            items: [
                {
                    width: o.iconWrapperWidth,
                    el: {
                        type: CenterAdaptLayout.xtype,
                        items: [this.checkbox, this.half],
                    },
                },
                {
                    el: this.text,
                }
            ],
        });
    }

    _setSelected(v) {
        this.checkbox.setSelected(!!v);
    }

    beforeClick() {
        const isHalf = this.isHalfSelected(),
            isSelected = this.isSelected();
        if (isHalf === true) {
            this.setSelected(true);
        } else {
            this.setSelected(!isSelected);
        }
    }

    setSelected(v) {
        this.checkbox.setSelected(v);
        this.setHalfSelected(false);
    }

    setHalfSelected(b) {
        this.halfSelected = !!b;
        if (b === true) {
            this.checkbox.setSelected(false);
            this.half.visible();
            this.checkbox.invisible();
        } else {
            this.half.invisible();
            this.checkbox.visible();
        }
    }

    isHalfSelected() {
        return !this.isSelected() && !!this.halfSelected;
    }

    isSelected() {
        return this.checkbox.isSelected();
    }

    setValue(selectedValues) {
        super.setValue(...arguments);
        const isAllChecked = this.options.isAllCheckedBySelectedValue.apply(this, arguments);
        this.setSelected(isAllChecked);
        !isAllChecked && this.setHalfSelected(this.options.isHalfCheckedBySelectedValue.apply(this, arguments));
    }

    doClick() {
        super.doClick(...arguments);
        if (this.isValid()) {
            this.fireEvent(MultiSelectBar.EVENT_CHANGE, this.isSelected(), this);
        }
    }
}
