import { contains, Controller, createWidget, isArray, shortcut, Widget } from "@/core";
import { Expander } from "@/base";
import { TreeExpanderPopup } from "./tree.expander.popup";

@shortcut()
export class TreeExpander extends Widget {
    static xtype = "bi.tree_expander";

    props = {
        baseCls: "bi-tree-expander",
        layer: 0, // 第几层级
        isLastNode: false, //   是不是最后一个
        isFirstNode: false, // 是不是第一个
        selectable: false,
        showLine: true,
    };

    render() {
        const self = this;
        const o = this.options;

        this.trigger = createWidget(o.el, {
            forceNotSelected: !o.selectable,
        });
        this.trigger.on(Controller.EVENT_CHANGE, function (type) {
            o.selectable && self.fireEvent(Controller.EVENT_CHANGE, arguments);
        });

        return {
            type: Expander.xtype,
            ref(_ref) {
                self.expander = _ref;
            },
            trigger: o.selectable ? "" : "click",
            el: this.trigger,
            isDefaultInit: o.isDefaultInit,
            popup: {
                type: TreeExpanderPopup.xtype,
                layer: o.layer || o.el.layer,
                isLastNode: o.isLastNode || o.el.isLastNode,
                isFirstNode: o.isFirstNode || o.el.isFirstNode,
                showLine: o.showLine,
                el: o.popup,
            },
            value: o.value,
            listeners: [
                {
                    eventName: Controller.EVENT_CHANGE,
                    action: (...args) => {
                        this.fireEvent(Controller.EVENT_CHANGE, ...args);
                    },
                }
            ],
        };
    }

    setValue(v) {
        if (contains(v, this.trigger.getValue())) {
            this.trigger.setSelected(true);
            this.expander.setValue([]);
        } else {
            this.trigger.setSelected(false);
            this.expander.setValue(v);
        }
    }

    getValue() {
        if (this.trigger.isSelected()) {
            const value = this.trigger.getValue();

            return isArray(value) ? value : [value];
        }

        return this.expander.getValue();
    }

    populate(items) {
        this.expander.populate(items);
    }

    getAllLeaves() {
        return this.expander && this.expander.getAllLeaves();
    }
}
