import { Label, IconButton, Trigger } from "@/base";
import { TriggerIconButton } from "../button";
import { HorizontalFillLayout, VerticalAdaptLayout, shortcut, isFunction, isKey, isNotEmptyString } from "@/core";

/**
 * 文字trigger
 *
 * Created by GUY on 2015/9/15.
 * @class TextTrigger
 * @extends Trigger
 */
@shortcut()
export class TextTrigger extends Trigger {
    static xtype = "bi.text_trigger";

    static EVENT_CLEAR = "EVENT_CLEAR";

    props() {
        return {
            baseCls: "bi-text-trigger",
            height: 24,
            textHgap: 6,
            textCls: "",
            allowClear: false,
            title: () => this.text.getText(),
            defaultText: "",
            text: "",
        };
    }

    render() {
        const o = this.options;

        const text = this.getText();

        const defaultText = this.getDefaultText();

        const label = {
            type: Label.xtype,
            ref: _ref => {
                this.text = _ref;
            },
            cls: `select-text-label ${o.textCls} ${
                !isNotEmptyString(text) && isNotEmptyString(defaultText) ? "bi-tips" : ""
            }`,
            textAlign: "left",
            height: o.height,
            text: text || o.defaultText,
            tipType: o.tipType,
            warningTitle: o.warningTitle,
            hgap: o.textHgap,
            vgap: o.textVgap,
            lgap: o.textLgap,
            rgap: o.textRgap,
            tgap: o.textTgap,
            bgap: o.textBgap,
            readonly: o.readonly,
        };

        const triggerButton = {
            type: TriggerIconButton.xtype,
            ref: _ref => {
                this.triggerButton = _ref;
            },
            width: o.triggerWidth || o.height,
        };

        return {
            type: HorizontalFillLayout.xtype,
            columnSize: ["fill", ""],
            items: [
                {
                    el: label,
                    width: "fill",
                },
                {
                    el: o.allowClear
                        ? {
                            type: VerticalAdaptLayout.xtype,
                            width: o.triggerWidth || o.height,
                            height: o.height,
                            horizontalAlign: "left",
                            scrollable: false,
                            items: [
                                {
                                    el: {
                                        type: IconButton.xtype,
                                        ref: _ref => {
                                            this.clearBtn = _ref;
                                        },
                                        cls: `close-h-font ${o.allowClear ? "clear-button" : ""}`,
                                        stopPropagation: true,
                                        width: o.triggerWidth || o.height,
                                        invisible: !isNotEmptyString(o.text),
                                        handler: () => {
                                            this.fireEvent(TextTrigger.EVENT_CLEAR);
                                        },
                                    },
                                },
                                {
                                    el: triggerButton,
                                }
                            ],
                        }
                        : triggerButton,
                }
            ],
        };
    }

    getText() {
        const o = this.options;

        return isFunction(o.text) ? o.text() : o.text;
    }

    getDefaultText() {
        const o = this.options;

        return isFunction(o.defaultText) ? o.defaultText() : o.defaultText;
    }

    getTextor() {
        return this.text;
    }

    setTextCls(cls) {
        const o = this.options;
        const oldCls = o.textCls;
        o.textCls = cls;
        this.text.element.removeClass(oldCls).addClass(cls);
    }

    setText(text) {
        if (this.options.allowClear) {
            this.clearBtn.setVisible(isNotEmptyString(text));
        }
        if (isKey(text)) {
            this.text.setText(text);
            this.text.element.removeClass("bi-tips");
        } else if (isKey(this.options.defaultText)) {
            this.text.setText(this.options.defaultText);
            this.text.element.addClass("bi-tips");
        } else {
            this.text.setText("");
            this.text.element.removeClass("bi-tips");
        }
    }

    setTipType(v) {
        this.text.options.tipType = v;
        this.options.tipType = v;
    }
}
