import { TextTrigger } from "./trigger.text";
import { shortcut, extend, emptyFn, createWidget, isFunction, isArray, Tree, each, contains, remove } from "@/core";
import { Trigger } from "@/base";

/**
 * 选择字段trigger
 *
 * Created by GUY on 2015/9/15.
 * @class SelectTextTrigger
 * @extends Trigger
 */
@shortcut()
export class SelectTextTrigger extends Trigger {
    static xtype = "bi.select_text_trigger";

    static EVENT_CLEAR = "EVENT_CLEAR";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-select-text-trigger",
            height: 24,
            allowClear: false,
            valueFormatter: emptyFn,
            defaultText: "",
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        const text = this._digest(o.value, o.items);
        this.trigger = createWidget({
            type: TextTrigger.xtype,
            element: this,
            height: o.height,
            readonly: o.readonly,
            text,
            defaultText: o.defaultText,
            textHgap: o.textHgap,
            textVgap: o.textVgap,
            textLgap: o.textLgap,
            textRgap: o.textRgap,
            textTgap: o.textTgap,
            textBgap: o.textBgap,
            tipType: o.tipType,
            title: null,
            allowClear: o.allowClear,
            listeners: [
                {
                    eventName: TextTrigger.EVENT_CLEAR,
                    action: () => {
                        this.setText("");
                        this.fireEvent(SelectTextTrigger.EVENT_CLEAR);
                    },
                },
            ],
        });
    }

    _digest(val, items) {
        const o = this.options;

        val = isArray(val) ? val.slice() : [val];

        const result = [];

        //  提升valueFormatter的优先级
        if (o.valueFormatter !== emptyFn && isFunction(o.valueFormatter)) {
            each(val, (index, v) => {
                result.push(o.valueFormatter(v));
            });

            return result.join(",");
        }

        const formatItems = Tree.transformToArrayFormat(items);
        each(formatItems, (i, item) => {
            if (contains(val, item.value) && !contains(result, item.text || item.value)) {
                result.push(item.text || item.value);
                remove(val, item.value);
            }
        });

        if (result.length > 0 && val.length === 0) {
            return result.join(",");
        } else {
            return isFunction(o.text) ? o.text() : o.text;
        }
    }

    setText(text) {
        this.options.text = text;
        this.trigger.setText(text);
    }

    setValue(val) {
        const formatText = this._digest(val, this.options.items);
        this.trigger.setText(formatText);
    }

    setTipType(v) {
        this.options.tipType = v;
        this.trigger.setTipType(v);
    }

    getTextor() {
        return this.trigger.getTextor();
    }

    populate(items) {
        this.options.items = items;
    }
}
