import { ListTreeView } from "./listtreeview";
import { cjkEncodeDO, delay, isEmpty, shortcut, extend } from "@/core";
import $ from "jquery";

/**
 * author: windy
 * 继承自treeView, 此树的父子节点的勾选状态互不影响, 此树不会有半选节点
 * 返回value格式为["A", ["A", "a"]]表示勾选了A且勾选了a
 * @class ListListAsyncTree
 * @extends TreeView
 */

@shortcut()
export class ListAsyncTree extends ListTreeView {
    static xtype = "bi.list_async_tree";

    // 配置属性
    _configSetting() {
        const paras = this.options.paras;
        const self = this;

        function beforeCheck(treeId, treeNode) {
            treeNode.half = false;
        }

        function onClick(event, treeId, treeNode) {
            const zTree = $.fn.zTree.getZTreeObj(treeId);
            const checked = treeNode.checked;
            self._checkValue(treeNode, !checked);
            zTree.checkNode(treeNode, !checked, true, true);
        }

        function beforeExpand(treeId, treeNode) {
            self._beforeExpandNode(treeId, treeNode);
        }

        function onCheck(event, treeId, treeNode) {
            self._selectTreeNode(treeId, treeNode);
        }

        return {
            async: {
                enable: false, // 很明显这棵树把异步请求关掉了，所有的异步请求都是手动控制的
                otherParam: cjkEncodeDO(paras),
            },
            check: {
                enable: true,
                chkboxType: { Y: "", N: "" },
            },
            data: {
                key: {
                    title: "title",
                    name: "text",
                },
                simpleData: {
                    enable: true,
                },
            },
            view: {
                showIcon: false,
                expandSpeed: "",
                nameIsHTML: true,
                dblClickExpand: false,
            },
            callback: {
                onCheck,
                beforeExpand,
                beforeCheck,
                onClick,
            },
        };
    }

    // 展开节点
    _beforeExpandNode(treeId, treeNode) {
        const self = this,
            o = this.options;
        const parentValues = treeNode.parentValues || self._getParentValues(treeNode);
        const op = extend({}, o.paras, {
            id: treeNode.id,
            times: 1,
            parentValues: parentValues.concat(this._getNodeValue(treeNode)),
        });

        function complete(d) {
            const nodes = d.items || [];
            if (nodes.length > 0) {
                callback(self._dealWidthNodes(nodes), !!d.hasNext);
            }
        }
        let times = 1;

        function callback(nodes, hasNext) {
            self.nodes.addNodes(treeNode, nodes);
            // 展开节点是没有分页的
            if (hasNext === true) {
                delay(() => {
                    times++;
                    op.times = times;
                    o.itemsCreator(op, complete);
                }, 100);
            }
        }

        if (!treeNode.children) {
            setTimeout(() => {
                o.itemsCreator(op, complete);
            }, 17);
        }
    }

    hasChecked() {
        return !isEmpty(this.options.paras.selectedValues) || super.hasChecked(...arguments);
    }

    // 生成树方法
    stroke(config) {
        delete this.options.keyword;
        extend(this.options.paras, config);
        const setting = this._configSetting();
        this._initTree(setting);
    }
}
