import {
    shortcut,
    Widget,
    Tree,
    createWidget,
    emptyFn,
    isNotNull,
    isNotEmptyArray,
    each,
    makeObject,
    isEmpty
} from "@/core";
import { TreeView } from "./treeview";

/**
 * 简单的多选树
 *
 * Created by GUY on 2016/2/16.
 * @class SimpleTreeView
 * @extends Widget
 */

@shortcut()
export class SimpleTreeView extends Widget {
    static xtype = "bi.simple_tree";

    static EVENT_CHANGE = "EVENT_CHANGE";

    props() {
        return {
            baseCls: "bi-simple-tree",
            itemsCreator: emptyFn,
            items: null,
        };
    }

    _init() {
        super._init(...arguments);
        const self = this,
            o = this.options;
        this.structure = new Tree();
        this.tree = createWidget({
            type: TreeView.xtype,
            element: this,
            itemsCreator(op, callback) {
                function fn(items) {
                    callback({
                        items,
                    });
                    self.structure.initTree(Tree.transformToTreeFormat(items));
                }
                if (isNotNull(o.items)) {
                    fn(o.items);
                } else {
                    o.itemsCreator(op, fn);
                }
            },
        });
        this.tree.on(TreeView.EVENT_CHANGE, function () {
            self.fireEvent(SimpleTreeView.EVENT_CHANGE, arguments);
        });
        if (isNotEmptyArray(o.items)) {
            this.populate();
        }
        if (isNotNull(o.value)) {
            this.setValue(o.value);
        }
    }

    populate(items, keyword) {
        if (items) {
            this.options.items = items;
        }
        this.tree.stroke({
            keyword,
        });
    }

    _digest(v) {
        v || (v = []);
        const self = this,
            map = {};
        const selected = [];
        each(v, (i, val) => {
            const node = self.structure.search(val, "value");
            if (node) {
                let p = node;
                p = p.getParent();
                if (p) {
                    if (!map[p.value]) {
                        map[p.value] = 0;
                    }
                    map[p.value]++;
                }

                while (p && p.getChildrenLength() <= map[p.value]) {
                    selected.push(p.value);
                    p = p.getParent();
                    if (p) {
                        if (!map[p.value]) {
                            map[p.value] = 0;
                        }
                        map[p.value]++;
                    }
                }
            }
        });

        return makeObject(v.concat(selected));
    }

    setValue(v) {
        this.tree.setValue(this._digest(v));
    }

    _getValue() {
        const result = [],
            val = this.tree.getValue();

        function track(nodes) {
            each(nodes, (key, node) => {
                if (isEmpty(node)) {
                    result.push(key);
                } else {
                    track(node);
                }
            });
        }
        track(val);

        return result;
    }

    empty() {
        this.tree.empty();
    }

    getValue() {
        const self = this,
            result = [],
            val = this._getValue();
        each(val, (i, key) => {
            const target = self.structure.search(key, "value");
            if (target) {
                self.structure._traverse(target, node => {
                    if (node.isLeaf()) {
                        result.push(node.value);
                    }
                });
            }
        });

        return result;
    }
}
