import { createWidget, debounce, has, OB, size, each, VerticalLayout } from "@/core";
import { LoadingBar } from "@/base";

/**
 * @author windy
 * @version 2.0
 * Created by windy on 2020/1/8
 * 提供节点分页加载方式
 */

export class TreeRenderPageService extends OB {
    nodeLists = {};

    _getLoadingBar(tId) {
        const tip = createWidget({
            type: LoadingBar.xtype,
            height: 25,
            handler: () => {
                this.refreshNodes(tId);
            },
        });
        tip.setLoaded();

        return tip;
    }

    pushNodeList(tId, populate) {
        const o = this.options;
        const tip = this._getLoadingBar(tId);
        if (!has(this.nodeLists, tId)) {
            this.nodeLists[tId] = {
                populate: debounce(populate, 0),
                options: {
                    times: 1,
                },
                loadWidget: tip,
            };
        } else {
            this.nodeLists[tId].loadWidget.destroy();
            this.nodeLists[tId].loadWidget = tip;
        }
        createWidget({
            type: VerticalLayout.xtype,
            element: o.subNodeListGetter(tId),
            items: [tip],
        });
    }

    refreshNodes(tId) {
        const nodeList = this.nodeLists[tId];
        nodeList.options.times++;
        nodeList.loadWidget.setLoading();
        nodeList.populate({
            times: nodeList.options.times,
        });
    }

    removeNodeList(tId) {
        this.nodeLists[tId] && this.nodeLists[tId].loadWidget.destroy();
        this.nodeLists[tId] && (this.nodeLists[tId].loadWidget = null);
        delete this.nodeLists[tId];
        if (size(this.nodeLists) === 0) {
            this.clear();
        }
    }

    clear() {
        each(this.nodeLists, tId => {
            this.removeNodeList(tId);
        });
        this.nodeLists = {};
    }
}
