import {
    Widget,
    extend,
    emptyFn,
    isNotNull,
    some,
    isNotEmptyArray,
    each,
    Func,
    uniq,
    makeObject,
    filter,
    difference,
    map,
    Selection
} from "@/core";
import { MultiSelectCombo } from "@/widget";

export class AbstractAllValueChooser extends Widget {
    _const = { perPage: 100 };

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            width: 200,
            height: 30,
            items: null,
            itemsCreator: emptyFn,
            cache: true,
        });
    }

    _valueFormatter(v) {
        let text = v;
        if (this.options.valueFormatter) {
            return this.options.valueFormatter(v);
        }
        if (isNotNull(this.items)) {
            some(this.items, (i, item) => {
                // 把value都换成字符串
                // 需要考虑到value也可能是数字
                if (item.value === v || `${item.value}` === v) {
                    text = item.text;

                    return true;
                }
            });
        }

        return text;
    }

    _getItemsByTimes(items, times) {
        const res = [];
        for (let i = (times - 1) * this._const.perPage; items[i] && i < times * this._const.perPage; i++) {
            res.push(items[i]);
        }

        return res;
    }

    _hasNextByTimes(items, times) {
        return times * this._const.perPage < items.length;
    }

    _itemsCreator(options, callback) {
        const self = this,
            o = this.options;
        if (!o.cache || !this.items) {
            o.itemsCreator({}, items => {
                self.items = items;
                call(items);
            });
        } else {
            call(this.items);
        }

        function call(items) {
            const keywords = (options.keywords || []).slice();
            if (options.keyword) {
                keywords.push(options.keyword);
            }
            let resultItems = items;
            if (isNotEmptyArray(keywords)) {
                resultItems = [];
                each(keywords, (i, kw) => {
                    const search = Func.getSearchResult(items, kw);
                    resultItems = resultItems.concat(search.match).concat(search.find);
                });
                resultItems = uniq(resultItems);
            }
            if (options.selectedValues) {
                // 过滤
                const values = makeObject(options.selectedValues, true);
                resultItems = filter(resultItems, (i, ob) => !values[ob.value]);
            }
            if (options.type === MultiSelectCombo.REQ_GET_ALL_DATA) {
                callback({
                    items: resultItems,
                });

                return;
            }
            if (options.type === MultiSelectCombo.REQ_GET_DATA_LENGTH) {
                callback({ count: resultItems.length });

                return;
            }
            callback({
                items: self._getItemsByTimes(resultItems, options.times),
                hasNext: self._hasNextByTimes(resultItems, options.times),
            });
        }
    }

    _assertValue(v) {
        v = v || {};
        let value = v;
        if (isNotNull(this.items)) {
            const isAllSelect = difference(map(this.items, "value"), v.value).length === 0;
            if (isAllSelect) {
                value = {
                    type: Selection.All,
                    value: [],
                };
            }
        }

        return value;
    }
}
