import { shortcut, extend, emptyFn, isNotNull, createWidget, bind, Selection, difference, map } from "@/core";
import { AbstractAllValueChooser } from "./abstract.allvaluechooser";
import { MultiSelectCombo } from "@/widget";

@shortcut()
export class AllValueChooserCombo extends AbstractAllValueChooser {
    static xtype = "bi.all_value_chooser_combo";

    static EVENT_CONFIRM = "EVENT_CONFIRM";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-all-value-chooser-combo",
            width: 200,
            height: 24,
            items: null,
            itemsCreator: emptyFn,
            cache: true,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        if (isNotNull(o.items)) {
            this.items = o.items;
        }
        this.combo = createWidget({
            type: MultiSelectCombo.xtype,
            simple: o.simple,
            text: o.text,
            element: this,
            itemsCreator: bind(this._itemsCreator, this),
            valueFormatter: bind(this._valueFormatter, this),
            width: o.width,
            height: o.height,
            defaultText: o.defaultText,
            value: this._assertValue({
                type: Selection.Multi,
                value: o.value || [],
            }),
        });

        this.combo.on(MultiSelectCombo.EVENT_CONFIRM, () => {
            this.fireEvent(AllValueChooserCombo.EVENT_CONFIRM);
        });
    }

    setValue(v) {
        this.combo.setValue(
            this._assertValue({
                type: Selection.Multi,
                value: v || [],
            })
        );
    }

    getValue() {
        return this.getAllValue();
    }

    getAllValue() {
        const val = this.combo.getValue() || {};
        if (val.type === Selection.Multi) {
            return val.value || [];
        }

        return difference(map(this.items, "value"), val.value || []);
    }

    populate(items) {
        // 直接用combo的populate不会作用到AbstractValueChooser上
        if (isNotNull(items)) {
            this.items = items;
        }
        this.combo.populate();
    }
}
