import { shortcut, extend, emptyFn, createWidget, bind, isNotNull, Selection, difference, map } from "@/core";
import { AbstractAllValueChooser } from "./abstract.allvaluechooser";
import { MultiSelectList } from "@/widget";

@shortcut()
export class AllValueChooserPane extends AbstractAllValueChooser {
    static xtype = "bi.all_value_chooser_pane";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-all-value-chooser-pane",
            width: 200,
            height: 30,
            items: null,
            itemsCreator: emptyFn,
            cache: true,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.list = createWidget({
            type: MultiSelectList.xtype,
            element: this,
            itemsCreator: bind(this._itemsCreator, this),
            valueFormatter: bind(this._valueFormatter, this),
            width: o.width,
            height: o.height,
        });

        this.list.on(MultiSelectList.EVENT_CHANGE, () => {
            this.fireEvent(AllValueChooserPane.EVENT_CHANGE);
        });

        if (isNotNull(o.items)) {
            this.items = o.items;
            this.list.populate();
        }
    }

    setValue(v) {
        this.list.setValue({
            type: Selection.Multi,
            value: v || [],
        });
    }

    getValue() {
        const val = this.list.getValue() || {};
        if (val.type === Selection.Multi) {
            return val.value || [];
        }

        return difference(map(this.items, "value"), val.value || []);
    }

    populate(items) {
        // 直接用combo的populate不会作用到AbstractValueChooser上
        if (isNotNull(items)) {
            this.items = items;
        }
        this.list.populate();
    }
}
