import { AbsoluteLayout, VerticalAdaptLayout, shortcut, Widget, extend, concat, isKey, VerticalAlign } from "@/core";
import { Label } from "@/base";

@shortcut()
export class FormField extends Widget {
    static xtype = "bi.form_field";

    props = {
        baseCls: "bi-form-field",
        label: "",
        el: {},
        headerCls: "",
        labelAlign: "right", // 文字默认右对齐
        validate() {
            return true;
        }, // 默认返回true
    };

    render() {
        const self = this,
            o = this.options;

        const field = {
            type: AbsoluteLayout.xtype,
            items: [
                {
                    el: extend({}, o.el, {
                        ref(_ref) {
                            self.field = _ref;
                            o.el.ref && o.el.ref.call(this, _ref);
                        },
                        height: o.el.height || 28,
                        listeners: concat(o.el.listeners, [
                            {
                                eventName: "EVENT_CHANGE",
                                action() {
                                    self.fireEvent("EVENT_CHANGE");
                                },
                            },
                            {
                                eventName: "EVENT_CONFIRM",
                                action() {
                                    self.fireEvent("EVENT_CONFIRM");
                                },
                            }
                        ]),
                    }),
                    left: 0,
                    bottom: 0,
                    right: 0,
                    top: 0,
                },
                {
                    el: {
                        type: Label.xtype,
                        cls: "error-tip bi-error",
                        ref(_ref) {
                            self.error = _ref;
                        },
                        invisible: true,
                    },
                    bottom: -20,
                    left: 0,
                    right: 0,
                    height: 20,
                }
            ],
        };

        return {
            type: VerticalAdaptLayout.xtype,
            columnSize: isKey(o.label) ? ["auto", "fill"] : ["fill"],
            verticalAlign: VerticalAlign.Stretch,
            items: isKey(o.label)
                ? [
                    {
                        el: {
                            type: Label.xtype,
                            textAlign: o.labelAlign,
                            text: o.label,
                            width: o.labelWidth,
                            cls: o.headerCls,
                            rgap: 20, // 表单文字与右侧输入间距均为20px
                        },
                    },
                    field
                ]
                : [field],
        };
    }

    getValue() {
        return this.field.getValue();
    }

    validate() {
        const isValid = this.validateWithNoTip();
        !isValid && this.error.setText(this.options.tip(this.field.getValue(), this.field));
        this.error.setVisible(!isValid);
        this.field.element[isValid ? "removeClass" : "addClass"]("bi-error");

        return isValid;
    }

    validateWithNoTip() {
        return this.options.validate(this.field.getValue(), this.field);
    }
}
