import {
    isEmpty,
    values,
    each,
    isNull,
    UUID,
    has,
    nextTick,
    last,
    concat,
    filter,
    Func,
    clone,
    any,
    isEqual,
    every,
    flatten,
    map,
    difference
} from "@/core";
import { AbstractTreeValueChooser } from "./abstract.treevaluechooser";

export class AbstractListTreeValueChooser extends AbstractTreeValueChooser {
    _reqDisplayTreeNode(op, callback) {
        const result = {};
        const selectedValues = op.selectedValues;

        if (selectedValues == null || isEmpty(selectedValues)) {
            callback({});
            
            return;
        }

        doCheck.call(this, [], this.tree.getRoot(), selectedValues);

        callback({
            items: values(result),
        });

        function doCheck(parentValues, node, selected) {
            each(selected, (idx, path) => {
                each(path, (id, value) => {
                    const nodeValue = value;
                    const node = this._getTreeNode(path.slice(0, id), nodeValue);
                    // 找不到就是新增值
                    if (isNull(node)) {
                        createOneJson(
                            {
                                id: UUID(),
                                text: nodeValue,
                                value: nodeValue,
                                isLeaf: true,
                            },
                            UUID()
                        );
                    } else {
                        if (!has(result, node.id)) {
                            createOneJson(node, node.parent && node.parent.id);
                        }
                        result[node.id].isLeaf !== true && (result[node.id].isLeaf = id === path.length - 1);
                    }
                });
            });
        }

        function createOneJson(node, pId) {
            result[node.id] = {
                id: node.id,
                pId,
                text: node.text,
                value: node.value,
                open: true,
                isLeaf: node.isLeaf,
            };
        }
    }

    _reqInitTreeNode(op, callback) {
        const self = this;
        let result = [];
        const keyword = op.keyword || "";
        const selectedValues = op.selectedValues;
        const lastSearchValue = op.lastSearchValue || ""; // 一次请求100个，但是搜索是拿全部的，lastSearchValue是上一次遍历到的节点索引
        const output = search();
        nextTick(() => {
            callback({
                hasNext: output.length > this._const.perPage,
                items: result,
                lastSearchValue: last(output),
            });
        });

        function search() {
            const children = self._getChildren([]);
            let start = children.length;
            if (lastSearchValue !== "") {
                for (let j = 0, len = start; j < len; j++) {
                    if (children[j].value === lastSearchValue) {
                        start = j + 1;
                        break;
                    }
                }
            } else {
                start = 0;
            }
            const output = [];
            let find;
            for (let i = start, len = children.length; i < len; i++) {
                if (output.length < self._const.perPage) {
                    find = nodeSearch(1, [], children[i].value, result);
                } else if (output.length === self._const.perPage) {
                    find = nodeSearch(1, [], children[i].value, []);
                }
                if (find[0] === true) {
                    output.push(children[i].value);
                }
                if (output.length > self._const.perPage) {
                    break;
                }
            }

            // 深层嵌套的比较麻烦，这边先实现的是在根节点添加
            if (op.times === 1) {
                const nodes = self._getAddedValueNode([], selectedValues);
                result = concat(
                    filter(nodes, (idx, node) => {
                        const find = Func.getSearchResult([node.text || node.value], keyword);
                        
                        return find.find.length > 0 || find.match.length > 0;
                    }),
                    result
                );
            }
            
            return output;
        };

        function createOneJson(parentValues, value, isOpen, checked, flag, result) {
            const node = self._getTreeNode(parentValues, value);
            result.push({
                id: node.id,
                pId: node.pId,
                text: node.text,
                value: node.value,
                title: node.title,
                isParent: node.getChildrenLength() > 0,
                open: isOpen,
                checked,
                halfCheck: false,
                flag,
                disabled: node.disabled,
            });
        };

        function nodeSearch(deep, parentValues, current, result) {
            let checked;
            if (self._isMatch(parentValues, current, keyword)) {
                checked = isSelected(parentValues, current);
                createOneJson(parentValues, current, false, checked, true, result);
                
                return [true, checked];
            }
            const newParents = clone(parentValues);
            newParents.push(current);
            const children = self._getChildren(newParents);

            let can = false;
            checked = false;

            each(children, (i, child) => {
                const state = nodeSearch(deep + 1, newParents, child.value, result);
                if (state[1] === true) {
                    checked = true;
                }
                if (state[0] === true) {
                    can = true;
                }
            });
            if (can === true) {
                checked = isSelected(parentValues, current);
                createOneJson(parentValues, current, true, checked, false, result);
            }
            
            return [can, checked];
        };

        // eslint-disable-next-line no-unused-vars
        function isHalf(parentValues, value) {
            const find = findSelectedObj(parentValues);
            if (find == null) {
                return null;
            }
            
            return any(find, (v, ob) => {
                if (v === value) {
                    if (ob != null && !isEmpty(ob)) {
                        return true;
                    }
                }
            });
        }

        // eslint-disable-next-line no-unused-vars
        function isAllSelected(parentValues, value) {
            const find = findSelectedObj(parentValues);
            if (find == null) {
                return null;
            }
            
            return any(find, (v, ob) => {
                if (v === value) {
                    if (ob != null && isEmpty(ob)) {
                        return true;
                    }
                }
            });
        }

        function isSelected(parentValues, value) {
            return any(selectedValues, (idx, array) => isEqual(parentValues, array.slice(0, parentValues.length)) && last(array) === value);
        }

        function findSelectedObj(parentValues) {
            let find = selectedValues;
            if (find == null) {
                return null;
            }
            every(parentValues, (i, v) => {
                find = find[v];
                if (find == null) {
                    return false;
                }
                
                return true;
            });
            
            return find;
        }
    }

    _reqTreeNode(op, callback) {
        const o = this.options;
        let result = [];
        const times = op.times;
        const parentValues = op.parentValues || [];
        const selectedValues = op.selectedValues || [];
        const valueMap = dealWithSelectedValue(parentValues, selectedValues);
        const nodes = this._getChildren(parentValues);
        for (let i = (times - 1) * this._const.perPage; nodes[i] && i < times * this._const.perPage; i++) {
            const checked = has(valueMap, nodes[i].value);
            result.push({
                id: nodes[i].id,
                pId: nodes[i].pId,
                value: nodes[i].value,
                text: nodes[i].text,
                times: 1,
                isParent: nodes[i].getChildrenLength() > 0,
                checked,
                halfCheck: false,
                open: o.open,
                disabled: nodes[i].disabled,
            });
        }
        // 如果指定节点全部打开
        if (o.open) {
            let allNodes = [];
            // 获取所有节点
            each(nodes, (idx, node) => {
                allNodes = concat(allNodes, this._getAllChildren(parentValues.concat([node.value])));
            });
            each(allNodes, (idx, node) => {
                const valueMap = dealWithSelectedValue(node.parentValues, selectedValues);
                const checked = has(valueMap, node.value);
                result.push({
                    id: node.id,
                    pId: node.pId,
                    value: node.value,
                    text: node.text,
                    times: 1,
                    isParent: node.getChildrenLength() > 0,
                    checked,
                    halfCheck: false,
                    open: o.open,
                    disabled: node.disabled,
                });
            });
        }
        // 深层嵌套的比较麻烦，这边先实现的是在根节点添加
        if (parentValues.length === 0 && times === 1) {
            result = concat(this._getAddedValueNode(parentValues, selectedValues), result);
        }
        nextTick(() => {
            callback({
                items: result,
                hasNext: nodes.length > times * this._const.perPage,
            });
        });

        function dealWithSelectedValue(parentValues, selectedValues) {
            const valueMap = {};
            each(selectedValues, (idx, v) => {
                if (isEqual(parentValues, v.slice(0, parentValues.length))) {
                    valueMap[last(v)] = [2, 0];
                }
            });
            
            return valueMap;
        }
    }

    _getAddedValueNode(parentValues, selectedValues) {
        const nodes = this._getChildren(parentValues);
        const values = flatten(
            filter(selectedValues, (idx, array) => array.length === 1)
        );
        
        return map(difference(values, map(nodes, "value")), (idx, v) => {
            return {
                id: UUID(),
                pId: nodes.length > 0 ? nodes[0].pId : UUID(),
                value: v,
                text: v,
                times: 1,
                isParent: false,
                checked: true,
                halfCheck: false,
            };
        });
    }
}
