import { shortcut, extend, emptyFn, isNotNull, createWidget, bind } from "@/core";
import { AbstractListTreeValueChooser } from "./abstract.treevaluechooser.list";
import { MultiTreeListCombo, MultiTreeCombo } from "@/widget";
import { TreeValueChooserInsertCombo } from "./combo.treevaluechooser.insert";

@shortcut()
export class ListTreeValueChooserInsertCombo extends AbstractListTreeValueChooser {
    static xtype = "bi.list_tree_value_chooser_insert_combo";

    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_CLICK_ITEM = "EVENT_CLICK_ITEM";
    static EVENT_SEARCHING = "EVENT_SEARCHING";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-list-tree-value-chooser-insert-combo",
            width: 200,
            height: 24,
            items: null,
            itemsCreator: emptyFn,
            isNeedAdjustWidth: true,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        if (isNotNull(o.items)) {
            this._initData(o.items);
        }
        this.combo = createWidget({
            type: MultiTreeListCombo.xtype,
            simple: o.simple,
            isNeedAdjustWidth: o.isNeedAdjustWidth,
            element: this,
            text: o.text,
            defaultText: o.defaultText,
            value: o.value,
            watermark: o.watermark,
            allowInsertValue: o.allowInsertValue,
            allowEdit: o.allowEdit,
            itemsCreator: bind(this._itemsCreator, this),
            valueFormatter: bind(this._valueFormatter, this),
            width: o.width,
            height: o.height,
            listeners: [
                {
                    eventName: MultiTreeListCombo.EVENT_FOCUS,
                    action: () => {
                        this.fireEvent(ListTreeValueChooserInsertCombo.EVENT_FOCUS);
                    },
                },
                {
                    eventName: MultiTreeListCombo.EVENT_BLUR,
                    action: () => {
                        this.fireEvent(ListTreeValueChooserInsertCombo.EVENT_BLUR);
                    },
                },
                {
                    eventName: MultiTreeListCombo.EVENT_STOP,
                    action: () => {
                        this.fireEvent(ListTreeValueChooserInsertCombo.EVENT_STOP);
                    },
                },
                {
                    eventName: MultiTreeListCombo.EVENT_CLICK_ITEM,
                    action: v => {
                        this.fireEvent(ListTreeValueChooserInsertCombo.EVENT_CLICK_ITEM, v);
                    },
                },
                {
                    eventName: MultiTreeListCombo.EVENT_SEARCHING,
                    action: () => {
                        this.fireEvent(ListTreeValueChooserInsertCombo.EVENT_SEARCHING);
                    },
                },
                {
                    eventName: MultiTreeListCombo.EVENT_CONFIRM,
                    action: () => {
                        this.fireEvent(ListTreeValueChooserInsertCombo.EVENT_CONFIRM);
                    },
                },
                {
                    eventName: MultiTreeCombo.EVENT_BEFORE_POPUPVIEW,
                    action: () => {
                        this.fireEvent(TreeValueChooserInsertCombo.EVENT_BEFORE_POPUPVIEW);
                    },
                }
            ],
        });
    }

    showView() {
        this.combo.showView();
    }

    hideView() {
        this.combo.hideView();
    }

    getSearcher() {
        return this.combo.getSearcher();
    }

    setValue(v) {
        this.combo.setValue(v);
    }

    getValue() {
        return this.combo.getValue();
    }

    populate(items) {
        if (isNotNull(items)) {
            this._initData(items);
        }
        this.combo.populate();
    }

    focus() {
        this.combo.focus();
    }

    blur() {
        this.combo.blur();
    }

    setWaterMark(v) {
        this.combo.setWaterMark(v);
    }
}
