import { shortcut, extend, emptyFn, isNotNull, createWidget, bind, Selection, difference, map } from "@/core";
import { AbstractValueChooser } from "./abstract.valuechooser";
import { MultiSelectCombo } from "@/widget";

@shortcut()
export class ValueChooserCombo extends AbstractValueChooser {
    static xtype = "bi.value_chooser_combo";

    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_SEARCHING = "EVENT_SEARCHING";
    static EVENT_CLICK_ITEM = "EVENT_CLICK_ITEM";
    static EVENT_CONFIRM = "EVENT_CONFIRM";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-value-chooser-combo",
            width: 200,
            height: 24,
            items: null,
            itemsCreator: emptyFn,
            cache: true,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        if (isNotNull(o.items)) {
            this.items = o.items;
        }
        this.combo = createWidget({
            type: MultiSelectCombo.xtype,
            simple: o.simple,
            element: this,
            allowEdit: o.allowEdit,
            text: o.text,
            defaultText: o.defaultText,
            value: this._assertValue(o.value),
            itemsCreator: bind(this._itemsCreator, this),
            valueFormatter: bind(this._valueFormatter, this),
            width: o.width,
            height: o.height,
            listeners: [
                {
                    eventName: MultiSelectCombo.EVENT_FOCUS,
                    action: () => {
                        this.fireEvent(ValueChooserCombo.EVENT_FOCUS);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_BLUR,
                    action: () => {
                        this.fireEvent(ValueChooserCombo.EVENT_BLUR);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_STOP,
                    action: () => {
                        this.fireEvent(ValueChooserCombo.EVENT_STOP);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_CLICK_ITEM,
                    action: () => {
                        this.fireEvent(ValueChooserCombo.EVENT_CLICK_ITEM);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_SEARCHING,
                    action: () => {
                        this.fireEvent(ValueChooserCombo.EVENT_SEARCHING);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_CONFIRM,
                    action: () => {
                        this.fireEvent(ValueChooserCombo.EVENT_CONFIRM);
                    },
                }
            ],
        });
    }

    setValue(v) {
        this.combo.setValue(this._assertValue(v));
    }

    getValue() {
        const val = this.combo.getValue() || {};
        
        return {
            type: val.type,
            value: val.value,
        };
    }

    getAllValue() {
        const val = this.combo.getValue() || {};
        if (val.type === Selection.Multi) {
            return val.value || [];
        }

        return difference(map(this.items, "value"), val.value || []);
    }

    populate(items) {
        // 直接用combo的populate不会作用到AbstractValueChooser上
        if (isNotNull(items)) {
            this.items = items;
        }
        this.combo.populate();
    }
}
