import { shortcut, isNotNull, bind } from "@/core";
import { AbstractValueChooser } from "./abstract.valuechooser";
import { MultiSelectCombo, MultiSelectNoBarCombo } from "@/widget";

@shortcut()
export class ValueChooserNoBarCombo extends AbstractValueChooser {
    static xtype = "bi.value_chooser_no_bar_combo";

    props = { baseCls: "bi-value-chooser-combo", width: 200, height: 24, items: null, cache: true };

    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_SEARCHING = "EVENT_SEARCHING";
    static EVENT_CLICK_ITEM = "EVENT_CLICK_ITEM";
    static EVENT_CONFIRM = "EVENT_CONFIRM";

    render() {
        const o = this.options;
        if (isNotNull(o.items)) {
            this.items = o.items;
        }

        return {
            type: MultiSelectNoBarCombo.xtype,
            simple: o.simple,
            allowEdit: o.allowEdit,
            text: o.text,
            defaultText: o.defaultText,
            value: this._assertValue(o.value),
            itemsCreator: bind(this._itemsCreator, this),
            valueFormatter: bind(this._valueFormatter, this),
            width: o.width,
            height: o.height,
            ref: _ref => {
                this.combo = _ref;
            },
            listeners: [
                {
                    eventName: MultiSelectCombo.EVENT_FOCUS,
                    action: () => {
                        this.fireEvent(ValueChooserNoBarCombo.EVENT_FOCUS);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_BLUR,
                    action: () => {
                        this.fireEvent(ValueChooserNoBarCombo.EVENT_BLUR);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_STOP,
                    action: () => {
                        this.fireEvent(ValueChooserNoBarCombo.EVENT_STOP);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_CLICK_ITEM,
                    action: () => {
                        this.fireEvent(ValueChooserNoBarCombo.EVENT_CLICK_ITEM);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_SEARCHING,
                    action: () => {
                        this.fireEvent(ValueChooserNoBarCombo.EVENT_SEARCHING);
                    },
                },
                {
                    eventName: MultiSelectCombo.EVENT_CONFIRM,
                    action: () => {
                        this.fireEvent(ValueChooserNoBarCombo.EVENT_CONFIRM);
                    },
                }
            ],
        };
    }

    setValue(v) {
        this.combo.setValue(v);
    }

    getValue() {
        return this.combo.getValue();
    }

    getAllValue() {
        return this.getValue();
    }

    populate(items) {
        // 直接用combo的populate不会作用到AbstractValueChooser上
        if (isNotNull(items)) {
            this.items = items;
        }
        this.combo.populate();
    }
}
