import { shortcut, extend, emptyFn, createWidget, bind, isNotNull, Selection, difference, map } from "@/core";
import { AbstractValueChooser } from "./abstract.valuechooser";
import { MultiSelectList } from "@/widget";

@shortcut()
export class ValueChooserPane extends AbstractValueChooser {
    static xtype = "bi.value_chooser_pane";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-value-chooser-pane",
            items: null,
            itemsCreator: emptyFn,
            cache: true,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.list = createWidget({
            type: MultiSelectList.xtype,
            element: this,
            value: o.value,
            itemsCreator: bind(this._itemsCreator, this),
            valueFormatter: bind(this._valueFormatter, this),
        });

        this.list.on(MultiSelectList.EVENT_CHANGE, () => {
            this.fireEvent(ValueChooserPane.EVENT_CHANGE);
        });
        if (isNotNull(o.items)) {
            this.items = o.items;
            this.list.populate();
        }
    }

    setValue(v) {
        this.list.setValue(v);
    }

    getValue() {
        const val = this.list.getValue() || {};
        
        return {
            type: val.type,
            value: val.value,
        };
    }

    getAllValue() {
        const val = this.combo.getValue() || {};
        if (val.type === Selection.Multi) {
            return val.value || [];
        }

        return difference(map(this.items, "value"), val.value || []);
    }

    populate(items) {
        // 直接用combo的populate不会作用到AbstractValueChooser上
        if (isNotNull(items)) {
            this.items = items;
        }
        this.list.populate();
    }
}
