/**
 * 可写的常量
 */
import { isNumber, nextTick, setNextTick } from "../2.base";
import { _global } from "../0.foundation";
import { Cache } from "../structure";
import { Element, setElement } from "../element/element";
import { setDom, DOM } from "../utils";

const PropertyDescriptors = {};

export function _defineVarProperties(libName) {
    Object.defineProperties(libName, PropertyDescriptors);
}

function addDescriptor(key, options) {
    options.enumerable = options.enumerable === undefined ? true : options.enumerable;
    options.configurable = options.configurable === undefined ? true : options.configurable;
    PropertyDescriptors[key] = options;
}

export let EVENT_RESPONSE_TIME = 200;
export const setEventResponseTime = v => {
    EVENT_RESPONSE_TIME = v;
};
addDescriptor("EVENT_RESPONSE_TIME", {
    get: () => EVENT_RESPONSE_TIME,
    set: setEventResponseTime,
});

export let pixUnit = "px";
export const setPixUnit = v => {
    pixUnit = v;
};
addDescriptor("pixUnit", {
    get: () => pixUnit,
    set: setPixUnit,
});

export let pixRatio = 1;
export const setPixRatio = v => {
    pixRatio = v;
};
addDescriptor("pixRatio", {
    get: () => pixRatio,
    set: setPixRatio,
});

export let StartOfWeek = 1;
export const setStartOfWeek = v => {
    if (![0, 1].includes(v)) {
        console.error("setStartOfWeek must be 0 or 1");
        return;
    }
    StartOfWeek = v;
};
addDescriptor("StartOfWeek", {
    get: () => StartOfWeek,
    set: setStartOfWeek,
});

export let BlankSplitChar = "\u200b \u200b";
export const setBlankSplitChar = v => {
    BlankSplitChar = v;
};
addDescriptor("BlankSplitChar", {
    get: () => BlankSplitChar,
    set: setBlankSplitChar,
});

// 一定返回最终的单位
export let pixFormat = (pix, border) => {
    if (!isNumber(pix)) {
        return pix;
    }
    if (pixUnit === "px") {
        return pix / pixRatio - (border || 0) + pixUnit;
    }
    const length = pix / pixRatio + pixUnit;
    if (border > 0) {
        return `calc(${length} - ${`${border}px`})`;
    }

    return length;
};
export const setPixFormat = v => {
    pixFormat = v;
};
addDescriptor("pixFormat", {
    get: () => pixFormat,
    set: setPixFormat,
});

export let toPix = (pix, border) => {
    if (!isNumber(pix)) {
        return pix;
    }
    if (pixUnit === "px") {
        return pix - (border || 0) * pixRatio;
    }
    if (border > 0) {
        return `calc(${pix / pixRatio + pixUnit} - ${`${border}px`})`;
    }

    return pix;
};
export let setToPix = v => {
    toPix = v;
};
addDescriptor("toPix", {
    get: () => toPix,
    set: setToPix,
});

const getCacheItem = key => {
    Cache.getItem(key, { typeConversion: true, defaultValue: true });
};
export let EVENT_BLUR = _global.localStorage ? getCacheItem("event.blur") : true;
export const setEventBlur = v => {
    EVENT_BLUR = v;
    Cache.setItem("event.blur", v);
};
addDescriptor("EVENT_BLUR", {
    get: () => EVENT_BLUR,
    set: setEventBlur,
});

addDescriptor("Element", {
    get: () => Element,
    set: setElement,
});

addDescriptor("DOM", {
    get: () => DOM,
    set: setDom,
});

addDescriptor("nextTick", {
    get: () => nextTick,
    set: setNextTick,
});

export let PasteLimit = 2000;
export const setPasteLimit = v => {
    PasteLimit = v;
};
addDescriptor("PasteLimit", {
    get: () => PasteLimit,
    set: setPasteLimit,
});
