/**
 * guy
 * popover弹出层控制器, z-index在100w层级
 */
import { Controller } from "./0.controller";
import { each, isNotNull } from "../2.base";
import { createWidget } from "../5.inject";

export class DrawerController extends Controller {
    constructor() {
        super();
        this._constructor();
        this.modal = this.options.modal;
    }
    props = {
        modal: true, // 模态窗口
        render: "body",
    }

    init() {
        this.floatManager = {};
        this.floatLayer = {};
        this.floatContainer = {};
        this.floatOpened = {};
        this.zindexMap = {};
    }

    create(name, options, context) {
        if (this.has(name)) {
            return this;
        }
        const popover = createWidget(options || {}, {
            type: "bi.drawer",
        }, context);
        this.add(name, popover, options, context);
        
        return this;
    }

    open(name) {
        if (!this.has(name)) {
            return this;
        }
        if (!this.floatOpened[name]) {
            this.floatOpened[name] = true;
            const container = this.floatContainer[name];
            const zIndex = BI.Popovers._getZIndex();
            container.element.css("zIndex", zIndex);
            this.modal && container.element.__hasZIndexMask__(this.zindexMap[name]) && container.element.__releaseZIndexMask__(this.zindexMap[name]);
            this.zindexMap[name] = zIndex;
            if (this.modal) {
                const mask = container.element.__buildZIndexMask__(BI.Popovers._getZIndex());
                mask.click(() => {
                    mask.destroy();
                    this.get(name).close();
                });
            }
            this.get(name).setZindex(BI.Popovers._getZIndex());
            this.floatContainer[name].visible();
            const popover = this.get(name);
            popover.show && popover.show();
        }
        
        return this;
    }

    close(name) {
        if (!this.has(name)) {
            return this;
        }
        if (this.floatOpened[name]) {
            delete this.floatOpened[name];
            this.floatContainer[name].invisible();
            this.modal && this.floatContainer[name].element.__releaseZIndexMask__(this.zindexMap[name]);
        }
        
        return this;
    }

    show(name) {
        return this.open(name);
    }

    hide(name) {
        return this.close(name);
    }

    isVisible(name) {
        return this.has(name) && this.floatOpened[name] === true;
    }

    add(name, popover, options, context) {
        options || (options = {});
        if (this.has(name)) {
            return this;
        }
        this.floatContainer[name] = createWidget({
            type: "bi.absolute",
            cls: "bi-popup-view",
            items: [{
                el: (this.floatLayer[name] = createWidget({
                    type: "bi.absolute",
                    items: [popover],
                }, context)),
                left: 0,
                right: 0,
                top: 0,
                bottom: 0,
            }],
        });
        this.floatManager[name] = popover;
        popover.on(BI.Drawer.EVENT_CLOSE, () => this.close(name));
        createWidget({
            type: "bi.absolute",
            element: options.container || this.options.render,
            items: [{
                el: this.floatContainer[name],
                left: 0,
                right: 0,
                top: 0,
                bottom: 0,
            }],
        });
        
        return this;
    }

    get(name) {
        return this.floatManager[name];
    }

    has(name) {
        return isNotNull(this.floatManager[name]);
    }

    remove(name) {
        if (!this.has(name)) {
            return this;
        }
        this.floatContainer[name].destroy();
        this.modal && this.floatContainer[name].element.__releaseZIndexMask__(this.zindexMap[name]);
        delete this.floatManager[name];
        delete this.floatLayer[name];
        delete this.zindexMap[name];
        delete this.floatContainer[name];
        delete this.floatOpened[name];
        
        return this;
    }

    removeAll() {
        each(this.floatContainer, (name, container) => {
            container.destroy();
            this.modal && this.floatContainer[name].element.__releaseZIndexMask__(this.zindexMap[name]);
        });
        this.floatManager = {};
        this.floatLayer = {};
        this.floatContainer = {};
        this.floatOpened = {};
        this.zindexMap = {};
        
        return this;
    }
}
