/**
 * window.resize 控制器
 *
 * Created by GUY on 2015/6/24.
 */
import { Controller } from "./0.controller";
import { isNull, each, debounce, isNotNull, isFunction } from "../2.base";
import { Widget } from "../4.widget";
import { _global } from "../0.foundation";
import { Layout } from "../wrapper/layout";
import $ from "jquery";

export class ResizeController extends Controller {
    init() {
        this.resizerManger = {};
    }

    _initResizeListener() {
        this.resizeHandler = debounce(ev => this._resize(ev), 30);
        if ("onorientationchange" in _global) {
            _global.onorientationchange = this.resizeHandler;
        } else {
            Widget._renderEngine.createElement(_global).resize(this.resizeHandler);
        }
    }

    _resize(ev) {
        each(this.resizerManger, (key, resizer) => {
            if (resizer instanceof $) {
                if (resizer.is(":visible")) {
                    resizer.trigger("__resize__");
                }
                
                return;
            }
            if (resizer instanceof Layout) {
                resizer.resize();
                
                return;
            }
            if (isFunction(resizer)) {
                resizer(ev);
            }
        });
    }

    add(name, resizer) {
        isNull(this.resizeHandler) && this._initResizeListener();

        if (this.has(name)) {
            return this;
        }
        this.resizerManger[name] = resizer;
        
        return () => this.remove(name);
    }

    get(name) {
        return this.resizerManger[name];
    }

    has(name) {
        return isNotNull(this.resizerManger[name]);
    }

    remove(name) {
        if (!this.has(name)) {
            return this;
        }
        delete this.resizerManger[name];
        
        return this;
    }
}
