import * as inject from "./5.inject";

const transFunc2Decorator = targetFunc => {
    return function () {
        return function decorator(Target) {
            targetFunc(Target.xtype, Target);
        };
    };
};

export const shortcut = transFunc2Decorator(inject.shortcut);
export const service = transFunc2Decorator(inject.service);
export const provider = transFunc2Decorator(inject.provider);
export const model = transFunc2Decorator(inject.model);

/**
 * 类注册_store属性
 * @param Model model类
 * @param opts 额外条件
 */
export function store(Model, opts = {}) {
    return function classDecorator(constructor) {
        return class extends constructor {
            _store() {
                const props = opts.props ? opts.props.apply(this) : undefined;

                return inject.Models.getModel(Model.xtype, props);
            }
        };
    };
}

/**
 * 注册mixin
 * ie8下不能使用
 */
export function mixin() {
    return function decorator(Target) {
        const mixin = {};

        Object.getOwnPropertyNames(Target.prototype).forEach(name => {
            if (name === "constructor") {
                return;
            }

            mixin[name] = Target.prototype[name];
        });

        Fix.mixin(Target.xtype, mixin);
    };
}

/**
 * 类注册mixins属性
 * ie8下不能使用
 * @param Mixins
 */
export function mixins(...Mixins) {
    return function classDecorator(constructor) {
        const mixins = [];

        Mixins.forEach(mixin => {
            mixin.xtype && mixins.push(mixin.xtype);
        });

        return class extends constructor {
            mixins = mixins;
        };
    };
}
