import { ResizeObserver as ResizeObserverPolyfill } from "@juggle/resize-observer";
import $ from "jquery";

const ResizeObserver = window.ResizeObserver || ResizeObserverPolyfill;

function addResizeListener(element, fn) {
    if (ResizeObserver) {
        if (!element.__resizeObserver__) {
            const resizeObserver = new ResizeObserver((() => {
                element.__resizeListeners__.forEach(listener => {
                    $(element).is(":visible") && listener();
                });
            }));
            resizeObserver.observe(element);
            element.__resizeObserver__ = resizeObserver;
        }
        if (!element.__resizeListeners__) {
            element.__resizeListeners__ = [];
        }
        element.__resizeListeners__.push(fn);
    }
}

function removeResizeListener(element, fn) {
    if (ResizeObserver) {
        if (BI.isNull(fn)) {
            element.__resizeListeners__ = [];
            element.__resizeObserver__ && element.__resizeObserver__.unobserve(element);
            element.__resizeObserver__ = null;

            return;
        }
        const index = element.__resizeListeners__.indexOf(fn);
        if (index >= 0) {
            element.__resizeListeners__.splice(index, 1);
            if (!element.__resizeListeners__.length) {
                element.__resizeObserver__ && element.__resizeObserver__.unobserve(element);
                element.__resizeObserver__ = null;
            }
        }
    }
}

export const ResizeDetector = {
    addResizeListener(widget, fn) {
        addResizeListener(widget.element[0], fn);

        return function () {
            // 如果fn是debounce需要cancel掉
            fn?.cancel?.();
            removeResizeListener(widget.element[0], fn);
        };
    },
    removeResizeListener(widget, fn) {
        // 如果fn是debounce需要cancel掉
        fn?.cancel?.();
        removeResizeListener(widget.element[0], fn);
    },
};
