function _aspect(type) {
    return function (target, methodName, advice) {
        let exist = target[methodName],
            dispatcher;

        if (!exist || exist.target != target) {
            dispatcher = target[methodName] = function () {
                // before methods
                let beforeArr = dispatcher.before;
                let args = arguments, next;
                for (let l = beforeArr.length; l--;) {
                    next = beforeArr[l].advice.apply(this, args);
                    if (next === false) {
                        return false;
                    }
                    args = next || args;
                }
                // target method
                let rs = dispatcher.method.apply(this, args);
                // after methods
                let afterArr = dispatcher.after;
                for (let i = 0, ii = afterArr.length; i < ii; i++) {
                    next = afterArr[i].advice.call(this, rs, args);
                    if (rs === false) {
                        return false;
                    }
                    args = next || args;
                }
                return rs;
            };

            dispatcher.before = [];
            dispatcher.after = [];

            if (exist) {
                dispatcher.method = exist;
            }
            dispatcher.target = target;
        }

        let aspectArr = (dispatcher || exist)[type];
        let obj = {
            advice: advice,
            _index: aspectArr.length,
            remove: function () {
                aspectArr.splice(this._index, 1);
            },
        };
        aspectArr.push(obj);

        return obj;
    };
}

export const aspect = {
    before: _aspect("before"),
    after: _aspect("after"),
};
