export const Cache = {
    _prefix: "bi",
    setUsername: function (username) {
        localStorage.setItem(Cache._prefix + ".username", (username + "" || "").toUpperCase());
    },
    getUsername: function () {
        return localStorage.getItem(Cache._prefix + ".username") || "";
    },
    _getKeyPrefix: function () {
        return Cache.getUsername() + "." + Cache._prefix + ".";
    },
    _generateKey: function (key) {
        return Cache._getKeyPrefix() + (key || "");
    },
    getItem: function (key, opt = {}) {
        const { defaultValue = null, typeConversion } = opt;
        const storageKey = Cache._generateKey(key);
        let value = localStorage.getItem(storageKey);

        if (value !== null && typeConversion) {
            if (value === "true") {
                value = true;
            } else if (value === "false") {
                value = false;
            } else if (Number.isNaN(Number(value))) {
                value = Number(value);
            }
        }

        return value !== null ? value : defaultValue;
    },
    setItem: function (key, value) {
        localStorage.setItem(Cache._generateKey(key), value);
    },
    removeItem: function (key) {
        localStorage.removeItem(Cache._generateKey(key));
    },
    clear: function () {
        for (let i = localStorage.length; i >= 0; i--) {
            const key = localStorage.key(i);
            if (key) {
                if (key.indexOf(Cache._getKeyPrefix()) === 0) {
                    localStorage.removeItem(key);
                }
            }
        }
    },
    keys: function () {
        const result = [];
        for (let i = localStorage.length; i >= 0; i--) {
            const key = localStorage.key(i);
            if (key) {
                const prefix = Cache._getKeyPrefix();
                if (key.indexOf(prefix) === 0) {
                    result[result.length] = key.substring(prefix.length);
                }
            }
        }
        return result;
    },

    addCookie: function (name, value, path, expiresHours) {
        let cookieString = name + "=" + encodeURI(value);
        // 判断是否设置过期时间
        if (expiresHours && expiresHours > 0) {
            const date = new Date();
            // expires是标准GMT格式时间，应该使用时间戳作为起始时间
            date.setTime(date.getTime() + expiresHours * 3600 * 1000);
            cookieString = cookieString + "; expires=" + date.toUTCString();
        }
        if (path) {
            cookieString = cookieString + "; path=" + path;
        }
        document.cookie = cookieString;
    },
    getCookie: function (name) {
        let arr, reg = new RegExp("(^| )" + name + "=([^;]*)(;|$)");
        if (arr = document.cookie.match(reg)) {
            return decodeURI(arr[2]);
        }
        return null;
    },
    deleteCookie: function (name, path) {
        const date = new Date();
        date.setTime(date.getTime() - 10000);
        let cookieString = name + "=v; expires=" + date.toUTCString();
        if (path) {
            cookieString = cookieString + "; path=" + path;
        }
        document.cookie = cookieString;
    },
};
