import {each, keys, map, size} from "../2.base";

class Section {
    constructor(height, width, x, y) {
        this.height = height;
        this.width = width;
        this.x = x;
        this.y = y;

        this._indexMap = {};
        this._indices = [];
    }

    addCellIndex(index) {
        if (!this._indexMap[index]) {
            this._indexMap[index] = true;
            this._indices.push(index);
        }
    }

    getCellIndices() {
        return this._indices;
    }
}

const SECTION_SIZE = 100;

export class SectionManager {
    constructor(sectionSize = SECTION_SIZE) {
        this._sectionSize = sectionSize;
        this._cellMetadata = [];
        this._sections = {};
    }

    getCellIndices(height, width, x, y) {
        const indices = {};

        each(this.getSections(height, width, x, y), function (i, section) {
            each(section.getCellIndices(), function (j, index) {
                indices[index] = index;
            });
        });

        return map(keys(indices), function (i, index) {
            return indices[index];
        });
    }

    getCellMetadata(index) {
        return this._cellMetadata[index];
    }

    getSections(height, width, x, y) {
        const sectionXStart = Math.floor(x / this._sectionSize);
        const sectionXStop = Math.floor((x + width - 1) / this._sectionSize);
        const sectionYStart = Math.floor(y / this._sectionSize);
        const sectionYStop = Math.floor((y + height - 1) / this._sectionSize);

        const sections = [];

        for (let sectionX = sectionXStart; sectionX <= sectionXStop; sectionX++) {
            for (let sectionY = sectionYStart; sectionY <= sectionYStop; sectionY++) {
                const key = sectionX + "." + sectionY;

                if (!this._sections[key]) {
                    this._sections[key] = new Section(this._sectionSize, this._sectionSize, sectionX * this._sectionSize, sectionY * this._sectionSize);
                }

                sections.push(this._sections[key]);
            }
        }

        return sections;
    }

    getTotalSectionCount() {
        return size(this._sections);
    }

    registerCell(cellMetadatum, index) {
        this._cellMetadata[index] = cellMetadatum;

        each(this.getSections(cellMetadatum.height, cellMetadatum.width, cellMetadatum.x, cellMetadatum.y), function (i, section) {
            section.addCellIndex(index);
        });
    }
}
