/**
 * @author windy
 * @version 2.0
 * Created by windy on 2021/6/30
 */
import { prepares } from "./0.foundation";
import { deepExtend, extend } from "./2.base";
import { OB } from "./3.ob";
import { Providers } from "./5.inject";
import { provider } from "./decorator";
import { isIE as checkIE } from "./platform/web";

// 系统参数常量
const system = {
    dependencies: {},
    layoutOptimize: false,
    responsiveMode: false,
    workerMode: false,
    size: {
        // 尺寸
        // 通用尺寸
        TOOL_BAR_HEIGHT: 24,
        LIST_ITEM_HEIGHT: 24,
        TRIGGER_HEIGHT: 24,
        TOAST_TOP: 10,
        H_GAP_SIZE: "M",
        V_GAP_SIZE: "S",
    },
    loadingCreator(config) {
        const loadingSize = (config ? config.loadingSize : "small") || "small";

        const isIE = checkIE();

        function getSize(v) {
            return Math.ceil(v / (loadingSize === "small" ? 2 : 1));
        }

        return {
            type: "bi.horizontal",
            cls: `bi-loading-widget${isIE ? " wave-loading hack" : ""}`,
            height: getSize(60),
            width: getSize(60),
            hgap: getSize(10),
            vgap: 2.5,
            items: isIE ? [] : [{
                type: "bi.layout",
                cls: "animate-rect rect1",
                height: getSize(50),
                width: getSize(5),
            }, {
                type: "bi.layout",
                cls: "animate-rect rect2",
                height: getSize(50),
                width: getSize(5),
            }, {
                type: "bi.layout",
                cls: "animate-rect rect3",
                height: getSize(50),
                width: getSize(5),
            }],
        };
    },
};

// 具体尺寸还没定，先写着
const sizeMap = {
    S: 10,
    M: 20,
    L: 24,
};

class SystemProviderOB extends OB {
    getSize () {
        const size = system.size;
        const H_GAP_SIZE = sizeMap[size.H_GAP_SIZE];
        const V_GAP_SIZE = sizeMap[size.V_GAP_SIZE];

        return extend({}, size, {
            H_GAP_SIZE,
            V_GAP_SIZE,
        });
    }

    getResponsiveMode () {
        return system.responsiveMode;
    }

    getWorkerMode () {
        return system.workerMode;
    }

    getLayoutOptimize () {
        return system.layoutOptimize;
    }

    getDependencies () {
        return system.dependencies;
    }

    getLoading(config) {
        return system.loadingCreator(config);
    }
}

@provider()
export class SystemProvider {
    static xtype = "bi.provider.system";

    SYSTEM = system;

    setSize(opt) {
        deepExtend(system, { size: opt });
    }

    setResponsiveMode(mode) {
        system.responsiveMode = !!mode;
    }

    setWorkerMode(mode) {
        system.workerMode = !!mode;
    }

    setLayoutOptimize(layoutOptimize) {
        system.layoutOptimize = layoutOptimize;
    }

    addDependency(moduleId, minVersion, maxVersion) {
        system.dependencies[moduleId] = {
            min: minVersion,
            max: maxVersion,
        };
    }

    addDependencies(moduleConfig) {
        extend(system.dependencies, moduleConfig);
    }

    setLoadingCreator = function(creator) {
        system.loadingCreator = creator;
    };

    $get() {
        return SystemProviderOB;
    }
}

export const SIZE_CONSANTS = {};
// 不再增加线型的配置了,之后不维护前置版本直接删掉，都用实线连接线
export const STYLE_CONSTANTS = {};

prepares.push(() => {
    extend(SIZE_CONSANTS, Providers.getProvider(SystemProvider.xtype).getSize());
    STYLE_CONSTANTS.LINK_LINE_TYPE = SIZE_CONSANTS.LIST_ITEM_HEIGHT === 24 ? "dashed" : "solid";
});
