import { shortcut } from "../../../decorator";
import { Layout } from "../../layout";
import { extend, isFunction } from "../../../2.base";

/**
 * absolute实现的居中布局
 * @class AbsoluteCenterLayout
 * @extends Layout
 */
@shortcut()
export class AbsoluteCenterLayout extends Layout {
    static xtype ="bi.absolute_center_adapt";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-abs-c-a",
            hgap: 0,
            lgap: 0,
            rgap: 0,
            vgap: 0,
            tgap: 0,
            bgap: 0,
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        w.element.css({
            position: "absolute",
            left: this._optimiseGap(o.hgap + o.lgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item)),
            right: this._optimiseGap(o.hgap + o.rgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item)),
            top: this._optimiseGap(o.vgap + o.tgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item)),
            bottom: this._optimiseGap(o.vgap + o.bgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item)),
            margin: "auto",
        });
        
        return w;
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
