import { shortcut } from "../../../decorator";
import { Layout } from "../../layout";
import { extend, isArray, each, map, stripEL } from "../../../2.base";
import { VerticalAlign } from "../../../constant";

@shortcut()
export class AbsoluteLeftRightVerticalAdaptLayout extends Layout {
    static xtype = "bi.absolute_left_right_vertical_adapt";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-abs-lr-v-a",
            verticalAlign: VerticalAlign.Middle,
            items: {},
            llgap: 0,
            lrgap: 0,
            lhgap: 0,
            ltgap: 0,
            lbgap: 0,
            lvgap: 0,
            rlgap: 0,
            rrgap: 0,
            rhgap: 0,
            rtgap: 0,
            rbgap: 0,
            rvgap: 0,
        });
    }

    render() {
        const o = this.options;
        super.render(...arguments);
        
        return {
            type: "bi.htape",
            innerHgap: o.innerHgap,
            innerVgap: o.innerVgap,
            ref: _ref => {
                this.layout = _ref;
            },
            verticalAlign: o.verticalAlign,
            items: this._formatItems(o.items),
            scrollx: o.scrollx,
            scrolly: o.scrolly,
            scrollable: o.scrollable,
        };
    }

    _formatItems(items) {
        const o = this.options;
        let left, right;
        if (isArray(items)) {
            each(items, (i, item) => {
                if (item.left) {
                    left = item.left;
                }
                if (item.right) {
                    right = item.right;
                }
            });
        }
        let leftItems = left || items.left || [];
        let rightItems = right || items.right || [];
        leftItems = map(leftItems, (i, item) => {
            const json = {
                el: stripEL(item),
                width: item.width,
            };
            if (o.lvgap + o.ltgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item) !== 0) {
                json.tgap = o.lvgap + o.ltgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item);
            }
            if (o.lhgap + o.llgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item) !== 0) {
                json.lgap = (i === 0 ? o.lhgap : 0) + o.llgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item);
            }
            if (o.lhgap + o.lrgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item) !== 0) {
                json.rgap = o.lhgap + o.lrgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item);
            }
            if (o.lvgap + o.lbgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item) !== 0) {
                json.bgap = o.lvgap + o.lbgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item);
            }
            
            return json;
        });
        rightItems = map(rightItems, (i, item) => {
            const json = {
                el: stripEL(item),
                width: item.width,
            };
            if (o.rvgap + o.rtgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item) !== 0) {
                json.tgap = o.rvgap + o.rtgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item);
            }
            if (o.rhgap + o.rlgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item) !== 0) {
                json.lgap = (i === 0 ? o.rhgap : 0) + o.rlgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item);
            }
            if (o.rhgap + o.rrgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item) !== 0) {
                json.rgap = o.rhgap + o.rrgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item);
            }
            if (o.rvgap + o.rbgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item) !== 0) {
                json.bgap = o.rvgap + o.rbgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item);
            }
            
            return json;
        });
        
        return leftItems.concat({}, rightItems);
    }

    resize() {
        this.layout.stroke(this._formatItems(this.options.items));
    }

    addItem() {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    populate(items) {
        this.layout.populate(this._formatItems(items));
    }
}

@shortcut()
export class AbsoluteRightVerticalAdaptLayout extends Layout {
    static xtype = "bi.absolute_right_vertical_adapt";

    props () {
        return extend(super.props(...arguments), {
            baseCls: "bi-abs-r-v-a",
            verticalAlign: VerticalAlign.Middle,
            items: [],
            lgap: 0,
            rgap: 0,
            hgap: 0,
            tgap: 0,
            bgap: 0,
            vgap: 0,
        });
    }

    render () {
        const o = this.options;
        super.render(...arguments);
        
        return {
            type: "bi.htape",
            ref: _ref => {
                this.layout = _ref;
            },
            verticalAlign: o.verticalAlign,
            items: [{}].concat(o.items),
            hgap: o.hgap,
            lgap: o.lgap,
            rgap: o.rgap,
            tgap: o.tgap,
            bgap: o.bgap,
            vgap: o.vgap,
            scrollx: o.scrollx,
            scrolly: o.scrolly,
            scrollable: o.scrollable,
        };
    }

    resize () {
        this.layout.stroke([{}].concat(this.options.items));
    }

    addItem () {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    populate (items) {
        this.layout.populate([{}].concat(items));
    }
}
