import { shortcut } from "../../../decorator";
import { Layout } from "../../layout";
import { extend, isArray, each } from "../../../2.base";
import { HorizontalAlign } from "../../../constant";

/**
 * 左右分离，垂直方向居中容器
 *          items:{
                left: [{el:{type:"bi.button"}}],
                right:[{el:{type:"bi.button"}}]
            }
 * @class LeftRightVerticalAdaptLayout
 * @extends Layout
 */
@shortcut()
export class LeftRightVerticalAdaptLayout extends Layout {
    static xtype = "bi.left_right_vertical_adapt";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-lr-v-a",
            items: {},
            llgap: 0,
            lrgap: 0,
            lhgap: 0,
            ltgap: 0,
            lbgap: 0,
            lvgap: 0,
            rlgap: 0,
            rrgap: 0,
            rhgap: 0,
            rtgap: 0,
            rbgap: 0,
            rvgap: 0,
        });
    }

    render() {
        const o = this.options;
        super.render(...arguments);
        const leftRight = this._getLeftRight(o.items);
        const layoutArray = [];
        if (leftRight.left || "left" in o.items) {
            layoutArray.push({
                type: "bi.left",
                lgap: o.innerHgap,
                vgap: o.innerVgap,
                items: [{
                    el: {
                        type: "bi.vertical_adapt",
                        ref: _ref => {
                            this.left = _ref;
                        },
                        height: "100%",
                        items: leftRight.left || o.items.left,
                        hgap: o.lhgap,
                        lgap: o.llgap,
                        rgap: o.lrgap,
                        tgap: o.ltgap,
                        bgap: o.lbgap,
                        vgap: o.lvgap,
                    },
                }],
            });
        }
        if (leftRight.right || "right" in o.items) {
            layoutArray.push({
                type: "bi.right",
                rgap: o.innerHgap,
                vgap: o.innerVgap,
                items: [{
                    el: {
                        type: "bi.vertical_adapt",
                        ref: _ref => {
                            this.right = _ref;
                        },
                        height: "100%",
                        items: leftRight.right || o.items.right,
                        hgap: o.rhgap,
                        lgap: o.rlgap,
                        rgap: o.rrgap,
                        tgap: o.rtgap,
                        bgap: o.rbgap,
                        vgap: o.rvgap,
                    },
                }],
            });
        }
        
        return layoutArray;
    }

    _getLeftRight(items) {
        let left, right;
        if (isArray(items)) {
            each(items, (i, item) => {
                if (item.left) {
                    left = item.left;
                }
                if (item.right) {
                    right = item.right;
                }
            });
        }
        
        return {
            left,
            right,
        };
    }

    resize() {
        const leftRight = this._getLeftRight(this.options.items);
        this.left.stroke(leftRight.left || this.options.items.left);
        this.right.stroke(leftRight.right || this.options.items.right);
    }

    addItem() {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    populate(items) {
        const leftRight = this._getLeftRight(items);
        this.left.populate(leftRight.left || items.left);
        this.right.populate(leftRight.right || items.right);
    }
}

@shortcut()
export class LeftVerticalAdaptLayout extends Layout {
    static xtype = "bi.left_vertical_adapt";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-l-v-a",
            items: [],
            columnSize: [],
            lgap: 0,
            rgap: 0,
            hgap: 0,
            tgap: 0,
            bgap: 0,
            vgap: 0,
        });
    }

    render() {
        const o = this.options;
        super.render(...arguments);
        
        return {
            type: "bi.vertical_adapt",
            ref: _ref => {
                this.layout = _ref;
            },
            items: o.items,
            columnSize: o.columnSize,
            hgap: o.hgap,
            lgap: o.lgap,
            rgap: o.rgap,
            tgap: o.tgap,
            bgap: o.bgap,
            vgap: o.vgap,
            innerHgap: o.innerHgap,
            innerVgap: o.innerVgap,
            scrollx: o.scrollx,
            scrolly: o.scrolly,
            scrollable: o.scrollable,
        };
    }

    resize() {
        this.layout.resize();
    }

    addItem() {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    populate(items) {
        this.layout.populate(...arguments);
    }
}

@shortcut()
export class RightVerticalAdaptLayout extends Layout {
    static xtype = "bi.right_vertical_adapt";
    
    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-r-v-a",
            items: [],
            columnSize: [],
            lgap: 0,
            rgap: 0,
            hgap: 0,
            tgap: 0,
            bgap: 0,
            vgap: 0,
        });
    }

    render() {
        const o = this.options;
        super.render(...arguments);
        
        return {
            type: "bi.vertical_adapt",
            ref: _ref => {
                this.layout = _ref;
            },
            horizontalAlign: HorizontalAlign.Right,
            items: o.items,
            columnSize: o.columnSize,
            hgap: o.hgap,
            lgap: o.lgap,
            rgap: o.rgap,
            tgap: o.tgap,
            bgap: o.bgap,
            vgap: o.vgap,
            innerHgap: o.innerHgap,
            innerVgap: o.innerVgap,
            scrollx: o.scrollx,
            scrolly: o.scrolly,
            scrollable: o.scrollable,
        };
    }

    resize() {
        this.layout.resize();
    }

    addItem() {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    populate(...args) {
        this.layout.populate(...args);
    }
}
