import { shortcut } from "../../../decorator";
import { Layout } from "../../layout";
import { extend } from "../../../2.base";
import { HorizontalAlign, VerticalAlign } from "../../../constant";

/**
 * 内联布局
 * @class InlineCenterAdaptLayout
 * @extends Layout
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Number} [hgap=0] 水平间隙
 * @cfg {Number} [vgap=0] 垂直间隙
 */
@shortcut()
export class InlineCenterAdaptLayout extends Layout {
    static xtype = "bi.inline_center_adapt";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-i-c-a",
            horizontalAlign: HorizontalAlign.Center,
            verticalAlign: VerticalAlign.Middle,
            columnSize: [],
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
        });
    }

    render() {
        const o = this.options;
        
        return {
            type: "bi.inline",
            ref: _ref => {
                this.layout = _ref;
            },
            items: o.items,
            horizontalAlign: o.horizontalAlign,
            verticalAlign: o.verticalAlign,
            columnSize: o.columnSize,
            hgap: o.hgap,
            vgap: o.vgap,
            lgap: o.lgap,
            rgap: o.rgap,
            tgap: o.tgap,
            bgap: o.bgap,
            innerHgap: o.innerHgap,
            innerVgap: o.innerVgap,
        };
    }

    resize() {
        this.layout.resize();
    }

    populate(...args) {
        this.layout.populate(...args);
    }
}
