import { shortcut } from "@/core/decorator";
import { extend, any, isEmptyObject, isNull, backAny } from "@/core/2.base";
import { HorizontalAlign, VerticalAlign } from "@/core/constant";
import { Layout } from "../../layout";

@shortcut()
export class AutoVerticalTapeLayout extends Layout {
    static xtype = "bi.vtape_auto";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-auto-htape",
            horizontalAlign: HorizontalAlign.Stretch,
            verticalAlign: VerticalAlign.Stretch,
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            rowSize: [],
            items: [],
        });
    }

    render() {
        const o = this.options;
        
        return {
            type: "bi.vtape",
            ref: _ref => {
                this.layout = _ref;
            },
            items: o.items,
            horizontalAlign: o.horizontalAlign,
            verticalAlign: o.verticalAlign,
            rowSize: o.rowSize,
            hgap: o.hgap,
            vgap: o.vgap,
            lgap: o.lgap,
            rgap: o.rgap,
            tgap: o.tgap,
            bgap: o.bgap,
            innerHgap: o.innerHgap,
            innerVgap: o.innerVgap,
        };
    }

    _handleResize() {
        const o = this.options;
        const items = o.items;
        const top = {}, bottom = {};
        top[0] = o.innerVgap;
        bottom[items.length - 1] = o.innerVgap;

        any(items, (i, item) => {
            if (isEmptyObject(item)) {
                return true;
            }
            const w = this.layout.getWidgetByName(this._getChildName(i));
            let rowSize = o.rowSize.length > 0 ? o.rowSize[i] : item.height;
            if (o.rowSize.length > 0) {
                if (item.height >= 1 && o.rowSize[i] >= 1 && o.rowSize[i] !== item.height) {
                    rowSize = item.height;
                }
            }
            if (isNull(top[i])) {
                let preRowSize = o.rowSize.length > 0 ? o.rowSize[i - 1] : items[i - 1].height;
                if (preRowSize === "") {
                    preRowSize = this.layout.getWidgetByName(this._getChildName(i - 1)).element.height();
                }
                top[i] = top[i - 1] + preRowSize + this._optimiseItemTgap(items[i - 1]) + this._optimiseItemBgap(items[i - 1]) + 2 * this._optimiseItemVgap(items[i - 1]) + o.vgap + o.tgap + o.bgap;
            }
            w.element.css({
                top: this._optimiseGap(top[i] + this._optimiseItemTgap(item) + this._optimiseItemVgap(item) + o.vgap + o.tgap),
            });

            if (rowSize === "fill") {
                return true;
            }
        });
        backAny(items, (i, item) => {
            if (isEmptyObject(item)) {
                return true;
            }
            const w = this.layout.getWidgetByName(this._getChildName(i));
            const rowSize = o.rowSize.length > 0 ? o.rowSize[i] : item.height;
            if (isNull(bottom[i])) {
                let nextRowSize = o.rowSize.length > 0 ? o.rowSize[i + 1] : items[i + 1].height;
                if (nextRowSize === "") {
                    nextRowSize = this.layout.getWidgetByName(this._getChildName(i + 1)).element.height();
                }
                bottom[i] = bottom[i + 1] + nextRowSize + this._optimiseItemTgap(items[i + 1]) + this._optimiseItemBgap(items[i + 1]) + 2 * this._optimiseItemVgap(items[i + 1]) + o.vgap + o.tgap + o.bgap;
            }
            w.element.css({
                bottom: this._optimiseGap(bottom[i] + this._optimiseItemBgap(item) + this._optimiseItemVgap(item) + o.vgap + o.bgap),
            });

            if (rowSize === "fill") {
                return true;
            }
        });
    }

    mounted() {
        if (window.ResizeObserver) {
            this.resizeObserver = new window.ResizeObserver(this._handleResize.bind(this));
            this.resizeObserver.observe(this.element[0]);
        }
        if (window.MutationObserver) {
            this.mutationObserver = new window.MutationObserver(this._handleResize.bind(this));
            this.mutationObserver.observe(this.element[0], {
                attributes: true,
                childList: true,
                subtree: true,
            });
        }
        this._handleResize();
    }

    destroyed() {
        this.resizeObserver && this.resizeObserver.unobserve(this.element[0]);
        this.mutationObserver && this.mutationObserver.disconnect();
    }

    resize() {
        this.layout.resize();
    }

    populate(...args) {
        this.layout.populate(...args);
    }
}
