import { shortcut } from "@/core/decorator";
import { extend, any, isEmptyObject, isNull, backAny, isFunction, compact, each } from "@/core/2.base";
import { _lazyCreateWidget } from "@/core/5.inject";
import { HorizontalAlign, VerticalAlign } from "@/core/constant";
import { Layout } from "../../layout";

@shortcut()
export class FloatHorizontalFillLayout extends Layout {
    static xtype = "bi.horizontal_float_fill";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-h-float-fill clearfix",
            horizontalAlign: HorizontalAlign.Stretch,
            verticalAlign: VerticalAlign.Stretch,
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            columnSize: [],
            items: [],
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    addItem(item) {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    stroke(items) {
        const o = this.options;
        items = compact(items);
        let rank = 0;

        const createWidget = (i, item, desc) => {
            let w;
            if (o.verticalAlign !== VerticalAlign.Stretch) {
                w = _lazyCreateWidget({
                    type: "bi.vertical_adapt",
                    horizontalAlign: HorizontalAlign.Stretch,
                    verticalAlign: o.verticalAlign,
                    columnSize: ["fill"],
                    items: [item],
                });
            } else {
                w = _lazyCreateWidget(item);
            }
            if (o.vgap + o.tgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item) !== 0) {
                w.element.css({
                    "margin-top": this._optimiseGap(o.vgap + o.tgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item)),
                });
            }
            if (desc) {
                if (o.hgap + o.rgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item) !== 0) {
                    w.element.css({
                        "margin-right": this._optimiseGap((i === o.items.length - 1 ? o.hgap : 0) + o.rgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item)),
                    });
                }
                if (o.hgap + o.lgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item) !== 0) {
                    w.element.css({
                        "margin-left": this._optimiseGap(o.hgap + o.lgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item)),
                    });
                }
            } else {
                if (o.hgap + o.lgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item) !== 0) {
                    w.element.css({
                        "margin-left": this._optimiseGap((i === 0 ? o.hgap : 0) + o.lgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item)),
                    });
                }
                if (o.hgap + o.rgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item) !== 0) {
                    w.element.css({
                        "margin-right": this._optimiseGap(o.hgap + o.rgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item)),
                    });
                }
            }
            if (o.vgap + o.bgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item) !== 0) {
                w.element.css({
                    "margin-bottom": this._optimiseGap(o.vgap + o.bgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item)),
                });
            }
            const top = o.vgap + o.tgap + this._optimiseItemTgap(item) + this._optimiseItemVgap(item),
                bottom = o.vgap + o.bgap + this._optimiseItemBgap(item) + this._optimiseItemVgap(item);
            if (o.verticalAlign === VerticalAlign.Stretch && isNull(item.height)) {
                w.element.css({
                    height: `calc(100% - ${this._optimiseGap(top + bottom)})`,
                });
            }
            w.element.css({
                position: "relative",
            });
            
            return w;
        };

        any(items, (i, item) => {
            if (isEmptyObject(item)) {
                return true;
            }
            const columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
            if (columnSize === "fill") {
                return true;
            }
            const w = createWidget(i, item);
            this.addWidget(this._getChildName(rank++), w);
            w.element.css({
                "float": "left",
            });
        });
        backAny(items, (i, item) => {
            if (isEmptyObject(item)) {
                return true;
            }
            const columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
            if (columnSize === "fill") {
                return true;
            }
            const w = createWidget(i, item, true);
            this.addWidget(this._getChildName(rank++), w);
            w.element.css({
                "float": "right",
            });
        });
        each(items, (i, item) => {
            const columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
            if (columnSize === "fill") {
                const w = createWidget(i, item);
                this.addWidget(this._getChildName(rank++), w);
            }
        });
    }

    resize() {
        // console.log("填充布局不需要resize");
    }

    update(opt) {
        return this.forceUpdate(opt);
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
