import { shortcut } from "@/core/decorator";
import { extend, isFunction, some } from "@/core/2.base";
import { Widget } from "@/core/4.widget";
import { HorizontalAlign, VerticalAlign } from "@/core/constant";
import { Layout } from "@/core/wrapper/layout";

/**
 *自适应水平和垂直方向都居中容器
 * Created by GUY on 2016/12/2.
 *
 * @class FlexHorizontalLayout
 * @extends Layout
 */
@shortcut()
export class FlexWrapperHorizontalLayout extends Layout {
    static xtype = "bi.flex_scrollable_horizontal";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-f-s-h",
            verticalAlign: VerticalAlign.Top,
            horizontalAlign: HorizontalAlign.Left,
            columnSize: [],
            scrollable: null,
            scrollx: true,
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        this.element.addClass(`v-${o.verticalAlign}`).addClass(`h-${o.horizontalAlign}`);
        this.$wrapper = Widget._renderEngine.createElement("<div>").addClass(`f-s-h-w v-${o.verticalAlign}`).addClass(`h-${o.horizontalAlign}`);
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    _hasFill() {
        const o = this.options;
        if (o.columnSize.length > 0) {
            return o.columnSize.indexOf("fill") >= 0 || o.columnSize.indexOf("auto") >= 0;
        }
        
        return some(o.items, (i, item) => {
            if (item.width === "fill" || item.width === "auto") {
                return true;
            }
        });
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        let columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
        if (o.columnSize.length > 0) {
            if (item.width >= 1 && o.columnSize[i] >= 1 && o.columnSize[i] !== item.width) {
                columnSize = null;
            }
        }
        w.element.css({
            position: "relative",
        });
        if (columnSize !== "auto") {
            if (columnSize === "fill" || columnSize === "") {
                if (o.horizontalAlign !== HorizontalAlign.Stretch) {
                    if (o.scrollable === true || o.scrollx === true) {
                        w.element.addClass("f-s-n");
                    }
                }
                // 当既有动态宽度和自适应宽度的时候只压缩自适应
                if (columnSize === "" && this._hasFill()) {
                    w.element.addClass("f-s-n");
                }
            } else {
                w.element.addClass("f-s-n");
            }
        }
        if (columnSize > 0) {
            w.element.width(this._optimiseGap(columnSize));
        }
        if (columnSize === "fill") {
            w.element.addClass("f-f");
            this.element.addClass("f-f");
        }
        if (columnSize === "" || columnSize === "auto") {
            w.element.addClass("f-auto");
            this.element.addClass("f-auto");
        }
        w.element.addClass("c-e");
        if (i === 0) {
            w.element.addClass("f-c");
        }
        if (i === o.items.length - 1) {
            w.element.addClass("l-c");
        }
        this._handleGap(w, item, i);
        
        return w;
    }

    appendFragment(frag) {
        this.$wrapper.append(frag);
        this.element.append(this.$wrapper);
    }

    _getWrapper() {
        return this.$wrapper;
    }

    populate(items) {
        super.populate(...arguments);
        this._mount();
    }
}
