import { shortcut } from "@/core/decorator";
import { extend, map, isEmptyObject, stripEL, isWidget } from "@/core/2.base";
import { HorizontalAlign } from "@/core/constant";
import { Layout } from "../../layout";

/**
 * absolute实现的居中布局
 * @class FloatAbsoluteHorizontalLayout
 * @extends Layout
 */
@shortcut()
export class FloatAbsoluteHorizontalLayout extends Layout {
    static xtype = "bi.absolute_horizontal_float";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-abs-h-fl",
            horizontalAlign: HorizontalAlign.Center,
            rowSize: [],
            vgap: 0,
            tgap: 0,
            bgap: 0,
        });
    }

    render() {
        const o = this.options;
        super.render(...arguments);
        
        return {
            type: "bi.vtape",
            horizontalAlign: o.horizontalAlign,
            rowSize: o.rowSize,
            items: this._formatItems(o.items),
            scrollx: o.scrollx,
            scrolly: o.scrolly,
            scrollable: o.scrollable,
            ref: _ref => {
                this.layout = _ref;
            },
            hgap: "50%",
            vgap: o.vgap,
            tgap: o.tgap,
            bgap: o.bgap,
            // lgap和rgap不传的话内部不会设置left和right
            lgap: o.lgap,
            rgap: o.rgap,
            innerHgap: o.innerHgap,
            innerVgap: o.innerVgap,
        };
    }

    _formatItems(items) {
        const o = this.options;
        if (o.horizontalAlign === HorizontalAlign.Left) {
            return items;
        }
        const cls = o.horizontalAlign === HorizontalAlign.Right ? "bi-abs-r-x-item" : "bi-abs-c-x-item";
        
        return map(items, (i, item) => {
            if (!item || isEmptyObject(item)) {
                return item;
            }
            const el = stripEL(item);
            if (isWidget(el)) {
                el.element.addClass(cls);
            } else {
                el.cls = (el.cls || "") + cls;
            }
            
            return item;
        });
    }

    resize() {
        this.layout.stroke(this._formatItems(this.options.items));
    }

    populate(items) {
        this.layout.populate(this._formatItems(items));
    }
}
