import { shortcut } from "@/core/decorator";
import { extend, map, isEmptyObject, stripEL, isWidget, isArray, each } from "@/core/2.base";
import { VerticalAlign } from "@/core/constant";
import { Layout } from "../../layout";

@shortcut()
export class FloatAbsoluteLeftRightVerticalAdaptLayout extends Layout {
    static xtype = "bi.absolute_left_right_vertical_float";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-abs-lr-v-fl",
            verticalAlign: VerticalAlign.Middle,
            items: {},
            llgap: 0,
            lrgap: 0,
            lhgap: 0,
            ltgap: 0,
            lbgap: 0,
            lvgap: 0,
            rlgap: 0,
            rrgap: 0,
            rhgap: 0,
            rtgap: 0,
            rbgap: 0,
            rvgap: 0,
        });
    }

    render() {
        const o = this.options;
        super.render(...arguments);
        
        return {
            type: "bi.htape",
            ref: _ref => {
                this.layout = _ref;
            },
            verticalAlign: o.verticalAlign,
            items: this._formatItems(o.items),
            vgap: "50%",
            innerHgap: o.innerHgap,
            innerVgap: o.innerVgap,
            scrollx: o.scrollx,
            scrolly: o.scrolly,
            scrollable: o.scrollable,
        };
    }

    _formatItems(items) {
        const o = this.options;
        let left, right;
        if (isArray(items)) {
            each(items, (i, item) => {
                if (item.left) {
                    left = item.left;
                }
                if (item.right) {
                    right = item.right;
                }
            });
        }
        let leftItems = left || items.left || [];
        let rightItems = right || items.right || [];
        leftItems = map(leftItems, (i, item) => {
            const el = stripEL(item);
            if (o.verticalAlign === VerticalAlign.Middle) {
                if (isWidget(el)) {
                    el.element.addClass("bi-abs-c-y-item");
                } else {
                    el.cls = `${el.cls || ""}bi-abs-c-y-item`;
                }
            }
            const json = {
                el,
                width: item.width,
            };
            // if (o.lvgap + o.ltgap + (item.tgap || 0) + (item.vgap || 0) !== 0) {
            //     json.tgap = o.lvgap + o.ltgap + (item.tgap || 0) + (item.vgap || 0);
            // }
            if (o.lhgap + o.llgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item) !== 0) {
                json.lgap = (i === 0 ? o.lhgap : 0) + o.llgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item);
            }
            if (o.lhgap + o.lrgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item) !== 0) {
                json.rgap = o.lhgap + o.lrgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item);
            }
            // if (o.lvgap + o.lbgap + (item.bgap || 0) + (item.vgap || 0) !== 0) {
            //     json.bgap = o.lvgap + o.lbgap + (item.bgap || 0) + (item.vgap || 0);
            // }

            return json;
        });
        rightItems = map(rightItems, (i, item) => {
            const el = stripEL(item);
            if (o.verticalAlign === VerticalAlign.Middle) {
                if (isWidget(el)) {
                    el.element.addClass("bi-abs-c-y-item");
                } else {
                    el.cls = `${el.cls || ""}bi-abs-c-y-item`;
                }
            }
            const json = {
                el,
                width: item.width,
            };
            // if (o.rvgap + o.rtgap + (item.tgap || 0) + (item.vgap || 0) !== 0) {
            //     json.tgap = o.rvgap + o.rtgap + (item.tgap || 0) + (item.vgap || 0);
            // }
            if (o.rhgap + o.rlgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item) !== 0) {
                json.lgap = (i === 0 ? o.rhgap : 0) + o.rlgap + this._optimiseItemLgap(item) + this._optimiseItemHgap(item);
            }
            if (o.rhgap + o.rrgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item) !== 0) {
                json.rgap = o.rhgap + o.rrgap + this._optimiseItemRgap(item) + this._optimiseItemHgap(item);
            }
            // if (o.rvgap + o.rbgap + (item.bgap || 0) + (item.vgap || 0) !== 0) {
            //     json.bgap = o.rvgap + o.rbgap + (item.bgap || 0) + (item.vgap || 0);
            // }

            return json;
        });
        
        return leftItems.concat({}, rightItems);
    }

    resize() {
        this.layout.stroke(this._formatItems(this.options.items));
    }

    addItem() {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    populate(items) {
        this.layout.populate(this._formatItems(items));
    }
}

@shortcut()
export class FloatAbsoluteRightVerticalAdaptLayout extends Layout {
    static xtype = "bi.absolute_right_vertical_float";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-abs-r-v-fl",
            verticalAlign: VerticalAlign.Middle,
            items: [],
            lgap: 0,
            rgap: 0,
            hgap: 0,
        });
    }

    render() {
        const o = this.options;
        super.render(...arguments);
        
        return {
            type: "bi.htape",
            ref: _ref => {
                this.layout = _ref;
            },
            verticalAlign: o.verticalAlign,
            items: [{}].concat(this._formatItems(o.items)),
            hgap: o.hgap,
            lgap: o.lgap,
            rgap: o.rgap,
            vgap: "50%",
            scrollx: o.scrollx,
            scrolly: o.scrolly,
            scrollable: o.scrollable,
        };
    }

    _formatItems(items) {
        if (this.options.verticalAlign !== VerticalAlign.Middle) {
            return items;
        }
        
        return map(items, (i, item) => {
            if (!item || isEmptyObject(item)) {
                return item;
            }
            const el = stripEL(item);
            if (isWidget(el)) {
                el.element.addClass("bi-abs-c-y-item");
            } else {
                el.cls = `${el.cls || ""}bi-abs-c-y-item`;
            }
            
            return item;
        });
    }

    resize() {
        this.layout.stroke([{}].concat(this._formatItems(this.options.items)));
    }

    addItem() {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    populate(items) {
        this.layout.populate([{}].concat(this._formatItems(items)));
    }
}
