import { shortcut } from "@/core/decorator";
import { extend, map, isEmptyObject, stripEL, isWidget } from "@/core/2.base";
import { VerticalAlign } from "@/core/constant";
import { Layout } from "../../layout";

/**
 * absolute实现的居中布局
 * @class FloatAbsoluteVerticalLayout
 * @extends Layout
 */
@shortcut()
export class FloatAbsoluteVerticalLayout extends Layout {
    static xtype = "bi.absolute_vertical_float";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-abs-h-fl",
            verticalAlign: VerticalAlign.Middle,
            columnSize: [],
            hgap: 0,
            lgap: 0,
            rgap: 0,
        });
    }

    render() {
        const o = this.options;
        super.render(...arguments);
        
        return {
            type: "bi.htape",
            verticalAlign: o.verticalAlign,
            columnSize: o.columnSize,
            items: this._formatItems(o.items),
            scrollx: o.scrollx,
            scrolly: o.scrolly,
            scrollable: o.scrollable,
            ref: _ref => {
                this.layout = _ref;
            },
            vgap: "50%",
            hgap: o.hgap,
            lgap: o.lgap,
            rgap: o.rgap,
            // tgap和bgap不传的话内部不会设置top和bottom
            tgap: o.tgap,
            bgap: o.bgap,
            innerHgap: o.innerHgap,
            innerVgap: o.innerVgap,
        };
    }

    _formatItems(items) {
        const o = this.options;
        if (o.verticalAlign === VerticalAlign.Top) {
            return items;
        }
        const cls = o.verticalAlign === VerticalAlign.Bottom ? "bi-abs-b-y-item" : "bi-abs-c-y-item";
        
        return map(items, (i, item) => {
            if (!item || isEmptyObject(item)) {
                return item;
            }
            const el = stripEL(item);
            if (isWidget(el)) {
                el.element.addClass(cls);
            } else {
                el.cls = (el.cls || "") + cls;
            }
            
            return item;
        });
    }

    resize() {
        this.layout.stroke(this._formatItems(this.options.items));
    }

    populate(items) {
        this.layout.populate(this._formatItems(items));
    }
}
