import { shortcut } from "@/core/decorator";
import { extend, map } from "@/core/2.base";
import { VerticalAlign, HorizontalAlign } from "@/core/constant";
import { Layout } from "../../layout";

/**
 * 浮动的水平居中布局
 */
@shortcut()
export class FloatHorizontalLayout extends Layout {
    static xtype = "bi.horizontal_float";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-h-fl",
            horizontalAlign: HorizontalAlign.Center,
            verticalAlign: VerticalAlign.Top,
            rowSize: [],
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
        });
    }

    render() {
        const o = this.options;
        if (o.verticalAlign === VerticalAlign.Top) {
            return {
                type: "bi.vertical",
                ref: _ref => {
                    this.layout = _ref;
                },
                items: this._formatItems(o.items),
                vgap: o.vgap,
                tgap: o.tgap,
                bgap: o.bgap,
                scrollx: o.scrollx,
                scrolly: o.scrolly,
                scrollable: o.scrollable,
            };
        }
        
        return {
            type: "bi.inline",
            items: [{
                el: {
                    type: "bi.vertical",
                    ref: _ref => {
                        this.layout = _ref;
                    },
                    items: this._formatItems(o.items),
                    vgap: o.vgap,
                    tgap: o.tgap,
                    bgap: o.bgap,
                },
            }],
            horizontalAlign: o.horizontalAlign,
            verticalAlign: o.verticalAlign,
            innerHgap: o.innerHgap,
            innerVgap: o.innerVgap,
            scrollx: o.scrollx,
            scrolly: o.scrolly,
            scrollable: o.scrollable,
        };
    }

    _formatItems(items) {
        const o = this.options;
        
        return map(items, (i, item) => {
            return {
                el: {
                    type: "bi.inline_horizontal_adapt",
                    horizontalAlign: o.horizontalAlign,
                    items: [item],
                    hgap: o.hgap,
                    lgap: o.lgap,
                    rgap: o.rgap,
                },
            };
        });
    }

    resize() {
        this.layout.stroke(this._formatItems(this.options.items));
    }

    populate(items) {
        this.layout.populate(this._formatItems(items));
    }
}
