import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, map, parseFloat, isKey, pick, isNotNull, isNumber } from "../../2.base";

/**
 * 固定子组件上下左右的布局容器
 */
@shortcut()
export class AbsoluteLayout extends Layout {
    static xtype = "bi.absolute";
    
    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-abs",
            hgap: null,
            vgap: null,
            lgap: null,
            rgap: null,
            tgap: null,
            bgap: null,
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        let left = 0, right = 0, top = 0, bottom = 0;
        let offsets = pick(item, ["top", "right", "bottom", "left"]);

        if (isKey(item.inset)) {
            const insets = map((`${item.inset}`).split(" "), (i, str) => parseFloat(str));
            switch (insets.length) {
            case 1:
                offsets = { top: insets[0], bottom: insets[0], left: insets[0], right: insets[0] };
                break;
            case 2:
                offsets = { top: insets[0], bottom: insets[0], left: insets[1], right: insets[1] };
                break;
            case 3:
                offsets = { top: insets[0], left: insets[1], right: insets[1], bottom: insets[2] };
                break;
            case 4:
            default:
                offsets = { top: insets[0], right: insets[1], bottom: insets[2], left: insets[3] };
                break;
            }
        }
        if (isNotNull(offsets.left)) {
            w.element.css({ left: isNumber(offsets.left) ? this._optimiseGap(offsets.left) : offsets.left });
            left += offsets.left;
        }
        if (isNotNull(offsets.right)) {
            w.element.css({ right: isNumber(offsets.right) ? this._optimiseGap(offsets.right) : offsets.right });
            right += offsets.right;
        }
        if (isNotNull(offsets.top)) {
            w.element.css({ top: isNumber(offsets.top) ? this._optimiseGap(offsets.top) : offsets.top });
            top += offsets.top;
        }
        if (isNotNull(offsets.bottom)) {
            w.element.css({ bottom: isNumber(offsets.bottom) ? this._optimiseGap(offsets.bottom) : offsets.bottom });
            bottom += offsets.bottom;
        }

        if (isNotNull(o.hgap)) {
            left += o.hgap;
            w.element.css({ left: this._optimiseGap(left) });
            right += o.hgap;
            w.element.css({ right: this._optimiseGap(right) });
        }
        if (isNotNull(o.vgap)) {
            top += o.vgap;
            w.element.css({ top: this._optimiseGap(top) });
            bottom += o.vgap;
            w.element.css({ bottom: this._optimiseGap(bottom) });
        }

        if (isNotNull(o.lgap)) {
            left += o.lgap;
            w.element.css({ left: this._optimiseGap(left) });
        }
        if (isNotNull(o.rgap)) {
            right += o.rgap;
            w.element.css({ right: this._optimiseGap(right) });
        }
        if (isNotNull(o.tgap)) {
            top += o.tgap;
            w.element.css({ top: this._optimiseGap(top) });
        }
        if (isNotNull(o.bgap)) {
            bottom += o.bgap;
            w.element.css({ bottom: this._optimiseGap(bottom) });
        }

        if (isNotNull(item.width)) {
            w.element.css({ width: isNumber(item.width) ? this._optimiseGap(item.width) : item.width });
        }
        if (isNotNull(item.height)) {
            w.element.css({ height: isNumber(item.height) ? this._optimiseGap(item.height) : item.height });
        }
        w.element.css({ position: "absolute" });
        
        return w;
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
