import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isNotNull, isFunction } from "../../2.base";
import { _lazyCreateWidget } from "../../5.inject";

/**
 * 上下的高度固定/左右的宽度固定，中间的高度/宽度自适应
 *
 * @class BorderLayout
 * @extends Layout
 */
@shortcut()
export class BorderLayout extends Layout {
    static xtype = "bi.border";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-border-layout",
            items: {},
        });
    }

    render() {
        super.render(...arguments);
        const self = this, o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            self.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    addItem(item) {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    stroke(regions) {
        let item;
        let top = 0;
        let bottom = 0;
        let left = 0;
        let right = 0;
        if ("north" in regions) {
            item = regions.north;
            if (isNotNull(item)) {
                if (item.el) {
                    if (!this.hasWidget(this._getChildName("north"))) {
                        const w = _lazyCreateWidget(item);
                        this.addWidget(this._getChildName("north"), w);
                    }
                    this.getWidgetByName(this._getChildName("north")).element.height(this._optimiseGap(item.height))
                        .css({
                            position: "absolute",
                            top: this._optimiseGap(item.top || 0),
                            left: this._optimiseGap(item.left || 0),
                            right: this._optimiseGap(item.right || 0),
                            bottom: "initial",
                        });
                }
                top = (item.height || 0) + (item.top || 0) + (item.bottom || 0);
            }
        }
        if ("south" in regions) {
            item = regions.south;
            if (isNotNull(item)) {
                if (item.el) {
                    if (!this.hasWidget(this._getChildName("south"))) {
                        const w = _lazyCreateWidget(item);
                        this.addWidget(this._getChildName("south"), w);
                    }
                    this.getWidgetByName(this._getChildName("south")).element.height(this._optimiseGap(item.height))
                        .css({
                            position: "absolute",
                            bottom: this._optimiseGap(item.bottom || 0),
                            left: this._optimiseGap(item.left || 0),
                            right: this._optimiseGap(item.right || 0),
                            top: "initial",
                        });
                }
                bottom = (item.height || 0) + (item.top || 0) + (item.bottom || 0);
            }
        }
        if ("west" in regions) {
            item = regions.west;
            if (isNotNull(item)) {
                if (item.el) {
                    if (!this.hasWidget(this._getChildName("west"))) {
                        const w = _lazyCreateWidget(item);
                        this.addWidget(this._getChildName("west"), w);
                    }
                    this.getWidgetByName(this._getChildName("west")).element.width(this._optimiseGap(item.width))
                        .css({
                            position: "absolute",
                            left: this._optimiseGap(item.left || 0),
                            top: this._optimiseGap(top),
                            bottom: this._optimiseGap(bottom),
                            right: "initial",
                        });
                }
                left = (item.width || 0) + (item.left || 0) + (item.right || 0);
            }
        }
        if ("east" in regions) {
            item = regions.east;
            if (isNotNull(item)) {
                if (item.el) {
                    if (!this.hasWidget(this._getChildName("east"))) {
                        const w = _lazyCreateWidget(item);
                        this.addWidget(this._getChildName("east"), w);
                    }
                    this.getWidgetByName(this._getChildName("east")).element.width(this._optimiseGap(item.width))
                        .css({
                            position: "absolute",
                            right: this._optimiseGap(item.right || 0),
                            top: this._optimiseGap(top),
                            bottom: this._optimiseGap(bottom),
                            left: "initial",
                        });
                }
                right = (item.width || 0) + (item.left || 0) + (item.right || 0);
            }
        }
        if ("center" in regions) {
            item = regions.center;
            if (isNotNull(item)) {
                if (!this.hasWidget(this._getChildName("center"))) {
                    const w = _lazyCreateWidget(item);
                    this.addWidget(this._getChildName("center"), w);
                }
                this.getWidgetByName(this._getChildName("center")).element
                    .css({
                        position: "absolute",
                        top: this._optimiseGap(top),
                        bottom: this._optimiseGap(bottom),
                        left: this._optimiseGap(left),
                        right: this._optimiseGap(right),
                    });
            }
        }
    }

    update(opt) {
        return this.forceUpdate(opt);
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
