import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, isArray, each } from "../../2.base";
import { Widget } from "../../4.widget";
import { _lazyCreateWidget } from "../../5.inject";

@shortcut()
export class GridLayout extends Layout {
    static xtype = "bi.grid";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-grid",
            columns: null,
            rows: null,
            items: [],
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    addItem() {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    stroke(items) {
        const o = this.options;
        const rows = o.rows || o.items.length, columns = o.columns || ((o.items[0] && o.items[0].length) | 0);
        const width = 100 / columns, height = 100 / rows;
        const els = [];
        for (let i = 0; i < rows; i++) {
            els[i] = [];
        }

        function firstElement (item, cls) {
            item.addClass(cls);

            return item;
        }

        function firstObject (item, cls) {
            item.cls = (item.cls || "") + cls;

            return item;
        }

        function first (item, row, col) {
            if (item instanceof Widget) {
                firstElement(item.element, row, col);
            } else if (item.el instanceof Widget) {
                firstElement(item.el.element, row, col);
            } else if (item.el) {
                firstObject(item.el, row, col);
            } else {
                firstObject(item, row, col);
            }
        }

        each(items, (i, item) => {
            if (isArray(item)) {
                each(item, (j, el) => {
                    els[i][j] = _lazyCreateWidget(el);
                });
                
                return;
            }
            els[item.row][item.column] = _lazyCreateWidget(item);
        });
        for (let i = 0; i < rows; i++) {
            for (let j = 0; j < columns; j++) {
                if (!els[i][j]) {
                    els[i][j] = _lazyCreateWidget({
                        type: "bi.layout",
                    });
                }
                first(els[i][j], this.getRowColumnCls(i, j, rows - 1, columns - 1));
                els[i][j].element.css({
                    position: "absolute",
                    top: `${height * i}%`,
                    left: `${width * j}%`,
                    right: `${100 - (width * (j + 1))}%`,
                    bottom: `${100 - (height * (i + 1))}%`,
                });
                this.addWidget(this._getChildName(`${i}_${j}`), els[i][j]);
            }
        }
    }

    resize() {
        // console.log("grid布局不需要resize")
    }

    update(opt) {
        return this.forceUpdate(opt);
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
