import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, sum } from "../../2.base";

/**
 * 靠左对齐的自由浮动布局
 * @class LatticeLayout
 * @extends Layout
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Number} [hgap=0] 水平间隙
 * @cfg {Number} [vgap=0] 垂直间隙
 */
@shortcut()
export class LatticeLayout extends Layout {
    static xtype = "bi.lattice";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-lattice clearfix",
            // columnSize: [0.2, 0.2, 0.6],
        });
    }

    render() {
        super.render(...arguments);
        const self = this, o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            self.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        let width;
        if (o.columnSize && o.columnSize[i]) {
            width = `${o.columnSize[i] / sum(o.columnSize) * 100}%`;
        } else {
            width = `${1 / this.options.items.length * 100}%`;
        }
        w.element.css({ position: "relative", "float": "left", width });
        
        return w;
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
