import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction, range, isArray, map, reduce, isEmpty, formatEL } from "../../2.base";
import { Widget } from "../../4.widget";
import { HorizontalAlign, VerticalAlign } from "../../constant";

/**
 * 上下的高度固定/左右的宽度固定，中间的高度/宽度自适应
 *
 * @class TableLayout
 * @extends Layout
 */
@shortcut()
export class TableLayout extends Layout {
    static xtype = "bi.table";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-t",
            // scrolly: true,
            columnSize: [],
            rowSize: [],
            horizontalAlign: HorizontalAlign.Stretch,
            verticalAlign: VerticalAlign.Stretch,
            // rowSize: 30,  // or [30,30,30]
            hgap: 0,
            vgap: 0,
            items: [],
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;

        const columnSize = o.columnSize.length > 0 ? o.columnSize : range(items[0].length).fill("");

        if (columnSize.length > 0) {
            const template = [];
            for (let i = 0; i < columnSize.length; i++) {
                if (columnSize[i] === "") {
                    template.push("auto");
                } else if (columnSize[i] === "fill") {
                    template.push("1fr");
                } else {
                    template.push(this._optimiseGap(columnSize[i]));
                }
            }
            this.element.css({
                "grid-template-columns": template.join(" "),
                "grid-template-rows": isArray(o.rowSize) ? map(o.rowSize, (i, size) => this._optimiseGap(size)).join(" ") : range(o.items.length).fill(this._optimiseGap(o.rowSize)).join(" "),
                "grid-row-gap": this._optimiseGap(o.vgap),
                "grid-column-gap": this._optimiseGap(o.hgap),
            });
        }

        return {
            type: "bi.default",
            ref: (_ref) => {
                this.layout = _ref;
            },
            items: this._formatItems(items),
        };
    }

    _formatItems(items) {
        const o = this.options;

        function firstElement (item, cls) {
            item.addClass(cls);

            return item;
        }

        function firstObject (item, cls) {
            item.cls = (item.cls || "") + cls;

            return item;
        }

        function first (item, cls) {
            if (item instanceof Widget) {
                return firstElement(item.element, cls);
            } else if (item.el instanceof Widget) {
                return firstElement(item.el.element, cls);
            } else if (item.el) {
                return firstObject(item.el, cls);
            } else {
                return firstObject(item, cls);
            }
        }

        function wrapLayout (item) {
            return {
                type: "bi.horizontal_fill",
                columnSize: ["fill"],
                horizontalAlign: o.horizontalAlign,
                verticalAlign: o.verticalAlign,
                items: [formatEL(item)],
            };
        }

        return reduce(items, (rowItems, result, rowIndex) => result.concat(map(rowItems, (colIndex, item) => {
            const cls = this.getRowColumnCls(rowIndex, colIndex, items.length - 1, rowItems.length - 1);
            if (isEmpty(item)) {
                return first(wrapLayout({
                    type: "bi.layout",
                }), cls);
            }

            return first(wrapLayout(item), cls);
        })), []);
    }

    resize() {
        // console.log("table布局不需要resize");
    }

    populate(items) {
        this.layout.populate(this._formatItems(items));
    }
}
