import { shortcut } from "../../decorator";
import { Layout } from "../layout";
import { extend, isFunction } from "../../2.base";
import { HorizontalAlign } from "../../constant";

/**
 * 垂直布局
 * @class VerticalLayout
 * @extends Layout
 */
@shortcut()
export class VerticalLayout extends Layout {
    static xtype = "bi.vertical";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-v",
            horizontalAlign: HorizontalAlign.Stretch,
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            scrolly: true,
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options;
        const items = isFunction(o.items) ? this.__watch(o.items, (context, newValue) => {
            this.populate(newValue);
        }) : o.items;
        this.populate(items);
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        w.element.css({
            position: "relative",
        });
        this._handleGap(w, item, null, i);
        if (o.horizontalAlign === HorizontalAlign.Center) {
            w.element.css({
                marginLeft: "auto",
                marginRight: "auto",
            });
        } else if (o.horizontalAlign === HorizontalAlign.Right) {
            w.element.css({
                marginLeft: "auto",
            });
        }
        
        return w;
    }

    populate(...args) {
        super.populate(...args);
        this._mount();
    }
}
