import { shortcut } from "@/core/decorator";
import { extend, each } from "@/core/2.base";
import { _lazyCreateWidget } from "@/core/5.inject";
import { Layout } from "../../layout";

/**
 * 浮动布局实现的居中容器
 * @class FloatCenterLayout
 * @extends Layout
 */
@shortcut()
export class FloatCenterLayout extends Layout {
    static xtype = "bi.float_center";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-float-center",
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options, items = o.items;
        const list = [], width = 100 / items.length;
        each(items, i => {
            const widget = _lazyCreateWidget({
                type: "bi.default",
            });
            widget.element.addClass(`center-element ${i === 0 ? "first-element " : ""}${i === items.length - 1 ? "last-element" : ""}`).css({
                width: `${width}%`,
                height: "100%",
            });
            list.push({
                el: widget,
            });
        });
        each(items, (i, item) => {
            if (item) {
                const w = _lazyCreateWidget(item);
                w.element.css({
                    position: "absolute",
                    left: this._optimiseGap(o.hgap + o.lgap),
                    right: this._optimiseGap(o.hgap + o.rgap),
                    top: this._optimiseGap(o.vgap + o.tgap),
                    bottom: this._optimiseGap(o.vgap + o.bgap),
                    width: "auto",
                    height: "auto",
                });
                list[i].el.addItem(w);
            }
        });
        
        return {
            type: "bi.left",
            ref: _ref => {
                this.layout = _ref;
            },
            items: list,
        };
    }

    resize() {
        // console.log("floatcenter布局不需要resize");
    }

    addItem(item) {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    populate(items) {
        this.layout.populate(...arguments);
    }
}
