import { shortcut } from "@/core/decorator";
import { extend, each } from "@/core/2.base";
import { _lazyCreateWidget } from "@/core/5.inject";
import { Layout } from "../../layout";

/**
 * 水平和垂直方向都居中容器, 非自适应，用于宽度高度固定的面板
 * @class HorizontalCenterLayout
 * @extends Layout
 */
@shortcut()
export class HorizontalCenterLayout extends Layout {
    static xtype = "bi.horizontal_center";

    props() {
        return extend(super.props(...arguments), {
            baseCls: "bi-h-center",
            hgap: 0,
            vgap: 0,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
        });
    }

    render() {
        super.render(...arguments);
        const o = this.options, items = o.items;
        const list = [];
        each(items, i => {
            list.push({
                column: i,
                row: 0,
                el: _lazyCreateWidget({
                    type: "bi.default",
                    cls: `center-element ${i === 0 ? "first-element " : ""}${i === items.length - 1 ? "last-element" : ""}`,
                }),
            });
        });
        each(items, (i, item) => {
            if (item) {
                const w = _lazyCreateWidget(item);
                w.element.css({
                    position: "absolute",
                    left: this._optimiseGap(o.hgap + o.lgap),
                    right: this._optimiseGap(o.hgap + o.rgap),
                    top: this._optimiseGap(o.vgap + o.tgap),
                    bottom: this._optimiseGap(o.vgap + o.bgap),
                    width: "auto",
                });
                list[i].el.addItem(w);
            }
        });
        
        return {
            type: "bi.grid",
            ref: _ref => {
                this.layout = _ref;
            },
            columns: list.length,
            rows: 1,
            items: list,
        };
    }

    resize() {
        // console.log("horizontal_center布局不需要resize");
    }

    addItem(item) {
        // do nothing
        throw new Error("Cannot add subwidget");
    }

    populate(items) {
        this.layout.populate(...arguments);
    }
}
