import { shortcut } from "@/core/decorator";
import { extend, count, isNotNull } from "@/core/2.base";
import { VerticalAlign } from "@/core/constant";
import { FlexHorizontalLayout } from "../flex";

/**
 * 横向黏性布局
 */
@shortcut()
export class HorizontalStickyLayout extends FlexHorizontalLayout {
    static xtype = "bi.horizontal_sticky";

    props() {
        return extend(super.props(...arguments), {
            extraCls: "bi-h-sticky",
            scrollx: true,
            // horizontalAlign: HorizontalAlign.Stretch,
            verticalAlign: VerticalAlign.Stretch,
        });
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        let columnSize = o.columnSize.length > 0 ? o.columnSize[i] : item.width;
        if (o.columnSize.length > 0) {
            if (item.width >= 1 && o.columnSize[i] >= 1 && o.columnSize[i] !== item.width) {
                columnSize = null;
            }
        }
        if (columnSize !== "fill") {
            let fillIndex;
            count(0, o.items.length, index => {
                if (isNotNull(fillIndex)) {
                    return;
                }
                if ((o.columnSize[index] === "fill" || o.items[index].width === "fill")) {
                    fillIndex = index;
                }
            });

            if (fillIndex > i) {
                w.element.css({
                    position: "sticky",
                    zIndex: 1,
                    left: 0,
                });
            } else {
                w.element.css({
                    position: "sticky",
                    zIndex: 1,
                    right: 0,
                });
            }
        } else {
            w.element.css({
                overflow: "",
            });
        }
        
        return w;
    }
}
