import { shortcut } from "@/core/decorator";
import { extend, count, isNotNull } from "@/core/2.base";
import { HorizontalAlign } from "@/core/constant";
import { FlexVerticalLayout } from "../flex";

/**
 * 纵向黏性布局
 */
@shortcut()
export class VerticalStickyLayout extends FlexVerticalLayout {
    static xtype = "bi.vertical_sticky";

    props() {
        return extend(super.props(...arguments), {
            extraCls: "bi-v-sticky",
            scrolly: true,
            horizontalAlign: HorizontalAlign.Stretch,
            // verticalAlign: VerticalAlign.Stretch
        });
    }

    _addElement(i, item) {
        const o = this.options;
        const w = super._addElement(...arguments);
        let rowSize = o.rowSize.length > 0 ? o.rowSize[i] : item.height;
        if (o.rowSize.length > 0) {
            if (item.height >= 1 && o.rowSize[i] >= 1 && o.rowSize[i] !== item.height) {
                rowSize = null;
            }
        }
        if (rowSize !== "fill") {
            let fillIndex;
            count(0, o.items.length, index => {
                if (isNotNull(fillIndex)) {
                    return;
                }
                if ((o.rowSize[index] === "fill" || o.items[index].height === "fill")) {
                    fillIndex = index;
                }
            });

            if (fillIndex > i) {
                w.element.css({
                    position: "sticky",
                    zIndex: 1,
                    top: 0,
                });
            } else {
                w.element.css({
                    position: "sticky",
                    zIndex: 1,
                    bottom: 0,
                });
            }
        } else {
            w.element.css({
                overflow: "",
            });
        }
        
        return w;
    }
}
