import {
    VerticalLayout,
    shortcut,
    Widget,
    map,
    extend,
    isFunction,
    contains,
    each,
    isKey,
    isNotEmptyArray
} from "@/core";
import { ArrowNode } from "@/case";
import { Expander } from "@/base";

@shortcut()
export class Collapse extends Widget {
    static xtype = "bi.collapse";

    static EVENT_EXPAND = "EVENT_EXPAND";

    props = {
        baseCls: "bi-collapse",
        items: [],
        value: [],
        trigger: "click",
        accordion: false,
        bordered: true,
        ghost: false,
        isDefaultInit: false,
        openMotion: {
            animation: "bi-slide-up",
            animationDuring: 200,
        },
    };

    render() {
        const o = this.options;

        const collapseCls = o.ghost ? "" : `bi-background ${o.bordered ? "bi-border bi-border-radius" : ""}`;

        this.expanders = {};

        return {
            type: VerticalLayout.xtype,
            cls: collapseCls,
            items: this._getItems(this.options.items),
        };
    }

    _getItems(items) {
        const o = this.options;

        return map(items, (index, item) => {
            const isActive = contains(this._getCurrentValue(o.value), item.value);
            const cls = o.ghost || index === 0 ? "" : "bi-border-top";

            const el = extend(
                {
                    type: ArrowNode.xtype,
                    height: 30,
                    text: item.text,
                    value: item.value,
                    open: isActive,
                },
                item.el
            );

            const popup = extend(
                {
                    animation: o.openMotion.animation,
                    animationDuring: o.openMotion.animationDuring,
                },
                item.popup
            );

            return extend(
                {
                    type: Expander.xtype,
                    cls,
                    isDefaultInit: o.isDefaultInit,
                    trigger: o.trigger,
                    listeners: [
                        {
                            eventName: Expander.EVENT_EXPAND,
                            action: () => {
                                this._hideOtherExpander(item.value);
                                this.fireEvent(Collapse.EVENT_EXPAND, item.value);
                            },
                        }
                    ],
                },
                item,
                {
                    el,
                    popup,
                    ref: _ref => {
                        isFunction(item.ref) && item.ref(_ref);
                        this.expanders[item.value] = _ref;
                    },
                }
            );
        });
    }

    _hideOtherExpander(expandKey) {
        if (this.options.accordion) {
            each(this.expanders, (key, expander) => {
                key !== `${expandKey}` && expander.hideView();
            });
        }
    }

    _getCurrentValue(v) {
        const values = isNotEmptyArray(v) ? v : isKey(v) ? [v] : [];

        return this.options.accordion ? values.slice(0, 1) : values;
    }

    getValue() {
        const value = [];
        each(this.expanders, (key, expander) => {
            expander.isExpanded() && value.push(key);
        });

        return value;
    }

    setValue(v) {
        const values = map(this._getCurrentValue(v), (idx, value) => `${value}`);
        each(this.expanders, (key, expander) => {
            contains(values, key) ? expander.showView() : expander.hideView();
        });
    }
}
