import { DateTriangleTrigger } from "./trigger.triangle.date";
import { MonthPopup } from "./popup.month";
import { Combo, Trigger } from "@/base";
import { VerticalLayout, shortcut, extend, createWidget } from "@/core";

/**
 * 日期控件中的月份下拉框
 *
 * Created by GUY on 2015/9/7.
 * @class MonthDateCombo
 * @extends Trigger
 */
@shortcut()
export class MonthDateCombo extends Trigger {
    static xtype = "bi.month_date_combo";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-month-combo",
            height: 24,
            container: null,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;

        this.trigger = createWidget({
            type: DateTriangleTrigger.xtype,
        });

        this.popup = createWidget({
            type: MonthPopup.xtype,
            allowMonths: o.allowMonths,
            behaviors: o.behaviors,
        });

        this.popup.on(MonthPopup.EVENT_CHANGE, () => {
            this.setValue(this.popup.getValue());
            this.combo.hideView();
            this.fireEvent(MonthDateCombo.EVENT_CHANGE);
        });

        this.combo = createWidget({
            type: Combo.xtype,
            offsetStyle: "center",
            container: o.container,
            element: this,
            isNeedAdjustHeight: false,
            isNeedAdjustWidth: false,
            el: this.trigger,
            popup: {
                minWidth: 100,
                stopPropagation: false,
                el: {
                    type: VerticalLayout.xtype,
                    hgap: 6,
                    vgap: 5,
                    items: [this.popup],
                },
            },
        });

        this.combo.on(Combo.EVENT_BEFORE_POPUPVIEW, () => {
            this.doBehavior();
        });
    }

    populate() {
        this.popup.populate(...arguments);
    }

    setValue(v) {
        this.trigger.setValue(v);
        this.popup.setValue(v);
    }

    getValue() {
        return this.popup.getValue();
    }
}
