import { DateTriangleTrigger } from "./trigger.triangle.date";
import { YearPopup } from "./popup.year";
import { Combo, Trigger } from "@/base";
import { shortcut, extend, createWidget } from "@/core";

/**
 * 年份下拉框
 *
 * Created by GUY on 2015/9/7.
 * @class YearDateCombo
 * @extends Trigger
 */
@shortcut()
export class YearDateCombo extends Trigger {
    static xtype = "bi.year_date_combo";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-year-combo",
            min: "1900-01-01", // 最小日期
            max: "2099-12-31", // 最大日期
            behaviors: {},
            height: 24,
            container: null,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;

        this.trigger = createWidget({
            type: DateTriangleTrigger.xtype,
        });

        this.popup = createWidget({
            type: YearPopup.xtype,
            behaviors: o.behaviors,
            min: o.min,
            max: o.max,
            width: 122,
        });

        this.popup.on(YearPopup.EVENT_CHANGE, () => {
            this.setValue(this.popup.getValue());
            this.combo.hideView();
            this.fireEvent(YearDateCombo.EVENT_CHANGE);
        });

        this.combo = createWidget({
            type: Combo.xtype,
            offsetStyle: "center",
            element: this,
            container: o.container,
            isNeedAdjustHeight: false,
            isNeedAdjustWidth: false,
            el: this.trigger,
            popup: {
                minWidth: 100,
                stopPropagation: false,
                el: this.popup,
            },
        });
        this.combo.on(Combo.EVENT_CHANGE, () => {
            this.fireEvent(YearDateCombo.EVENT_CHANGE);
        });
        // BI-22551 popup未初始化传入的behavior无效
        this.combo.on(Combo.EVENT_BEFORE_POPUPVIEW, () => {
            this.doBehavior();
            this.fireEvent(YearDateCombo.EVENT_BEFORE_POPUPVIEW);
        });
    }

    setMinDate(minDate) {
        this.popup.setMinDate(minDate);
    }

    setMaxDate(maxDate) {
        this.popup.setMaxDate(maxDate);
    }

    setValue(v) {
        this.trigger.setValue(v);
        this.popup.setValue(v);
    }

    getValue() {
        return this.popup.getValue();
    }
}
