import { IconButton } from "@/base";
import { YearDateCombo } from "./combo.year.date";
import { MonthDateCombo } from "./combo.month.date";
import {
    HTapeLayout,
    CenterAdaptLayout,
    shortcut,
    Widget,
    extend,
    createWidget,
    contains,
    isNull,
    getDate,
    filter,
    range,
    checkDateVoid,
    parseDateTime,
    parseInt,
    print
} from "@/core";

/**
 * Created by GUY on 2015/9/7.
 * @class DatePicker
 * @extends Widget
 */
@shortcut()
export class DatePicker extends Widget {
    static xtype = "bi.date_picker";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_BEFORE_YEAR_MONTH_POPUPVIEW = "EVENT_BEFORE_YEAR_MONTH_POPUPVIEW";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: "bi-date-picker",
            height: 40,
            min: "1900-01-01", // 最小日期
            max: "2099-12-31", // 最大日期
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this._year = getDate().getFullYear();
        this._month = getDate().getMonth() + 1;
        this.left = createWidget({
            type: IconButton.xtype,
            cls: "pre-page-h-font",
            width: 24,
            height: 24,
        });
        this.left.on(IconButton.EVENT_CHANGE, () => {
            if (this._month === 1) {
                this.setValue({
                    year: this.year.getValue() - 1 || getDate().getFullYear() - 1,
                    month: 12,
                });
            } else {
                this.setValue({
                    year: this.year.getValue() || getDate().getFullYear(),
                    month: this.month.getValue() - 1 || getDate().getMonth(),
                });
            }
            this.fireEvent(DatePicker.EVENT_CHANGE);
            // this._checkLeftValid();
            // this._checkRightValid();
        });

        this.right = createWidget({
            type: IconButton.xtype,
            cls: "next-page-h-font",
            width: 24,
            height: 24,
        });

        this.right.on(IconButton.EVENT_CHANGE, () => {
            if (this._month === 12) {
                this.setValue({
                    year: this.year.getValue() + 1 || getDate().getFullYear() + 1,
                    month: 1,
                });
            } else {
                this.setValue({
                    year: this.year.getValue() || getDate().getFullYear(),
                    month: this.month.getValue() + 1 || getDate().getMonth() + 2,
                });
            }
            this.fireEvent(DatePicker.EVENT_CHANGE);
            // this._checkLeftValid();
            // this._checkRightValid();
        });

        this.year = createWidget({
            type: YearDateCombo.xtype,
            behaviors: o.behaviors,
            min: o.min,
            max: o.max,
        });
        this.year.on(YearDateCombo.EVENT_CHANGE, () => {
            this.setValue({
                year: this.year.getValue(),
                month: this._refreshMonth(),
            });
            this.fireEvent(DatePicker.EVENT_CHANGE);
        });
        this.year.on(YearDateCombo.EVENT_BEFORE_POPUPVIEW, () => {
            this.fireEvent(DatePicker.EVENT_BEFORE_YEAR_MONTH_POPUPVIEW);
        });
        this.month = createWidget({
            type: MonthDateCombo.xtype,
            behaviors: o.behaviors,
            allowMonths: this._getAllowMonths(),
        });
        this.month.on(MonthDateCombo.EVENT_CHANGE, () => {
            this.setValue({
                year: this.year.getValue() || this._year,
                month: this.month.getValue(),
            });
            this.fireEvent(DatePicker.EVENT_CHANGE);
        });
        this.month.on(YearDateCombo.EVENT_BEFORE_POPUPVIEW, () => {
            this.fireEvent(DatePicker.EVENT_BEFORE_YEAR_MONTH_POPUPVIEW);
        });

        createWidget({
            type: HTapeLayout.xtype,
            element: this,
            items: [
                {
                    el: {
                        type: CenterAdaptLayout.xtype,
                        items: [this.left],
                    },
                    width: 24,
                },
                {
                    el: {
                        type: CenterAdaptLayout.xtype,
                        hgap: 10,
                        items: [this.year, this.month],
                    },
                },
                {
                    el: {
                        type: CenterAdaptLayout.xtype,
                        items: [this.right],
                    },
                    width: 24,
                }
            ],
        });
        this.setValue({
            year: this._year,
            month: this._month,
        });
    }

    _refreshMonth(defaultMonth) {
        let month = this.month.getValue();
        this.month.populate(this._getAllowMonths());
        const allowMonth = this._getAllowMonths();
        if (!contains(allowMonth, month)) {
            month = defaultMonth || allowMonth[0];
        }
        this.month.setValue(month);

        return month;
    }

    _getAllowMonths() {
        const obj = this._getCheckMinMaxDate();
        const year = this.year.getValue() || this._year;

        return filter(range(1, 13), (idx, v) => !checkDateVoid(year, v, 1, obj.min, obj.max)[0]);
    }

    _checkLeftValid() {
        const obj = this._getCheckMinMaxDate();
        const year = this._month === 1 ? this._year - 1 : this._year;
        const month = this._month === 1 ? 12 : this._month - 1;
        const valid = isNull(checkDateVoid(year, month, 1, obj.min, obj.max)[0]);
        this.left.setEnable(valid);

        return valid;
    }

    _checkRightValid() {
        const obj = this._getCheckMinMaxDate();
        const year = this._month === 12 ? this._year + 1 : this._year;
        const month = this._month === 12 ? 1 : this._month + 1;
        const valid = isNull(checkDateVoid(year, month, 1, obj.min, obj.max)[0]);
        this.right.setEnable(valid);

        return valid;
    }

    _getCheckMinMaxDate() {
        const o = this.options;
        const minDate = parseDateTime(o.min, "%Y-%X-%d");
        const maxDate = parseDateTime(o.max, "%Y-%X-%d");
        minDate.setDate(1);
        maxDate.setDate(1);

        return {
            min: print(minDate, "%Y-%X-%d"),
            max: print(maxDate, "%Y-%X-%d"),
        };
    }

    setMinDate(minDate) {
        this.options.min = minDate;
        this.year.setMinDate(minDate);
        this._refreshMonth(this._month);
        // this._checkLeftValid();
        // this._checkRightValid();
    }

    setMaxDate(maxDate) {
        this.options.max = maxDate;
        this.year.setMaxDate(maxDate);
        this._refreshMonth(this._month);
        // this._checkLeftValid();
        // this._checkRightValid();
    }

    setValue(ob) {
        this._year = parseInt(ob.year);
        this._month = parseInt(ob.month);
        this.year.setValue(ob.year);
        this._refreshMonth(this._month);
        this.month.setValue(ob.month);
        // this._checkLeftValid();
        // this._checkRightValid();
    }

    getValue() {
        return {
            year: this.year.getValue(),
            month: this.month.getValue(),
        };
    }
}
