import { YearDateCombo } from "./combo.year.date";
import { HTapeLayout, CenterAdaptLayout, shortcut, Widget, extend, createWidget, getDate, parseDateTime } from "@/core";
import { IconButton } from "@/base";

@shortcut()
export class YearPicker extends Widget {
    static xtype = "bi.year_picker";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: "bi-year-picker",
            behaviors: {},
            height: 40,
            min: "1900-01-01", // 最小日期
            max: "2099-12-31", // 最大日期
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this._year = getDate().getFullYear();
        this.left = createWidget({
            type: IconButton.xtype,
            cls: "pre-page-h-font",
            width: 25,
            height: 25,
        });
        this.left.on(IconButton.EVENT_CHANGE, () => {
            this.setValue(this.year.getValue() - 1);
            this.fireEvent(YearPicker.EVENT_CHANGE);
            // this._checkLeftValid();
            // this._checkRightValid();
        });

        this.right = createWidget({
            type: IconButton.xtype,
            cls: "next-page-h-font",
            width: 25,
            height: 25,
        });

        this.right.on(IconButton.EVENT_CHANGE, () => {
            this.setValue(this.year.getValue() + 1);
            this.fireEvent(YearPicker.EVENT_CHANGE);
            // this._checkLeftValid();
            // this._checkRightValid();
        });

        this.year = createWidget({
            type: YearDateCombo.xtype,
            min: o.min,
            behaviors: o.behaviors,
            max: o.max,
            width: 50,
        });
        this.year.on(YearDateCombo.EVENT_CHANGE, () => {
            this.setValue(this.year.getValue());
            this.fireEvent(YearPicker.EVENT_CHANGE);
        });
        this.year.on(YearDateCombo.EVENT_BEFORE_POPUPVIEW, () => {
            this.fireEvent(YearPicker.EVENT_BEFORE_POPUPVIEW);
        });

        createWidget({
            type: HTapeLayout.xtype,
            element: this,
            items: [
                {
                    el: {
                        type: CenterAdaptLayout.xtype,
                        items: [this.left],
                    },
                    width: 25,
                },
                {
                    type: CenterAdaptLayout.xtype,
                    items: [
                        {
                            el: this.year,
                        }
                    ],
                },
                {
                    el: {
                        type: CenterAdaptLayout.xtype,
                        items: [this.right],
                    },
                    width: 25,
                }
            ],
        });
        this.setValue(this._year);
    }

    _checkLeftValid() {
        const o = this.options;
        const valid = this._year > parseDateTime(o.min, "%Y-%X-%d").getFullYear();
        this.left.setEnable(valid);

        return valid;
    }

    _checkRightValid() {
        const o = this.options;
        const valid = this._year < parseDateTime(o.max, "%Y-%X-%d").getFullYear();
        this.right.setEnable(valid);

        return valid;
    }

    setMinDate(minDate) {
        this.options.min = minDate;
        this.year.setMinDate(minDate);
        // this._checkLeftValid();
        // this._checkRightValid();
    }

    setMaxDate(maxDate) {
        this.options.max = maxDate;
        this.year.setMaxDate(maxDate);
        // this._checkLeftValid();
        // this._checkRightValid();
    }

    setValue(v) {
        this._year = v;
        this.year.setValue(v);
        // this._checkLeftValid();
        // this._checkRightValid();
    }

    getValue() {
        return this.year.getValue();
    }
}
