import { Calendar } from "@/case";
import { DatePicker } from "./picker.date";
import { Navigation } from "@/base";
import {
    VerticalLayout,
    AbsoluteLayout,
    Layout,
    shortcut,
    Widget,
    createWidget,
    bind,
    getDate,
    each,
    isNotEmptyString
} from "@/core";

/**
 * Created by GUY on 2015/9/7.
 * @class DateCalendarPopup
 * @extends Widget
 */
@shortcut()
export class DateCalendarPopup extends Widget {
    static xtype = "bi.date_calendar_popup";

    props = {
        baseCls: "bi-date-calendar-popup",
        min: "1900-01-01",
        max: "2099-12-31",
        selectedTime: null,
    };

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_BEFORE_YEAR_MONTH_POPUPVIEW = "EVENT_BEFORE_YEAR_MONTH_POPUPVIEW";

    _createNav(v) {
        const date = Calendar.getDateJSONByPage(v);
        const calendar = createWidget({
            type: Calendar.xtype,
            logic: {
                dynamic: true,
            },
            min: this.options.min,
            max: this.options.max,
            year: date.year,
            month: date.month,
            // BI-45616 此处为确定当前应该展示哪个年月对应的Calendar, day不是关键数据, 给1号就可
            day: 1,
        });

        return calendar;
    }

    render() {
        const o = this.options;
        this.today = getDate();
        this._year = this.today.getFullYear();
        this._month = this.today.getMonth() + 1;
        this._day = this.today.getDate();

        this.selectedTime = o.selectedTime || {
            year: this._year,
            month: this._month,
            day: this._day,
        };
        this.datePicker = createWidget({
            type: DatePicker.xtype,
            behaviors: o.behaviors,
            min: o.min,
            max: o.max,
        });

        this.calendar = createWidget({
            direction: "top",
            logic: {
                dynamic: true,
            },
            type: Navigation.xtype,
            tab: this.datePicker,
            cardCreator: bind(this._createNav, this),

            afterCardCreated() {},

            afterCardShow: () => {
                this.calendar.setValue(this.selectedTime);
            },
        });

        this.datePicker.on(DatePicker.EVENT_CHANGE, () => {
            this.selectedTime = this.datePicker.getValue();
            this.selectedTime.day = 1;
            this.calendar.setSelect(Calendar.getPageByDateJSON(this.selectedTime));
        });

        this.datePicker.on(DatePicker.EVENT_BEFORE_YEAR_MONTH_POPUPVIEW, () => {
            this.fireEvent(DateCalendarPopup.EVENT_BEFORE_YEAR_MONTH_POPUPVIEW);
        });

        this.calendar.on(Navigation.EVENT_CHANGE, () => {
            this.selectedTime = this.calendar.getValue();
            this.setValue(this.selectedTime);
            this.fireEvent(DateCalendarPopup.EVENT_CHANGE);
        });

        return [
            {
                type: VerticalLayout.xtype,
                items: [
                    {
                        el: this.calendar,
                        hgap: 12,
                        bgap: 7,
                    }
                ],
            },
            {
                type: AbsoluteLayout.xtype,
                items: [
                    {
                        el: {
                            type: Layout.xtype,
                            cls: "bi-split-top",
                        },
                        height: 1,
                        top: 40,
                        left: 0,
                        right: 0,
                    }
                ],
            }
        ];
    }

    _checkMin() {
        const o = this.options;
        each(this.calendar.getAllCard(), (idx, calendar) => {
            calendar.setMinDate(o.min);
        });
    }

    _checkMax() {
        const o = this.options;
        each(this.calendar.getAllCard(), (idx, calendar) => {
            calendar.setMaxDate(o.max);
        });
    }

    setMinDate(minDate) {
        if (isNotEmptyString(this.options.min)) {
            this.options.min = minDate;
            this.datePicker.setMinDate(minDate);
            this._checkMin();
        }
    }

    setMaxDate(maxDate) {
        if (isNotEmptyString(this.options.max)) {
            this.options.max = maxDate;
            this.datePicker.setMaxDate(maxDate);
            this._checkMax();
        }
    }

    setValue(timeOb) {
        this.datePicker.setValue(timeOb);
        this.calendar.setSelect(Calendar.getPageByDateJSON(timeOb));
        this.calendar.setValue(timeOb);
        this.selectedTime = timeOb;
    }

    getValue() {
        return this.selectedTime;
    }
}
