import { ButtonGroup, TextItem } from "@/base";
import {
    CenterAdaptLayout,
    shortcut,
    Widget,
    extend,
    Controller,
    createWidget,
    Events,
    createItems,
    toPix,
    map,
    contains,
    getDate,
    LogicFactory,
    parseInt,
    SIZE_CONSANTS
} from "@/core";

/**
 * 月份展示面板
 *
 * Created by GUY on 2015/9/2.
 * @class MonthPopup
 * @extends Widget
 */
@shortcut()
export class MonthPopup extends Widget {
    static xtype = "bi.month_popup";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-month-popup",
            behaviors: {},
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;

        this.selectedMonth = getDate().getMonth() + 1;

        this.month = createWidget({
            type: ButtonGroup.xtype,
            element: this,
            behaviors: o.behaviors,
            items: createItems(this._getItems(o.allowMonths), {}),
            layouts: [
                LogicFactory.createLogic(
                    "table",
                    extend(
                        {
                            dynamic: true,
                        },
                        {
                            columns: 2,
                            rows: 6,
                            columnSize: [1 / 2, 1 / 2],
                            rowSize: SIZE_CONSANTS.LIST_ITEM_HEIGHT + 1,
                        }
                    )
                ),
                {
                    type: CenterAdaptLayout.xtype,
                    vgap: 2,
                }
            ],
            value: o.value,
        });

        this.month.on(Controller.EVENT_CHANGE, (...args) => {
            const [type, value] = args;
            this.selectedMonth = value;
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
            if (type === Events.CLICK) {
                this.fireEvent(MonthPopup.EVENT_CHANGE);
            }
        });
    }

    _getItems(m) {
        Widget.pushContext(this);
        // 纵向排列月
        const month = [1, 7, 2, 8, 3, 9, 4, 10, 5, 11, 6, 12];
        let items = [];
        items.push(month.slice(0, 2));
        items.push(month.slice(2, 4));
        items.push(month.slice(4, 6));
        items.push(month.slice(6, 8));
        items.push(month.slice(8, 10));
        items.push(month.slice(10, 12));
        items = map(items, (i, item) =>
            map(item, (j, td) => {
                return {
                    type: TextItem.xtype,
                    cls: "bi-border-radius bi-list-item-select",
                    textAlign: "center",
                    whiteSpace: "nowrap",
                    once: false,
                    forceSelected: true,
                    height: toPix(SIZE_CONSANTS.LIST_ITEM_HEIGHT, 1),
                    width: 30,
                    value: td,
                    text: td,
                    disabled: !contains(m, td),
                };
            })
        );
        Widget.popContext();

        return items;
    }

    populate(months) {
        this.month.populate(this._getItems(months));
    }

    getValue() {
        return this.selectedMonth;
    }

    setValue(v) {
        v = parseInt(v);
        this.selectedMonth = v;
        this.month.setValue([v]);
    }
}
