import { DatePicker, DateCalendarPopup } from "../date/calendar";
import { Navigation } from "@/base";
import {
    VTapeLayout,
    AbsoluteLayout,
    Layout,
    shortcut,
    Widget,
    extend,
    createWidget,
    bind,
    isNull,
    each,
    isNotEmptyString,
    getDate,
    getMonthDays
} from "@/core";
import { Calendar } from "@/case";

@shortcut()
export class StaticDatePaneCard extends Widget {
    static xtype = "bi.static_date_pane_card";

    static EVENT_BEFORE_YEAR_MONTH_POPUPVIEW = "EVENT_BEFORE_YEAR_MONTH_POPUPVIEW";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: "bi-date-pane",
            min: "1900-01-01", // 最小日期
            max: "2099-12-31", // 最大日期
            selectedTime: null,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;

        this.today = getDate();
        this._year = this.today.getFullYear();
        this._month = this.today.getMonth() + 1;

        this.selectedTime = o.selectedTime || {
            year: this._year,
            month: this._month,
        };

        this.datePicker = createWidget({
            type: DatePicker.xtype,
            behaviors: o.behaviors,
            min: o.min,
            max: o.max,
        });
        this.datePicker.on(DatePicker.EVENT_CHANGE, () => {
            const value = this.datePicker.getValue();
            const monthDay = getMonthDays(getDate(value.year, value.month - 1, 1));
            let day = this.selectedTime.day || 0;
            if (day > monthDay) {
                day = monthDay;
            }
            this.selectedTime = {
                year: value.year,
                month: value.month,
            };
            day !== 0 && (this.selectedTime.day = day);
            this.calendar.setSelect(Calendar.getPageByDateJSON(this.selectedTime));
            this.calendar.setValue(this.selectedTime);
            day !== 0 && this.fireEvent(DateCalendarPopup.EVENT_CHANGE);
        });
        this.datePicker.on(DatePicker.EVENT_BEFORE_YEAR_MONTH_POPUPVIEW, () => {
            this.fireEvent(StaticDatePaneCard.EVENT_BEFORE_YEAR_MONTH_POPUPVIEW);
        });

        this.calendar = createWidget({
            direction: "custom",
            // logic: {
            //     dynamic: false
            // },
            type: Navigation.xtype,
            tab: this.datePicker,
            cardCreator: bind(this._createNav, this),
        });
        this.calendar.on(Navigation.EVENT_CHANGE, () => {
            this.selectedTime = this.calendar.getValue();
            this.calendar.empty();
            this.setValue(this.selectedTime);
            this.fireEvent(DateCalendarPopup.EVENT_CHANGE);
        });
        this.setValue(o.selectedTime);

        createWidget({
            type: VTapeLayout.xtype,
            element: this,
            items: [
                {
                    el: this.datePicker,
                    height: 40,
                },
                this.calendar
            ],
            hgap: 10,
        });

        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: {
                        type: Layout.xtype,
                        cls: "bi-split-top",
                    },
                    height: 1,
                    top: 40,
                    left: 0,
                    right: 0,
                }
            ],
        });
    }

    _createNav(v) {
        const date = Calendar.getDateJSONByPage(v);
        const calendar = createWidget({
            type: Calendar.xtype,
            logic: {
                dynamic: false,
            },
            min: this.options.min,
            max: this.options.max,
            year: date.year,
            month: date.month,
            day: this.selectedTime.day,
        });

        return calendar;
    }

    _getNewCurrentDate() {
        const today = getDate();

        return {
            year: today.getFullYear(),
            month: today.getMonth() + 1,
        };
    }

    _setCalenderValue(date) {
        this.calendar.setSelect(Calendar.getPageByDateJSON(date));
        this.calendar.setValue(date);
        this.selectedTime = date;
    }

    _setDatePicker(timeOb) {
        if (isNull(timeOb) || isNull(timeOb.year) || isNull(timeOb.month)) {
            this.datePicker.setValue(this._getNewCurrentDate());
        } else {
            this.datePicker.setValue(timeOb);
        }
    }

    _setCalendar(timeOb) {
        if (isNull(timeOb) || isNull(timeOb.day)) {
            this.calendar.empty();
            this._setCalenderValue(this._getNewCurrentDate());
        } else {
            this._setCalenderValue(timeOb);
        }
    }

    _checkMin() {
        const o = this.options;
        each(this.calendar.getAllCard(), (idx, calendar) => {
            calendar.setMinDate(o.min);
        });
    }

    _checkMax() {
        const o = this.options;
        each(this.calendar.getAllCard(), (idx, calendar) => {
            calendar.setMaxDate(o.max);
        });
    }

    setMinDate(minDate) {
        if (isNotEmptyString(this.options.min)) {
            this.options.min = minDate;
            this.datePicker.setMinDate(minDate);
            this._checkMin();
        }
    }

    setMaxDate(maxDate) {
        if (isNotEmptyString(this.options.max)) {
            this.options.max = maxDate;
            this.datePicker.setMaxDate(maxDate);
            this._checkMax();
        }
    }

    setValue(timeOb) {
        this._setDatePicker(timeOb);
        this._setCalendar(timeOb);
    }

    getValue() {
        return this.selectedTime;
    }
}
