import {
    VTapeLayout,
    CenterLayout,
    shortcut,
    Widget,
    createItems,
    i18nText,
    isEmptyObject,
    isNull,
    isEmptyString,
    isNotEmptyObject,
    getDate
} from "@/core";
import { LinearSegment } from "@/case";
import { Tab, TextButton, ButtonGroup } from "@/base";
import { StaticDatePaneCard } from "./card.static.datepane";
import { DynamicDateCard, DynamicDateHelper, DynamicDateCombo, DynamicDatePopup } from "../dynamicdate";

@shortcut()
export class DynamicDatePane extends Widget {
    static xtype = "bi.dynamic_date_pane";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_BEFORE_YEAR_MONTH_POPUPVIEW = "EVENT_BEFORE_YEAR_MONTH_POPUPVIEW";
    static Static = 1;
    static Dynamic = 2;

    props = {
        baseCls: "bi-dynamic-date-pane",
        minDate: "1900-01-01",
        maxDate: "2099-12-31",
        supportDynamic: true,
        isPreview: false, // 是否是预览状态
    };

    render() {
        const o = this.options;

        return {
            type: VTapeLayout.xtype,
            items: [
                {
                    el: {
                        type: LinearSegment.xtype,
                        invisible: !o.supportDynamic,
                        cls: "bi-split-bottom",
                        height: 30,
                        items: createItems(
                            [
                                {
                                    text: i18nText("BI-Multi_Date_YMD"),
                                    value: DynamicDatePane.Static,
                                },
                                {
                                    text: i18nText("BI-Basic_Dynamic_Title"),
                                    value: DynamicDatePane.Dynamic,
                                }
                            ],
                            {
                                textAlign: "center",
                            }
                        ),
                        listeners: [
                            {
                                eventName: ButtonGroup.EVENT_CHANGE,
                                action: () => {
                                    const value = this.switcher.getValue()[0];
                                    this.dateTab.setSelect(value);
                                    switch (value) {
                                        case DynamicDatePane.Static:
                                            this._setStaticValueByDynamicValue(this.dynamicPane.getValue());
                                            break;
                                        case DynamicDatePane.Dynamic:
                                            this.dynamicPane.setValue({
                                                year: 0,
                                            });
                                            break;
                                        default:
                                            break;
                                    }
                                    this.fireEvent(DynamicDatePane.EVENT_CHANGE);
                                },
                            }
                        ],
                        ref: _ref => {
                            this.switcher = _ref;
                        },
                    },
                    height: o.supportDynamic ? 30 : 0,
                },
                {
                    type: Tab.xtype,
                    ref: _ref => {
                        this.dateTab = _ref;
                    },
                    showIndex: DynamicDatePane.Static,
                    cardCreator: v => {
                        switch (v) {
                            case DynamicDatePane.Static:
                                return {
                                    type: StaticDatePaneCard.xtype,
                                    min: o.minDate,
                                    max: o.maxDate,
                                    behaviors: o.behaviors,
                                    listeners: [
                                        {
                                            eventName: "EVENT_CHANGE",
                                            action: () => {
                                                this.fireEvent(DynamicDatePane.EVENT_CHANGE);
                                            },
                                        },
                                        {
                                            eventName: "EVENT_BEFORE_YEAR_MONTH_POPUPVIEW",
                                            action: () => {
                                                this.fireEvent(DynamicDatePane.EVENT_BEFORE_YEAR_MONTH_POPUPVIEW);
                                            },
                                        }
                                    ],
                                    ref: _ref => {
                                        this.ymd = _ref;
                                    },
                                };
                            case DynamicDatePane.Dynamic:
                            default:
                                return {
                                    type: VTapeLayout.xtype,
                                    items: [
                                        {
                                            type: DynamicDateCard.xtype,
                                            min: o.minDate,
                                            max: o.maxDate,
                                            ref: _ref => {
                                                this.dynamicPane = _ref;
                                            },
                                        },
                                        {
                                            el: {
                                                type: CenterLayout.xtype,
                                                items: [
                                                    {
                                                        type: TextButton.xtype,
                                                        cls: "bi-high-light bi-border-top",
                                                        shadow: true,
                                                        text: i18nText("BI-Basic_Clear"),
                                                        textHeight: 23,
                                                        listeners: [
                                                            {
                                                                eventName: TextButton.EVENT_CHANGE,
                                                                action: () => {
                                                                    this.setValue();
                                                                    this.fireEvent(DynamicDatePane.EVENT_CHANGE);
                                                                },
                                                            }
                                                        ],
                                                    },
                                                    {
                                                        type: TextButton.xtype,
                                                        cls: "bi-border-left bi-high-light bi-border-top",
                                                        textHeight: 23,
                                                        shadow: true,
                                                        text: i18nText("BI-Basic_OK"),
                                                        listeners: [
                                                            {
                                                                eventName: TextButton.EVENT_CHANGE,
                                                                action: () => {
                                                                    const type = this.dateTab.getSelect();
                                                                    if (type === DynamicDateCombo.Dynamic) {
                                                                        this.dynamicPane.checkValidation(true) &&
                                                                            this.fireEvent(
                                                                                DynamicDatePopup.EVENT_CHANGE
                                                                            );
                                                                    } else {
                                                                        this.fireEvent(DynamicDatePane.EVENT_CHANGE);
                                                                    }
                                                                },
                                                            }
                                                        ],
                                                    }
                                                ],
                                            },
                                            height: 24,
                                        }
                                    ],
                                };
                        }
                    },
                }
            ],
        };
    }

    created() {
        this._initValue(this.options.value);
    }

    _checkValueValid(value) {
        return isNull(value) || isEmptyObject(value) || isEmptyString(value);
    }

    _checkValue(v) {
        switch (v.type) {
            case DynamicDateCombo.Dynamic:
                return isNotEmptyObject(v.value);
            case DynamicDateCombo.Static:
            default:
                return true;
        }
    }

    _initValue(v) {
        v = v || {};
        const type = v.type || DynamicDateCombo.Static;
        const value = v.value || v;

        const { isPreview } = this.options;

        // 如果是初始化且是预览状态 直接打开静态页面
        this.switcher.setValue(isPreview ? DynamicDateCombo.Static : type);
        this.dateTab.setSelect(isPreview ? DynamicDateCombo.Static : type);
        switch (type) {
            case DynamicDateCombo.Dynamic:
                if (isPreview) {
                    this._setStaticValueByDynamicValue(value);
                } else {
                    this.dynamicPane.setValue(value);
                }
                break;
            case DynamicDateCombo.Static:
            default:
                this._setStaticValue(value);
                break;
        }
    }

    setMinDate(minDate) {
        if (this.options.minDate !== minDate) {
            this.options.minDate = minDate;
            this.ymd.setMinDate(minDate);
        }
    }

    setMaxDate(maxDate) {
        if (this.options.maxDate !== maxDate) {
            this.options.maxDate = maxDate;
            this.ymd.setMaxDate(maxDate);
        }
    }

    setValue(v) {
        v = v || {};
        const type = v.type || DynamicDateCombo.Static;
        const value = v.value || v;
        this.switcher.setValue(type);
        this.dateTab.setSelect(type);
        switch (type) {
            case DynamicDateCombo.Dynamic:
                this.dynamicPane.setValue(value);
                break;
            case DynamicDateCombo.Static:
            default:
                this._setStaticValue(value);
                break;
        }
    }

    _setStaticValue(value) {
        if (this._checkValueValid(value)) {
            const date = getDate();
            this.ymd.setValue({
                year: date.getFullYear(),
                month: date.getMonth() + 1,
            });
        } else {
            this.ymd.setValue(value);
        }
    }

    _setStaticValueByDynamicValue(value) {
        const date = DynamicDateHelper.getCalculation(value);
        this.ymd.setValue({
            year: date.getFullYear(),
            month: date.getMonth() + 1,
            day: date.getDate(),
        });
    }

    getValue() {
        const type = this.dateTab.getSelect();

        return {
            type,
            value: type === DynamicDatePane.Static ? this.dateTab.getValue() : this.dynamicPane.getValue(),
        };
    }
}
