import { DateTimeTrigger } from "./datetime.trigger";
import { DateTimePopup } from "./datetime.popup";
import { Combo, IconButton, Single } from "@/base";
import { HTapeLayout, shortcut, extend, getDate, isNotNull, createWidget } from "@/core";

@shortcut()
export class DateTimeCombo extends Single {
    static xtype = "bi.date_time_combo";

    constants = {
        popupHeight: 290,
        popupWidth: 270,
        comboAdjustHeight: 1,
        border: 1,
        iconWidth: 24,
    };

    static EVENT_CANCEL = "EVENT_CANCEL";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";

    _defaultConfig(conf) {
        return extend(super._defaultConfig(...arguments), {
            baseCls: `bi-date-time-combo bi-focus-shadow ${
                conf.simple ? "bi-border-bottom" : "bi-border bi-border-radius"
            }`,
            width: 200,
            height: 24,
            minDate: "1900-01-01",
            maxDate: "2099-12-31",
        });
    }

    _init() {
        super._init(...arguments);
        const opts = this.options;
        const date = getDate();
        this.storeValue = isNotNull(opts.value)
            ? opts.value
            : {
                year: date.getFullYear(),
                month: date.getMonth() + 1,
                day: date.getDate(),
                hour: date.getHours(),
                minute: date.getMinutes(),
                second: date.getSeconds(),
            };
        this.trigger = createWidget({
            type: DateTimeTrigger.xtype,
            height: opts.height,
            min: opts.minDate,
            max: opts.maxDate,
            value: opts.value,
        });

        this.popup = createWidget({
            type: DateTimePopup.xtype,
            behaviors: opts.behaviors,
            min: opts.minDate,
            max: opts.maxDate,
            value: opts.value,
        });
        this.setValue(this.storeValue);

        this.popup.on(DateTimePopup.BUTTON_CANCEL_EVENT_CHANGE, () => {
            this.setValue(this.storeValue);
            this.hidePopupView();
            this.fireEvent(DateTimeCombo.EVENT_CANCEL);
        });
        this.popup.on(DateTimePopup.BUTTON_OK_EVENT_CHANGE, () => {
            this.storeValue = this.popup.getValue();
            this.setValue(this.storeValue);
            this.hidePopupView();
            this.fireEvent(DateTimeCombo.EVENT_CONFIRM);
        });
        this.combo = createWidget({
            type: Combo.xtype,
            container: opts.container,
            toggle: false,
            isNeedAdjustHeight: false,
            isNeedAdjustWidth: false,
            el: this.trigger,
            adjustLength: this.constants.comboAdjustHeight,
            popup: {
                el: this.popup,
                width: this.constants.popupWidth,
                stopPropagation: false,
            },
            // DEC-4250 和复选下拉一样，点击不收起
            hideChecker(e) {
                return triggerBtn.element.find(e.target).length === 0;
            },
        });
        this.combo.on(Combo.EVENT_BEFORE_POPUPVIEW, () => {
            this.popup.setValue(this.storeValue);
            this.fireEvent(DateTimeCombo.EVENT_BEFORE_POPUPVIEW);
        });

        const triggerBtn = createWidget({
            type: IconButton.xtype,
            cls: "bi-trigger-icon-button date-font",
            width: this.constants.iconWidth,
            height: opts.height,
        });
        triggerBtn.on(IconButton.EVENT_CHANGE, () => {
            if (this.combo.isViewVisible()) {
                // this.combo.hideView();
            } else {
                this.combo.showView();
            }
        });

        createWidget({
            type: HTapeLayout.xtype,
            columnSize: ["", this.constants.iconWidth],
            element: this,
            items: [this.combo, triggerBtn],
        });
    }

    setValue(v) {
        this.storeValue = v;
        this.popup.setValue(v);
        this.trigger.setValue(v);
    }

    getValue() {
        return this.storeValue;
    }

    hidePopupView() {
        this.combo.hideView();
    }
}
