import { TextButton } from "@/base";
import { DateCalendarPopup } from "../date/calendar";
import {
    GridLayout,
    VTapeLayout,
    CenterAdaptLayout,
    shortcut,
    Widget,
    extend,
    createWidget,
    i18nText,
    isNull,
    getDate
} from "@/core";
import { DynamicDateTimeSelect } from "../dynamicdatetime";

@shortcut()
export class DateTimePopup extends Widget {
    static xtype = "bi.date_time_popup";

    static BUTTON_OK_EVENT_CHANGE = "BUTTON_OK_EVENT_CHANGE";
    static BUTTON_CANCEL_EVENT_CHANGE = "BUTTON_CANCEL_EVENT_CHANGE";
    static CALENDAR_EVENT_CHANGE = "CALENDAR_EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-date-time-popup",
            width: 268,
            height: 374,
        });
    }

    _init() {
        super._init(...arguments);
        const opts = this.options;
        this.cancelButton = createWidget({
            type: TextButton.xtype,
            cls: "multidate-popup-button bi-border-top bi-border-right",
            shadow: true,
            text: i18nText("BI-Basic_Cancel"),
        });
        this.cancelButton.on(TextButton.EVENT_CHANGE, () => {
            this.fireEvent(DateTimePopup.BUTTON_CANCEL_EVENT_CHANGE);
        });

        this.okButton = createWidget({
            type: TextButton.xtype,
            cls: "multidate-popup-button bi-border-top",
            shadow: true,
            text: i18nText("BI-Basic_OK"),
        });
        this.okButton.on(TextButton.EVENT_CHANGE, () => {
            this.fireEvent(DateTimePopup.BUTTON_OK_EVENT_CHANGE);
        });

        this.dateCombo = createWidget({
            type: DateCalendarPopup.xtype,
            behaviors: opts.behaviors,
            min: this.options.min,
            max: this.options.max,
        });
        this.dateCombo.on(DateCalendarPopup.EVENT_CHANGE, () => {
            this.fireEvent(DateTimePopup.CALENDAR_EVENT_CHANGE);
        });

        this.dateButton = createWidget({
            type: GridLayout.xtype,
            items: [[this.cancelButton, this.okButton]],
        });
        createWidget({
            element: this,
            type: VTapeLayout.xtype,
            items: [
                {
                    el: this.dateCombo,
                },
                {
                    el: {
                        type: CenterAdaptLayout.xtype,
                        cls: "bi-split-top",
                        items: [
                            {
                                type: DynamicDateTimeSelect.xtype,
                                ref: _ref => {
                                    this.timeSelect = _ref;
                                },
                            }
                        ],
                    },
                    height: 50,
                },
                {
                    el: this.dateButton,
                    height: 30,
                }
            ],
        });
        this.setValue(opts.value);
    }

    setValue(v) {
        const value = v;
        let date;
        if (isNull(value)) {
            date = getDate();
            this.dateCombo.setValue({
                year: date.getFullYear(),
                month: date.getMonth() + 1,
                day: date.getDate(),
            });
            this.timeSelect.setValue({
                hour: date.getHours(),
                minute: date.getMinutes(),
                second: date.getSeconds(),
            });
        } else {
            this.dateCombo.setValue({
                year: value.year,
                month: value.month,
                day: value.day,
            });
            this.timeSelect.setValue({
                hour: value.hour,
                minute: value.minute,
                second: value.second,
            });
        }
    }

    getValue() {
        return extend(
            {
                year: this.dateCombo.getValue().year,
                month: this.dateCombo.getValue().month,
                day: this.dateCombo.getValue().day,
            },
            this.timeSelect.getValue()
        );
    }
}
