import { cloneDeep, createWidget, each, extend, isArray, isNull, shortcut, some, Widget } from "@/core";
import { DownListPopup } from "./popup.downlist";
import { Combo } from "@/base";
import { IconTrigger } from "@/case";

function transformItems(items) {
    if (!items) return items;
    let result = cloneDeep(items);
    const isComplexItmes = some(items, (_, item) => isArray(item));
    // 传一维数组，帮转二维
    if (!isComplexItmes) {
        result = [result];
    }
    // 帮转 el
    each(result, (_, arr) => {
        each(arr, (_, item) => {
            if (item.children && !item.el) {
                item.el = {
                    text: item.text,
                    icon: item.icon,
                    cls: item.cls,
                    iconCls1: item.iconCls1,
                    value: item.value,
                };
            }
        });
    });

    return result;
}

@shortcut()
export class DownListCombo extends Widget {
    static xtype = "bi.down_list_combo";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_SON_VALUE_CHANGE = "EVENT_SON_VALUE_CHANGE";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-down-list-combo",
            height: 24,
            items: [],
            adjustLength: 0,
            direction: "bottom",
            trigger: "click",
            container: null,
            stopPropagation: false,
            el: {},
            popup: {},
            minWidth: 140,
            maxHeight: 1000,
            destroyWhenHide: false,
            isDefaultInit: true,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;

        this.downlistcombo = createWidget({
            element: this,
            type: Combo.xtype,
            trigger: o.trigger,
            isNeedAdjustWidth: false,
            isDefaultInit: o.isDefaultInit,
            container: o.container,
            adjustLength: o.adjustLength,
            direction: o.direction,
            belowMouse: o.belowMouse,
            stopPropagation: o.stopPropagation,
            destroyWhenHide: o.destroyWhenHide,
            el: {
                type: IconTrigger.xtype,
                extraCls: o.iconCls,
                width: o.width,
                height: o.height,
                ...o.el,
            },
            popup: {
                el: {
                    type: DownListPopup.xtype,
                    ref: _ref => {
                        this.popupView = _ref;
                    },
                    items: transformItems(o.items),
                    chooseType: o.chooseType,
                    value: o.value,
                    listeners: [
                        {
                            eventName: DownListPopup.EVENT_CHANGE,
                            action: value => {
                                this.fireEvent(DownListCombo.EVENT_CHANGE, value);
                                this.downlistcombo.hideView();
                            },
                        },
                        {
                            eventName: DownListPopup.EVENT_SON_VALUE_CHANGE,
                            action: (value, fatherValue) => {
                                this.fireEvent(DownListCombo.EVENT_SON_VALUE_CHANGE, value, fatherValue);
                                this.downlistcombo.hideView();
                            },
                        },
                    ],
                },
                stopPropagation: o.stopPropagation,
                maxHeight: o.maxHeight,
                minWidth: o.minWidth,
                ...o.popup,
            },
        });

        this.downlistcombo.on(Combo.EVENT_BEFORE_POPUPVIEW, () => {
            this.fireEvent(DownListCombo.EVENT_BEFORE_POPUPVIEW);
        });
    }

    hideView() {
        this.downlistcombo.hideView();
    }

    showView(e) {
        this.downlistcombo.showView(e);
    }

    populate(items) {
        this.popupView.populate(items);
    }

    setValue(v) {
        this.popupView.setValue(v);
    }

    getValue() {
        if (isNull(this.popupView)) {
            return this.options.value;
        } else {
            return this.popupView.getValue();
        }
    }

    adjustWidth() {
        this.downlistcombo.adjustWidth();
    }

    adjustHeight() {
        this.downlistcombo.adjustHeight();
    }
}
