import { Label, Icon, BasicButton } from "@/base";
import {
    CenterAdaptLayout,
    shortcut,
    extend,
    createWidget,
    isPlainObject,
    LogicFactory,
    Direction,
    SIZE_CONSANTS
} from "@/core";

@shortcut()
export class DownListItem extends BasicButton {
    static xtype = "bi.down_list_item";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: "bi-down-list-item bi-list-item-active",
            cls: "",
            height: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
            logic: {
                dynamic: true,
            },
            selected: false,
            iconHeight: null,
            iconWidth: null,
            textHgap: 0,
            textVgap: 0,
            textLgap: 0,
            textRgap: 0,
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.text = createWidget({
            type: Label.xtype,
            cls: "list-item-text",
            textAlign: "left",
            hgap: o.textHgap,
            vgap: o.textVgap,
            lgap: o.textLgap,
            rgap: o.textRgap,
            text: o.text,
            value: o.value,
            keyword: o.keyword,
            height: o.height,
        });

        const icon = isPlainObject(o.icon)
            ? o.icon
            : {
                type: Icon.xtype,
                width: o.iconWidth,
                height: o.iconHeight,
            };

        this.icon = createWidget({
            type: CenterAdaptLayout.xtype,
            width: 36,
            height: o.height,
            items: [
                {
                    el: icon,
                }
            ],
        });

        createWidget(
            extend(
                {
                    element: this,
                },
                LogicFactory.createLogic(
                    LogicFactory.createLogicTypeByDirection(Direction.Left),
                    extend(o.logic, {
                        items: LogicFactory.createLogicItemsByDirection(Direction.Left, this.icon, this.text),
                    })
                )
            )
        );
    }

    setValue() {
        if (!this.isReadOnly()) {
            this.text.setValue(...arguments);
        }
    }

    getValue() {
        return this.text.getValue();
    }

    setText() {
        this.text.setText(...arguments);
    }

    getText() {
        return this.text.getText();
    }

    doClick() {
        super.doClick(...arguments);
        if (this.isValid()) {
            this.fireEvent(DownListItem.EVENT_CHANGE, this.getValue(), this);
        }
    }

    doRedMark() {
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    doHighLight() {
        this.text.doHighLight(...arguments);
    }

    unHighLight() {
        this.text.unHighLight(...arguments);
    }
}
