import { Label, IconButton, BasicButton } from "@/base";
import {
    CenterAdaptLayout,
    HorizontalFillLayout,
    shortcut,
    extend,
    createWidget,
    isNotNull,
    isArray,
    any,
    contains,
    isPlainObject,
    first,
    BlankSplitChar
} from "@/core";

@shortcut()
export class DownListGroupItem extends BasicButton {
    static xtype = "bi.down_list_group_item";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: `${conf.baseCls || ""} bi-down-list-group-item`,
            logic: {
                dynamic: false,
            },
            // invalid: true,
            iconCls1: "dot-e-font",
            icon: "",
            iconCls2: "pull-right-e-font",
        });
    }

    render() {
        const o = this.options;
        this.text = createWidget({
            type: Label.xtype,
            cls: "list-group-item-text",
            textAlign: "left",
            text: o.text,
            title: o.title,
            value: o.value,
            height: o.height,
        });

        if (isPlainObject(o.icon)) {
            this.icon1 = createWidget({
                width: 36,
                height: o.height,
                type: CenterAdaptLayout.xtype,
                items: [o.icon],
            });
        } else {
            this.icon1 = createWidget({
                type: IconButton.xtype,
                cls: o.iconCls1,
                width: 36,
                height: o.height,
                iconHeight: o.iconHeight,
                iconWidth: 36,
                disableSelected: true,
                selected: this._digest(o.value),
            });
        }

        this.icon2 = createWidget({
            type: IconButton.xtype,
            cls: o.iconCls2,
            width: 24,
            forceNotSelected: true,
        });

        this.element.hover(
            () => {
                if (this.isEnabled()) {
                    this.hover();
                }
            },
            () => {
                if (this.isEnabled()) {
                    this.dishover();
                }
            }
        );

        return {
            type: HorizontalFillLayout.xtype,
            columnSize: [36, "fill", 24],
            items: [this.icon1, this.text, this.icon2],
        };
    }

    _getLevel() {
        const child = first(this.options.childValues);

        return isNotNull(child) ? `${child}`.split(BlankSplitChar).length : 0;
    }

    _digest(v) {
        const o = this.options;
        v = isArray(v) ? v : [v];
        const level = this._getLevel();

        return any(v, (idx, value) =>
            contains(o.childValues, `${value}`.split(BlankSplitChar).slice(0, level).join(BlankSplitChar))
        );
    }

    hover() {
        super.hover(...arguments);
        this.icon1.element.addClass("hover");
        this.icon2.element.addClass("hover");
    }

    dishover() {
        super.dishover(...arguments);
        this.icon1.element.removeClass("hover");
        this.icon2.element.removeClass("hover");
    }

    doClick() {
        super.doClick(...arguments);
        if (this.isValid()) {
            this.fireEvent(DownListGroupItem.EVENT_CHANGE, this.getValue());
        }
    }

    doRedMark() {
        this.text.doRedMark(...arguments);
    }

    unRedMark() {
        this.text.unRedMark(...arguments);
    }

    setValue(v) {
        this.icon1.setSelected && this.icon1.setSelected(this._digest(v));
    }
}
