import { ButtonTree, Pane } from "@/base";
import {
    BlankSplitChar,
    contains,
    createItems,
    createWidget,
    deepClone,
    each,
    extend,
    isEmpty,
    isNotEmptyArray,
    isNotEmptyString,
    isNotNull,
    Layout,
    map,
    Selection,
    shortcut,
    SIZE_CONSANTS,
    some,
    VerticalLayout,
} from "@/core";
import { DownListGroup } from "./group.downlist";

@shortcut()
export class DownListPopup extends Pane {
    static xtype = "bi.down_list_popup";

    constants = {
        nextIcon: "pull-right-e-font",
        height: 24,
        iconHeight: 12,
        iconWidth: 12,
        hgap: 0,
        vgap: 0,
        border: 1,
    };

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_SON_VALUE_CHANGE = "EVENT_SON_VALUE_CHANGE";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: "bi-down-list-popup",
            items: [],
            chooseType: Selection.Multi,
        });
    }

    _init() {
        super._init(...arguments);
        this.singleValues = [];
        this.childValueMap = {};
        this.fatherValueMap = {};
        this.items = [];
        const o = this.options,
            children = this._createPopupItems(o.items);
        this.popup = createWidget({
            type: ButtonTree.xtype,
            items: createItems(
                children,
                {},
                {
                    adjustLength: -2,
                },
            ),
            layouts: [
                {
                    type: VerticalLayout.xtype,
                    hgap: this.constants.hgap,
                    vgap: this.constants.vgap,
                },
            ],
            value: this._digest(o.value),
            chooseType: o.chooseType,
        });

        this.popup.on(ButtonTree.EVENT_CHANGE, (value, object) => {
            let changedValue = value;
            if (isNotNull(this.childValueMap[value])) {
                changedValue = this.childValueMap[value];
                this.fireEvent(DownListPopup.EVENT_SON_VALUE_CHANGE, changedValue, this.fatherValueMap[value]);
            } else {
                this.fireEvent(DownListPopup.EVENT_CHANGE, changedValue, object);
            }

            if (!contains(this.singleValues, changedValue)) {
                const item = this.getValue();
                const result = [];
                each(item, (i, valueObject) => {
                    if (valueObject.value !== changedValue) {
                        result.push(valueObject);
                    }
                });
                this.setValue(result);
            }
        });

        createWidget({
            type: VerticalLayout.xtype,
            element: this,
            items: [this.popup],
            vgap: 5,
        });
    }

    _createPopupItems(items) {
        const result = [];
        // 不能修改populate进来的item的引用
        each(items, (i, it) => {
            const item_done = {
                type: DownListGroup.xtype,
                items: [],
            };

            const storeItem = [];

            each(it, (i, sourceItem) => {
                const item = extend({}, sourceItem);
                if (isNotEmptyArray(sourceItem.children) && !isEmpty(sourceItem.el)) {
                    item.type = "bi.combo_group";
                    // popup未初始化返回的是options中的value, 在经过buttontree的getValue concat之后，无法区分值来自options
                    // 还是item自身, 这边控制defaultInit为true来避免这个问题
                    item.isDefaultInit = true;
                    item.cls = "down-list-group";
                    item.trigger = "hover";
                    item.isNeedAdjustWidth = false;
                    item.el = sourceItem.el;
                    item.el.title = sourceItem.el.title;
                    item.el.type = "bi.down_list_group_item";
                    item.el.logic = {
                        dynamic: true,
                    };
                    item.el.height = sourceItem.el.height || SIZE_CONSANTS.LIST_ITEM_HEIGHT;
                    item.el.iconCls2 = this.constants.nextIcon;
                    item.popup = {
                        lgap: 1,
                        el: {
                            type: ButtonTree.xtype,
                            chooseType: 0,
                            layouts: [
                                {
                                    type: VerticalLayout.xtype,
                                },
                            ],
                        },
                        innerVgap: 5,
                        maxHeight: 378,
                    };
                    this._createChildren(item, sourceItem);
                } else {
                    item.type = sourceItem.type || "bi.down_list_item";
                    item.title = sourceItem.title;
                    item.textRgap = 10;
                    item.isNeedAdjustWidth = false;
                    item.logic = {
                        dynamic: true,
                    };
                }
                const el_done = {};
                el_done.el = item;
                item_done.items.push(el_done);
                storeItem.push(item);
            });
            each(item_done.items, (i, item) => {
                this.singleValues.push(item.el.value);
            });

            result.push(item_done);
            this.items.push(storeItem);
            if (this._needSpliter(i, items.length)) {
                const spliter_container = createWidget({
                    type: VerticalLayout.xtype,
                    items: [
                        {
                            el: {
                                type: Layout.xtype,
                                cls: "bi-down-list-spliter bi-split-top cursor-pointer",
                                height: 0,
                            },
                        },
                    ],
                    cls: "bi-down-list-spliter-container cursor-pointer",
                    vgap: 5,
                    hgap: 12,
                });
                result.push(spliter_container);
            }
        });

        return result;
    }

    _createChildren(targetItem, sourceItem) {
        targetItem.el.childValues = [];
        targetItem.items = targetItem.children = [];
        each(sourceItem.children, (i, child) => {
            const item = extend({}, child);
            const fatherValue = deepClone(targetItem.el.value);
            const childValue = deepClone(item.value);
            this.singleValues.push(item.value);
            item.type = item.type || "bi.down_list_item";
            item.extraCls = " child-down-list-item";
            item.title = item.title;
            item.textRgap = 10;
            item.isNeedAdjustWidth = false;
            item.logic = {
                dynamic: true,
            };
            item.father = fatherValue;
            item.childValue = item.value;
            this.fatherValueMap[this._createChildValue(fatherValue, childValue)] = fatherValue;
            this.childValueMap[this._createChildValue(fatherValue, childValue)] = childValue;
            item.value = this._createChildValue(fatherValue, childValue);
            targetItem.el.childValues.push(item.value);
            targetItem.items.push(item);
        });
    }

    _needSpliter(i, itemLength) {
        return i < itemLength - 1;
    }

    _createChildValue(fatherValue, childValue) {
        return fatherValue + BlankSplitChar + childValue;
    }

    _digest(valueItem) {
        const valueArray = [];
        each(valueItem, (i, item) => {
            let value;
            if (isNotNull(item.childValue)) {
                value = this._createChildValue(item.value, item.childValue);
            } else {
                value = item.value;
            }
            valueArray.push(value);
        });

        return valueArray;
    }

    _checkValues(values) {
        const value = [];
        each(this.items, (idx, itemGroup) => {
            each(itemGroup, (id, item) => {
                if (isNotNull(item.children)) {
                    const childValues = map(item.children, "value");
                    const v = joinValue(childValues, values[idx]);
                    if (isNotEmptyString(v)) {
                        value.push(v);
                    }
                } else {
                    if (item.value === values[idx][0]) {
                        value.push(values[idx][0]);
                    }
                }
            });
        });

        return value;

        function joinValue(sources, targets) {
            let value = "";
            some(sources, (idx, s) =>
                some(targets, (id, t) => {
                    if (s === t) {
                        value = s;

                        return true;
                    }
                }),
            );

            return value;
        }
    }

    populate(items) {
        super.populate(...arguments);
        this.items = [];
        this.childValueMap = {};
        this.fatherValueMap = {};
        this.singleValues = [];
        const children = this._createPopupItems(items);
        const popupItem = createItems(
            children,
            {},
            {
                adjustLength: -2,
            },
        );
        this.popup.populate(popupItem);
    }

    setValue(valueItem) {
        this.popup.setValue(this._digest(valueItem));
    }

    _getValue() {
        const v = [];
        each(this.popup.getAllButtons(), (i, item) => {
            i % 2 === 0 && v.push(item.getValue());
        });

        return v;
    }

    getValue() {
        const result = [];
        const values = this._checkValues(this._getValue());
        each(values, (i, value) => {
            const valueItem = {};
            if (isNotNull(this.childValueMap[value])) {
                const fartherValue = this.fatherValueMap[value];
                valueItem.childValue = this.childValueMap[value];
                valueItem.value = fartherValue;
            } else {
                valueItem.value = value;
            }
            result.push(valueItem);
        });

        return result;
    }
}
