import {
    isNotNull,
    parseInt,
    getDate,
    i18nText,
    size,
    getOffsetQuarter,
    getOffsetMonth,
    getOffsetDate,
    getLastDateOfMonth,
    getWeekStartDate,
    getWeekEndDate,
    getQuarterStartDate,
    getQuarterEndDate,
    print
} from "@/core";
import { DynamicDateCard } from "./dynamicdate.card";

export const DynamicDateHelper = {
    getCalculation(obj) {
        const date = getDate();

        return this.getCalculationByDate(date, obj);
    },

    getDescription(obj) {
        let value = "";
        let endText = "";
        if (isNotNull(obj.year)) {
            if (parseInt(obj.year) !== 0) {
                value +=
                    Math.abs(obj.year) +
                    i18nText("BI-Basic_Year") +
                    (obj.year < 0 ? i18nText("BI-Basic_Front") : i18nText("BI-Basic_Behind"));
            }
            endText = getPositionText(i18nText("BI-Basic_Year"), obj.position);
        }
        if (isNotNull(obj.quarter)) {
            if (parseInt(obj.quarter) !== 0) {
                value +=
                    Math.abs(obj.quarter) +
                    i18nText("BI-Basic_Single_Quarter") +
                    (obj.quarter < 0 ? i18nText("BI-Basic_Front") : i18nText("BI-Basic_Behind"));
            }
            endText = getPositionText(i18nText("BI-Basic_Single_Quarter"), obj.position);
        }
        if (isNotNull(obj.month)) {
            if (parseInt(obj.month) !== 0) {
                value +=
                    Math.abs(obj.month) +
                    i18nText("BI-Basic_Month") +
                    (obj.month < 0 ? i18nText("BI-Basic_Front") : i18nText("BI-Basic_Behind"));
            }
            endText = getPositionText(i18nText("BI-Basic_Month"), obj.position);
        }
        if (isNotNull(obj.week)) {
            if (parseInt(obj.week) !== 0) {
                value +=
                    Math.abs(obj.week) +
                    i18nText("BI-Basic_Week") +
                    (obj.week < 0 ? i18nText("BI-Basic_Front") : i18nText("BI-Basic_Behind"));
            }
            endText = getPositionText(i18nText("BI-Basic_Week"), obj.position);
        }
        if (isNotNull(obj.day)) {
            if (parseInt(obj.day) !== 0) {
                value +=
                    Math.abs(obj.day) +
                    i18nText("BI-Basic_Day") +
                    (obj.day < 0 ? i18nText("BI-Basic_Front") : i18nText("BI-Basic_Behind"));
            }
            endText = size(obj) === 1 ? getPositionText(i18nText("BI-Basic_Month"), obj.position) : "";
        }
        if (isNotNull(obj.workDay) && parseInt(obj.workDay) !== 0) {
            value +=
                Math.abs(obj.workDay) +
                i18nText("BI-Basic_Work_Day") +
                (obj.workDay < 0 ? i18nText("BI-Basic_Front") : i18nText("BI-Basic_Behind"));
        }

        return value + endText;

        function getPositionText(baseText, position) {
            switch (position) {
            case DynamicDateCard.OFFSET.BEGIN:
                return baseText + i18nText("BI-Basic_Begin_Start");
            case DynamicDateCard.OFFSET.END:
                return baseText + i18nText("BI-Basic_End_Stop");
            case DynamicDateCard.OFFSET.CURRENT:
            default:
                return i18nText("BI-Basic_Current_Day");
            }
        }
    },

    getCalculationByDate(date, obj) {
        if (isNotNull(obj.year)) {
            date = getDate(date.getFullYear() + parseInt(obj.year), date.getMonth(), date.getDate());
        }
        if (isNotNull(obj.quarter)) {
            date = getOffsetQuarter(date, parseInt(obj.quarter));
        }
        if (isNotNull(obj.month)) {
            date = getOffsetMonth(date, parseInt(obj.month));
        }
        if (isNotNull(obj.week)) {
            date = getOffsetDate(date, parseInt(obj.week) * 7);
        }
        if (isNotNull(obj.day)) {
            date = getOffsetDate(date, parseInt(obj.day));
        }
        if (isNotNull(obj.workDay)) {
            // 配置了节假日就按照节假日计算工作日偏移，否则按正常的天去算
            if (isNotNull(BI.holidays)) {
                const count = Math.abs(obj.workDay);
                for (let i = 0; i < count; i++) {
                    date = getOffsetDate(date, obj.workDay < 0 ? -1 : 1);
                    if (isNotNull(BI.holidays[print(date, "%Y-%X-%d")])) {
                        i--;
                    }
                }
            } else {
                date = getOffsetDate(date, parseInt(obj.workDay));
            }
        }
        if (isNotNull(obj.position) && obj.position !== DynamicDateCard.OFFSET.CURRENT) {
            date = this.getBeginDate(date, obj);
        }

        return getDate(date.getFullYear(), date.getMonth(), date.getDate());
    },

    getBeginDate(date, obj) {
        if (isNotNull(obj.day)) {
            return obj.position === DynamicDateCard.OFFSET.BEGIN
                ? getDate(date.getFullYear(), date.getMonth(), 1)
                : getDate(date.getFullYear(), date.getMonth(), getLastDateOfMonth(date).getDate());
        }
        if (isNotNull(obj.week)) {
            return obj.position === DynamicDateCard.OFFSET.BEGIN ? getWeekStartDate(date) : getWeekEndDate(date);
        }
        if (isNotNull(obj.month)) {
            return obj.position === DynamicDateCard.OFFSET.BEGIN
                ? getDate(date.getFullYear(), date.getMonth(), 1)
                : getDate(date.getFullYear(), date.getMonth(), getLastDateOfMonth(date).getDate());
        }
        if (isNotNull(obj.quarter)) {
            return obj.position === DynamicDateCard.OFFSET.BEGIN ? getQuarterStartDate(date) : getQuarterEndDate(date);
        }
        if (isNotNull(obj.year)) {
            return obj.position === DynamicDateCard.OFFSET.BEGIN
                ? getDate(date.getFullYear(), 0, 1)
                : getDate(date.getFullYear(), 11, 31);
        }

        return date;
    },
};
