import { HTapeLayout, shortcut, Widget, toPix, isNaturalNumber, i18nText, SIZE_CONSANTS } from "@/core";
import { SignEditor, TextValueCombo } from "@/case";
import { Label } from "@/base";
import { DynamicDateCard } from "./dynamicdate.card";

@shortcut()
export class DynamicDateParamItem extends Widget {
    static xtype = "bi.dynamic_date_param_item";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_INPUT_CHANGE = "EVENT_INPUT_CHANGE";

    props() {
        return {
            baseCls: "bi-dynamic-date-param-item",
            dateType: DynamicDateCard.TYPE.YEAR,
            validationChecker() {
                return true;
            },
            value: 0,
            offset: 0,
            height: SIZE_CONSANTS.TOOL_BAR_HEIGHT,
        };
    }

    render() {
        const o = this.options;

        return {
            type: HTapeLayout.xtype,
            items: [
                {
                    el: {
                        type: SignEditor.xtype,
                        cls: "bi-border bi-focus-shadow bi-border-radius",
                        height: toPix(SIZE_CONSANTS.TOOL_BAR_HEIGHT, 2),
                        validationChecker(v) {
                            return isNaturalNumber(v);
                        },
                        value: o.value,
                        ref: _ref => {
                            this.editor = _ref;
                        },
                        errorText() {
                            return i18nText("BI-Please_Input_Natural_Number");
                        },
                        allowBlank: false,
                        listeners: [
                            {
                                eventName: SignEditor.EVENT_CONFIRM,
                                action: () => {
                                    this.fireEvent(DynamicDateParamItem.EVENT_CHANGE);
                                },
                            },
                            {
                                eventName: SignEditor.EVENT_CHANGE,
                                action: () => {
                                    this.fireEvent(DynamicDateParamItem.EVENT_INPUT_CHANGE);
                                },
                            }
                        ],
                    },
                    width: 60,
                },
                {
                    el: {
                        type: Label.xtype,
                        height: SIZE_CONSANTS.TOOL_BAR_HEIGHT,
                        text: this._getText(),
                    },
                    width: o.dateType === DynamicDateCard.TYPE.WORK_DAY ? 60 : 20,
                },
                {
                    type: TextValueCombo.xtype,
                    height: SIZE_CONSANTS.TOOL_BAR_HEIGHT,
                    items: [
                        {
                            text: i18nText("BI-Basic_Front"),
                            value: 0,
                        },
                        {
                            text: i18nText("BI-Basic_Behind"),
                            value: 1,
                        }
                    ],
                    ref: _ref => {
                        this.offsetCombo = _ref;
                    },
                    container: null,
                    value: o.offset,
                    listeners: [
                        {
                            eventName: TextValueCombo.EVENT_CHANGE,
                            action: () => {
                                this.fireEvent(DynamicDateParamItem.EVENT_CHANGE);
                            },
                        }
                    ],
                }
            ],
        };
    }

    _getText() {
        let text = "";
        switch (this.options.dateType) {
            case DynamicDateCard.TYPE.YEAR:
                text = i18nText("BI-Basic_Year");
                break;
            case DynamicDateCard.TYPE.QUARTER:
                text = i18nText("BI-Basic_Single_Quarter");
                break;
            case DynamicDateCard.TYPE.MONTH:
                text = i18nText("BI-Basic_Month");
                break;
            case DynamicDateCard.TYPE.WEEK:
                text = i18nText("BI-Basic_Week");
                break;
            case DynamicDateCard.TYPE.DAY:
                text = i18nText("BI-Basic_Day");
                break;
            case DynamicDateCard.TYPE.WORK_DAY:
            default:
                text = i18nText("BI-Basic_Work_Day");
                break;
        }

        return text;
    }

    checkValidation() {
        return isNaturalNumber(this.editor.getValue());
    }

    setValue(v) {
        v = v || {};
        v.value = v.value || 0;
        v.offset = v.offset || 0;
        this.editor.setValue(v.value);
        this.offsetCombo.setValue(v.offset);
    }

    getValue() {
        return {
            dateType: this.options.dateType,
            value: this.editor.getValue(),
            offset: this.offsetCombo.getValue()[0],
        };
    }
}
