import { shortcut, Widget, extend, emptyFn, createWidget, toPix, Controller } from "@/core";
import { Editor } from "@/base";

@shortcut()
export class TextEditor extends Widget {
    static xtype = "bi.text_editor";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_CLICK = "EVENT_CLICK";
    static EVENT_KEY_DOWN = "EVENT_KEY_DOWN";
    static EVENT_SPACE = "EVENT_SPACE";
    static EVENT_BACKSPACE = "EVENT_BACKSPACE";
    static EVENT_START = "EVENT_START";
    static EVENT_PAUSE = "EVENT_PAUSE";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_CHANGE_CONFIRM = "EVENT_CHANGE_CONFIRM";
    static EVENT_VALID = "EVENT_VALID";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_ENTER = "EVENT_ENTER";
    static EVENT_RESTRICT = "EVENT_RESTRICT";
    static EVENT_REMOVE = "EVENT_REMOVE";
    static EVENT_EMPTY = "EVENT_EMPTY";

    _defaultConfig(config) {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            extraCls: `bi-text-editor bi-focus-shadow ${config.simple ? "bi-border-bottom" : "bi-border"}`,
            hgap: 4,
            vgap: 2,
            lgap: 0,
            rgap: 0,
            tgap: 0,
            bgap: 0,
            validationChecker: emptyFn,
            quitChecker: emptyFn,
            allowBlank: false,
            watermark: "",
            errorText: "",
            height: 24,
        });
    }

    render() {
        const o = this.options;
        const border = o.simple ? 1 : 2;
        this.editor = createWidget({
            type: Editor.xtype,
            element: this,
            width: toPix(o.width, border),
            height: toPix(o.height, border),
            simple: o.simple,
            hgap: o.hgap,
            vgap: o.vgap,
            lgap: o.lgap,
            rgap: o.rgap,
            tgap: o.tgap,
            bgap: o.bgap,
            value: o.value,
            title: o.title,
            tipType: o.tipType,
            validationChecker: o.validationChecker,
            quitChecker: o.quitChecker,
            allowBlank: o.allowBlank,
            watermark: o.watermark,
            errorText: o.errorText,
            inputType: o.inputType,
            autocomplete: o.autocomplete,
            autoTrim: o.autoTrim,
        });
        this.editor.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });

        this.editor.on(Editor.EVENT_FOCUS, () => {
            this.fireEvent(TextEditor.EVENT_FOCUS);
        });
        this.editor.on(Editor.EVENT_BLUR, () => {
            this.fireEvent(TextEditor.EVENT_BLUR);
        });
        this.editor.on(Editor.EVENT_CLICK, () => {
            this.fireEvent(TextEditor.EVENT_CLICK);
        });
        this.editor.on(Editor.EVENT_CHANGE, () => {
            this.fireEvent(TextEditor.EVENT_CHANGE);
        });
        this.editor.on(Editor.EVENT_KEY_DOWN, v => {
            this.fireEvent(TextEditor.EVENT_KEY_DOWN);
        });
        this.editor.on(Editor.EVENT_SPACE, v => {
            this.fireEvent(TextEditor.EVENT_SPACE);
        });
        this.editor.on(Editor.EVENT_BACKSPACE, v => {
            this.fireEvent(TextEditor.EVENT_BACKSPACE);
        });

        this.editor.on(Editor.EVENT_VALID, () => {
            this.element.removeClass("error");
            this.fireEvent(TextEditor.EVENT_VALID);
        });
        this.editor.on(Editor.EVENT_CONFIRM, () => {
            this.fireEvent(TextEditor.EVENT_CONFIRM);
        });
        this.editor.on(Editor.EVENT_CHANGE_CONFIRM, () => {
            this.fireEvent(TextEditor.EVENT_CHANGE_CONFIRM);
        });
        this.editor.on(Editor.EVENT_REMOVE, v => {
            this.fireEvent(TextEditor.EVENT_REMOVE);
        });
        this.editor.on(Editor.EVENT_START, () => {
            this.fireEvent(TextEditor.EVENT_START);
        });
        this.editor.on(Editor.EVENT_PAUSE, () => {
            this.fireEvent(TextEditor.EVENT_PAUSE);
        });
        this.editor.on(Editor.EVENT_STOP, () => {
            this.fireEvent(TextEditor.EVENT_STOP);
        });
        this.editor.on(Editor.EVENT_ERROR, (...args) => {
            this.element.addClass("error");
            this.fireEvent(TextEditor.EVENT_ERROR, ...args);
        });
        this.editor.on(Editor.EVENT_ENTER, () => {
            this.fireEvent(TextEditor.EVENT_ENTER);
        });
        this.editor.on(Editor.EVENT_RESTRICT, () => {
            this.fireEvent(TextEditor.EVENT_RESTRICT);
        });
        this.editor.on(Editor.EVENT_EMPTY, () => {
            this.fireEvent(TextEditor.EVENT_EMPTY);
        });
    }

    setWaterMark(v) {
        this.options.watermark = v;
        this.editor.setWaterMark(v);
    }

    focus() {
        this.editor.focus();
    }

    blur() {
        this.editor.blur();
    }

    setErrorText(text) {
        this.editor.setErrorText(text);
    }

    getErrorText() {
        return this.editor.getErrorText();
    }

    isValid() {
        return this.editor.isValid();
    }

    setValue(v) {
        this.editor.setValue(v);
    }

    getValue() {
        return this.editor.getValue();
    }
}
