import { Widget, extend, parseInt, parseFloat } from "@/core";

export class AccurateCalculationModel extends Widget {
    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "",
        });
    }

    _init() {
        super._init(...arguments);
    }

    _getMagnitude(n) {
        let magnitude = "1";
        for (let i = 0; i < n; i++) {
            magnitude += "0";
        }

        return parseInt(magnitude);
    }

    _formatDecimal(stringNumber1, stringNumber2) {
        if (stringNumber1.numDecimalLength === stringNumber2.numDecimalLength) {
            return;
        }
        let magnitudeDiff = stringNumber1.numDecimalLength - stringNumber2.numDecimalLength;
        let needAddZero;
        if (magnitudeDiff > 0) {
            needAddZero = stringNumber2;
        } else {
            needAddZero = stringNumber1;
            magnitudeDiff = 0 - magnitudeDiff;
        }
        for (let i = 0; i < magnitudeDiff; i++) {
            if (needAddZero.numDecimal === "0" && i === 0) {
                continue;
            }
            needAddZero.numDecimal += "0";
        }
    }

    _stringNumberFactory(num) {
        const strNum = num.toString();
        const numStrArray = strNum.split(".");
        const numInteger = numStrArray[0];
        let numDecimal;
        let numDecimalLength;
        if (numStrArray.length === 1) {
            numDecimal = "0";
            numDecimalLength = 0;
        } else {
            numDecimal = numStrArray[1];
            numDecimalLength = numStrArray[1].length;
        }

        return {
            numInteger,
            numDecimal,
            numDecimalLength,
        };
    }

    _accurateSubtraction(num1, num2) {
        // num1-num2 && num1>num2
        const stringNumber1 = this._stringNumberFactory(num1);
        const stringNumber2 = this._stringNumberFactory(num2);
        // 整数部分计算
        let integerResult = parseInt(stringNumber1.numInteger) - parseInt(stringNumber2.numInteger);
        // 小数部分
        this._formatDecimal(stringNumber1, stringNumber2);
        const decimalMaxLength = getDecimalMaxLength(stringNumber1, stringNumber2);

        let decimalResultTemp;
        let decimalResult;

        if (parseInt(stringNumber1.numDecimal) >= parseInt(stringNumber2.numDecimal)) {
            decimalResultTemp = (parseInt(stringNumber1.numDecimal) - parseInt(stringNumber2.numDecimal)).toString();
            decimalResult = addZero(decimalResultTemp, decimalMaxLength);
        } else {
            // 否则借位
            integerResult--;
            const borrow = this._getMagnitude(decimalMaxLength);
            decimalResultTemp = (
                borrow +
                parseInt(stringNumber1.numDecimal) -
                parseInt(stringNumber2.numDecimal)
            ).toString();
            decimalResult = addZero(decimalResultTemp, decimalMaxLength);
        }
        const result = `${integerResult}.${decimalResult}`;

        return parseFloat(result);

        function getDecimalMaxLength(num1, num2) {
            if (num1.numDecimal.length >= num2.numDecimal.length) {
                return num1.numDecimal.length;
            }

            return num2.numDecimal.length;
        }

        function addZero(resultTemp, length) {
            const diff = length - resultTemp.length;
            for (let i = 0; i < diff; i++) {
                resultTemp = `0${resultTemp}`;
            }

            return resultTemp;
        }
    }

    _accurateAddition(num1, num2) {
        // 加法结合律
        const stringNumber1 = this._stringNumberFactory(num1);
        const stringNumber2 = this._stringNumberFactory(num2);
        // 整数部分计算
        let integerResult = parseInt(stringNumber1.numInteger) + parseInt(stringNumber2.numInteger);
        // 小数部分
        this._formatDecimal(stringNumber1, stringNumber2);

        let decimalResult = (parseInt(stringNumber1.numDecimal) + parseInt(stringNumber2.numDecimal)).toString();

        if (decimalResult !== "0") {
            if (decimalResult.length <= stringNumber1.numDecimal.length) {
                decimalResult = addZero(decimalResult, stringNumber1.numDecimal.length);
            } else {
                integerResult++; // 进一
                decimalResult = decimalResult.slice(1);
            }
        }
        const result = `${integerResult}.${decimalResult}`;

        return parseFloat(result);

        function addZero(resultTemp, length) {
            const diff = length - resultTemp.length;
            for (let i = 0; i < diff; i++) {
                resultTemp = `0${resultTemp}`;
            }

            return resultTemp;
        }
    }

    _accurateMultiplication(num1, num2) {
        // 乘法分配律
        const stringNumber1 = this._stringNumberFactory(num1);
        const stringNumber2 = this._stringNumberFactory(num2);
        // 整数部分计算
        const integerResult = parseInt(stringNumber1.numInteger) * parseInt(stringNumber2.numInteger);
        // num1的小数和num2的整数
        const dec1Int2 = this._accurateDivisionTenExponent(
            parseInt(stringNumber1.numDecimal) * parseInt(stringNumber2.numInteger),
            stringNumber1.numDecimalLength
        );
        // num1的整数和num2的小数
        const int1dec2 = this._accurateDivisionTenExponent(
            parseInt(stringNumber1.numInteger) * parseInt(stringNumber2.numDecimal),
            stringNumber2.numDecimalLength
        );
        // 小数*小数
        const dec1dec2 = this._accurateDivisionTenExponent(
            parseInt(stringNumber1.numDecimal) * parseInt(stringNumber2.numDecimal),
            stringNumber1.numDecimalLength + stringNumber2.numDecimalLength
        );

        return this._accurateAddition(
            this._accurateAddition(this._accurateAddition(integerResult, dec1Int2), int1dec2),
            dec1dec2
        );
    }

    _accurateDivisionTenExponent(num, n) {
        // num/10^n && n>0
        const stringNumber = this._stringNumberFactory(num);
        let integerResult, partDecimalResult;
        if (stringNumber.numInteger.length > n) {
            integerResult = stringNumber.numInteger.slice(0, stringNumber.numInteger.length - n);
            partDecimalResult = stringNumber.numInteger.slice(-n);
        } else {
            integerResult = "0";
            partDecimalResult = addZero(stringNumber.numInteger, n);
        }
        const result = `${integerResult}.${partDecimalResult}${stringNumber.numDecimal}`;

        return parseFloat(result);

        function addZero(resultTemp, length) {
            const diff = length - resultTemp.length;
            for (let i = 0; i < diff; i++) {
                resultTemp = `0${resultTemp}`;
            }

            return resultTemp;
        }
    }

    accurateSubtraction(num1, num2) {
        if (num1 >= 0 && num2 >= 0) {
            if (num1 >= num2) {
                return this._accurateSubtraction(num1, num2);
            }

            return -this._accurateSubtraction(num2, num1);
        }
        if (num1 >= 0 && num2 < 0) {
            return this._accurateAddition(num1, -num2);
        }
        if (num1 < 0 && num2 >= 0) {
            return -this._accurateAddition(-num1, num2);
        }
        if (num1 < 0 && num2 < 0) {
            if (num1 >= num2) {
                return this._accurateSubtraction(-num2, -num1);
            }

            return this._accurateSubtraction(-num1, -num2);
        }
    }

    accurateAddition(num1, num2) {
        if (num1 >= 0 && num2 >= 0) {
            return this._accurateAddition(num1, num2);
        }
        if (num1 >= 0 && num2 < 0) {
            return this.accurateSubtraction(num1, -num2);
        }
        if (num1 < 0 && num2 >= 0) {
            return this.accurateSubtraction(num2, -num1);
        }
        if (num1 < 0 && num2 < 0) {
            return -this._accurateAddition(-num1, -num2);
        }
    }

    accurateMultiplication(num1, num2) {
        if (num1 >= 0 && num2 >= 0) {
            return this._accurateMultiplication(num1, num2);
        }
        if (num1 >= 0 && num2 < 0) {
            return -this._accurateMultiplication(num1, -num2);
        }
        if (num1 < 0 && num2 >= 0) {
            return -this._accurateMultiplication(-num1, num2);
        }
        if (num1 < 0 && num2 < 0) {
            return this._accurateMultiplication(-num1, -num2);
        }
    }

    accurateDivisionTenExponent(num1, n) {
        if (num1 >= 0) {
            return this._accurateDivisionTenExponent(num1, n);
        }

        return -this._accurateDivisionTenExponent(-num1, n);
    }
}
