import {
    shortcut,
    extend,
    Selection,
    createWidget,
    createItems,
    isNotNull,
    contains,
    each,
    VerticalLayout,
    isNotEmptyArray,
    isEmpty,
    Layout,
    deepClone,
    isArray,
    isNotEmptyString,
    some,
    concat,
    BlankSplitChar,
    SIZE_CONSANTS
} from "@/core";
import { Pane, ButtonTree, ComboGroup } from "@/base";
import { DownListGroup, DownListGroupItem, DownListItem } from "../downlist";

@shortcut()
export class MultiLayerDownListPopup extends Pane {
    static xtype = "bi.multi_layer_down_list_popup";

    constants = {
        nextIcon: "pull-right-e-font",
        height: 25,
        iconHeight: 12,
        iconWidth: 12,
        hgap: 0,
        vgap: 0,
        border: 1,
    };

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_SON_VALUE_CHANGE = "EVENT_SON_VALUE_CHANGE";

    _defaultConfig() {
        const conf = super._defaultConfig(...arguments);

        return extend(conf, {
            baseCls: "bi-down-list-popup",
            items: [],
            chooseType: Selection.Multi,
        });
    }

    _init() {
        super._init(...arguments);
        this.singleValues = [];
        this.childValueMap = {};
        this.fatherValueMap = {};
        this.items = [];
        const self = this,
            o = this.options,
            children = this._createPopupItems(o.items);
        this.popup = createWidget({
            type: ButtonTree.xtype,
            items: createItems(
                children,
                {},
                {
                    adjustLength: -2,
                }
            ),
            layouts: [
                {
                    type: VerticalLayout.xtype,
                    hgap: this.constants.hgap,
                    vgap: this.constants.vgap,
                }
            ],
            value: this._digest(o.value),
            chooseType: o.chooseType,
        });

        this.popup.on(ButtonTree.EVENT_CHANGE, (value, object) => {
            let changedValue = value;
            if (isNotNull(self.childValueMap[value])) {
                changedValue = self.childValueMap[value];
                const fatherValue = self.fatherValueMap[value];
                const fatherArrayValue = `${fatherValue}`.split(BlankSplitChar);
                self.fireEvent(
                    MultiLayerDownListPopup.EVENT_SON_VALUE_CHANGE,
                    changedValue,
                    fatherArrayValue.length > 1 ? fatherArrayValue : fatherValue
                );
            } else {
                self.fireEvent(MultiLayerDownListPopup.EVENT_CHANGE, changedValue, object);
            }

            if (!contains(self.singleValues, changedValue)) {
                const item = self.getValue();
                const result = [];
                each(item, (i, valueObject) => {
                    if (valueObject.value != changedValue) {
                        result.push(valueObject);
                    }
                });
                self.setValue(result);
            }
        });

        createWidget({
            type: VerticalLayout.xtype,
            element: this,
            items: [this.popup],
            vgap: 5,
        });
    }

    _createPopupItems(items) {
        const self = this,
            result = [];
        each(items, (i, it) => {
            const item_done = {
                type: DownListGroup.xtype,
                items: [],
            };
            const storeItem = [];

            each(it, (i, sourceItem) => {
                const item = extend({}, sourceItem);
                if (isNotEmptyArray(sourceItem.children) && !isEmpty(sourceItem.el)) {
                    item.type = ComboGroup.xtype;
                    item.cls = "down-list-group";
                    item.trigger = "hover";
                    item.isNeedAdjustWidth = false;
                    item.el = sourceItem.el;
                    item.el.title = sourceItem.el.title;
                    item.el.type = DownListGroupItem.xtype;
                    item.el.logic = {
                        dynamic: true,
                    };
                    item.el.height = sourceItem.el.height || SIZE_CONSANTS.LIST_ITEM_HEIGHT;
                    item.el.iconCls2 = self.constants.nextIcon;
                    item.popup = {
                        lgap: 1,
                        el: {
                            type: ButtonTree.xtype,
                            chooseType: 0,
                            layouts: [
                                {
                                    type: VerticalLayout.xtype,
                                }
                            ],
                        },
                        innerVgap: 5,
                        maxHeight: 378,
                    };
                    self._createChildren(item, sourceItem);
                } else {
                    item.type = sourceItem.type || DownListItem.xtype;
                    item.title = sourceItem.title;
                    item.textRgap = 10;
                    item.isNeedAdjustWidth = false;
                    item.logic = {
                        dynamic: true,
                    };
                }
                const el_done = {};
                el_done.el = item;
                item_done.items.push(el_done);
                storeItem.push(item);
            });
            if (self._isGroup(item_done.items)) {
                each(item_done.items, (i, item) => {
                    self.singleValues.push(item.el.value);
                });
            }

            result.push(item_done);
            self.items.push(storeItem);
            if (self._needSpliter(i, items.length)) {
                const spliter_container = createWidget({
                    type: VerticalLayout.xtype,
                    items: [
                        {
                            el: {
                                type: Layout.xtype,
                                cls: "bi-down-list-spliter bi-border-top cursor-pointer",
                                height: 0,
                            },
                        }
                    ],
                    cls: "bi-down-list-spliter-container cursor-pointer",
                    vgap: 5,
                    hgap: 12,
                });
                result.push(spliter_container);
            }
        });

        return result;
    }

    _createChildren(targetItem, sourceItem) {
        const self = this;
        this._formatEL(targetItem).el.childValues = [];
        targetItem.items = targetItem.children = [];
        each(sourceItem.children, (i, child) => {
            const item = child.el ? extend({}, child.el, { children: child.children }) : extend({}, child);
            const fatherValue = deepClone(self._formatEL(targetItem).el.value);
            const childValue = deepClone(item.value);
            self.singleValues.push(item.value);
            item.type = item.type || DownListItem.xtype;
            item.extraCls = " child-down-list-item";
            item.textRgap = 10;
            item.isNeedAdjustWidth = false;
            item.logic = {
                dynamic: true,
            };
            item.father = fatherValue;
            self.fatherValueMap[self._createChildValue(fatherValue, childValue)] = fatherValue;
            self.childValueMap[self._createChildValue(fatherValue, childValue)] = childValue;
            item.value = self._createChildValue(fatherValue, childValue);
            self._formatEL(targetItem).el.childValues.push(item.value);
            if (isNotEmptyArray(child.children)) {
                item.type = DownListGroupItem.xtype;
                item.iconCls2 = self.constants.nextIcon;
                item.height = child.height || SIZE_CONSANTS.LIST_ITEM_HEIGHT;
                self._createChildren(item, child);
            }
            targetItem.items.push(item);
        });
    }

    _formatEL(obj) {
        if (obj && obj.el) {
            return obj;
        }

        return {
            el: obj,
        };
    }

    _isGroup(i) {
        return i.length > 1;
    }

    _needSpliter(i, itemLength) {
        return i < itemLength - 1;
    }

    _createChildValue(fatherValue, childValue) {
        let fValue = fatherValue;
        if (isArray(fatherValue)) {
            fValue = fatherValue.join(BlankSplitChar);
        }

        return fValue + BlankSplitChar + childValue;
    }

    _digest(valueItem) {
        const self = this;
        const valueArray = [];
        each(valueItem, (i, item) => {
            let value;
            if (isNotNull(item.childValue)) {
                value = self._createChildValue(item.value, item.childValue);
            } else {
                value = item.value;
            }
            valueArray.push(value);
        });

        return valueArray;
    }

    _checkValues(values) {
        const value = [];
        each(this.items, (idx, itemGroup) => {
            each(itemGroup, (id, item) => {
                if (isNotNull(item.children)) {
                    const childValues = getChildrenValue(item);
                    const v = joinValue(childValues, values[idx]);
                    if (isNotEmptyString(v)) {
                        value.push(v);
                    }
                } else {
                    if (item.value === values[idx][0]) {
                        value.push(values[idx][0]);
                    }
                }
            });
        });

        return value;

        function joinValue(sources, targets) {
            let value = "";
            some(sources, (idx, s) =>
                some(targets, (id, t) => {
                    if (s === t) {
                        value = s;

                        return true;
                    }
                })
            );

            return value;
        }

        function getChildrenValue(item) {
            let children = [];
            if (isNotNull(item.children)) {
                each(item.children, (idx, child) => {
                    children = concat(children, getChildrenValue(child));
                });
            } else {
                children.push(item.value);
            }

            return children;
        }
    }

    populate(items) {
        super.populate(...arguments);
        const self = this;
        self.childValueMap = {};
        self.fatherValueMap = {};
        self.singleValues = [];
        this.items = [];
        const children = self._createPopupItems(items);
        const popupItem = createItems(
            children,
            {},
            {
                adjustLength: -2,
            }
        );
        self.popup.populate(popupItem);
    }

    setValue(valueItem) {
        this.popup.setValue(this._digest(valueItem));
    }

    _getValue() {
        const v = [];
        each(this.popup.getAllButtons(), (i, item) => {
            i % 2 === 0 && v.push(item.getValue());
        });

        return v;
    }

    getValue() {
        const self = this,
            result = [];
        const values = this._checkValues(this._getValue());
        each(values, (i, value) => {
            const valueItem = {};
            if (isNotNull(self.childValueMap[value])) {
                const fartherValue = self.fatherValueMap[value];
                valueItem.childValue = self.childValueMap[value];
                const fatherArrayValue = `${fartherValue}`.split(BlankSplitChar);
                valueItem.value = fatherArrayValue.length > 1 ? fatherArrayValue : fartherValue;
            } else {
                valueItem.value = value;
            }
            result.push(valueItem);
        });

        return result;
    }
}
