import {
    shortcut,
    Widget,
    i18nText,
    emptyFn,
    createWidget,
    Controller,
    VerticalLayout,
    isEmptyArray,
    isArray
} from "@/core";
import { MultiLayerSelectLevelTree } from "./multilayerselecttree.leveltree";
import { TextButton } from "@/base";

@shortcut()
export class MultiLayerSelectTreeInsertSearchPane extends Widget {
    static xtype = "bi.multilayer_select_tree_insert_search_pane";

    static EVENT_ADD_ITEM = "EVENT_ADD_ITEM";
    static EVENT_CHANGE = "EVENT_CHANGE";

    props() {
        return {
            baseCls: "bi-multilayer-select-tree-popup",
            tipText: i18nText("BI-No_Selected_Item"),
            isDefaultInit: false,
            itemsCreator: emptyFn,
            items: [],
            value: "",
        };
    }

    render() {
        const self = this,
            o = this.options;
        this.tree = createWidget({
            type: MultiLayerSelectLevelTree.xtype,
            isDefaultInit: o.isDefaultInit,
            items: o.items,
            itemsCreator:
                o.itemsCreator === emptyFn
                    ? emptyFn
                    : function (op, callback) {
                        o.itemsCreator(op, res => {
                            callback(res);
                            self.setKeyword(o.keywordGetter());
                        });
                    },
            keywordGetter: o.keywordGetter,
            value: o.value,
            scrollable: null,
            listeners: [
                {
                    eventName: Controller.EVENT_CHANGE,
                    action() {
                        self.fireEvent(Controller.EVENT_CHANGE, arguments);
                    },
                },
                {
                    eventName: MultiLayerSelectLevelTree.EVENT_CHANGE,
                    action() {
                        self.fireEvent(MultiLayerSelectTreeInsertSearchPane.EVENT_CHANGE);
                    },
                }
            ],
        });

        return {
            type: VerticalLayout.xtype,
            scrolly: false,
            scrollable: true,
            vgap: 5,
            items: [
                {
                    type: TextButton.xtype,
                    invisible: true,
                    text: i18nText("BI-Basic_Click_To_Add_Text", ""),
                    height: 24,
                    cls: "bi-high-light",
                    hgap: 5,
                    ref(_ref) {
                        self.addNotMatchTip = _ref;
                    },
                    handler() {
                        self.fireEvent(MultiLayerSelectTreeInsertSearchPane.EVENT_ADD_ITEM, o.keywordGetter());
                    },
                },
                this.tree
            ],
        };
    }

    setKeyword(keyword) {
        const showTip = isEmptyArray(this.tree.getAllLeaves());
        this.addNotMatchTip.setVisible(showTip);
        showTip && this.addNotMatchTip.setText(i18nText("BI-Basic_Click_To_Add_Text", keyword));
    }

    getValue() {
        return this.tree.getValue();
    }

    setValue(v) {
        v = isArray(v) ? v : [v];
        this.tree.setValue(v);
    }

    populate(items) {
        this.tree.populate(items);
    }
}
