import {
    shortcut,
    Widget,
    extend,
    emptyFn,
    createWidget,
    map,
    i18nText,
    VerticalAdaptLayout,
    VTapeLayout,
    Selection
} from "@/core";
import { TextButton, Label } from "@/base";
import { DisplaySelectedList } from "./multiselect.display";

@shortcut()
export class MultiSelectCheckPane extends Widget {
    static xtype = "bi.multi_select_check_pane";

    constants = { height: 12, lgap: 10, tgap: 10, bgap: 5 };

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-multi-select-check-pane bi-background",
            items: [],
            itemsCreator: emptyFn,
            valueFormatter: emptyFn,
            onClickContinueSelect: emptyFn,
        });
    }

    _init() {
        super._init(...arguments);

        const self = this,
            opts = this.options;

        this.storeValue = opts.value || {};
        this.display = createWidget({
            type: DisplaySelectedList.xtype,
            items: opts.items,
            itemsCreator(op, callback) {
                op = extend(op || {}, {
                    selectedValues: self.storeValue.value,
                });
                if (self.storeValue.type === Selection.Multi) {
                    callback({
                        items: map(self.storeValue.value, (i, v) => {
                            const txt = opts.valueFormatter(v) || v;

                            return {
                                text: txt,
                                value: v,
                                title: txt,
                            };
                        }),
                    });

                    return;
                }
                opts.itemsCreator(op, callback);
            },
        });

        this.continueSelect = createWidget({
            type: TextButton.xtype,
            title: i18nText("BI-Continue_Select"),
            text: i18nText("BI-Continue_Select"),
            cls: "multi-select-check-selected bi-high-light",
        });

        this.continueSelect.on(TextButton.EVENT_CHANGE, () => {
            opts.onClickContinueSelect();
        });

        createWidget({
            type: VTapeLayout.xtype,
            element: this,
            items: [
                {
                    height: this.constants.height,
                    el: {
                        type: VerticalAdaptLayout.xtype,
                        columnSize: ["auto", "auto"],
                        cls: "multi-select-continue-select",
                        items: [
                            {
                                el: {
                                    type: Label.xtype,
                                    title: i18nText("BI-Selected_Data"),
                                    text: i18nText("BI-Selected_Data"),
                                },
                                lgap: this.constants.lgap,
                            },
                            {
                                el: this.continueSelect,
                                hgap: this.constants.lgap,
                            }
                        ],
                    },
                    tgap: this.constants.tgap,
                },
                {
                    height: "fill",
                    el: this.display,
                    tgap: this.constants.bgap,
                }
            ],
        });
    }

    setValue(v) {
        this.storeValue = v || {};
    }

    empty() {
        this.display.empty();
    }

    populate() {
        this.display.populate(...arguments);
    }
}
