import { shortcut, extend, emptyFn, createWidget, VerticalLayout, createItems } from "@/core";
import { Pane, ButtonGroup, Loader, IconTextItem } from "@/base";
import { ListPane } from "@/case";

@shortcut()
export class DisplaySelectedList extends Pane {
    static xtype = "bi.display_selected_list";

    constants = { height: 24, lgap: 10 };

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-display-list",
            itemsCreator: emptyFn,
            items: [],
        });
    }

    _init() {
        super._init(...arguments);

        const self = this,
            opts = this.options;

        this.hasNext = false;
        let cacheItems = [];

        this.button_group = createWidget({
            type: ListPane.xtype,
            element: this,
            el: {
                type: Loader.xtype,
                isDefaultInit: false,
                logic: {
                    dynamic: true,
                    scrolly: true,
                },
                items: this._createItems(opts.items),
                chooseType: ButtonGroup.CHOOSE_TYPE_MULTI,
                layouts: [
                    {
                        type: VerticalLayout.xtype,
                        lgap: 10,
                    }
                ],
            },
            itemsCreator(options, callback) {
                if (options.times === 1) {
                    cacheItems = [];
                }

                if (cacheItems.length > 0) {
                    const renderedItems = cacheItems.slice(0, 100);
                    cacheItems = cacheItems.slice(100);
                    self.hasNext = cacheItems.length > 0;
                    callback(self._createItems(renderedItems));

                    return;
                }

                opts.itemsCreator(options, ob => {
                    self.hasNext = !!ob.hasNext;
                    const firstItemsCount = 100 + (ob.items.length % 100);
                    if (ob.items.length > 100) {
                        cacheItems = ob.items.slice(firstItemsCount);
                        self.hasNext = firstItemsCount !== ob.items.length;
                    }
                    callback(self._createItems(ob.items.slice(0, firstItemsCount)));
                });
            },
            hasNext() {
                return self.hasNext;
            },
        });
    }

    _createItems(items) {
        return createItems(items, {
            type: IconTextItem.xtype,
            cls: "cursor-default check-font icon-size-12 display-list-item bi-tips",
            once: true,
            invalid: true,
            selected: true,
            height: this.constants.height,
            logic: {
                dynamic: true,
            },
        });
    }

    empty() {
        this.button_group.empty();
    }

    populate(items) {
        if (arguments.length === 0) {
            this.button_group.populate();
        } else {
            this.button_group.populate(this._createItems(items));
        }
    }
}
