import {
    shortcut,
    extend,
    emptyFn,
    createWidget,
    Controller,
    i18nText,
    isNotNull,
    isNotEmptyString,
    nextTick,
    Selection
} from "@/core";
import { Single, TextButton } from "@/base";
import { MultiSelectCombo } from "../multiselect.combo";

@shortcut()
export class MultiSelectCheckSelectedButton extends Single {
    static xtype = "bi.multi_select_check_selected_button";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-multi-select-check-selected-button",
            itemsCreator: emptyFn,
        });
    }

    _init() {
        super._init(...arguments);
        const self = this,
            o = this.options;
        this.numberCounter = createWidget({
            type: TextButton.xtype,
            element: this,
            hgap: 4,
            text: "0",
            textAlign: "center",
            textHeight: 16,
            cls: "bi-high-light-background count-tip",
        });
        this.numberCounter.on(Controller.EVENT_CHANGE, function () {
            self.fireEvent(Controller.EVENT_CHANGE, arguments);
        });

        this.numberCounter.on(TextButton.EVENT_CHANGE, function () {
            self.fireEvent(MultiSelectCheckSelectedButton.EVENT_CHANGE, arguments);
        });

        this.numberCounter.element.hover(
            () => {
                self.numberCounter.setTag(self.numberCounter.getText());
                self.numberCounter.setText(i18nText("BI-Check_Selected"));
            },
            () => {
                self.numberCounter.setText(self.numberCounter.getTag());
            }
        );
        this.setVisible(false);
        if (isNotNull(o.value)) {
            this.setValue(o.value);
        }
    }

    _populate(ob) {
        const self = this,
            o = this.options;
        if (ob.type === Selection.All) {
            o.itemsCreator(
                {
                    type: MultiSelectCombo.REQ_GET_DATA_LENGTH,
                },
                res => {
                    if (self.options.value.type !== Selection.All) {
                        return;
                    }
                    if (isNotEmptyString(res.count)) {
                        nextTick(() => {
                            self.numberCounter.setText(res.count);
                            self.setVisible(true);
                        });

                        return;
                    }
                    const length = res.count - ob.value.length;
                    nextTick(() => {
                        self.numberCounter.setText(length);
                        self.setVisible(length > 0);
                    });
                }
            );

            return;
        }
        nextTick(() => {
            self.numberCounter.setText(ob.value.length);
            self.setVisible(ob.value.length > 0);
        });
    }

    _assertValue(ob) {
        ob || (ob = {});
        ob.type || (ob.type = Selection.Multi);
        ob.value || (ob.value = []);

        return ob;
    }

    setValue(ob) {
        ob = this._assertValue(ob);
        this.options.value = ob;
        this._populate(ob);
    }

    populate() {
        this._populate(this._assertValue(this.options.value));
    }

    getValue() {}
}
