import {
    shortcut,
    Widget,
    extend,
    emptyFn,
    deepClone,
    isKey,
    Selection,
    remove,
    pushDistinct,
    createWidget,
    isNotEmptyString,
    last,
    initial,
    endWith,
    AbsoluteLayout,
    isEmptyString,
    makeObject,
    map,
    each,
    isNotNull,
    Func,
    concat,
    values,
    filter,
    contains,
    isNull,
    VTapeLayout,
    SIZE_CONSANTS,
    BlankSplitChar
} from "@/core";
import { Searcher } from "@/base";
import { MultiSelectBar } from "@/case";
import { MultiSelectLoader } from "../multiselect/multiselect.loader";
import { MultiSelectSearchPane } from "../multiselect/search/multiselect.search.pane";
import { SelectPatchEditor } from "../multiselect/trigger/editor/editor.patch";
import { SearchEditor } from "../editor/editor.search";

@shortcut()
export class MultiSelectList extends Widget {
    static xtype = "bi.multi_select_list";

    static REQ_GET_DATA_LENGTH = "1";
    static REQ_GET_ALL_DATA = "-1";
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-multi-select-list",
            itemsCreator: emptyFn,
            valueFormatter: emptyFn,
            searcherHeight: 24,
            itemHeight: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
        });
    }

    _init() {
        super._init(...arguments);

        const self = this,
            o = this.options;
        this.storeValue = this._assertValue(deepClone(o.value) || {});

        function assertShowValue() {
            isKey(self._startValue) &&
                (self.storeValue.type === Selection.All
                    ? remove(self.storeValue.value, self._startValue)
                    : pushDistinct(self.storeValue.value, self._startValue));
            // self.trigger.setValue(self.storeValue);
        }

        this.adapter = createWidget({
            type: MultiSelectLoader.xtype,
            cls: "popup-multi-select-list bi-border-left bi-border-right bi-border-bottom",
            itemsCreator: o.itemsCreator,
            valueFormatter: o.valueFormatter,
            itemHeight: o.itemHeight,
            logic: {
                dynamic: false,
            },
            value: o.value,
            isDefaultInit: true,
            // onLoaded: o.onLoaded,
            el: {},
        });
        this.adapter.on(MultiSelectLoader.EVENT_CHANGE, function () {
            self.storeValue = this.getValue();
            self._adjust(() => {
                assertShowValue();
                self.fireEvent(MultiSelectList.EVENT_CHANGE);
            });
        });

        this.searcherPane = createWidget({
            type: MultiSelectSearchPane.xtype,
            cls: "bi-border-left bi-border-right bi-border-bottom",
            valueFormatter: o.valueFormatter,
            keywordGetter() {
                return self.trigger.getKeyword();
            },
            itemsCreator(op, callback) {
                const keyword = self.trigger.getKeyword();
                if (isNotEmptyString(keyword)) {
                    op.keywords = [keyword];
                    o.itemsCreator(op, callback);
                }
            },
            itemHeight: o.itemHeight,
        });
        this.searcherPane.setVisible(false);

        this.trigger = createWidget({
            type: Searcher.xtype,
            el: {
                type: SelectPatchEditor.xtype,
                el: {
                    type: SearchEditor.xtype,
                },
                ref(ref) {
                    self.editor = ref;
                },
                height: o.searcherHeight,
            },
            isAutoSearch: false,
            isAutoSync: false,
            onSearch(op, callback) {
                callback();
            },
            adapter: this.adapter,
            popup: this.searcherPane,
            height: 200,
            masker: false,
            listeners: [
                {
                    eventName: Searcher.EVENT_START,
                    action() {
                        self._showSearcherPane();
                        self._setStartValue("");
                        this.setValue(deepClone(self.storeValue));
                    },
                },
                {
                    eventName: Searcher.EVENT_STOP,
                    action() {
                        self._showAdapter();
                        self._setStartValue("");
                        self.adapter.setValue(self.storeValue);
                        // 需要刷新回到初始界面，否则搜索的结果不能放在最前面
                        self.adapter.populate();
                    },
                },
                {
                    eventName: Searcher.EVENT_PAUSE,
                    action() {
                        self._showAdapter();
                        self.fireEvent(MultiSelectList.EVENT_CHANGE);
                    },
                },
                {
                    eventName: Searcher.EVENT_SEARCHING,
                    action() {
                        let keywords = this.getKeyword();
                        const lastKeyword = last(keywords);
                        keywords = initial(keywords || []);
                        if (keywords.length > 0) {
                            self._joinKeywords(keywords, () => {
                                if (endWith(lastKeyword, BlankSplitChar)) {
                                    self.adapter.setValue(self.storeValue);
                                    assertShowValue();
                                    self.adapter.populate();
                                    self._setStartValue("");
                                } else {
                                    self.adapter.setValue(self.storeValue);
                                    assertShowValue();
                                }
                                self.fireEvent(MultiSelectList.EVENT_CHANGE);
                            });
                        }
                    },
                },
                {
                    eventName: Searcher.EVENT_CHANGE,
                    action(value, obj) {
                        if (obj instanceof MultiSelectBar) {
                            self._joinAll(this.getValue(), () => {
                                assertShowValue();
                                self.fireEvent(MultiSelectList.EVENT_CHANGE);
                            });
                        } else {
                            self._join(this.getValue(), () => {
                                assertShowValue();
                                self.fireEvent(MultiSelectList.EVENT_CHANGE);
                            });
                        }
                    },
                }
            ],
        });

        createWidget({
            type: VTapeLayout.xtype,
            element: this,
            items: [
                {
                    el: this.trigger,
                    height: o.searcherHeight,
                },
                {
                    el: this.adapter,
                    height: "fill",
                }
            ],
        });
        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: this.searcherPane,
                    top: o.searcherHeight,
                    bottom: 0,
                    left: 0,
                    right: 0,
                }
            ],
        });
    }

    _getKeywords() {
        const val = this.editor.getValue();
        let keywords = val.split(/\u200b\s\u200b/);
        if (isEmptyString(keywords[keywords.length - 1])) {
            keywords = keywords.slice(0, keywords.length - 1);
        }
        if (/\u200b\s\u200b$/.test(val)) {
            return keywords.concat([BlankSplitChar]);
        }

        return keywords;
    }

    _showAdapter() {
        this.adapter.setVisible(true);
        this.searcherPane.setVisible(false);
    }

    _showSearcherPane() {
        this.searcherPane.setVisible(true);
        this.adapter.setVisible(false);
    }

    _defaultState() {
        this.trigger.stopEditing();
    }

    _assertValue(val) {
        val || (val = {});
        val.type || (val.type = Selection.Multi);
        val.value || (val.value = []);

        return val;
    }

    _makeMap(values) {
        return makeObject(values || []);
    }

    _joinKeywords(keywords, callback) {
        const self = this,
            o = this.options;
        this._assertValue(this.storeValue);
        // 和复选下拉框同步，allData做缓存是会爆炸的
        o.itemsCreator(
            {
                type: MultiSelectList.REQ_GET_ALL_DATA,
                keywords,
            },
            ob => {
                const values = map(ob.items, "value");
                digest(values);
            }
        );

        function digest(items) {
            const selectedMap = self._makeMap(items);
            each(keywords, (i, val) => {
                if (isNotNull(selectedMap[val])) {
                    self.storeValue.type === Selection.Multi
                        ? pushDistinct(self.storeValue.value, val)
                        : remove(self.storeValue.value, val);
                }
            });
            self._adjust(callback);
        }
    }

    _joinAll(res, callback) {
        const self = this,
            o = this.options;
        this._assertValue(res);
        if (this.storeValue.type === res.type) {
            const result = Func.getSearchResult(
                map(this.storeValue.value, (_i, v) => {
                    return {
                        text: o.valueFormatter(v) || v,
                        value: v,
                    };
                }),
                this.trigger.getKeyword()
            );
            let change = false;
            const tempMap = this._makeMap(this.storeValue.value);
            each(concat(result.match, result.find), (i, obj) => {
                const v = obj.value;
                if (isNotNull(tempMap[v])) {
                    change = true;
                    delete tempMap[v];
                }
            });
            change && (this.storeValue.value = values(tempMap));
            this._adjust(callback);

            return;
        }
        o.itemsCreator(
            {
                type: MultiSelectList.REQ_GET_ALL_DATA,
                keywords: [this.trigger.getKeyword()],
                selectedValues: filter(this.storeValue.value, (_i, v) => !contains(res.value, v)),
            },
            ob => {
                const items = map(ob.items, "value");
                const selectedMap = self._makeMap(self.storeValue.value);
                const notSelectedMap = self._makeMap(res.value);
                const newItems = [];
                each(items, (i, item) => {
                    if (isNotNull(selectedMap[items[i]])) {
                        delete selectedMap[items[i]];
                    }
                    if (isNull(notSelectedMap[items[i]])) {
                        newItems.push(item);
                    }
                });
                self.storeValue.value = newItems.concat(values(selectedMap));
                self._adjust(callback);
            }
        );
    }

    _adjust(callback) {
        const self = this,
            o = this.options;
        if (!this._count) {
            o.itemsCreator(
                {
                    type: MultiSelectList.REQ_GET_DATA_LENGTH,
                },
                res => {
                    self._count = res.count;
                    adjust();
                    callback();
                }
            );
        } else {
            adjust();
            callback();
        }

        function adjust() {
            if (self.storeValue.type === Selection.All && self.storeValue.value.length >= self._count) {
                self.storeValue = {
                    type: Selection.Multi,
                    value: [],
                };
            } else if (self.storeValue.type === Selection.Multi && self.storeValue.value.length >= self._count) {
                self.storeValue = {
                    type: Selection.All,
                    value: [],
                };
            }
        }
    }

    _join(res, callback) {
        const self = this;
        this._assertValue(res);
        this._assertValue(this.storeValue);
        if (this.storeValue.type === res.type) {
            const map = this._makeMap(this.storeValue.value);
            each(res.value, (i, v) => {
                if (!map[v]) {
                    pushDistinct(self.storeValue.value, v);
                    map[v] = v;
                }
            });
            let change = false;
            each(res.assist, (i, v) => {
                if (isNotNull(map[v])) {
                    change = true;
                    delete map[v];
                }
            });
            change && (this.storeValue.value = values(map));
            self._adjust(callback);

            return;
        }
        this._joinAll(res, callback);
    }

    _setStartValue(value) {
        this._startValue = value;
        this.adapter.setStartValue(value);
    }

    isAllSelected() {
        return this.adapter.isAllSelected();
    }

    resize() {}

    setValue(v) {
        this.storeValue = v || {};
        this._assertValue(this.storeValue);
        this.adapter.setValue(this.storeValue);
        this.trigger.setValue(this.storeValue);
    }

    getValue() {
        return deepClone(this.storeValue);
    }

    populate() {
        this.adapter.populate(...arguments);
        this.trigger.populate(...arguments);
    }
}
