import { MultiTreeSearchPane } from "../multitree/trigger/multi.tree.search.pane";
import { VerticalFillLayout, shortcut, extend, emptyFn, createWidget, nextTick, AbsoluteLayout } from "@/core";
import { Single, Searcher } from "@/base";
import { MultiSelectTreePopup } from "./multiselecttree.popup";

@shortcut()
export class MultiSelectTree extends Single {
    static xtype = "bi.multi_select_tree";

    _constant = { EDITOR_HEIGHT: 24 };

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-multi-select-tree",
            itemsCreator: emptyFn,
            simple: false,
            searcherPaneIsSelectedAny: true,
        });
    }

    _init() {
        super._init(...arguments);
        const self = this,
            o = this.options;
        this.storeValue = { value: {} };

        this.adapter = createWidget({
            type: MultiSelectTreePopup.xtype,
            itemsCreator: o.itemsCreator,
            showLine: o.showLine,
            showIcon: o.showIcon,
        });
        this.adapter.on(MultiSelectTreePopup.EVENT_CHANGE, () => {
            if (self.searcher.isSearching()) {
                self.storeValue = { value: self.searcherPane.getValue() };
            } else {
                self.storeValue = { value: self.adapter.getValue() };
            }
            self.setSelectedValue(self.storeValue.value);
            self.fireEvent(MultiSelectTree.EVENT_CHANGE);
        });

        // 搜索中的时候用的是parttree，同adapter中的synctree不一样
        this.searcherPane = createWidget({
            type: MultiTreeSearchPane.xtype,
            cls: "bi-border-left bi-border-right bi-border-bottom",
            showIcon: o.showIcon,
            isSelectedAny: o.searcherPaneIsSelectedAny,
            keywordGetter() {
                return self.searcher.getKeyword();
            },
            itemsCreator(op, callback) {
                op.keyword = self.searcher.getKeyword();
                o.itemsCreator(op, callback);
            },
        });
        this.searcherPane.setVisible(false);

        this.searcher = createWidget({
            type: Searcher.xtype,
            simple: o.simple,
            isAutoSearch: false,
            isAutoSync: false,
            onSearch(op, callback) {
                callback({
                    keyword: self.searcher.getKeyword(),
                });
            },
            adapter: this.adapter,
            popup: this.searcherPane,
            masker: false,
            listeners: [
                {
                    eventName: Searcher.EVENT_START,
                    action() {
                        self._showSearcherPane();
                        // self.storeValue = {value: self.adapter.getValue()};
                        // self.searcherPane.setSelectedValue(self.storeValue.value);
                    },
                },
                {
                    eventName: Searcher.EVENT_STOP,
                    action() {
                        self._showAdapter();
                        // self.storeValue = {value: self.searcherPane.getValue()};
                        // self.adapter.setSelectedValue(self.storeValue.value);
                        nextTick(() => {
                            self.adapter.populate();
                        });
                    },
                },
                {
                    eventName: Searcher.EVENT_CHANGE,
                    action() {
                        if (self.searcher.isSearching()) {
                            self.storeValue = {
                                value: self.searcherPane.getValue(),
                            };
                        } else {
                            self.storeValue = {
                                value: self.adapter.getValue(),
                            };
                        }
                        self.setSelectedValue(self.storeValue.value);
                        self.fireEvent(MultiSelectTree.EVENT_CHANGE);
                    },
                },
                {
                    eventName: Searcher.EVENT_PAUSE,
                    action() {
                        self._showAdapter();
                        // BI-64732 pause 和stop一致, 都应该刷新adapter
                        nextTick(() => {
                            self.adapter.populate();
                        });
                    },
                }
            ],
        });

        createWidget({
            type: VerticalFillLayout.xtype,
            element: this,
            items: [
                {
                    el: this.searcher,
                    height: "",
                },
                {
                    el: this.adapter,
                    height: "fill",
                }
            ],
        });
        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: this.searcherPane,
                    top: this._constant.EDITOR_HEIGHT,
                    bottom: 0,
                    left: 0,
                    right: 0,
                }
            ],
        });
    }

    _showAdapter() {
        this.adapter.setVisible(true);
        this.searcherPane.setVisible(false);
    }

    _showSearcherPane() {
        this.searcherPane.setVisible(true);
        this.adapter.setVisible(false);
    }

    resize() {}

    setSelectedValue(v) {
        this.storeValue.value = v || {};
        this.adapter.setSelectedValue(v);
        this.searcherPane.setSelectedValue(v);
        this.searcher.setValue({
            value: v || {},
        });
    }

    setValue(v) {
        this.adapter.setValue(v);
    }

    stopSearch() {
        this.searcher.stopSearch();
    }

    updateValue(v) {
        this.adapter.updateValue(v);
    }

    getValue() {
        return this.storeValue.value;
    }

    populate() {
        this.adapter.populate();
    }
}
