import { shortcut, extend, emptyFn, createWidget, i18nText } from "@/core";
import { Pane } from "@/base";
import { AsyncTree, MultiPopupView, TreeView } from "@/case";

@shortcut()
export class MultiTreePopup extends Pane {
    static xtype = "bi.multi_tree_popup_view";

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_CLICK_CONFIRM = "EVENT_CLICK_CONFIRM";
    static EVENT_CLICK_CLEAR = "EVENT_CLICK_CLEAR";
    static EVENT_AFTERINIT = "EVENT_AFTERINIT";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-multi-tree-popup",
            maxWidth: "auto",
            minWidth: 140,
            maxHeight: 400,
            onLoaded: emptyFn,
            el: {
                type: AsyncTree.xtype,
            },
        });
    }

    _init() {
        super._init(...arguments);

        const self = this,
            opts = this.options,
            v = opts.value;

        this.selectedValues = {};

        this.tree = createWidget(opts.el, {
            type: AsyncTree.xtype,
            showLine: opts.showLine,
            height: 400,
            cls: "popup-view-tree",
            itemsCreator: opts.itemsCreator,
            onLoaded: opts.onLoaded,
            value: v.value || {},
        });

        this.popupView = createWidget({
            type: MultiPopupView.xtype,
            element: this,
            stopPropagation: false,
            maxWidth: opts.maxWidth,
            minWidth: opts.minWidth,
            maxHeight: opts.maxHeight,
            buttons: [i18nText("BI-Basic_Clears"), i18nText("BI-Basic_OK")],
            el: this.tree,
        });

        this.popupView.on(MultiPopupView.EVENT_CLICK_TOOLBAR_BUTTON, index => {
            switch (index) {
            case 0:
                self.fireEvent(MultiTreePopup.EVENT_CLICK_CLEAR);
                break;
            case 1:
                self.fireEvent(MultiTreePopup.EVENT_CLICK_CONFIRM);
                break;
            
            default:
                break;
            }
        });

        this.tree.on(TreeView.EVENT_CHANGE, () => {
            self.fireEvent(MultiTreePopup.EVENT_CHANGE);
        });

        this.tree.on(TreeView.EVENT_AFTERINIT, () => {
            self.fireEvent(MultiTreePopup.EVENT_AFTERINIT);
        });
    }

    getValue() {
        return this.tree.getValue();
    }

    setValue(v) {
        v || (v = {});
        this.tree.setSelectedValue(v.value);
    }

    populate(config) {
        this.tree.stroke(config);
    }

    hasChecked() {
        return this.tree.hasChecked();
    }

    setDirection(direction, position) {
        this.popupView.setDirection(direction, position);
    }

    resetHeight(h) {
        this.popupView.resetHeight(h);
    }

    resetWidth(w) {
        this.popupView.resetWidth(w);
    }
}
