import { shortcut, extend, i18nText, Controller } from "@/core";
import { Pane } from "@/base";
import { TreeView, PartTree } from "@/case";

@shortcut()
export class MultiTreeSearchPane extends Pane {
    static xtype = "bi.multi_tree_search_pane";

    props = { 
        baseCls: "bi-multi-tree-search-pane bi-card",
        isSelectedAny: true,
    };

    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_CLICK_CONFIRM = "EVENT_CLICK_CONFIRM";
    static EVENT_CLICK_CLEAR = "EVENT_CLICK_CLEAR";
    static EVENT_CLICK_TREE_NODE = "EVENT_CLICK_TREE_NODE";

    render() {
        const self = this,
            opts = this.options;

        return extend(
            {
                type: PartTree.xtype,
                element: this,
                tipText: i18nText("BI-No_Select"),
                isSelectedAny: opts.isSelectedAny,
                itemsCreator(op, callback) {
                    op.keyword = opts.keywordGetter();
                    opts.itemsCreator(op, callback);
                },
                value: opts.value,
                showIcon: opts.showIcon,
                listeners: [
                    {
                        eventName: Controller.EVENT_CHANGE,
                        action() {
                            self.fireEvent(Controller.EVENT_CHANGE, arguments);
                        },
                    },
                    {
                        eventName: TreeView.EVENT_CHANGE,
                        action() {
                            self.fireEvent(MultiTreeSearchPane.EVENT_CHANGE);
                        },
                    },
                    {
                        eventName: PartTree.EVENT_CLICK_TREE_NODE,
                        action() {
                            self.fireEvent(MultiTreeSearchPane.EVENT_CLICK_TREE_NODE);
                        },
                    }
                ],
                ref(_ref) {
                    self.partTree = _ref;
                },
            },
            opts.el
        );
    }

    hasChecked() {
        return this.partTree.hasChecked();
    }

    setValue(v) {
        this.setSelectedValue(v.value);
    }

    setSelectedValue(v) {
        v || (v = {});
        this.partTree.setSelectedValue(v);
    }

    getValue() {
        return this.partTree.getValue();
    }

    empty() {
        this.partTree.empty();
    }

    populate(op) {
        this.partTree.stroke(...arguments);
    }
}
