import { shortcut, Widget, extend, emptyFn, createWidget, isNotNull, Selection, size, each } from "@/core";
import { Searcher } from "@/base";
import { MultiSelectEditor } from "@/widget/multiselect/trigger/editor.multiselect";
import { MultiSelectSearchPane } from "@/widget/multiselect/search/multiselect.search.pane";

@shortcut()
export class SearchMultiSelectSearcher extends Widget {
    static xtype = "bi.search_multi_select_searcher";

    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";
    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_START = "EVENT_START";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_PAUSE = "EVENT_PAUSE";
    static EVENT_SEARCHING = "EVENT_SEARCHING";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-multi-select-searcher",
            itemsCreator: emptyFn,
            el: {},
            popup: {},
            valueFormatter: emptyFn,
            adapter: null,
            masker: {},
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.editor = createWidget(o.el, {
            type: MultiSelectEditor.xtype,
            height: o.height,
            text: o.text,
            tipType: o.tipType,
            warningTitle: o.warningTitle,
        });

        this.searcher = createWidget({
            type: Searcher.xtype,
            element: this,
            height: o.height,
            isAutoSearch: false,
            isAutoSync: false,
            onSearch: (op, callback) => {
                callback();
            },
            el: this.editor,

            popup: extend(
                {
                    type: MultiSelectSearchPane.xtype,
                    valueFormatter: o.valueFormatter,
                    keywordGetter: () => this.editor.getValue(),
                    itemsCreator: (op, callback) => {
                        const keyword = this.editor.getValue();
                        op.keywords = [keyword];
                        o.itemsCreator(op, callback);
                    },
                    value: o.value,
                },
                o.popup
            ),

            adapter: o.adapter,
            masker: o.masker,
        });
        this.searcher.on(Searcher.EVENT_START, () => {
            this.fireEvent(SearchMultiSelectSearcher.EVENT_START);
        });
        this.searcher.on(Searcher.EVENT_PAUSE, () => {
            this.fireEvent(SearchMultiSelectSearcher.EVENT_PAUSE);
        });
        this.searcher.on(Searcher.EVENT_STOP, () => {
            this.fireEvent(SearchMultiSelectSearcher.EVENT_STOP);
        });
        this.searcher.on(Searcher.EVENT_CHANGE, (...args) => {
            this.fireEvent(SearchMultiSelectSearcher.EVENT_CHANGE, ...args);
        });
        this.searcher.on(Searcher.EVENT_SEARCHING, () => {
            const keywords = this.searcher.getKeywords();
            this.fireEvent(SearchMultiSelectSearcher.EVENT_SEARCHING, keywords);
        });
        if (isNotNull(o.value)) {
            this.setState(o.value);
        }
    }

    adjustView() {
        this.searcher.adjustView();
    }

    isSearching() {
        return this.searcher.isSearching();
    }

    stopSearch() {
        this.searcher.stopSearch();
    }

    getKeyword() {
        return this.editor.getValue();
    }

    hasMatched() {
        return this.searcher.hasMatched();
    }

    hasChecked() {
        return this.searcher.getView() && this.searcher.getView().hasChecked();
    }

    setAdapter(adapter) {
        this.searcher.setAdapter(adapter);
    }

    setState(obj) {
        let state;
        const o = this.options;
        const ob = {};
        ob.type = obj.type;
        ob.value = o.allValueGetter() || [];
        ob.assist = obj.assist;
        if (ob.type === Selection.All) {
            if (ob.value.length === 0) {
                this.editor.setState(Selection.All);
            } else if (size(ob.assist) <= 20) {
                state = "";
                each(ob.assist, (i, v) => {
                    if (i === 0) {
                        state += `${o.valueFormatter(`${v}`) || v}`;
                    } else {
                        state += `,${o.valueFormatter(`${v}`) || v}`;
                    }
                });
                this.editor.setState(state);
            } else {
                this.editor.setState(Selection.Multi);
            }
        } else {
            if (ob.value.length === 0) {
                this.editor.setState(Selection.None);
            } else if (size(ob.value) <= 20) {
                state = "";
                each(ob.value, (i, v) => {
                    if (i === 0) {
                        state += `${o.valueFormatter(`${v}`) || v}`;
                    } else {
                        state += `,${o.valueFormatter(`${v}`) || v}`;
                    }
                });
                this.editor.setState(state);
            } else {
                this.editor.setState(Selection.Multi);
            }
        }
    }

    setTipType(v) {
        this.editor.setTipType(v);
    }

    setValue(ob) {
        this.setState(ob);
        this.searcher.setValue(ob);
    }

    getKey() {
        return this.editor.getValue();
    }

    getValue() {
        return this.searcher.getValue();
    }

    populate(...items) {
        this.searcher.populate(items);
    }
}
