import {
    shortcut,
    Widget,
    extend,
    emptyFn,
    createWidget,
    i18nText,
    isUndefined,
    Controller,
    VerticalLayout,
    map,
    isArray,
    isKey,
    Func,
    SIZE_CONSANTS
} from "@/core";
import { ButtonGroup, Loader } from "@/base";
import { SingleSelectList } from "../singleselect.list";
import { SingleSelectItem, SingleSelectRadioItem } from "@/case";

@shortcut()
export class SingleSelectSearchLoader extends Widget {
    static xtype = "bi.single_select_search_loader";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-search-loader",
            allowNoSelect: false,
            logic: {
                dynamic: false,
            },
            itemsCreator: emptyFn,
            keywordGetter: emptyFn,
            valueFormatter: emptyFn,
        });
    }

    _init() {
        super._init(...arguments);

        const self = this,
            opts = this.options;
        let hasNext = false;

        this.button_group = createWidget({
            type: SingleSelectList.xtype,
            allowNoSelect: opts.allowNoSelect,
            element: this,
            logic: {
                dynamic: false,
            },
            value: opts.value,
            el: {
                tipText: i18nText("BI-No_Select"),
                el: {
                    type: Loader.xtype,
                    isDefaultInit: false,
                    logic: {
                        dynamic: true,
                        scrolly: true,
                    },
                    el: {
                        chooseType: ButtonGroup.CHOOSE_TYPE_SINGLE,
                        behaviors: {
                            redmark() {
                                return true;
                            },
                        },
                        layouts: [
                            {
                                type: VerticalLayout.xtype,
                            }
                        ],
                    },
                },
            },
            itemsCreator(op, callback) {
                self.storeValue &&
                    (op = extend(op || {}, {
                        selectedValues: [self.storeValue],
                    }));
                opts.itemsCreator(op, ob => {
                    const keyword = (ob.keyword = opts.keywordGetter());
                    hasNext = ob.hasNext;
                    let firstItems = [];
                    if (op.times === 1 && !isUndefined(self.storeValue)) {
                        const json = self._filterValues(self.storeValue);
                        firstItems = self._createItems(json);
                    }
                    const context = {
                        tipText: ob.tipText,
                    };
                    callback(firstItems.concat(self._createItems(ob.items)), keyword || "", context);
                    if (op.times === 1 && self.storeValue) {
                        self.setValue(self.storeValue);
                    }
                });
            },
            hasNext() {
                return hasNext;
            },
        });
        this.button_group.on(Controller.EVENT_CHANGE, function () {
            self.fireEvent(Controller.EVENT_CHANGE, arguments);
        });
        this.button_group.on(SingleSelectList.EVENT_CHANGE, function () {
            self.fireEvent(SingleSelectSearchLoader.EVENT_CHANGE, arguments);
        });
    }

    _createItems(items) {
        const o = this.options;

        return map(items, (i, item) =>
            extend(
                {
                    type: o.allowNoSelect ? SingleSelectItem.xtype : SingleSelectRadioItem.xtype,
                    logic: o.logic,
                    cls: "bi-list-item-active",
                    height: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
                    selected: false,
                    iconWrapperWidth: 26,
                    hgap: o.allowNoSelect ? 10 : 0,
                    title: item.title || item.text,
                },
                item
            )
        );
    }

    _filterValues(src) {
        const o = this.options;
        const keyword = o.keywordGetter();
        let values = src || [];
        const newValues = map(isArray(values) ? values : [values], (i, v) => {
            return {
                text: o.valueFormatter(v) || v,
                value: v,
            };
        });
        if (isKey(keyword)) {
            const search = Func.getSearchResult(newValues, keyword);
            values = search.match.concat(search.find);
        }

        return map(values, (i, v) => {
            return {
                text: v.text,
                title: v.text,
                value: v.value,
                selected: false,
            };
        });
    }

    setValue(v) {
        // 暂存的值一定是新的值，不然v改掉后，storeValue也跟着改了
        this.storeValue = v;
        this.button_group.setValue(v);
    }

    getValue() {
        return this.button_group.getValue();
    }

    getAllButtons() {
        return this.button_group.getAllButtons();
    }

    empty() {
        this.button_group.empty();
    }

    populate(items) {
        this.button_group.populate(...arguments);
    }

    resetHeight(h) {
        this.button_group.resetHeight(h);
    }

    resetWidth(w) {
        this.button_group.resetWidth(w);
    }
}
