import {
    shortcut,
    Widget,
    extend,
    emptyFn,
    createWidget,
    i18nText,
    Controller,
    VerticalFillLayout,
    VerticalLayout
} from "@/core";
import { Label } from "@/base";
import { SingleSelectSearchLoader } from "./singleselect.search.loader";

@shortcut()
export class SingleSelectSearchInsertPane extends Widget {
    static xtype = "bi.single_select_search_insert_pane";

    constants = { height: 25, lgap: 10, tgap: 5 };

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-search-pane-insert bi-card",
            allowNoSelect: false,
            itemsCreator: emptyFn,
            valueFormatter: emptyFn,
            keywordGetter: emptyFn,
        });
    }

    _init() {
        super._init(...arguments);
        const self = this,
            o = this.options;

        this.addNotMatchTip = createWidget({
            type: Label.xtype,
            text: i18nText("BI-Basic_Press_Enter_To_Add_Text", ""),
            height: this.constants.height,
            cls: "bi-keyword-red-mark",
            hgap: 5,
        });

        this.loader = createWidget({
            type: SingleSelectSearchLoader.xtype,
            allowNoSelect: o.allowNoSelect,
            keywordGetter: o.keywordGetter,
            valueFormatter: o.valueFormatter,
            itemsCreator(op, callback) {
                o.itemsCreator.apply(self, [
                    op,
                    function (res) {
                        callback(res);
                        self.setKeyword(o.keywordGetter());
                    }
                ]);
            },
            value: o.value,
        });
        this.loader.on(Controller.EVENT_CHANGE, function () {
            self.fireEvent(Controller.EVENT_CHANGE, arguments);
        });

        this.resizer = createWidget({
            type: VerticalFillLayout.xtype,
            rowSize: ["", "fill"],
            element: this,
            items: [
                {
                    type: VerticalLayout.xtype,
                    items: [this.addNotMatchTip],
                    height: this.constants.height,
                },
                {
                    el: this.loader,
                }
            ],
        });
    }

    setKeyword(keyword) {
        this.addNotMatchTip.setText(i18nText("BI-Basic_Press_Enter_To_Add_Text", keyword));
    }

    hasMatched() {
        return false;
    }

    setValue(v) {
        this.loader.setValue(v);
    }

    getValue() {
        return this.loader.getValue();
    }

    empty() {
        this.loader.empty();
    }

    populate(items) {
        this.loader.populate(...arguments);
    }
}
