import { shortcut, Widget, extend, emptyFn, createWidget, i18nText, Controller, VerticalFillLayout } from "@/core";
import { Label } from "@/base";
import { SingleSelectSearchLoader } from "./singleselect.search.loader";

@shortcut()
export class SingleSelectSearchPane extends Widget {
    static xtype = "bi.single_select_search_pane";

    constants = { height: 25, lgap: 10, tgap: 5 };

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-search-pane bi-card",
            allowNoSelect: false,
            itemsCreator: emptyFn,
            valueFormatter: emptyFn,
            keywordGetter: emptyFn,
        });
    }

    _init() {
        super._init(...arguments);
        const self = this,
            o = this.options;

        this.tooltipClick = createWidget({
            type: Label.xtype,
            invisible: true,
            text: i18nText("BI-Click_Blank_To_Select"),
            cls: "single-select-toolbar",
            height: this.constants.height,
        });

        this.loader = createWidget({
            type: SingleSelectSearchLoader.xtype,
            allowNoSelect: o.allowNoSelect,
            keywordGetter: o.keywordGetter,
            valueFormatter: o.valueFormatter,
            itemsCreator(op, callback) {
                o.itemsCreator.apply(self, [
                    op,
                    function (res) {
                        callback(res);
                        self.setKeyword(o.keywordGetter());
                    }
                ]);
            },
            value: o.value,
        });
        this.loader.on(Controller.EVENT_CHANGE, function () {
            self.fireEvent(Controller.EVENT_CHANGE, arguments);
        });

        this.resizer = createWidget({
            type: VerticalFillLayout.xtype,
            rowSize: ["", "fill"],
            element: this,
            items: [
                {
                    el: this.tooltipClick,
                },
                {
                    el: this.loader,
                }
            ],
        });
        this.tooltipClick.setVisible(false);
    }

    setKeyword(keyword) {
        let btn;
        const o = this.options;
        const isVisible =
            this.loader.getAllButtons().length > 0 &&
            (btn = this.loader.getAllButtons()[0]) &&
            keyword === (o.valueFormatter(btn.getValue()) || btn.getValue());
        if (isVisible !== this.tooltipClick.isVisible()) {
            this.tooltipClick.setVisible(isVisible);
            this.resizer.attr("items")[0].height = isVisible ? this.constants.height : 0;
            this.resizer.resize();
        }
    }

    hasMatched() {
        return this.tooltipClick.isVisible();
    }

    setValue(v) {
        this.loader.setValue(v);
    }

    getValue() {
        return this.loader.getValue();
    }

    empty() {
        this.loader.empty();
    }

    populate(items) {
        this.loader.populate(...arguments);
    }
}
