import { SingleSelectSearchInsertPane } from "./search";
import {
    shortcut,
    extend,
    emptyFn,
    i18nText,
    isKey,
    createWidget,
    toPix,
    isNotNull,
    nextTick,
    AbsoluteLayout,
    makeObject
} from "@/core";
import { Single, Combo } from "@/base";
import { SingleSelectTrigger } from "./singleselect.trigger";
import { SingleSelectPopupView } from "./singleselect.popup.view";
import { TriggerIconButton } from "@/case";

@shortcut()
export class SingleSelectInsertCombo extends Single {
    static xtype = "bi.single_select_insert_combo";

    static REQ_GET_DATA_LENGTH = 0;
    static REQ_GET_ALL_DATA = -1;
    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";
    static EVENT_STOP = "EVENT_STOP";
    static EVENT_SEARCHING = "EVENT_SEARCHING";
    static EVENT_CLICK_ITEM = "EVENT_CLICK_ITEM";
    static EVENT_CONFIRM = "EVENT_CONFIRM";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-combo",
            allowNoSelect: false,
            itemsCreator: emptyFn,
            itemWrapper: emptyFn,
            valueFormatter: emptyFn,
            height: 24,
            allowEdit: true,
            watermark: i18nText("BI-Basic_Search_And_Patch_Paste"),
        });
    }

    _init() {
        const self = this,
            o = this.options;
        super._init(...arguments);
        const assertShowValue = () => {
            isKey(this._startValue) && (this.storeValue = this._startValue);
            this.trigger.getSearcher().setState(this.storeValue);
        };
        this.storeValue = o.value;
        // 标记正在请求数据
        this.requesting = false;

        this.trigger = createWidget({
            type: SingleSelectTrigger.xtype,
            watermark: o.watermark,
            height: toPix(o.height, o.simple ? 1 : 2),
            allowNoSelect: o.allowNoSelect,
            allowEdit: o.allowEdit,
            // adapter: this.popup,
            valueFormatter: o.valueFormatter,
            itemsCreator(op, callback) {
                o.itemsCreator(op, function (res) {
                    if (op.times === 1 && isNotNull(op.keywords)) {
                        // 预防trigger内部把当前的storeValue改掉
                        self.trigger.setValue(self.getValue());
                    }
                    callback.apply(self, arguments);
                });
            },
            text: o.text,
            value: this.storeValue,
            searcher: {
                popup: {
                    type: SingleSelectSearchInsertPane.xtype,
                },
            },
        });

        this.trigger.on(SingleSelectTrigger.EVENT_FOCUS, () => {
            self.fireEvent(SingleSelectInsertCombo.EVENT_FOCUS);
        });
        this.trigger.on(SingleSelectTrigger.EVENT_BLUR, () => {
            self.fireEvent(SingleSelectInsertCombo.EVENT_BLUR);
        });

        this.trigger.on(SingleSelectTrigger.EVENT_START, function () {
            self._setStartValue();
            this.getSearcher().setValue(self.storeValue);
        });
        this.trigger.on(SingleSelectTrigger.EVENT_STOP, () => {
            self._setStartValue();
            self.fireEvent(SingleSelectInsertCombo.EVENT_STOP);
        });
        this.trigger.on(SingleSelectTrigger.EVENT_PAUSE, () => {
            self.storeValue = self.trigger.getSearcher().getKeyword();
            assertShowValue();
            self._defaultState();
        });
        this.trigger.on(SingleSelectTrigger.EVENT_SEARCHING, () => {
            self._dataChange = true;
            self.fireEvent(SingleSelectInsertCombo.EVENT_SEARCHING);
        });

        this.trigger.on(SingleSelectTrigger.EVENT_CHANGE, function (value, obj) {
            self.storeValue = this.getValue();
            assertShowValue();
            self._defaultState();
            self._dataChange = true;
        });
        this.trigger.on(SingleSelectTrigger.EVENT_COUNTER_CLICK, () => {
            if (!self.combo.isViewVisible()) {
                self.combo.showView();
            }
        });

        this.combo = createWidget({
            type: Combo.xtype,
            cls: o.simple ? "bi-border-bottom" : "bi-border bi-border-radius",
            container: o.container,
            toggle: false,
            el: this.trigger,
            adjustLength: 1,
            popup: {
                type: SingleSelectPopupView.xtype,
                allowNoSelect: o.allowNoSelect,
                ref() {
                    self.popup = this;
                    self.trigger.setAdapter(this);
                },
                listeners: [
                    {
                        eventName: SingleSelectPopupView.EVENT_CHANGE,
                        action() {
                            self._dataChange = true;
                            self.storeValue = this.getValue();
                            self._adjust(() => {
                                assertShowValue();
                                self._defaultState();
                            });
                            self.fireEvent(SingleSelectInsertCombo.EVENT_CLICK_ITEM);
                        },
                    }
                ],
                itemsCreator: o.itemsCreator,
                itemWrapper: o.itemWrapper,
                valueFormatter: o.valueFormatter,
                onLoaded() {
                    nextTick(() => {
                        self.combo.adjustWidth();
                        self.combo.adjustHeight();
                        self.trigger.getSearcher().adjustView();
                    });
                },
            },
            hideChecker(e) {
                return triggerBtn.element.find(e.target).length === 0;
            },
            value: o.value,
        });

        this.combo.on(Combo.EVENT_BEFORE_POPUPVIEW, function () {
            if (!this.isViewVisible()) {
                self._dataChange = false; // 标记数据是否发生变化
            }
            this.setValue(self.storeValue);
            nextTick(() => {
                self.populate();
            });
        });
        // 当退出的时候如果还在处理请求，则等请求结束后再对外发确定事件
        this.wants2Quit = false;
        this.combo.on(Combo.EVENT_AFTER_HIDEVIEW, () => {
            // important:关闭弹出时又可能没有退出编辑状态
            self.trigger.stopEditing();
            if (self.requesting === true) {
                self.wants2Quit = true;
            } else {
                self._dataChange && self.fireEvent(SingleSelectInsertCombo.EVENT_CONFIRM);
            }
        });

        const triggerBtn = createWidget({
            type: TriggerIconButton.xtype,
            width: o.height,
            height: o.height,
            cls: "single-select-trigger-icon-button",
        });
        triggerBtn.on(TriggerIconButton.EVENT_CHANGE, () => {
            if (self.combo.isViewVisible()) {
                self.combo.hideView();
            } else {
                self.combo.showView();
            }
        });
        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: this.combo,
                    left: 0,
                    right: 0,
                    top: 0,
                    bottom: 0,
                },
                {
                    el: triggerBtn,
                    right: 0,
                    top: 0,
                    bottom: 0,
                }
            ],
        });
    }

    _defaultState() {
        this.trigger.stopEditing();
        this.combo.hideView();
    }

    _assertValue(val) {}

    _makeMap(values) {
        return makeObject(values || []);
    }

    _adjust(callback) {
        const self = this;
        adjust();
        callback();

        function adjust() {
            if (self.wants2Quit === true) {
                self._dataChange && self.fireEvent(SingleSelectInsertCombo.EVENT_CONFIRM);
                self.wants2Quit = false;
            }
            self.requesting = false;
        }
    }

    _setStartValue(value) {
        this._startValue = value;
        this.popup.setStartValue(value);
    }

    setValue(v) {
        this.storeValue = v;
        this._assertValue(this.storeValue);
        this.combo.setValue(this.storeValue);
    }

    getValue() {
        return this.storeValue;
    }

    populate() {
        this.combo.populate(...arguments);
    }
}
