import { shortcut, Widget, extend, emptyFn, createWidget, Controller, Events, i18nText, Direction, LogicFactory, pixFormat } from "@/core";
import { ListPane, SingleSelectItem } from "@/case";

@shortcut()
export class SingleSelectList extends Widget {
    static xtype = "bi.single_select_list";

    _constants = { itemHeight: 24 };

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-select-list",
            direction: Direction.Top, // toolbar的位置
            logic: {
                dynamic: true,
            },
            items: [],
            itemsCreator: emptyFn,
            hasNext: emptyFn,
            onLoaded: emptyFn,
            el: {
                type: ListPane.xtype,
            },
            allowNoSelect: false,
        });
    }

    _init() {
        super._init(...arguments);
        const self = this,
            o = this.options;

        this.list = createWidget(o.el, {
            type: ListPane.xtype,
            items: o.items,
            itemsCreator(op, callback) {
                op.times === 1 && self.toolbar && self.toolbar.setVisible(false);
                o.itemsCreator(op, function (items) {
                    callback.apply(self, arguments);
                    if (op.times === 1) {
                        self.toolbar && self.toolbar.setVisible(items && items.length > 0);
                        self.toolbar && self.toolbar.setEnable(items && items.length > 0);
                    }
                });
            },
            onLoaded: o.onLoaded,
            hasNext: o.hasNext,
            value: o.value,
        });

        this.list.on(Controller.EVENT_CHANGE, function (type, value, obj) {
            if (type === Events.CLICK) {
                self.fireEvent(SingleSelectList.EVENT_CHANGE, value, obj);
            }
            self.fireEvent(Controller.EVENT_CHANGE, arguments);
        });

        createWidget(
            extend(
                {
                    element: this,
                },
                LogicFactory.createLogic(
                    LogicFactory.createLogicTypeByDirection(o.direction),
                    extend(
                        {
                            scrolly: true,
                        },
                        o.logic,
                        {
                            items: o.allowNoSelect
                                ? LogicFactory.createLogicItemsByDirection(
                                    o.direction,
                                    {
                                        type: SingleSelectItem.xtype,
                                        cls: "bi-list-item-active",
                                        height: this._constants.itemHeight,
                                        forceNotSelected: true,
                                        text: i18nText("BI-Basic_No_Select"),
                                        ref(_ref) {
                                            self.toolbar = _ref;
                                        },
                                        listeners: [
                                            {
                                                eventName: Controller.EVENT_CHANGE,
                                                action(type) {
                                                    if (type === Events.CLICK) {
                                                        self.list.setValue();
                                                        self.fireEvent(SingleSelectList.EVENT_CHANGE);
                                                    }
                                                    self.fireEvent(Controller.EVENT_CHANGE, arguments);
                                                },
                                            }
                                        ],
                                    },
                                    this.list
                                )
                                : LogicFactory.createLogicItemsByDirection(o.direction, this.list),
                        }
                    )
                )
            )
        );
    }

    hasPrev() {
        return this.list.hasPrev();
    }

    hasNext() {
        return this.list.hasNext();
    }

    prependItems(items) {
        this.list.prependItems(...arguments);
    }

    addItems(items) {
        this.list.addItems(...arguments);
    }

    setValue(v) {
        this.list.setValue([v]);
    }

    getValue() {
        return this.list.getValue()[0];
    }

    empty() {
        this.list.empty();
    }

    populate(items) {
        this.list.populate(...arguments);
    }

    resetHeight(h) {
        this.list.resetHeight
            ? this.list.resetHeight(h)
            : this.list.element.css({
                "max-height": pixFormat(h - (this.options.allowNoSelect ? this._constants.itemHeight : 0)),
            });
    }

    setNotSelectedValue() {
        this.list.setNotSelectedValue(...arguments);
    }

    getNotSelectedValue() {
        return this.list.getNotSelectedValue();
    }

    getAllButtons() {
        return this.list.getAllButtons();
    }

    getAllLeaves() {
        return this.list.getAllLeaves();
    }

    getSelectedButtons() {
        return this.list.getSelectedButtons();
    }

    getNotSelectedButtons() {
        return this.list.getNotSelectedButtons();
    }

    getIndexByValue(value) {
        return this.list.getIndexByValue(value);
    }

    getNodeById(id) {
        return this.list.getNodeById(id);
    }

    getNodeByValue(value) {
        return this.list.getNodeByValue(value);
    }
}
