import {
    shortcut,
    Widget,
    extend,
    emptyFn,
    createWidget,
    isUndefined,
    map,
    isKey,
    Controller,
    VerticalLayout,
    delay,
    SIZE_CONSANTS
} from "@/core";
import { ButtonGroup, Loader } from "@/base";
import { SingleSelectList } from "./singleselect.list";
import { SingleSelectItem, SingleSelectRadioItem } from "@/case";

@shortcut()
export class SingleSelectLoader extends Widget {
    static xtype = "bi.single_select_loader";

    _constants = { itemVgap: 5 };

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-loader",
            logic: {
                dynamic: true,
            },
            el: {
                height: 400,
            },
            allowNoSelect: false,
            valueFormatter: emptyFn,
            itemsCreator: emptyFn,
            itemWrapper: emptyFn,
            onLoaded: emptyFn,
        });
    }

    _init() {
        super._init(...arguments);

        const self = this,
            opts = this.options;
        let hasNext = false;
        this.storeValue = opts.value;
        this.button_group = createWidget({
            type: SingleSelectList.xtype,
            allowNoSelect: opts.allowNoSelect,
            logic: opts.logic,
            el: extend(
                {
                    onLoaded: opts.onLoaded,
                    el: {
                        type: Loader.xtype,
                        isDefaultInit: false,
                        logic: {
                            dynamic: true,
                            scrolly: true,
                        },
                        el: {
                            chooseType: ButtonGroup.CHOOSE_TYPE_SINGLE,
                            behaviors: {
                                redmark() {
                                    return true;
                                },
                            },
                            layouts: [
                                {
                                    type: VerticalLayout.xtype,
                                }
                            ],
                        },
                    },
                },
                opts.el
            ),
            itemsCreator(op, callback) {
                const startValue = self._startValue;
                !isUndefined(self.storeValue) &&
                    (op = extend(op || {}, {
                        selectedValues: [self.storeValue],
                    }));
                opts.itemsCreator(op, ob => {
                    hasNext = ob.hasNext;
                    let firstItems = [];
                    if (op.times === 1 && !isUndefined(self.storeValue)) {
                        const json = map([self.storeValue], (i, v) => {
                            const txt = opts.valueFormatter(v) || v;

                            return (
                                opts.itemWrapper({
                                    text: txt,
                                    value: v,
                                    title: txt,
                                    selected: true,
                                }) || {
                                    text: txt,
                                    value: v,
                                    title: txt,
                                    selected: true,
                                }
                            );
                        });
                        firstItems = self._createItems(json);
                    }
                    callback(firstItems.concat(self._createItems(ob.items)), ob.keyword || "");
                    if (op.times === 1 && self.storeValue) {
                        isKey(startValue) && (self.storeValue = startValue);
                        self.setValue(self.storeValue);
                    }
                    op.times === 1 && self._scrollToTop();
                });
            },
            hasNext() {
                return hasNext;
            },
            value: this.storeValue,
        });

        createWidget({
            type: VerticalLayout.xtype,
            element: this,
            items: [this.button_group],
            vgap: this._constants.itemVgap,
        });

        this.button_group.on(Controller.EVENT_CHANGE, function () {
            self.fireEvent(Controller.EVENT_CHANGE, arguments);
        });
        this.button_group.on(SingleSelectList.EVENT_CHANGE, function () {
            self.fireEvent(SingleSelectLoader.EVENT_CHANGE, arguments);
        });
    }

    _createItems(items) {
        const o = this.options;

        return map(items, (i, item) =>
            extend(
                {
                    type: o.allowNoSelect ? SingleSelectItem.xtype : SingleSelectRadioItem.xtype,
                    logic: o.logic,
                    cls: "bi-list-item-active",
                    height: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
                    selected: false,
                    iconWrapperWidth: 26,
                    textHgap: o.allowNoSelect ? 10 : 0,
                    title: item.title || item.text,
                },
                item
            )
        );
    }

    _scrollToTop() {
        const self = this;
        delay(() => {
            self.button_group.element.scrollTop(0);
        }, 30);
    }

    _assertValue(val) {}

    setStartValue(v) {
        this._startValue = v;
    }

    setValue(v) {
        this.storeValue = v;
        this._assertValue(this.storeValue);
        this.button_group.setValue(this.storeValue);
    }

    getValue() {
        return this.button_group.getValue();
    }

    getAllButtons() {
        return this.button_group.getAllButtons();
    }

    empty() {
        this.button_group.empty();
    }

    populate(items) {
        this.button_group.populate(...arguments);
    }

    resetHeight(h) {
        this.button_group.resetHeight(h - this._constants.itemVgap * 2);
    }

    resetWidth(w) {
        this.button_group.resetWidth(w);
    }
}
