import { SingleSelectLoader } from "./singleselect.loader";
import { shortcut, Widget, extend, emptyFn, createWidget } from "@/core";
import { MultiPopupView } from "@/case";
import { PopupView } from "@/base";

@shortcut()
export class SingleSelectPopupView extends Widget {
    static xtype = "bi.single_select_popup_view";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-popup-view",
            allowNoSelect: false,
            maxWidth: "auto",
            minWidth: 135,
            maxHeight: 400,
            valueFormatter: emptyFn,
            itemsCreator: emptyFn,
            itemWrapper: emptyFn,
            onLoaded: emptyFn,
        });
    }

    _init() {
        super._init(...arguments);
        const opts = this.options;

        this.loader = createWidget({
            type: SingleSelectLoader.xtype,
            allowNoSelect: opts.allowNoSelect,
            itemsCreator: opts.itemsCreator,
            itemWrapper: opts.itemWrapper,
            valueFormatter: opts.valueFormatter,
            onLoaded: opts.onLoaded,
            value: opts.value,
        });

        this.popupView = createWidget({
            type: PopupView.xtype,
            stopPropagation: false,
            maxWidth: opts.maxWidth,
            minWidth: opts.minWidth,
            maxHeight: opts.maxHeight,
            element: this,
            el: this.loader,
            value: opts.value,
        });

        this.popupView.on(MultiPopupView.EVENT_CHANGE, () => {
            this.fireEvent(SingleSelectPopupView.EVENT_CHANGE);
        });
    }

    setStartValue(v) {
        this.loader.setStartValue(v);
    }

    setValue(v) {
        this.popupView.setValue(v);
    }

    getValue() {
        return this.popupView.getValue();
    }

    populate(items) {
        this.popupView.populate(...arguments);
    }

    resetHeight(h) {
        this.popupView.resetHeight(h);
    }

    resetWidth(w) {
        this.popupView.resetWidth(w);
    }

    setDirection(direction, position) {
        this.popupView.setDirection(direction, position);
    }
}
