import { SingleSelectSearchInsertPane } from "./search";
import {
    shortcut,
    extend,
    emptyFn,
    isKey,
    createWidget,
    isNotEmptyString,
    i18nText,
    deepClone,
    VerticalFillLayout,
    AbsoluteLayout,
    makeObject
} from "@/core";
import { Single, Searcher } from "@/base";
import { SingleSelectLoader } from "./singleselect.loader";
import { SelectPatchEditor } from "../multiselect";
import { SearchEditor } from "../editor";

@shortcut()
export class SingleSelectInsertList extends Single {
    static xtype = "bi.single_select_insert_list";

    static REQ_GET_DATA_LENGTH = 0;
    static REQ_GET_ALL_DATA = -1;
    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-insert-list",
            allowNoSelect: false,
            itemsCreator: emptyFn,
            itemWrapper: emptyFn,
            valueFormatter: emptyFn,
            searcherHeight: 24,
            simple: false,
        });
    }

    _init() {
        super._init(...arguments);

        const self = this,
            o = this.options;
        this.storeValue = o.value;

        const assertShowValue = () => {
            isKey(this._startValue) && (this.storeValue = this._startValue);
            // this.trigger.setValue(this.storeValue);
        };

        this.adapter = createWidget({
            type: SingleSelectLoader.xtype,
            allowNoSelect: o.allowNoSelect,
            cls: "popup-single-select-list bi-border-left bi-border-right bi-border-bottom",
            itemsCreator: o.itemsCreator,
            valueFormatter: o.valueFormatter,
            itemWrapper: o.itemWrapper,
            logic: {
                dynamic: true,
            },
            // onLoaded: o.onLoaded,
            el: {},
            value: o.value,
        });
        this.adapter.on(SingleSelectLoader.EVENT_CHANGE, function () {
            self.storeValue = this.getValue();
            assertShowValue();
            self.fireEvent(SingleSelectInsertList.EVENT_CHANGE);
        });

        this.searcherPane = createWidget({
            type: SingleSelectSearchInsertPane.xtype,
            allowNoSelect: o.allowNoSelect,
            cls: "bi-border-left bi-border-right bi-border-bottom",
            valueFormatter: o.valueFormatter,
            keywordGetter() {
                return self.trigger.getKeyword();
            },
            itemsCreator(op, callback) {
                op.keywords = [self.trigger.getKeyword()];
                if (isNotEmptyString(op.keywords[0])) {
                    this.setKeyword(op.keywords[0]);
                    o.itemsCreator(op, callback);
                }
            },
        });
        this.searcherPane.setVisible(false);

        this.trigger = createWidget({
            type: Searcher.xtype,
            el: {
                type: SelectPatchEditor.xtype,
                el: {
                    type: SearchEditor.xtype,
                    watermark: i18nText("BI-Basic_Search_And_Patch_Paste"),
                    simple: o.simple,
                },
                ref(ref) {
                    self.editor = ref;
                },
                height: o.searcherHeight,
            },
            isAutoSearch: false,
            isAutoSync: false,
            onSearch(op, callback) {
                callback();
            },
            adapter: this.adapter,
            popup: this.searcherPane,
            masker: false,
            value: o.value,
            listeners: [
                {
                    eventName: Searcher.EVENT_START,
                    action() {
                        self._showSearcherPane();
                        self._setStartValue();
                        this.setValue(deepClone(self.storeValue));
                    },
                },
                {
                    eventName: Searcher.EVENT_STOP,
                    action() {
                        self._showAdapter();
                        self._setStartValue();
                        self.adapter.setValue(self.storeValue);
                        // 需要刷新回到初始界面，否则搜索的结果不能放在最前面
                        self.adapter.populate();
                    },
                },
                {
                    eventName: Searcher.EVENT_PAUSE,
                    action() {
                        const keyword = this.getKeyword();
                        self.storeValue = keyword;
                        self._showAdapter();
                        self.adapter.setValue(self.storeValue);
                        self._setStartValue(keyword);
                        assertShowValue();
                        self.adapter.populate();
                        self._setStartValue();
                        self.fireEvent(SingleSelectInsertList.EVENT_CHANGE);
                    },
                },
                {
                    eventName: Searcher.EVENT_CHANGE,
                    action() {
                        self.storeValue = this.getValue();
                        self.fireEvent(SingleSelectInsertList.EVENT_CHANGE);
                    },
                }
            ],
        });

        createWidget({
            type: VerticalFillLayout.xtype,
            rowSize: ["", "fill"],
            element: this,
            items: [
                {
                    el: this.trigger,
                },
                {
                    el: this.adapter,
                }
            ],
        });
        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: this.searcherPane,
                    top: 24,
                    bottom: 0,
                    left: 0,
                    right: 0,
                }
            ],
        });
    }

    _showAdapter() {
        this.adapter.setVisible(true);
        this.searcherPane.setVisible(false);
    }

    _showSearcherPane() {
        this.searcherPane.setVisible(true);
        this.adapter.setVisible(false);
    }

    _defaultState() {
        this.trigger.stopEditing();
    }

    _assertValue() {}

    _makeMap(values) {
        return makeObject(values || []);
    }

    _setStartValue(value) {
        this._startValue = value;
        this.adapter.setStartValue(value);
    }

    isAllSelected() {
        return this.adapter.isAllSelected();
    }

    resize() {
        // this.trigger.getCounter().adjustView();
        // this.trigger.adjustView();
    }

    setValue(v) {
        this.storeValue = v;
        this.adapter.setValue(this.storeValue);
        this.trigger.setValue(this.storeValue);
    }

    getValue() {
        return deepClone(this.storeValue);
    }

    populate() {
        this._count = null;
        this._allData = null;
        this.adapter.populate(...arguments);
        this.trigger.populate(...arguments);
    }
}
