import {
    shortcut,
    Widget,
    extend,
    i18nText,
    createWidget,
    Controller,
    isEmptyString,
    isEmptyArray,
    BlankSplitChar
} from "@/core";
import { StateEditor } from "@/case";
import { SelectPatchEditor } from "../../multiselect";

@shortcut()
export class SingleSelectEditor extends Widget {
    static xtype = "bi.single_select_editor";

    static EVENT_FOCUS = "EVENT_FOCUS";
    static EVENT_BLUR = "EVENT_BLUR";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-select-editor",
            el: {},
            text: i18nText("BI-Basic_Please_Select"),
            watermark: i18nText("BI-Basic_Search"),
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.editor = createWidget(o.el, {
            type: SelectPatchEditor.xtype,
            element: this,
            height: o.height,
            watermark: o.watermark,
            allowBlank: true,
            value: o.value,
            defaultText: o.text,
            text: o.text,
        });

        this.editor.on(Controller.EVENT_CHANGE, (...args) => {
            this.fireEvent(Controller.EVENT_CHANGE, ...args);
        });
        this.editor.on(StateEditor.EVENT_FOCUS, () => {
            this.fireEvent(SingleSelectEditor.EVENT_FOCUS);
        });
        this.editor.on(StateEditor.EVENT_BLUR, () => {
            this.fireEvent(SingleSelectEditor.EVENT_BLUR);
        });
    }

    focus() {
        this.editor.focus();
    }

    blur() {
        this.editor.blur();
    }

    setState(state) {
        this.editor.setState(state);
    }

    setValue(v) {
        this.editor.setValue(v);
    }

    getValue() {
        return this.editor.getValue();
    }

    getKeywords() {
        const val = this.editor.getValue();
        let keywords = val.split(/\u200b\s\u200b/);
        if (isEmptyString(keywords[keywords.length - 1])) {
            keywords = keywords.slice(0, keywords.length - 1);
        }
        if (/\u200b\s\u200b$/.test(val)) {
            return keywords.concat([BlankSplitChar]);
        }

        return keywords;
    }

    getKeyword() {
        const val = this.editor.getValue();
        let keywords = val.split(/\u200b\s\u200b/);
        if (isEmptyString(keywords[keywords.length - 1])) {
            keywords = keywords.slice(0, keywords.length - 1);
        }

        return isEmptyArray(keywords) ? "" : keywords[keywords.length - 1];
    }

    populate(items) {}
}
