import { SliderIconButton } from "./button/iconbutton.slider";
import {
    shortcut,
    createWidget,
    parseFloat,
    toPix,
    Layout,
    VerticalAdaptLayout,
    AbsoluteLayout,
    VerticalLayout,
    HorizontalAutoLayout,
    clamp,
    isNumeric,
    isNull,
    parseInt,
    isNotEmptyString,
    MouseMoveTracker
} from "@/core";
import { Single, Label } from "@/base";

@shortcut()
export class SingleSliderLabel extends Single {
    static xtype = "bi.single_slider_label";
    _constant = {
        EDITOR_WIDTH: 90,
        EDITOR_HEIGHT: 20,
        HEIGHT: 20,
        SLIDER_WIDTH_HALF: 15,
        SLIDER_WIDTH: 30,
        SLIDER_HEIGHT: 30,
        TRACK_HEIGHT: 24,
        TRACK_GAP_HALF: 7,
        TRACK_GAP: 14,
    };
    props = {
        baseCls: "bi-single-slider-label bi-slider-track",
        digit: false,
        unit: "",
        value: "",
        min: 0,
        max: 100,
    };

    static EVENT_CHANGE = "EVENT_CHANGE";

    beforeMount() {
        const { value, min, max } = this.options;
        this.setMinAndMax({
            min,
            max,
        });
        this.setValue(value);
        this.populate();
    }

    render() {
        const o = this.options;
        const c = this._constant;
        this.enable = false;
        this.value = "";

        this.grayTrack = createWidget({
            type: Layout.xtype,
            cls: "gray-track",
            height: 6,
        });
        this.blueTrack = createWidget({
            type: Layout.xtype,
            cls: "blue-track bi-high-light-background",
            height: 6,
        });
        this.track = this._createTrackWrapper();

        this.slider = createWidget({
            type: SliderIconButton.xtype,
        });
        this._draggable(this.slider);
        const sliderVertical = createWidget({
            type: VerticalAdaptLayout.xtype,
            columnSize: ["fill"],
            items: [
                {
                    type: AbsoluteLayout.xtype,
                    items: [
                        {
                            el: this.slider,
                            top: 8,
                        }
                    ],
                    height: c.SLIDER_HEIGHT,
                }
            ],
            hgap: c.SLIDER_WIDTH_HALF,
            height: c.SLIDER_HEIGHT,
        });
        sliderVertical.element.click(e => {
            if (this.enable && this.isEnabled() && sliderVertical.element[0] === e.originalEvent.target) {
                const offset = e.clientX - this.element.offset().left - c.SLIDER_WIDTH_HALF;
                const trackLength = this.track.element[0].scrollWidth - c.TRACK_GAP;
                let percent = 0;
                if (offset < 0) {
                    percent = 0;
                }
                if (offset > 0 && offset < trackLength) {
                    percent = (offset * 100) / this._getGrayTrackLength();
                }
                if (offset >= trackLength) {
                    percent = 100;
                }
                const significantPercent = parseFloat(percent.toFixed(1));
                this._setAllPosition(significantPercent);
                let v = this._getValueByPercent(significantPercent);
                v = o.digit === false ? v : v.toFixed(o.digit);
                this.label.setText(v + o.unit);
                this.value = v;
                this.fireEvent(SingleSliderLabel.EVENT_CHANGE);
            }
        });
        this.label = createWidget({
            type: Label.xtype,
            height: c.HEIGHT,
            width: toPix(c.EDITOR_WIDTH, 2),
        });

        this._setVisible(false);

        return {
            type: AbsoluteLayout.xtype,
            items: [
                {
                    el: {
                        type: VerticalLayout.xtype,
                        items: [
                            {
                                type: AbsoluteLayout.xtype,
                                items: [
                                    {
                                        el: this.track,
                                        width: "100%",
                                        height: c.TRACK_HEIGHT,
                                    }
                                ],
                            }
                        ],
                        hgap: c.TRACK_GAP_HALF,
                        height: c.TRACK_HEIGHT,
                    },
                    top: 13,
                    left: 0,
                    width: "100%",
                },
                {
                    el: sliderVertical,
                    top: 10,
                    left: 0,
                    width: "100%",
                },
                {
                    el: {
                        type: VerticalLayout.xtype,
                        items: [
                            {
                                type: HorizontalAutoLayout.xtype,
                                items: [this.label],
                            }
                        ],
                        height: c.EDITOR_HEIGHT,
                    },
                    top: 0,
                    left: 0,
                    width: "100%",
                }
            ],
        };
    }

    _draggable(widget) {
        const o = this.options;
        let startDrag = false;
        let size = 0,
            offset = 0,
            defaultSize = 0;
        const mouseMoveTracker = new MouseMoveTracker(
            deltaX => {
                if (mouseMoveTracker.isDragging()) {
                    startDrag = true;
                    offset += deltaX;
                    size = optimizeSize(defaultSize + offset);
                    widget.element.addClass("dragging");
                    const percent = (size * 100) / this._getGrayTrackLength();
                    const significantPercent = parseFloat(percent.toFixed(1)); // 直接对计算出来的百分数保留到小数点后一位，相当于分成了1000份。
                    this._setBlueTrack(significantPercent);
                    this._setLabelPosition(significantPercent);
                    this._setSliderPosition(significantPercent);
                    let v = this._getValueByPercent(significantPercent);
                    v = o.digit === false ? v : v.toFixed(o.digit);
                    this.label.setValue(v + o.unit);
                    this.value = v;
                    this.fireEvent(SingleSliderLabel.EVENT_CHANGE);
                }
            },
            () => {
                if (startDrag === true) {
                    size = optimizeSize(size);
                    const percent = (size * 100) / this._getGrayTrackLength();
                    const significantPercent = parseFloat(percent.toFixed(1));
                    this._setSliderPosition(significantPercent);
                    size = 0;
                    offset = 0;
                    defaultSize = size;
                    startDrag = false;
                }
                widget.element.removeClass("dragging");
                mouseMoveTracker.releaseMouseMoves();
                this.fireEvent(SingleSliderLabel.EVENT_CHANGE);
            },
            window
        );
        widget.element.on("mousedown", function (event) {
            if (!widget.isEnabled()) {
                return;
            }
            defaultSize = this.offsetLeft;
            optimizeSize(defaultSize);
            mouseMoveTracker.captureMouseMoves(event);
        });

        const optimizeSize = s => clamp(s, 0, this._getGrayTrackLength());
    }

    _createTrackWrapper() {
        return createWidget({
            type: AbsoluteLayout.xtype,
            items: [
                {
                    el: {
                        type: VerticalLayout.xtype,
                        items: [
                            {
                                type: AbsoluteLayout.xtype,
                                items: [
                                    {
                                        el: this.grayTrack,
                                        top: 0,
                                        left: 0,
                                        width: "100%",
                                    },
                                    {
                                        el: this.blueTrack,
                                        top: 0,
                                        left: 0,
                                        width: "0%",
                                    }
                                ],
                            }
                        ],
                        hgap: 8,
                        height: 8,
                    },
                    top: 8,
                    left: 0,
                    width: "100%",
                }
            ],
        });
    }

    _checkValidation(v) {
        return isNumeric(v) && !(isNull(v) || v < this.min || v > this.max);
    }

    _setBlueTrack(percent) {
        this.blueTrack.element.css({ width: `${percent}%` });
    }

    _setLabelPosition(percent) {
        // this.label.element.css({left: percent + "%"});
    }

    _setSliderPosition(percent) {
        this.slider.element.css({ left: `${percent}%` });
    }

    _setAllPosition(percent) {
        this._setSliderPosition(percent);
        this._setLabelPosition(percent);
        this._setBlueTrack(percent);
    }

    _setVisible(visible) {
        this.slider.setVisible(visible);
        this.label.setVisible(visible);
    }

    _getGrayTrackLength() {
        return this.grayTrack.element[0].scrollWidth;
    }

    _getValueByPercent(percent) {
        const thousandth = parseInt(percent * 10);

        return ((this.max - this.min) * thousandth) / 1000 + this.min;
    }

    _getPercentByValue(v) {
        return ((v - this.min) * 100) / (this.max - this.min);
    }

    _setEnable(b) {
        super._setEnable.apply(this, [b]);
        if (b) {
            this.blueTrack.element.removeClass("disabled-blue-track").addClass("blue-track");
        } else {
            this.blueTrack.element.removeClass("blue-track").addClass("disabled-blue-track");
        }
    }

    getValue() {
        return this.value;
    }

    setValue(v) {
        const o = this.options;
        v = parseFloat(v);
        v = o.digit === false ? v : v.toFixed(o.digit);
        if (!isNaN(v)) {
            if (this._checkValidation(v)) {
                this.value = v;
            }
            if (v > this.max) {
                this.value = this.max;
            }
            if (v < this.min) {
                this.value = this.min;
            }
        }
    }

    setMinAndMax(v) {
        const minNumber = parseFloat(v.min);
        const maxNumber = parseFloat(v.max);
        if (!isNaN(minNumber) && !isNaN(maxNumber) && maxNumber > minNumber) {
            this.min = minNumber;
            this.max = maxNumber;
        }
    }

    reset() {
        this._setVisible(false);
        this.enable = false;
        this.value = "";
        this.min = 0;
        this.max = 0;
        this._setBlueTrack(0);
    }

    populate() {
        const o = this.options;
        if (!isNaN(this.min) && !isNaN(this.max)) {
            this._setVisible(true);
            this.enable = true;
            if (isNumeric(this.value) || isNotEmptyString(this.value)) {
                this.label.setValue(this.value + o.unit);
                this._setAllPosition(this._getPercentByValue(this.value));
            } else {
                this.label.setValue(this.max + o.unit);
                this._setAllPosition(100);
            }
        }
    }
}
