import { shortcut, extend, i18nText, createWidget, Controller, isArray, VerticalLayout } from "@/core";
import { Pane } from "@/base";
import { LevelTree } from "@/case";

@shortcut()
export class SingleTreePopup extends Pane {
    static xtype = "bi.single_level_tree";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-single-level-tree",
            tipText: i18nText("BI-No_Selected_Item"),
            items: [],
            value: "",
        });
    }

    _init() {
        super._init(...arguments);

        const self = this,
            o = this.options;

        this.tree = createWidget({
            type: LevelTree.xtype,
            expander: {
                isDefaultInit: true,
            },
            items: o.items,
            value: o.value,
            chooseType: Selection.Single,
        });

        createWidget({
            type: VerticalLayout.xtype,
            element: this,
            vgap: 5,
            items: [this.tree],
        });

        this.tree.on(Controller.EVENT_CHANGE, function () {
            self.fireEvent(Controller.EVENT_CHANGE, arguments);
        });

        this.tree.on(LevelTree.EVENT_CHANGE, () => {
            self.fireEvent(SingleTreePopup.EVENT_CHANGE);
        });

        this.check();
    }

    getValue() {
        return this.tree.getValue();
    }

    setValue(v) {
        v = isArray(v) ? v : [v];
        this.tree.setValue(v);
    }

    populate(items) {
        super.populate(...arguments);
        this.tree.populate(items);
    }
}
