import { DownListSelectTextTrigger } from "./trigger.textvaluedownlist";
import {
    shortcut,
    Widget,
    extend,
    isNotNull,
    createWidget,
    Selection,
    toPix,
    isNull,
    deepClone,
    each,
    flatten,
    has,
} from "@/core";
import { DownListCombo } from "../downlist";

@shortcut()
export class TextValueDownListCombo extends Widget {
    static xtype = "bi.text_value_down_list_combo";

    static EVENT_CHANGE = "EVENT_CHANGE";

    _defaultConfig(config) {
        return extend(super._defaultConfig(...arguments), {
            baseCls:
                "bi-text-value-down-list-combo " + (config.simple ? "bi-border-bottom" : "bi-border bi-border-radius"),
            height: 24,
        });
    }

    _init() {
        const o = this.options;
        super._init(...arguments);
        this._createValueMap();

        let value;
        if (isNotNull(o.value)) {
            value = this._digest(o.value);
        }

        this.combo = createWidget({
            type: DownListCombo.xtype,
            element: this,
            chooseType: Selection.Single,
            adjustLength: 2,
            width: toPix(o.width, 2),
            height: toPix(o.height, 2),
            el: {
                type: DownListSelectTextTrigger.xtype,
                ref: _ref => {
                    this.trigger = _ref;
                },
                cls: "text-value-down-list-trigger",
                height: toPix(o.height, 2),
                items: o.items,
                text: o.text,
                value,
            },
            value: isNull(value) ? [] : [value],
            items: deepClone(o.items),
        });

        this.combo.on(DownListCombo.EVENT_CHANGE, () => {
            const currentVal = this.combo.getValue()[0].value;
            if (currentVal !== this.value) {
                this.setValue(currentVal);
                this.fireEvent(TextValueDownListCombo.EVENT_CHANGE);
            }
        });

        this.combo.on(DownListCombo.EVENT_SON_VALUE_CHANGE, () => {
            const currentVal = this.combo.getValue()[0].childValue;
            if (currentVal !== this.value) {
                this.setValue(currentVal);
                this.fireEvent(TextValueDownListCombo.EVENT_CHANGE);
            }
        });
    }

    _createValueMap() {
        this.valueMap = {};
        each(flatten(this.options.items), (idx, item) => {
            if (has(item, "el")) {
                each(item.children, (id, it) => {
                    this.valueMap[it.value] = { value: item.el.value, childValue: it.value };
                });
            } else {
                this.valueMap[item.value] = { value: item.value };
            }
        });
    }

    _digest(v) {
        this.value = v;

        return this.valueMap[v];
    }

    setValue(v) {
        v = this._digest(v);
        this.combo.setValue([v]);
        this.trigger?.setValue(v);
    }

    getValue() {
        const v = this.combo.getValue()[0];

        return [v.childValue || v.value];
    }

    populate(items) {
        this.options.items = flatten(items);
        this.combo.populate(items);
        this._createValueMap();
    }
}
