import { shortcut, extend, createWidget, isNull, flatten, deepClone, each, has, concat } from "@/core";
import { Trigger } from "@/base";
import { SelectTextTrigger } from "@/case";

@shortcut()
export class DownListSelectTextTrigger extends Trigger {
    static xtype = "bi.down_list_select_text_trigger";

    _defaultConfig() {
        return extend(super._defaultConfig(...arguments), {
            baseCls: "bi-down-list-select-text-trigger",
            height: 24,
            text: "",
        });
    }

    _init() {
        super._init(...arguments);
        const o = this.options;
        this.trigger = createWidget({
            type: SelectTextTrigger.xtype,
            element: this,
            height: o.height,
            items: this._formatItemArray(o.items),
            text: o.text,
            value: isNull(o.value) ? "" : o.value.childValue || o.value.value,
        });
    }

    _formatItemArray() {
        const sourceArray = flatten(deepClone(this.options.items));
        let targetArray = [];
        each(sourceArray, (idx, item) => {
            if (has(item, "el")) {
                each(item.children, (id, it) => {
                    it.text = `${item.el.text}(${it.text})`;
                });
                targetArray = concat(targetArray, item.children);
            } else {
                targetArray.push(item);
            }
        });

        return targetArray;
    }

    setValue(vals) {
        this.trigger.setValue(vals.childValue || vals.value);
    }

    populate(items) {
        this.trigger.populate(this._formatItemArray(items));
    }
}
