import {
    HorizontalFillLayout,
    shortcut,
    toPix,
    getDate,
    isNotEmptyString,
    isEqual,
    isEmptyString,
    AbsoluteLayout
} from "@/core";
import { TimeTrigger } from "./time.trigger";
import { Single, IconButton, Combo } from "@/base";
import { TimePopup } from "./datetime.popup";

@shortcut()
export class TimeCombo extends Single {
    static xtype = "bi.time_combo";

    static EVENT_KEY_DOWN = "EVENT_KEY_DOWN";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_CHANGE = "EVENT_CHANGE";
    static EVENT_VALID = "EVENT_VALID";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";

    constants = {
        popupHeight: 80,
        popupWidth: 240,
        comboAdjustHeight: 1,
        border: 1,
        iconWidth: 24,
    };

    props = {
        baseCls: "bi-time-combo",
        height: 24,
        format: "",
        allowEdit: false,
        isNeedAdjustHeight: false,
        isNeedAdjustWidth: false,
    };

    _init() {
        super._init(...arguments);
    }

    render() {
        const opts = this.options;
        this.storeTriggerValue = "";
        this.storeValue = opts.value;

        const popup = {
            type: TimePopup.xtype,
            value: opts.value,
            listeners: [
                {
                    eventName: TimePopup.BUTTON_CLEAR_EVENT_CHANGE,
                    action: () => {
                        this.setValue();
                        this.hidePopupView();
                        this.fireEvent(TimeCombo.EVENT_CONFIRM);
                    },
                },
                {
                    eventName: TimePopup.BUTTON_OK_EVENT_CHANGE,
                    action: () => {
                        this.setValue(this.popup.getValue());
                        this.hidePopupView();
                        this.fireEvent(TimeCombo.EVENT_CONFIRM);
                    },
                },
                {
                    eventName: TimePopup.BUTTON_NOW_EVENT_CHANGE,
                    action: () => {
                        this._setNowTime();
                    },
                }
            ],
            ref: _ref => {
                this.popup = _ref;
            },
        };

        return {
            type: AbsoluteLayout.xtype,
            items: [
                {
                    el: {
                        type: Combo.xtype,
                        cls: "bi-border bi-border-radius",
                        container: opts.container,
                        toggle: false,
                        isNeedAdjustHeight: opts.isNeedAdjustHeight,
                        isNeedAdjustWidth: opts.isNeedAdjustWidth,
                        el: {
                            type: HorizontalFillLayout.xtype,
                            columnSize: ["fill", this.constants.iconWidth],
                            height: toPix(opts.height, 2),
                            items: [
                                {
                                    type: TimeTrigger.xtype,
                                    height: toPix(opts.height, 2),
                                    allowEdit: opts.allowEdit,
                                    watermark: opts.watermark,
                                    format: opts.format,
                                    value: opts.value,
                                    ref: _ref => {
                                        this.trigger = _ref;
                                    },
                                    listeners: [
                                        {
                                            eventName: "EVENT_KEY_DOWN",
                                            action: () => {
                                                if (this.combo.isViewVisible()) {
                                                    this.combo.hideView();
                                                }
                                                this.fireEvent(TimeCombo.EVENT_KEY_DOWN, arguments);
                                            },
                                        },
                                        {
                                            eventName: "EVENT_STOP",
                                            action: () => {
                                                if (!this.combo.isViewVisible()) {
                                                    this.combo.showView();
                                                }
                                            },
                                        },
                                        {
                                            eventName: "EVENT_FOCUS",
                                            action: () => {
                                                this.storeTriggerValue = this.trigger.getKey();
                                                if (!this.combo.isViewVisible()) {
                                                    this.combo.showView();
                                                }
                                                this.fireEvent("EVENT_FOCUS");
                                            },
                                        },
                                        {
                                            eventName: "EVENT_BLUR",
                                            action: () => {
                                                this.fireEvent("EVENT_BLUR");
                                            },
                                        },
                                        {
                                            eventName: "EVENT_ERROR",
                                            action: () => {
                                                const date = getDate();
                                                this.storeValue = {
                                                    hour: date.getHours(),
                                                    minute: date.getMinutes(),
                                                    second: date.getSeconds(),
                                                };
                                                this.fireEvent("EVENT_ERROR");
                                            },
                                        },
                                        {
                                            eventName: "EVENT_VALID",
                                            action: () => {
                                                this.fireEvent("EVENT_VALID");
                                            },
                                        },
                                        {
                                            eventName: "EVENT_CHANGE",
                                            action: () => {
                                                this.fireEvent("EVENT_CHANGE");
                                            },
                                        },
                                        {
                                            eventName: "EVENT_CONFIRM",
                                            action: () => {
                                                if (this.combo.isViewVisible()) {
                                                    return;
                                                }
                                                const dateStore = this.storeTriggerValue;
                                                const dateObj = this.trigger.getKey();
                                                if (isNotEmptyString(dateObj) && !isEqual(dateObj, dateStore)) {
                                                    this.storeValue = this.trigger.getValue();
                                                    this.setValue(this.trigger.getValue());
                                                } else if (isEmptyString(dateObj)) {
                                                    this.storeValue = null;
                                                    this.trigger.setValue();
                                                }
                                                this.fireEvent("EVENT_CONFIRM");
                                            },
                                        }
                                    ],
                                },
                                {
                                    el: {
                                        type: IconButton.xtype,
                                        cls: "bi-trigger-icon-button time-font",
                                        width: this.constants.iconWidth,
                                        listeners: [
                                            {
                                                eventName: IconButton.EVENT_CHANGE,
                                                action: () => {
                                                    if (this.combo.isViewVisible()) {
                                                        // this.combo.hideView();
                                                    } else {
                                                        this.combo.showView();
                                                    }
                                                },
                                            }
                                        ],
                                        ref: _ref => {
                                            this.triggerBtn = _ref;
                                        },
                                    },
                                }
                            ],
                        },
                        adjustLength: this.constants.comboAdjustHeight,
                        popup: {
                            el: popup,
                            width: opts.isNeedAdjustWidth ? opts.width : this.constants.popupWidth,
                            stopPropagation: false,
                        },
                        hideChecker: e => this.triggerBtn.element.find(e.target).length === 0,
                        listeners: [
                            {
                                eventName: Combo.EVENT_BEFORE_POPUPVIEW,
                                action: () => {
                                    this.popup.setValue(this.storeValue);
                                    this.fireEvent(TimeCombo.EVENT_BEFORE_POPUPVIEW);
                                },
                            }
                        ],
                        ref: _ref => {
                            this.combo = _ref;
                        },
                    },
                    top: 0,
                    left: 0,
                    right: 0,
                    bottom: 0,
                }
            ],
        };
    }

    setValue(v) {
        this.storeValue = v;
        this.trigger.setValue(v);
    }

    getValue() {
        return this.storeValue;
    }

    hidePopupView() {
        this.combo.hideView();
    }

    _setNowTime() {
        const date = getDate();
        const nowTome = {
            hour: date.getHours(),
            minute: date.getMinutes(),
            second: date.getSeconds(),
        };
        this.setValue(nowTome);
        this.hidePopupView();
        this.fireEvent(TimeCombo.EVENT_CONFIRM);
    }

    focus() {
        this.trigger.focus();
    }

    blur() {
        this.trigger.blur();
    }

    setWaterMark(v) {
        this.trigger.setWaterMark(v);
    }
}
