import { checkDateVoid, i18nText, isNotEmptyString, parseDateTime, shortcut, VerticalLayout, Widget, SIZE_CONSANTS } from "@/core";
import { Bubbles, Label } from "@/base";
import { DynamicDateCard } from "@/widget/dynamicdate/dynamicdate.card";
import { DynamicDateHelper } from "@/widget/dynamicdate/dynamicdate.caculate";
import { DynamicDateParamItem } from "@/widget/dynamicdate/dynamicdate.param.item";

/**
 * 年份展示面板
 *
 * Created by GUY on 2015/9/2.
 * @class YearCard
 * @extends Trigger
 */

@shortcut()
export class DynamicYearCard extends Widget {
    static xtype = "bi.dynamic_year_card";
    static EVENT_CHANGE = "EVENT_CHANGE";
    props = {
        baseCls: "bi-year-card",
    };
    render() {
        return {
            type: VerticalLayout.xtype,
            ref: _ref => {
                this.wrapper = _ref;
            },
            items: [
                {
                    type: Label.xtype,
                    text: i18nText("BI-Multi_Date_Relative_Current_Time"),
                    textAlign: "left",
                    height: SIZE_CONSANTS.LIST_ITEM_HEIGHT,
                },
                {
                    type: DynamicDateParamItem.xtype,
                    ref: _ref => {
                        this.item = _ref;
                    },
                    listeners: [
                        {
                            eventName: "EVENT_CHANGE",
                            action: () => {
                                this.fireEvent("EVENT_CHANGE");
                            },
                        },
                        {
                            eventName: "EVENT_INPUT_CHANGE",
                            action: () => {
                                Bubbles.hide("dynamic-year-error");
                            },
                        }
                    ],
                }
            ],
            vgap: 10,
            hgap: 10,
        };
    }
    _checkDate() {
        const o = this.options;
        const date = DynamicDateHelper.getCalculation(this._getValue());

        return !checkDateVoid(date.getFullYear(), date.getMonth() + 1, date.getDate(), o.min, o.max)[0];
    }

    _createValue(type, v) {
        return {
            dateType: type,
            value: Math.abs(v),
            offset: v > 0 ? 1 : 0,
        };
    }

    _getErrorText() {
        const o = this.options;
        const start = parseDateTime(o.min, "%Y-%X-%d");
        const end = parseDateTime(o.max, "%Y-%X-%d");

        return i18nText("BI-Basic_Year_Range_Error", start.getFullYear(), end.getFullYear());
    }

    setMinDate(minDate) {
        if (isNotEmptyString(this.options.min)) {
            this.options.min = minDate;
        }
    }

    setMaxDate(maxDate) {
        if (isNotEmptyString(this.options.max)) {
            this.options.max = maxDate;
        }
    }

    setValue(v) {
        v = v || { year: 0 };
        this.item.setValue(this._createValue(DynamicDateCard.TYPE.YEAR, v.year));
    }

    _getValue() {
        const value = this.item.getValue();

        return {
            year: value.offset === 0 ? -value.value : +value.value,
        };
    }

    getInputValue() {
        return this._getValue();
    }

    getValue() {
        return this.checkValidation() ? this._getValue() : {};
    }

    checkValidation(show) {
        let errorText;
        let invalid = !this.item.checkValidation();
        if (invalid) {
            errorText = i18nText("BI-Please_Input_Natural_Number");
        } else {
            invalid = !this._checkDate(this._getValue());
            errorText = this._getErrorText();
        }
        invalid && show && Bubbles.show("dynamic-year-error", errorText, this.item, {});

        return !invalid;
    }
}
