import { IconButton, Combo } from "@/base";
import { shortcut, Widget, toPix, getDate, isNotNull, AbsoluteLayout, HorizontalFillLayout, extend, createWidget } from "@/core";
import { DynamicYearTrigger } from "@/widget/year/trigger.year";
import { DynamicYearPopup } from "@/widget/year/popup.year";
import { DynamicDateCombo } from "@/widget/dynamicdate/dynamicdate.combo";

@shortcut()
export class DynamicYearCombo extends Widget {
    static xtype = "bi.dynamic_year_combo";
    static EVENT_CONFIRM = "EVENT_CONFIRM";
    static EVENT_BEFORE_POPUPVIEW = "EVENT_BEFORE_POPUPVIEW";
    static EVENT_ERROR = "EVENT_ERROR";
    static EVENT_VALID = "EVENT_VALID";
    static EVENT_FOCUS = "EVENT_FOCUS";
    
    static Static = 1;
    static Dynamic = 2;
    
    _const = {
        iconWidth: 24,
    };

    props = {
        baseCls: "bi-year-combo",
        behaviors: {},
        minDate: "1900-01-01", // 最小日期
        maxDate: "2099-12-31", // 最大日期
        height: 24,
        supportDynamic: true,
        isPreview: false, // 是否是预览状态
    };

    _init() {
        const o = this.options;
        super._init(...arguments);
        this.storeValue = o.value;
        const border = o.simple ? 1 : 2;
        this.trigger = createWidget({
            type: DynamicYearTrigger.xtype,
            simple: o.simple,
            min: o.minDate,
            max: o.maxDate,
            height: toPix(o.height, border),
            value: o.value || "",
            watermark: o.watermark,
        });
        this.trigger.on(DynamicYearTrigger.EVENT_KEY_DOWN, () => {
            if (this.combo.isViewVisible()) {
                this.combo.hideView();
            }
        });
        this.trigger.on(DynamicYearTrigger.EVENT_FOCUS, () => {
            this.storeTriggerValue = this.getKey();
            this.fireEvent(DynamicYearCombo.EVENT_FOCUS);
        });
        this.trigger.on(DynamicYearTrigger.EVENT_START, () => {
            this.combo.isViewVisible() && this.combo.hideView();
        });
        this.trigger.on(DynamicYearTrigger.EVENT_STOP, () => {
            this.combo.showView();
        });
        this.trigger.on(DynamicYearTrigger.EVENT_ERROR, () => {
            this.combo.isViewVisible() && this.combo.hideView();
            this.comboWrapper.element.addClass("error");
            this.fireEvent(DynamicYearCombo.EVENT_ERROR);
        });
        this.trigger.on(DynamicYearTrigger.EVENT_VALID, () => {
            this.comboWrapper.element.removeClass("error");
            this.fireEvent(DynamicYearCombo.EVENT_VALID);
        });
        this.trigger.on(DynamicYearTrigger.EVENT_CONFIRM, () => {
            if (this.combo.isViewVisible()) {
                return;
            }
            if (this.getKey() && this.getKey() !== this.storeTriggerValue) {
                this.storeValue = this.trigger.getValue();
                this.setValue(this.storeValue);
            } else if (!this.getKey()) {
                this.storeValue = null;
                this.setValue();
            }
            this._checkDynamicValue(this.storeValue);
            this.fireEvent(DynamicYearCombo.EVENT_CONFIRM);
        });
        this.combo = createWidget({
            type: Combo.xtype,
            container: o.container,
            isNeedAdjustHeight: false,
            isNeedAdjustWidth: false,
            el: this.trigger,
            destroyWhenHide: true,
            adjustLength: 1,
            popup: {
                minWidth: 85,
                stopPropagation: false,
                el: {
                    type: DynamicYearPopup.xtype,
                    supportDynamic: o.supportDynamic,
                    isPreview: o.isPreview,
                    ref: _ref => {
                        this.popup = _ref;
                    },
                    listeners: [
                        {
                            eventName: DynamicYearPopup.EVENT_CHANGE,
                            action: () => {
                                this.setValue(this.popup.getValue());
                                this.combo.hideView();
                                this.fireEvent(DynamicYearCombo.EVENT_CONFIRM);
                            },
                        },
                        {
                            eventName: DynamicYearPopup.BUTTON_CLEAR_EVENT_CHANGE,
                            action: () => {
                                this.setValue();
                                this.combo.hideView();
                                this.fireEvent(DynamicYearCombo.EVENT_CONFIRM);
                            },
                        },
                        {
                            eventName: DynamicYearPopup.BUTTON_lABEL_EVENT_CHANGE,
                            action: () => {
                                const date = getDate();
                                this.setValue({ type: DynamicYearCombo.Static, value: { year: date.getFullYear() } });
                                this.combo.hideView();
                                this.fireEvent(DynamicDateCombo.EVENT_CONFIRM);
                            },
                        },
                        {
                            eventName: DynamicYearPopup.BUTTON_OK_EVENT_CHANGE,
                            action: () => {
                                this.setValue(this.popup.getValue());
                                this.combo.hideView();
                                this.fireEvent(DynamicDateCombo.EVENT_CONFIRM);
                            },
                        }
                    ],
                    behaviors: o.behaviors,
                    min: o.minDate,
                    max: o.maxDate,
                },
                value: o.value || "",
            },
        });
        this.combo.on(Combo.EVENT_BEFORE_POPUPVIEW, () => {
            this.popup.setMinDate(o.minDate);
            this.popup.setMaxDate(o.maxDate);
            this.popup.setValue(this.storeValue);
            this.fireEvent(DynamicYearCombo.EVENT_BEFORE_POPUPVIEW);
        });

        createWidget({
            type: AbsoluteLayout.xtype,
            element: this,
            items: [
                {
                    el: {
                        type: HorizontalFillLayout.xtype,
                        columnSize: ["", "fill"],
                        cls: `${o.simple ? "bi-border-bottom" : "bi-border bi-border-radius"} bi-focus-shadow`,
                        ref: _ref => {
                            this.comboWrapper = _ref;
                        },
                        items: [
                            {
                                el: {
                                    type: IconButton.xtype,
                                    cls: "bi-trigger-icon-button date-change-h-font",
                                    width: this._const.iconWidth,
                                    height: toPix(o.height, border),
                                    ref: _ref => {
                                        this.changeIcon = _ref;
                                    },
                                },
                            },
                            this.combo
                        ],
                    },
                    top: 0,
                    left: 0,
                    right: 0,
                    bottom: 0,
                }
            ],
        });
        this._checkDynamicValue(o.value);
    }

    _checkDynamicValue(v) {
        let type = null;
        if (isNotNull(v)) {
            type = v.type;
        }
        switch (type) {
            case DynamicYearCombo.Dynamic:
                this.changeIcon.setVisible(true);
                break;
            default:
                this.changeIcon.setVisible(false);
                break;
        }
    }

    setMinDate(minDate) {
        const o = this.options;
        o.minDate = minDate;
        this.trigger.setMinDate(minDate);
        this.popup && this.popup.setMinDate(minDate);
    }

    setMaxDate(maxDate) {
        const o = this.options;
        o.maxDate = maxDate;
        this.trigger.setMaxDate(maxDate);
        this.popup && this.popup.setMaxDate(maxDate);
    }

    hideView() {
        this.combo.hideView();
    }

    getKey() {
        return `${this.trigger.getKey()}`;
    }

    setValue(v) {
        this.storeValue = v;
        this.trigger.setValue(v);
        this._checkDynamicValue(v);
    }

    getValue() {
        return this.storeValue;
    }

    isStateValid() {
        return this.trigger.isValid();
    }

    setWaterMark(v) {
        this.trigger.setWaterMark(v);
    }
}
extend(DynamicYearCombo, {
    Static: 1,
    Dynamic: 2,
});
