import {
    shortcut,
    Widget,
    chunk,
    map,
    toPix,
    extend,
    VerticalLayout,
    CenterAdaptLayout,
    parseDateTime,
    each,
    checkDateVoid,
    contains,
    getDate,
    parseInt,
    SIZE_CONSANTS,
    LogicFactory
} from "@/core";
import { TextItem, ButtonGroup } from "@/base";
import { YearPicker } from "../date/calendar";

@shortcut()
export class StaticYearMonthCard extends Widget {
    static xtype = "bi.static_year_month_card";

    static EVENT_CHANGE = "EVENT_CHANGE";

    props = { baseCls: "bi-static-year-month-card", behaviors: {} };

    _createMonths() {
        // 纵向排列月
        const month = [1, 7, 2, 8, 3, 9, 4, 10, 5, 11, 6, 12];
        const items = chunk(month, 2);

        return map(items, (i, item) =>
            map(item, (j, td) => {
                return {
                    type: TextItem.xtype,
                    cls: "bi-list-item-select",
                    textAlign: "center",
                    whiteSpace: "nowrap",
                    once: false,
                    forceSelected: true,
                    height: toPix(SIZE_CONSANTS.LIST_ITEM_HEIGHT, 1),
                    width: 38,
                    value: td,
                    text: td,
                    ref: _ref => {
                        this.monthMap[j === 0 ? i : i + 6] = _ref;
                    },
                };
            })
        );
    }

    render() {
        const o = this.options;
        this.monthMap = {};

        return {
            type: VerticalLayout.xtype,
            items: [
                {
                    type: YearPicker.xtype,
                    cls: "bi-split-bottom",
                    min: o.min,
                    max: o.max,
                    ref: _ref => {
                        this.yearPicker = _ref;
                    },
                    behaviors: o.behaviors,
                    height: 30,
                    listeners: [
                        {
                            eventName: YearPicker.EVENT_CHANGE,
                            action: () => {
                                const value = this.yearPicker.getValue();
                                this._checkMonthStatus(value);
                                this._setYear(value);
                            },
                        }
                    ],
                },
                {
                    el: {
                        type: ButtonGroup.xtype,
                        behaviors: o.behaviors,
                        ref: _ref => {
                            this.month = _ref;
                        },
                        items: this._createMonths(),
                        layouts: [
                            LogicFactory.createLogic(
                                "table",
                                extend(
                                    {
                                        dynamic: true,
                                    },
                                    {
                                        columns: 2,
                                        rows: 6,
                                        columnSize: [1 / 2, 1 / 2],
                                        rowSize: SIZE_CONSANTS.LIST_ITEM_HEIGHT + 1,
                                    }
                                )
                            ),
                            {
                                type: CenterAdaptLayout.xtype,
                                vgap: 1,
                                hgap: 2,
                            }
                        ],
                        value: o.value,
                        listeners: [
                            {
                                eventName: ButtonGroup.EVENT_CHANGE,
                                action: () => {
                                    this.selectedYear = this.yearPicker.getValue();
                                    this.selectedMonth = this.month.getValue()[0];
                                    this.fireEvent(StaticYearMonthCard.EVENT_CHANGE);
                                },
                            }
                        ],
                    },
                    vgap: 5,
                }
            ],
        };
    }

    created() {
        this._checkMonthStatus(this.selectedYear);
    }

    _checkMonthStatus(year) {
        const o = this.options;
        const minDate = parseDateTime(o.min, "%Y-%X-%d"),
            maxDate = parseDateTime(o.max, "%Y-%X-%d");
        const minYear = minDate.getFullYear(),
            maxYear = maxDate.getFullYear();
        let minMonth = 0;
        let maxMonth = 11;
        minYear === year && (minMonth = minDate.getMonth());
        maxYear === year && (maxMonth = maxDate.getMonth());
        const yearInvalid = year < minYear || year > maxYear;
        each(this.monthMap, (month, obj) => {
            const monthInvalid = month < minMonth || month > maxMonth;
            obj.setEnable(!yearInvalid && !monthInvalid);
        });
    }

    _setYear(year) {
        const o = this.options;

        const dateVoid = checkDateVoid(year, this.selectedMonth, 1, o.min, o.max);

        //  在切换年的时候,如果月份不在区间内了,取消选中
        if (contains(["y", "m"], dateVoid[0])) {
            this.selectedYear = year;
            this.month.setValue();

            return;
        }

        this.selectedYear = year;
        this.month.setValue(this.selectedMonth);
    }

    setMinDate(minDate) {
        if (this.options.min !== minDate) {
            this.options.min = minDate;
            this.yearPicker.setMinDate(minDate);
            this._checkMonthStatus(this.selectedYear);
        }
    }

    setMaxDate(maxDate) {
        if (this.options.max !== maxDate) {
            this.options.max = maxDate;
            this.yearPicker.setMaxDate(maxDate);
            this._checkMonthStatus(this.selectedYear);
        }
    }

    getValue() {
        return {
            year: this.selectedYear,
            month: this.selectedMonth,
        };
    }

    setValue(obj) {
        const o = this.options;
        const newObj = {};
        newObj.year = obj.year || 0;
        newObj.month = obj.month || 0;
        if (newObj.year === 0 || newObj.month === 0 || checkDateVoid(newObj.year, newObj.month, 1, o.min, o.max)[0]) {
            const year = newObj.year || getDate().getFullYear();
            this.selectedYear = year;
            this.selectedMonth = "";
            this.yearPicker.setValue(year);
            this.month.setValue();
        } else {
            this.selectedYear = parseInt(newObj.year);
            this.selectedMonth = parseInt(newObj.month);
            this.yearPicker.setValue(this.selectedYear);
            this.month.setValue(this.selectedMonth);
        }
        this._checkMonthStatus(this.selectedYear);
    }
}
