import {
    shortcut,
    Widget,
    map,
    extend,
    VerticalLayout,
    parseDateTime,
    parseInt,
    each,
    checkDateVoid,
    getDate,
    getQuarterName,
    getQuarter,
    SIZE_CONSANTS
} from "@/core";
import { TextItem, ButtonGroup } from "@/base";
import { YearPicker } from "../date/calendar";

@shortcut()
export class StaticYearQuarterCard extends Widget {
    static xtype = "bi.static_year_quarter_card";

    props = { baseCls: "bi-static-year-quarter-card", behaviors: {} };

    static EVENT_CHANGE = "EVENT_CHANGE";

    _createQuarter() {
        const items = [
            {
                text: getQuarterName(1),
                value: 1,
            },
            {
                text: getQuarterName(2),
                value: 2,
            },
            {
                text: getQuarterName(3),
                value: 3,
            },
            {
                text: getQuarterName(4),
                value: 4,
            }
        ];
        
        return map(items, (j, item) => extend(item, {
            type: TextItem.xtype,
            cls: "bi-border-radius bi-list-item-select",
            textAlign: "center",
            whiteSpace: "nowrap",
            once: false,
            forceSelected: true,
            height: SIZE_CONSANTS.TOOL_BAR_HEIGHT,
            ref: _ref => {
                this.quarterMap[j + 1] = _ref;
            },
        }));
    }

    render() {
        const o = this.options;
        this.quarterMap = {};
        
        return {
            type: VerticalLayout.xtype,
            items: [
                {
                    type: YearPicker.xtype,
                    cls: "bi-split-bottom",
                    ref: _ref => {
                        this.yearPicker = _ref;
                    },
                    min: o.min,
                    max: o.max,
                    behaviors: o.behaviors,
                    height: 30,
                    listeners: [
                        {
                            eventName: YearPicker.EVENT_CHANGE,
                            action: () => {
                                const value = this.yearPicker.getValue();
                                this._checkQuarterStatus(value);
                                this.setValue({
                                    year: value,
                                    quarter: this.selectedQuarter,
                                });
                            },
                        }
                    ],
                },
                {
                    el: {
                        type: ButtonGroup.xtype,
                        behaviors: o.behaviors,
                        ref: _ref => {
                            this.quarter = _ref;
                        },
                        items: this._createQuarter(),
                        layouts: [
                            {
                                type: VerticalLayout.xtype,
                                vgap: 10,
                                hgap: 12,
                            }
                        ],
                        value: o.value,
                        listeners: [
                            {
                                eventName: ButtonGroup.EVENT_CHANGE,
                                action: () => {
                                    this.selectedYear =
                                        this.yearPicker.getValue();
                                    this.selectedQuarter = this.quarter.getValue()[0];
                                    this.fireEvent(
                                        StaticYearQuarterCard.EVENT_CHANGE
                                    );
                                },
                            }
                        ],
                    },
                    vgap: 5,
                }
            ],
        };
    }

    _checkQuarterStatus(year) {
        const o = this.options;
        const minDate = parseDateTime(o.min, "%Y-%X-%d"),
            maxDate = parseDateTime(o.max, "%Y-%X-%d");
        const minYear = minDate.getFullYear(),
            maxYear = maxDate.getFullYear();
        let minQuarter = 1;
        let maxQuarter = 4;
        minYear === year && (minQuarter = parseInt(getQuarter(minDate)));
        maxYear === year && (maxQuarter = parseInt(getQuarter(maxDate)));
        const yearInvalid = year < minYear || year > maxYear;
        each(this.quarterMap, (quarter, obj) => {
            const quarterInvalid = quarter < minQuarter || quarter > maxQuarter;
            obj.setEnable(!yearInvalid && !quarterInvalid);
        });
    }

    setMinDate(minDate) {
        if (this.options.min !== minDate) {
            this.options.min = minDate;
            this.yearPicker.setMinDate(minDate);
            this._checkQuarterStatus(this.selectedYear);
        }
    }

    setMaxDate(maxDate) {
        if (this.options.max !== maxDate) {
            this.options.max = maxDate;
            this.yearPicker.setMaxDate(maxDate);
            this._checkQuarterStatus(this.selectedYear);
        }
    }

    getValue() {
        return {
            year: this.selectedYear,
            quarter: this.selectedQuarter,
        };
    }

    setValue(obj) {
        const o = this.options;
        const newObj = {};
        newObj.year = obj.year || 0;
        newObj.quarter = obj.quarter || 0;
        if (
            newObj.quarter === 0 ||
            newObj.year === 0 ||
            checkDateVoid(newObj.year, newObj.quarter, 1, o.min, o.max)[0]
        ) {
            const year = newObj.year || getDate().getFullYear();
            this.selectedYear = year;
            this.selectedQuarter = "";
            this.yearPicker.setValue(year);
            this.quarter.setValue();
        } else {
            this.selectedYear = parseInt(newObj.year);
            this.selectedQuarter = parseInt(newObj.quarter);
            this.yearPicker.setValue(this.selectedYear);
            this.quarter.setValue(this.selectedQuarter);
        }
        this._checkQuarterStatus(this.selectedYear);
    }
}
