import {
    shortcut,
    Widget,
    toPix,
    i18nText,
    VerticalLayout,
    GridLayout,
    print,
    getDate,
    checkDateVoid,
    createItems,
    getQuarter,
    SIZE_CONSANTS
} from "@/core";
import { DynamicYearQuarterCombo } from "./combo.yearquarter";
import { TextButton, Tab } from "@/base";
import { DynamicDateCombo, DynamicDatePopup, DynamicDateHelper } from "../dynamicdate";
import { DynamicYearCard } from "../year/card.dynamic.year";
import { LinearSegment } from "@/case";
import { DynamicYearQuarterCard } from "./card.dynamic.yearquarter";
import { StaticYearQuarterCard } from "./card.static.yearquarter";

@shortcut()
export class DynamicYearQuarterPopup extends Widget {
    static xtype = "bi.dynamic_year_quarter_popup";

    constants = { tabHeight: 40, buttonHeight: 24 };
    props = {
        baseCls: "bi-year-quarter-popup",
        behaviors: {},
        min: "1900-01-01",
        max: "2099-12-31",
        width: 180,
        supportDynamic: true,
        isPreview: false, // 是否是预览状态
    };

    static BUTTON_CLEAR_EVENT_CHANGE = "BUTTON_CLEAR_EVENT_CHANGE";
    static BUTTON_lABEL_EVENT_CHANGE = "BUTTON_lABEL_EVENT_CHANGE";
    static BUTTON_OK_EVENT_CHANGE = "BUTTON_OK_EVENT_CHANGE";
    static EVENT_CHANGE = "EVENT_CHANGE";

    render() {
        this.storeValue = { type: DynamicYearQuarterCombo.Static };

        return {
            type: VerticalLayout.xtype,
            items: [
                {
                    el: this._getTabJson(),
                },
                {
                    el: {
                        type: GridLayout.xtype,
                        items: [
                            [
                                {
                                    type: TextButton.xtype,
                                    cls: "bi-split-top bi-high-light",
                                    shadow: true,
                                    textHeight: toPix(
                                        SIZE_CONSANTS.TOOL_BAR_HEIGHT,
                                        1
                                    ),
                                    text: i18nText("BI-Basic_Clear"),
                                    listeners: [
                                        {
                                            eventName: TextButton.EVENT_CHANGE,
                                            action: () => {
                                                this.fireEvent(
                                                    DynamicYearQuarterPopup.BUTTON_CLEAR_EVENT_CHANGE
                                                );
                                            },
                                        }
                                    ],
                                },
                                {
                                    type: TextButton.xtype,
                                    cls: "bi-split-left bi-split-right bi-high-light bi-split-top",
                                    textHeight: toPix(
                                        SIZE_CONSANTS.TOOL_BAR_HEIGHT,
                                        1
                                    ),
                                    shadow: true,
                                    text: i18nText("BI-Basic_Current_Quarter"),
                                    disabled: this._checkTodayValid(),
                                    ref: _ref => {
                                        this.textButton = _ref;
                                    },
                                    listeners: [
                                        {
                                            eventName: TextButton.EVENT_CHANGE,
                                            action: () => {
                                                this.fireEvent(
                                                    DynamicYearQuarterPopup.BUTTON_lABEL_EVENT_CHANGE
                                                );
                                            },
                                        }
                                    ],
                                },
                                {
                                    type: TextButton.xtype,
                                    cls: "bi-split-top bi-high-light",
                                    shadow: true,
                                    textHeight: toPix(
                                        SIZE_CONSANTS.TOOL_BAR_HEIGHT,
                                        1
                                    ),
                                    text: i18nText("BI-Basic_OK"),
                                    listeners: [
                                        {
                                            eventName: TextButton.EVENT_CHANGE,
                                            action: () => {
                                                const type =
                                                    this.dateTab.getSelect();
                                                if (
                                                    type ===
                                                    DynamicDateCombo.Dynamic
                                                ) {
                                                    this.dynamicPane.checkValidation(
                                                        true
                                                    ) &&
                                                        this.fireEvent(
                                                            DynamicDatePopup.BUTTON_OK_EVENT_CHANGE
                                                        );
                                                } else {
                                                    this.fireEvent(DynamicYearQuarterPopup
                                                        .BUTTON_OK_EVENT_CHANGE
                                                    );
                                                }
                                            },
                                        }
                                    ],
                                }
                            ]
                        ],
                        height: SIZE_CONSANTS.TOOL_BAR_HEIGHT,
                    },
                }
            ],
        };
    }

    _setInnerValue() {
        if (this.dateTab.getSelect() === DynamicYearQuarterCombo.Static) {
            this.textButton.setValue(i18nText("BI-Basic_Current_Quarter"));
            this.textButton.setEnable(!this._checkTodayValid());
        } else {
            let date = DynamicDateHelper.getCalculation(
                this.dynamicPane.getInputValue()
            );
            date = print(date, "%Y-%Q");
            this.textButton.setValue(date);
            this.textButton.setEnable(false);
        }
    }

    _checkTodayValid() {
        const o = this.options;
        const today = getDate();

        return !!checkDateVoid(
            today.getFullYear(),
            today.getMonth() + 1,
            today.getDate(),
            o.min,
            o.max
        )[0];
    }

    _getTabJson() {
        const o = this.options;

        return {
            type: Tab.xtype,
            logic: {
                dynamic: true,
            },
            ref: _ref => {
                this.dateTab = _ref;
            },
            tab: {
                type: LinearSegment.xtype,
                cls: "bi-split-bottom",
                invisible: !o.supportDynamic,
                height: this.constants.tabHeight,
                items: createItems(
                    [
                        {
                            text: i18nText("BI-Basic_Year_Quarter"),
                            value: DynamicYearQuarterCombo.Static,
                        },
                        {
                            text: i18nText("BI-Basic_Dynamic_Title"),
                            value: DynamicYearQuarterCombo.Dynamic,
                        }
                    ],
                    {
                        textAlign: "center",
                    }
                ),
            },
            cardCreator: v => {
                switch (v) {
                    case DynamicYearQuarterCombo.Dynamic:
                        return {
                            type: DynamicYearQuarterCard.xtype,
                            cls: "dynamic-year-quarter-pane",
                            min: this.options.min,
                            max: this.options.max,
                            listeners: [
                                {
                                    eventName: "EVENT_CHANGE",
                                    action: () => {
                                        this._setInnerValue(this.year, v);
                                    },
                                }
                            ],
                            ref: _ref => {
                                this.dynamicPane = _ref;
                            },
                        };
                    case DynamicYearQuarterCombo.Static:
                    default:
                        return {
                            type: StaticYearQuarterCard.xtype,
                            behaviors: o.behaviors,
                            min: this.options.min,
                            max: this.options.max,
                            listeners: [
                                {
                                    eventName: DynamicYearCard.EVENT_CHANGE,
                                    action: () => {
                                        this.fireEvent(
                                            DynamicYearQuarterPopup.EVENT_CHANGE
                                        );
                                    },
                                }
                            ],
                            ref: _ref => {
                                this.year = _ref;
                            },
                        };
                }
            },
            listeners: [
                {
                    eventName: Tab.EVENT_CHANGE,
                    action: () => {
                        const v = this.dateTab.getSelect();
                        let date;
                        switch (v) {
                            case DynamicYearQuarterCombo.Static:
                                date = DynamicDateHelper.getCalculation(
                                    this.dynamicPane.getValue()
                                );
                                this.year.setValue({
                                    year: date.getFullYear(),
                                    quarter: getQuarter(date),
                                });
                                this._setInnerValue();
                                break;
                            case DynamicYearQuarterCombo.Dynamic:
                            default:
                                if (
                                    this.storeValue &&
                                    this.storeValue.type ===
                                    DynamicYearQuarterCombo.Dynamic
                                ) {
                                    this.dynamicPane.setValue(
                                        this.storeValue.value
                                    );
                                } else {
                                    this.dynamicPane.setValue({
                                        year: 0,
                                    });
                                }
                                this._setInnerValue();
                                break;
                        }
                    },
                }
            ],
        };
    }

    setMinDate(minDate) {
        if (this.options.min !== minDate) {
            this.options.min = minDate;
            this.year && this.year.setMinDate(minDate);
            this.dynamicPane && this.dynamicPane.setMinDate(minDate);
        }
    }

    setMaxDate(maxDate) {
        if (this.options.max !== maxDate) {
            this.options.max = maxDate;
            this.year && this.year.setMaxDate(maxDate);
            this.dynamicPane && this.dynamicPane.setMaxDate(maxDate);
        }
    }

    setValue(v) {
        this.storeValue = v;
        v = v || {};
        const type = v.type || DynamicDateCombo.Static;
        const value = v.value || v;
        const { isPreview } = this.options;
        let date;

        // 如果是预览状态 直接打开静态页面
        this.dateTab.setSelect(isPreview ? DynamicDateCombo.Static : type);
        switch (type) {
            case DynamicDateCombo.Dynamic:
                if (isPreview) {
                    date = DynamicDateHelper.getCalculation(value);
                    this.year.setValue({
                        year: date.getFullYear(),
                        quarter: getQuarter(date),
                    });
                } else {
                    this.dynamicPane.setValue(value);
                }
                this._setInnerValue();
                break;
            case DynamicDateCombo.Static:
            default:
                this.year.setValue(value);
                this.textButton.setValue(i18nText("BI-Basic_Current_Quarter"));
                this.textButton.setEnable(!this._checkTodayValid());
                break;
        }
    }

    getValue() {
        return {
            type: this.dateTab.getSelect(),
            value: this.dateTab.getValue(),
        };
    }
}
